@@ -1,7 +1,7 @@
 /*
  *  WINGs WMConnection function library
  * 
- *  Copyright (c) 1999 Dan Pascu
+ *  Copyright (c) 1999-2000 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
  * TODO:
  * - decide if we want to support connections with external sockets, else
  *   clean up the structure of the unneeded members.
+ * - decide what to do with all wwarning() calls that are still there.
  *
  */
 
@@ -53,11 +54,16 @@
 # define SHUT_RDWR              2
 #endif
 
-/* for SunOS */
+/* For SunOS */
 #ifndef SA_RESTART
 # define SA_RESTART             0
 #endif
 
+/* For Solaris */
+#ifndef INADDR_NONE
+# define INADDR_NONE           -1
+#endif
+
 /* Stuff for setting the sockets into non-blocking mode. */
 /*#ifdef	__POSIX_SOURCE
 # define NONBLOCK_OPT           O_NONBLOCK
@@ -67,19 +73,19 @@
 
 #define NONBLOCK_OPT            O_NONBLOCK
 
-
 #define NETBUF_SIZE             4096
 
-
 #define DEF_TIMEOUT             600   /* 600 seconds == 10 minutes */
 
 
+
 int WCErrorCode = 0;
 
-
-
 static Bool SigInitialized = False;
 
+static unsigned int DefaultTimeout = DEF_TIMEOUT;
+static unsigned int OpenTimeout = DEF_TIMEOUT;
+
 
 
 typedef struct TimeoutData {
@@ -88,7 +94,6 @@ typedef struct TimeoutData {
 } TimeoutData;
 
 
-
 typedef struct W_Connection {
     int sock;                     /* the socket we speak through */
 
@@ -102,12 +107,14 @@ typedef struct W_Connection {
     void *clientData;             /* client data */
     unsigned int uflags;          /* flags for the client */
 
-    WMBag *outputQueue;
+    WMArray *outputQueue;
     unsigned bufPos;
 
     TimeoutData sendTimeout;
+    TimeoutData openTimeout;
 
     WMConnectionState state;
+    WMConnectionTimeoutState timeoutState;
 
     char *address;
     char *service;
@@ -121,18 +128,34 @@ typedef struct W_Connection {
 
 
 
-
 static void
 clearOutputQueue(WMConnection *cPtr) /*FOLD00*/
 {
-    int i;
-
     cPtr->bufPos = 0;
+    WMEmptyArray(cPtr->outputQueue);
+}
 
-    for (i=0; i<WMGetBagItemCount(cPtr->outputQueue); i++)
-        WMReleaseData(WMGetFromBag(cPtr->outputQueue, i));
 
-    WMEmptyBag(cPtr->outputQueue);
+static void
+openTimeout(void *cdata) /*FOLD00*/
+{
+    WMConnection *cPtr = (WMConnection*) cdata;
+
+    cPtr->openTimeout.handler = NULL;
+    if (cPtr->handler.write) {
+        WMDeleteInputHandler(cPtr->handler.write);
+        cPtr->handler.write = NULL;
+    }
+    if (cPtr->state != WCConnected) {
+        cPtr->state = WCTimedOut;
+        cPtr->timeoutState = WCTWhileOpening;
+        if (cPtr->delegate && cPtr->delegate->didTimeout) {
+            (*cPtr->delegate->didTimeout)(cPtr->delegate, cPtr);
+        } else {
+            WMCloseConnection(cPtr);
+            cPtr->state = WCTimedOut; /* the above set state to WCClosed */
+        }
+    }
 }
 
 
@@ -140,17 +163,22 @@ static void
 sendTimeout(void *cdata) /*FOLD00*/
 {
     WMConnection *cPtr = (WMConnection*) cdata;
-    TimeoutData *tPtr = &cPtr->sendTimeout;
 
-    tPtr->handler = NULL;
+    cPtr->sendTimeout.handler = NULL;
     if (cPtr->handler.write) {
         WMDeleteInputHandler(cPtr->handler.write);
         cPtr->handler.write = NULL;
     }
-    if (WMGetBagItemCount(cPtr->outputQueue)>0) {
+    if (WMGetArrayItemCount(cPtr->outputQueue)>0) {
         clearOutputQueue(cPtr);
-        if (cPtr->delegate && cPtr->delegate->didTimeout)
+        cPtr->state = WCTimedOut;
+        cPtr->timeoutState = WCTWhileSending;
+        if (cPtr->delegate && cPtr->delegate->didTimeout) {
             (*cPtr->delegate->didTimeout)(cPtr->delegate, cPtr);
+        } else {
+            WMCloseConnection(cPtr);
+            cPtr->state = WCTimedOut; /* the above set state to WCClosed */
+        }
     }
 }
 
@@ -165,16 +193,20 @@ inputHandler(int fd, int mask, void *clientData) /*FOLD00*/
 
     if ((mask & WIWriteMask)) {
         if (cPtr->state == WCInProgress) {
+            Bool failed;
             int	result;
-            int	len = sizeof(result);
+            int len = sizeof(result);
 
+            WCErrorCode = 0;
             if (getsockopt(cPtr->sock, SOL_SOCKET, SO_ERROR,
                            (void*)&result, &len) == 0 && result != 0) {
                 cPtr->state = WCFailed;
                 WCErrorCode = result;
+                failed = True;
                 /* should call wsyserrorwithcode(result, ...) here? */
             } else {
                 cPtr->state = WCConnected;
+                failed = False;
             }
 
             if (cPtr->handler.write) {
@@ -182,10 +214,18 @@ inputHandler(int fd, int mask, void *clientData) /*FOLD00*/
                 cPtr->handler.write = NULL;
             }
 
+            if (cPtr->openTimeout.handler) {
+                WMDeleteTimerHandler(cPtr->openTimeout.handler);
+                cPtr->openTimeout.handler = NULL;
+            }
+
             if (cPtr->delegate && cPtr->delegate->didInitialize)
                 (*cPtr->delegate->didInitialize)(cPtr->delegate, cPtr);
 
-            if (cPtr->state == WCFailed)
+            /* we use failed and not cPtr->state here, because cPtr may be
+             * destroyed by the delegate called above if the connection failed
+             */
+            if (failed)
                 return;
         } else if (cPtr->state == WCConnected) {
             WMFlushConnection(cPtr);
@@ -217,7 +257,7 @@ setSocketNonBlocking(int sock, Bool flag) /*FOLD00*/
     state = fcntl(sock, F_GETFL, 0);
 
     if (state < 0) {
-        wsyserror("Failed to get socket flags with fcntl.");
+        /* set WCErrorCode here? -Dan*/
         return False;
     }
 
@@ -234,7 +274,7 @@ setSocketNonBlocking(int sock, Bool flag) /*FOLD00*/
     }
 
     if (fcntl(sock, F_SETFL, state) < 0) {
-        wsyserror("Failed to set socket flags with fcntl.");
+        /* set WCErrorCode here? -Dan*/
         return False;
     }
 
@@ -260,7 +300,7 @@ getSocketAddress(char* name, char* service, char* protocol) /*FOLD00*/
     static struct sockaddr_in socketaddr;
     struct servent *sp;
 
-    if (!protocol || protocol[0]=='\0')
+    if (!protocol || protocol[0]==0)
         protocol = "tcp";
 
     memset(&socketaddr, 0, sizeof(struct sockaddr_in));
@@ -271,7 +311,7 @@ getSocketAddress(char* name, char* service, char* protocol) /*FOLD00*/
      * Otherwise we expect the given name to be an address unless it is
      * NULL (any address).
      */
-    if (name && name[0]!='\0') {
+    if (name && name[0]!=0) {
         WMHost *host = WMGetHostWithName(name);
 
         if (!host)
@@ -291,7 +331,7 @@ getSocketAddress(char* name, char* service, char* protocol) /*FOLD00*/
         socketaddr.sin_addr.s_addr = htonl(INADDR_ANY);
     }
 
-    if (!service || service[0]=='\0') {
+    if (!service || service[0]==0) {
         socketaddr.sin_port = 0;
     } else if ((sp = getservbyname(service, protocol))==0) {
         char *endptr;
@@ -299,7 +339,7 @@ getSocketAddress(char* name, char* service, char* protocol) /*FOLD00*/
 
         portNumber = strtoul(service, &endptr, 10);
 
-        if (service[0]!='\0' && *endptr=='\0' && portNumber<65536) {
+        if (service[0]!=0 && *endptr==0 && portNumber<65536) {
             socketaddr.sin_port = htons(portNumber);
         } else {
             return NULL;
@@ -322,12 +362,18 @@ createConnectionWithSocket(int sock, Bool closeOnRelease) /*FOLD00*/
     wretain(cPtr);
     memset(cPtr, 0, sizeof(WMConnection));
 
+    fcntl(sock, F_SETFD, FD_CLOEXEC); /* by default close on exec */
+
     cPtr->sock = sock;
-    cPtr->sendTimeout.timeout = DEF_TIMEOUT;
+    cPtr->openTimeout.timeout = OpenTimeout;
+    cPtr->openTimeout.handler = NULL;
+    cPtr->sendTimeout.timeout = DefaultTimeout;
     cPtr->sendTimeout.handler = NULL;
     cPtr->closeOnRelease = closeOnRelease;
-    cPtr->outputQueue = WMCreateBag(16);
+    cPtr->outputQueue =
+        WMCreateArrayWithDestructor(16, (WMFreeDataProc*)WMReleaseData);
     cPtr->state = WCNotConnected;
+    cPtr->timeoutState = WCTNone;
 
     /* ignore dead pipe */
     if (!SigInitialized) {
@@ -347,7 +393,8 @@ WMCreateConnectionWithSocket(int sock, Bool closeOnRelease) /*FOLD00*/
 {
     WMConnection *cPtr;
     struct sockaddr_in clientname;
-    int size, n;
+    int size;
+    int n;
 
     cPtr = createConnectionWithSocket(sock, closeOnRelease);
     cPtr->wasNonBlocking = WMIsConnectionNonBlocking(cPtr);
@@ -395,10 +442,12 @@ WMCreateConnectionAsServerAtAddress(char *host, char *service, char *protocol) /
 {
     WMConnection *cPtr;
     struct sockaddr_in *socketaddr;
-    int sock, size, on;
+    int sock, on;
+    int size;
+
+    WCErrorCode = 0;
 
     if ((socketaddr = getSocketAddress(host, service, protocol)) == NULL) {
-        WCErrorCode = 0;
         wwarning("Bad address-service-protocol combination");
         return NULL;
     }
@@ -407,7 +456,6 @@ WMCreateConnectionAsServerAtAddress(char *host, char *service, char *protocol) /
     sock = socket(PF_INET, SOCK_STREAM, 0);
     if (sock<0) {
         WCErrorCode = errno;
-        wsyserror("Unable to create socket");
         return NULL;
     }
 
@@ -421,17 +469,12 @@ WMCreateConnectionAsServerAtAddress(char *host, char *service, char *protocol) /
 
     if (bind(sock, (struct sockaddr *)socketaddr, sizeof(*socketaddr)) < 0) {
         WCErrorCode = errno;
-        wsyserror("Unable to bind to address '%s:%hu'",
-                  inet_ntoa(socketaddr->sin_addr),
-                  ntohs(socketaddr->sin_port));
         close(sock);
         return NULL;
     }
 
     if (listen(sock, 10) < 0) {
         WCErrorCode = errno;
-        wsyserror("Unable to listen on port '%hu'",
-                  ntohs(socketaddr->sin_port));
         close(sock);
         return NULL;
     }
@@ -441,7 +484,6 @@ WMCreateConnectionAsServerAtAddress(char *host, char *service, char *protocol) /
     size = sizeof(*socketaddr);
     if (getsockname(sock, (struct sockaddr*)socketaddr, &size) < 0) {
         WCErrorCode = errno;
-        wsyserror("Unable to get socket address");
         close(sock);
         return NULL;
     }
@@ -463,17 +505,14 @@ WMCreateConnectionToAddress(char *host, char *service, char *protocol) /*FOLD00*
     struct sockaddr_in *socketaddr;
     int sock;
 
-    if (service==NULL || service[0]=='\0') {
-        WCErrorCode = 0;
-        wwarning("Bad argument - service is not specified");
-        return NULL;
-    }
+    WCErrorCode = 0;
 
-    if (host==NULL || host[0]=='\0')
+    wassertrv(service!=NULL && service[0]!=0, NULL);
+
+    if (host==NULL || host[0]==0)
         host = "localhost";
 
     if ((socketaddr = getSocketAddress(host, service, protocol)) == NULL) {
-        WCErrorCode = 0;
         wwarning("Bad address-service-protocol combination");
         return NULL;
     }
@@ -482,16 +521,12 @@ WMCreateConnectionToAddress(char *host, char *service, char *protocol) /*FOLD00*
     sock = socket(PF_INET, SOCK_STREAM, 0);
     if (sock<0) {
         WCErrorCode = errno;
-        wsyserror("Unable to create socket");
         return NULL;
     }
     /* make socket blocking while we connect. */
     setSocketNonBlocking(sock, False);
     if (connect(sock, (struct sockaddr*)socketaddr, sizeof(*socketaddr)) < 0) {
         WCErrorCode = errno;
-        wsyserror("Unable to make connection to address '%s:%hu'",
-                  inet_ntoa(socketaddr->sin_addr),
-                  ntohs(socketaddr->sin_port));
         close(sock);
         return NULL;
     }
@@ -509,22 +544,18 @@ WMConnection*
 WMCreateConnectionToAddressAndNotify(char *host, char *service, char *protocol) /*FOLD00*/
 {
     WMConnection *cPtr;
-    /*TimeoutData *tPtr;*/
     struct sockaddr_in *socketaddr;
     int sock;
     Bool isNonBlocking;
 
-    if (service==NULL || service[0]=='\0') {
-        WCErrorCode = 0;
-        wwarning("Bad argument - service is not specified");
-        return NULL;
-    }
+    WCErrorCode = 0;
 
-    if (host==NULL || host[0]=='\0')
+    wassertrv(service!=NULL && service[0]!=0, NULL);
+
+    if (host==NULL || host[0]==0)
         host = "localhost";
 
     if ((socketaddr = getSocketAddress(host, service, protocol)) == NULL) {
-        WCErrorCode = 0;
         wwarning("Bad address-service-protocol combination");
         return NULL;
     }
@@ -533,16 +564,12 @@ WMCreateConnectionToAddressAndNotify(char *host, char *service, char *protocol)
     sock = socket(PF_INET, SOCK_STREAM, 0);
     if (sock<0) {
         WCErrorCode = errno;
-        wsyserror("Unable to create socket");
         return NULL;
     }
     isNonBlocking = setSocketNonBlocking(sock, True);
     if (connect(sock, (struct sockaddr*)socketaddr, sizeof(*socketaddr)) < 0) {
         if (errno!=EINPROGRESS) {
             WCErrorCode = errno;
-            wsyserror("Unable to make connection to address '%s:%hu'",
-                      inet_ntoa(socketaddr->sin_addr),
-                      ntohs(socketaddr->sin_port));
             close(sock);
             return NULL;
         }
@@ -552,12 +579,12 @@ WMCreateConnectionToAddressAndNotify(char *host, char *service, char *protocol)
     cPtr->state = WCInProgress;
     cPtr->isNonBlocking = isNonBlocking;
 
-    /*tPtr = &cPtr->sendTimeout;
-     tPtr->handler = WMAddTimerHandler(tPtr->timeout*1000, connectTimeout, cPtr);
-    */
     cPtr->handler.write = WMAddInputHandler(cPtr->sock, WIWriteMask,
                                             inputHandler, cPtr);
 
+    cPtr->openTimeout.handler =
+        WMAddTimerHandler(cPtr->openTimeout.timeout*1000, openTimeout, cPtr);
+
     setConnectionAddress(cPtr, socketaddr);
 
     return cPtr;
@@ -573,12 +600,15 @@ removeAllHandlers(WMConnection *cPtr) /*FOLD00*/
         WMDeleteInputHandler(cPtr->handler.write);
     if (cPtr->handler.exception)
         WMDeleteInputHandler(cPtr->handler.exception);
+    if (cPtr->openTimeout.handler)
+        WMDeleteTimerHandler(cPtr->openTimeout.handler);
     if (cPtr->sendTimeout.handler)
         WMDeleteTimerHandler(cPtr->sendTimeout.handler);
 
     cPtr->handler.read = NULL;
     cPtr->handler.write = NULL;
     cPtr->handler.exception = NULL;
+    cPtr->openTimeout.handler = NULL;
     cPtr->sendTimeout.handler = NULL;
 }
 
@@ -592,8 +622,7 @@ WMDestroyConnection(WMConnection *cPtr) /*FOLD00*/
     }
 
     removeAllHandlers(cPtr);
-    clearOutputQueue(cPtr);
-    WMFreeBag(cPtr->outputQueue);
+    WMFreeArray(cPtr->outputQueue); /* will also free the items with the destructor */
 
     if (cPtr->address) {
         wfree(cPtr->address);
@@ -629,21 +658,13 @@ WMAcceptConnection(WMConnection *listener) /*FOLD00*/
     int newSock;
     WMConnection *newConnection;
 
-    if (listener->state!=WCListening) {
-        wwarning("Called 'WMAcceptConnection()' on a non-listening connection");
-        WCErrorCode = 0;
-        return NULL;
-    }
+    WCErrorCode = 0;
+    wassertrv(listener && listener->state==WCListening, NULL);
 
     size = sizeof(clientname);
     newSock = accept(listener->sock, (struct sockaddr*) &clientname, &size);
     if (newSock<0) {
-        if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
-            WCErrorCode = errno;
-            wsyserror("Could not accept connection");
-        } else {
-            WCErrorCode = 0;
-        }
+        WCErrorCode = ((errno!=EAGAIN && errno!=EWOULDBLOCK) ? errno : 0);
         return NULL;
     }
 
@@ -671,7 +692,7 @@ WMGetConnectionService(WMConnection *cPtr) /*FOLD00*/
 
 
 char*
-WMGetConnectionProtocol(WMConnection *cPtr)
+WMGetConnectionProtocol(WMConnection *cPtr) /*FOLD00*/
 {
     return cPtr->protocol;
 }
@@ -691,6 +712,13 @@ WMGetConnectionState(WMConnection *cPtr) /*FOLD00*/
 }
 
 
+WMConnectionTimeoutState
+WMGetConnectionTimeoutState(WMConnection *cPtr) /*FOLD00*/
+{
+    return cPtr->timeoutState;
+}
+
+
 Bool
 WMEnqueueConnectionData(WMConnection *cPtr, WMData *data) /*FOLD00*/
 {
@@ -700,7 +728,7 @@ WMEnqueueConnectionData(WMConnection *cPtr, WMData *data) /*FOLD00*/
     if (cPtr->state!=WCConnected)
         return False;
 
-    WMPutInBag(cPtr->outputQueue, WMRetainData(data));
+    WMAddToArray(cPtr->outputQueue, WMRetainData(data));
     return True;
 }
 
@@ -720,19 +748,19 @@ WMSendConnectionData(WMConnection *cPtr, WMData *data) /*FOLD00*/
 
     /* If we have no data just flush the queue, else try to send data */
     if (data && WMGetDataLength(data)>0) {
-        WMPutInBag(cPtr->outputQueue, WMRetainData(data));
+        WMAddToArray(cPtr->outputQueue, WMRetainData(data));
         /* If there already was something in queue, and also a write input
          * handler is established, it means we were unable to send, so
          * return and let the write handler notify us when we can send.
          */
-        if (WMGetBagItemCount(cPtr->outputQueue)>1 && cPtr->handler.write)
+        if (WMGetArrayItemCount(cPtr->outputQueue)>1 && cPtr->handler.write)
             return 0;
     }
 
     totalTransfer = 0;
 
-    while (WMGetBagItemCount(cPtr->outputQueue) > 0) {
-        data = WMGetFromBag(cPtr->outputQueue, 0);
+    while (WMGetArrayItemCount(cPtr->outputQueue) > 0) {
+        data = WMGetFromArray(cPtr->outputQueue, 0);
         dataBytes = (const unsigned char *)WMDataBytes(data);
         len = WMGetDataLength(data);
         pos = cPtr->bufPos; /* where we're left last time */
@@ -759,7 +787,6 @@ WMSendConnectionData(WMConnection *cPtr, WMData *data) /*FOLD00*/
                 default:
                     WCErrorCode = errno;
                     cPtr->state = WCDied;
-                    /*clearOutputQueue(cPtr);*/
                     removeAllHandlers(cPtr);
                     if (cPtr->delegate && cPtr->delegate->didDie)
                         (*cPtr->delegate->didDie)(cPtr->delegate, cPtr);
@@ -769,8 +796,7 @@ WMSendConnectionData(WMConnection *cPtr, WMData *data) /*FOLD00*/
             pos += bytes;
             totalTransfer += bytes;
         }
-        WMReleaseData(data);
-        WMDeleteFromBag(cPtr->outputQueue, 0);
+        WMDeleteFromArray(cPtr->outputQueue, 0);
         cPtr->bufPos = 0;
         if (tPtr->handler) {
             WMDeleteTimerHandler(tPtr->handler);
@@ -870,7 +896,6 @@ WMIsConnectionNonBlocking(WMConnection *cPtr) /*FOLD00*/
     state = fcntl(cPtr->sock, F_GETFL, 0);
 
     if (state < 0) {
-        wsyserror("Failed to get socket flags with fcntl.");
         /* If we can't use fcntl on socket, this probably also means we could
          * not use fcntl to set non-blocking mode, and since a socket defaults
          * to blocking when created, return False as the best assumption */
@@ -885,17 +910,33 @@ WMIsConnectionNonBlocking(WMConnection *cPtr) /*FOLD00*/
 #endif
 
 
-void
+Bool
 WMSetConnectionNonBlocking(WMConnection *cPtr, Bool flag) /*FOLD00*/
 {
-    if (cPtr->sock < 0)
-        return;
+    wassertrv(cPtr!=NULL && cPtr->sock>=0, False);
 
     if (cPtr->isNonBlocking == flag)
-        return;
+        return True;
 
-    if (setSocketNonBlocking(cPtr->sock, flag)==True)
+    if (setSocketNonBlocking(cPtr->sock, flag)==True) {
         cPtr->isNonBlocking = flag;
+        return True;
+    }
+
+    return False;
+}
+
+
+Bool
+WMSetConnectionCloseOnExec(WMConnection *cPtr, Bool flag)
+{
+    wassertrv(cPtr!=NULL && cPtr->sock>=0, False);
+
+    if (fcntl(cPtr->sock, F_SETFD, (flag ? FD_CLOEXEC : 0)) < 0) {
+        return False;
+    }
+
+    return True;
 }
 
 
@@ -927,13 +968,43 @@ WMSetConnectionFlags(WMConnection *cPtr, unsigned int flags) /*FOLD00*/
 }
 
 
-void
-WMSetConnectionSendTimeout(WMConnection *cPtr, unsigned int timeout) /*FOLD00*/
+WMArray*
+WMGetConnectionUnsentData(WMConnection *cPtr)
 {
-    if (timeout == 0)
-        timeout = DEF_TIMEOUT;
+    return cPtr->outputQueue;
+}
 
-    cPtr->sendTimeout.timeout = timeout;
+
+void
+WMSetConnectionDefaultTimeout(unsigned int timeout) /*FOLD00*/
+{
+    if (timeout == 0) {
+        DefaultTimeout = DEF_TIMEOUT;
+    } else {
+        DefaultTimeout = timeout;
+    }
+}
+
+
+void
+WMSetConnectionOpenTimeout(unsigned int timeout) /*FOLD00*/
+{
+    if (timeout == 0) {
+        OpenTimeout = DefaultTimeout;
+    } else {
+        OpenTimeout = timeout;
+    }
+}
+
+
+void
+WMSetConnectionSendTimeout(WMConnection *cPtr, unsigned int timeout) /*FOLD00*/
+{
+    if (timeout == 0) {
+        cPtr->sendTimeout.timeout = DefaultTimeout;
+    } else {
+        cPtr->sendTimeout.timeout = timeout;
+    }
 }
 
 
