@@ -12,9 +12,6 @@
 #define PATH_MAX 1024
 #endif
 
-#ifndef FLOPPY_PATH
-#define FLOPPY_PATH "/floppy"
-#endif
 
 typedef struct W_FilePanel {
     WMWindow *win;
@@ -45,7 +42,6 @@ typedef struct W_FilePanel {
     struct {
 	unsigned int canExit:1;
 	unsigned int canceled:1;       /* clicked on cancel */
-	unsigned int done:1;
 	unsigned int filtered:1;
 	unsigned int canChooseFiles:1;
 	unsigned int canChooseDirectories:1;
@@ -103,18 +99,18 @@ static int
 closestListItem(WMList *list, char *text, Bool exact)
 {
     WMListItem *item;
-    WMBag *items = WMGetListItems(list);
-    WMBagIterator i;
-    int len = strlen(text);
+    WMArray *items = WMGetListItems(list);
+    int i, len = strlen(text);
 
     if (len==0)
 	return -1;
 
-    WM_ITERATE_BAG(items, item, i) {
+    for(i=0; i<WMGetArrayItemCount(items); i++) {
+        item = WMGetFromArray(items, i);
         if (strlen(item->text) >= len &&
             ((exact && strcmp(item->text, text)==0) ||
              (!exact && strncmp(item->text, text, len)==0))) {
-	    return WMBagIndexForIterator(items, i);
+	    return i;
 	}
     }
 
@@ -183,7 +179,8 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
 {
     WMFilePanel *fPtr;
     WMFont *largeFont;
-    
+    WMPixmap *icon;
+
     fPtr = wmalloc(sizeof(WMFilePanel));
     memset(fPtr, 0, sizeof(WMFilePanel));
 
@@ -201,7 +198,13 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
     WMResizeWidget(fPtr->iconLabel, 64, 64);
     WMMoveWidget(fPtr->iconLabel, 0, 0);
     WMSetLabelImagePosition(fPtr->iconLabel, WIPImageOnly);
-    WMSetLabelImage(fPtr->iconLabel, scrPtr->applicationIcon);
+    icon = WMCreateApplicationIconBlendedPixmap(scrPtr, (RColor*)NULL);
+    if (icon) {
+        WMSetLabelImage(fPtr->iconLabel, icon);
+        WMReleasePixmap(icon);
+    } else {
+        WMSetLabelImage(fPtr->iconLabel, scrPtr->applicationIconPixmap);
+    }
     
     fPtr->titleLabel = WMCreateLabel(fPtr->win);
     WMResizeWidget(fPtr->titleLabel, PWIDTH-64, 64);
@@ -217,6 +220,7 @@ makeFilePanel(WMScreen *scrPtr, char *name, char *title)
     WMSetFrameRelief(fPtr->line, WRGroove);
 
     fPtr->browser = WMCreateBrowser(fPtr->win);
+    WMSetBrowserAllowEmptySelection(fPtr->browser, True);
     WMSetBrowserDelegate(fPtr->browser, &browserDelegate);
     WMSetBrowserAction(fPtr->browser, browserClick, fPtr);
     WMSetBrowserDoubleAction(fPtr->browser, browserDClick, fPtr);
@@ -373,7 +377,6 @@ WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
                                 char *path, char *name, char **fileTypes)
 {
     WMScreen *scr = WMWidgetScreen(panel->win);
-    XEvent event;
 
     if (name && !owner) {
         WMSetWindowTitle(panel->win, name);
@@ -383,7 +386,6 @@ WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
 
     WMSetFilePanelDirectory(panel, path);
 
-    panel->flags.done = 0;
     switch(panel->flags.panelType) {
     case WP_OPEN:
         if (fileTypes)
@@ -404,15 +406,9 @@ WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
 
     WMSetLabelText(panel->titleLabel, name);
 
-    scr->modalView = W_VIEW(panel->win);
     WMMapWidget(panel->win);
 
-    scr->modal = 1;
-    while (!panel->flags.done) {	
-	WMNextEvent(scr->display, &event);
-	WMHandleEvent(&event);
-    }
-    scr->modal = 0;
+    WMRunModalLoop(scr, W_VIEW(panel->win));
 
     /* Must withdraw window because the next time we map
      * it, it might have a different transient owner. 
@@ -655,41 +651,19 @@ showError(WMScreen *scr, WMWindow *owner, char *s, char *file)
 static void
 createDir(WMButton *bPre, WMFilePanel *panel)
 {
-    char *directory_name;
-    char *directory;
-    char *file;
-    char *s;
+    char *dirName, *directory, *file, *s;
     WMScreen *scr = WMWidgetScreen(panel->win);
-    WMInputPanel *_panel;
 
-    _panel = WMCreateInputPanel(scr, panel->win,
-            "Create Directory", "Enter directory name", "", "OK", "Cancel");
-    scr->modalView = W_VIEW(_panel->win);
-    WMMapWidget(_panel->win);
-    scr->modal = 1;
-    while (!_panel->done || WMScreenPending(scr)) {
-        XEvent event;
-        WMNextEvent(scr->display, &event);
-        WMHandleEvent(&event);
-    }
-    scr->modal = 0;
-
-    if (_panel->result == WAPRDefault)
-        directory_name = WMGetTextFieldText(_panel->text);
-    else {
-        WMDestroyInputPanel(_panel);
+    dirName = WMRunInputPanel(scr, panel->win, "Create Directory",
+                              "Enter directory name", "", "OK", "Cancel");
+    if (!dirName)
         return;
-    }
-
-    WMDestroyInputPanel(_panel);
 
     directory = getCurrentFileName(panel);
-    {
-        char *s = strrchr(directory,'/');
-        if (s) s[1] = 0;
-    }
+    s = strrchr(directory,'/');
+    if (s) s[1] = 0;
 
-    if (directory_name[0] == '/') {
+    if (dirName[0] == '/') {
         directory[0] = 0;
     } else {
         while ((s = strstr(directory,"//"))) {
@@ -699,15 +673,15 @@ createDir(WMButton *bPre, WMFilePanel *panel)
         }
         if ((s = strrchr(directory, '/')) && !s[1]) s[0] = 0;
     }
-    while ((s = strstr(directory_name,"//"))) {
+    while ((s = strstr(dirName,"//"))) {
         int i;
         for (i = 2;s[i] == '/';i++);
         strcpy(s, &s[i-1]);
     }
-    if ((s = strrchr(directory_name, '/')) && !s[1]) s[0] = 0;
+    if ((s = strrchr(dirName, '/')) && !s[1]) s[0] = 0;
 
-    file = wmalloc(strlen(directory_name)+strlen(directory)+1);
-    sprintf(file, "%s/%s", directory, directory_name);
+    file = wmalloc(strlen(dirName)+strlen(directory)+1);
+    sprintf(file, "%s/%s", directory, dirName);
     while ((s = strstr(file,"//"))) {
         int i;
         for (i = 2;s[i] == '/';i++);
@@ -728,10 +702,9 @@ createDir(WMButton *bPre, WMFilePanel *panel)
     }
     else WMSetFilePanelDirectory(panel, file);
 
-    wfree(directory_name);
+    wfree(dirName);
     wfree(directory);
     wfree(file);
-
 }
 
 static void
@@ -846,20 +819,20 @@ goUnmount(WMButton *bPtr, WMFilePanel *panel)
 static void
 goFloppy(WMButton *bPtr, WMFilePanel *panel)
 {
-    char *file;
     struct stat filestat;
     WMScreen *scr = WMWidgetScreen(panel->win);
 
-    file = FLOPPY_PATH;
-    if (stat(FLOPPY_PATH,&filestat)) {
-        showError(scr, panel->win, "An error occured browsing '%s'.", file);
+    if (stat(WINGsConfiguration.floppyPath, &filestat)) {
+        showError(scr, panel->win, "An error occured browsing '%s'.",
+                  WINGsConfiguration.floppyPath);
         return;
     } else if (!S_ISDIR(filestat.st_mode)) {
-        showError(scr, panel->win, "'%s' is not a directory.", file);
+        showError(scr, panel->win, "'%s' is not a directory.",
+                  WINGsConfiguration.floppyPath);
         return;
     }
     
-    WMSetFilePanelDirectory(panel, FLOPPY_PATH);
+    WMSetFilePanelDirectory(panel, WINGsConfiguration.floppyPath);
 }
 
 
@@ -1004,7 +977,7 @@ buttonClick(WMButton *bPtr, WMFilePanel *panel)
 
     range.count = range.position = 0;
     WMSelectTextFieldRange(panel->fileField, range);
-    panel->flags.done = 1;
+    WMBreakModalLoop(WMWidgetScreen(bPtr));
 }
 
 
