@@ -4,10 +4,98 @@
 
 #include <wraster.h>
 #include <assert.h>
+#include <X11/Xlocale.h>
 
 static char *makeFontSetOfSize(char *fontset, int size);
 
 
+
+/* XLFD pattern matching */
+static char*
+xlfd_get_element (const char *xlfd, int index)
+{
+    const char *p = xlfd;
+    while (*p != 0) {
+	if (*p == '-' && --index == 0) {
+	    const char *end = strchr(p + 1, '-');
+	    char *buf;
+	    size_t len;
+	    if (end == 0) end = p + strlen(p);
+	    len = end - (p + 1);
+	    buf = wmalloc(len);
+	    memcpy(buf, p + 1, len);
+	    buf[len] = 0;
+	    return buf;
+	}
+	p++;
+    }
+    return strdup("*");
+}
+
+/* XLFD pattern matching */
+static char*
+generalize_xlfd (const char *xlfd)
+{
+    char *buf;
+    int len;
+    char *weight = xlfd_get_element(xlfd, 3);
+    char *slant  = xlfd_get_element(xlfd, 4);
+    char *pxlsz  = xlfd_get_element(xlfd, 7);
+
+#define Xstrlen(A) ((A)?strlen(A):0)
+    len = Xstrlen(xlfd)+Xstrlen(weight)+Xstrlen(slant)+Xstrlen(pxlsz)*2+50;
+#undef Xstrlen
+
+    buf = wmalloc(len + 1);
+    snprintf(buf, len + 1, "%s,-*-*-%s-%s-*-*-%s-*-*-*-*-*-*-*,"
+	     "-*-*-*-*-*-*-%s-*-*-*-*-*-*-*,*",
+	     xlfd, weight, slant, pxlsz, pxlsz);
+
+    wfree(pxlsz);
+    wfree(slant);
+    wfree(weight);
+
+    return buf;
+}
+
+/* XLFD pattern matching */
+static XFontSet
+W_CreateFontSetWithGuess(Display *dpy, char *xlfd, char ***missing,
+			int *nmissing, char **def_string)
+{
+    XFontSet fs = XCreateFontSet(dpy, xlfd, missing, nmissing, def_string);
+
+    if (fs != NULL && *nmissing == 0) return fs;
+
+    /* for non-iso8859-1 language and iso8859-1 specification
+       (this fontset is only for pattern analysis) */
+    if (fs == NULL) {
+	char *old_locale = setlocale(LC_CTYPE, NULL);
+	if (*nmissing != 0) XFreeStringList(*missing);
+	setlocale(LC_CTYPE, "C");
+	fs = XCreateFontSet(dpy, xlfd, missing, nmissing, def_string);
+	setlocale(LC_CTYPE, old_locale);
+    }
+
+    /* make XLFD font name for pattern analysis */
+    if (fs != NULL) {
+	XFontStruct **fontstructs;
+	char **fontnames;
+	if (XFontsOfFontSet(fs, &fontstructs, &fontnames) > 0)
+	    xlfd = fontnames[0];
+    }
+
+    xlfd = generalize_xlfd (xlfd);
+
+    if (*nmissing != 0) XFreeStringList(*missing);
+    if (fs != 0) XFreeFontSet(dpy, fs);
+
+    fs = XCreateFontSet(dpy, xlfd, missing, nmissing, def_string);
+
+    wfree(xlfd);
+    return fs;
+}
+
 WMFont*
 WMCreateFontSet(WMScreen *scrPtr, char *fontName)
 {
@@ -33,8 +121,8 @@ WMCreateFontSet(WMScreen *scrPtr, char *fontName)
 
     font->screen = scrPtr;
 
-    font->font.set = XCreateFontSet(display, fontName, &missing, &nmissing,
-				    &defaultString);
+    font->font.set = W_CreateFontSetWithGuess(display, fontName, &missing,
+					     &nmissing, &defaultString);
     if (nmissing > 0 && font->font.set) {
 	int i;
 	
@@ -363,3 +451,141 @@ makeFontSetOfSize(char *fontset, int size)
     return newfs;
 }
 
+
+static void
+changeFontProp(char *fname, char *newprop, int which)
+{
+	char before[128], prop[128], after[128];
+	char *ptr, *bptr;
+	int part=0;
+	
+	if(!fname || !prop)
+		return;
+
+	ptr = fname;
+	bptr = before;
+	while (*ptr) {
+		if(*ptr == '-') {
+    	    *bptr = 0;
+			if(part==which) bptr = prop;
+			else if(part==which+1) bptr = after;
+    	  	*bptr++ = *ptr;
+			part++;
+		} else {
+    	  *bptr++ = *ptr;
+		}  ptr++;
+	}*bptr = 0;
+	snprintf(fname, 255, "%s-%s%s", before, newprop, after); 
+}
+
+
+WMFont *
+WMNormalizeFont(WMScreen *scr, WMFont *font)
+{   
+    WMFont *newfont=NULL;
+    char fname[256];
+    
+    if(!scr || !font)
+        return NULL;
+    
+    snprintf(fname, 255, font->name); 
+    changeFontProp(fname, "medium", 2);
+	changeFontProp(fname, "r", 3);
+    newfont = WMCreateNormalFont(scr, fname);
+    
+    if(!newfont)
+        return NULL;
+    
+    return newfont;
+}
+
+
+WMFont *
+WMStrengthenFont(WMScreen *scr, WMFont *font) 
+{
+	WMFont *newfont=NULL;
+	char fname[256];
+
+	if(!scr || !font)
+        return NULL;
+
+	snprintf(fname, 255, font->name);
+	changeFontProp(fname, "bold", 2);
+	newfont = WMCreateNormalFont(scr, fname);
+
+	if(!newfont)
+        return NULL;
+
+	return newfont;
+}
+
+
+WMFont *
+WMUnstrengthenFont(WMScreen *scr, WMFont *font) 
+{
+	WMFont *newfont=NULL;
+	char fname[256];
+
+	if(!scr || !font)
+        return NULL;
+
+	snprintf(fname, 255, font->name);
+	changeFontProp(fname, "medium", 2);
+	newfont = WMCreateNormalFont(scr, fname);
+
+	if(!newfont)
+        return NULL;
+
+	return newfont;
+}
+
+
+WMFont *
+WMEmphasizeFont(WMScreen *scr, WMFont *font) 
+{
+	WMFont *newfont=NULL;
+	char fname[256];
+
+	if(!scr || !font)
+        return NULL;
+
+	snprintf(fname, 255, font->name);
+	changeFontProp(fname, "o", 3);
+	newfont = WMCreateNormalFont(scr, fname);
+
+	if(!newfont)
+        return NULL;
+
+	return newfont;
+}
+
+
+WMFont *
+WMUnemphasizeFont(WMScreen *scr, WMFont *font) 
+{
+	WMFont *newfont=NULL;
+	char fname[256];
+
+	if(!scr || !font)
+        return NULL;
+
+	snprintf(fname, 255, font->name);
+	changeFontProp(fname, "r", 3);
+	newfont = WMCreateNormalFont(scr, fname);
+
+	if(!newfont)
+        return NULL;
+
+	return newfont;
+}
+
+WMFont *
+WMGetFontOfSize(WMScreen *scr, WMFont *font, int size) 
+{
+    if(!scr || !font || size<1)
+        return NULL;
+
+    return font;
+}
+
+
