@@ -11,7 +11,7 @@ alertPanelOnClick(WMWidget *self, void *clientData)
 {
     WMAlertPanel *panel = clientData;
 
-    panel->done = 1;
+    WMBreakModalLoop(WMWidgetScreen(self));
     if (self == panel->defBtn) {
 	panel->result = WAPRDefault;
     } else if (self == panel->othBtn) {
@@ -26,16 +26,18 @@ static void
 handleKeyPress(XEvent *event, void *clientData)
 {
     WMAlertPanel *panel = (WMAlertPanel*)clientData;
-
-    if (event->xkey.keycode == panel->retKey && panel->defBtn) {
+    KeySym ksym;
+    
+    XLookupString(&event->xkey, NULL, 0, &ksym, NULL);
+        
+    if (ksym == XK_Return && panel->defBtn) {
 	WMPerformButtonClick(panel->defBtn);
-    }
-    if (event->xkey.keycode == panel->escKey) {
+    } else if (ksym == XK_Escape) {
 	if (panel->altBtn || panel->othBtn) {
 	    WMPerformButtonClick(panel->othBtn ? panel->othBtn : panel->altBtn);
 	} else {
 	    panel->result = WAPRDefault;
-	    panel->done=1;
+	    WMBreakModalLoop(WMWidgetScreen(panel->win));
 	}
     }
 }
@@ -52,18 +54,30 @@ WMRunAlertPanel(WMScreen *scrPtr, WMWindow *owner,
     panel = WMCreateAlertPanel(scrPtr, owner, title, msg, defaultButton,
 			       alternateButton, otherButton);
 
-    scrPtr->modalView = W_VIEW(panel->win);
+    {
+	int px, py;
+	WMView *view = WMWidgetView(panel->win);
+	
+	if (owner) {
+	    WMView *oview = WMWidgetView(owner);
+	    WMPoint pt = WMGetViewScreenPosition(oview);
+
+	    px = (W_VIEW_WIDTH(oview)-W_VIEW_WIDTH(view))/2;
+	    py = (W_VIEW_HEIGHT(oview)-W_VIEW_HEIGHT(view))/2;
+	    
+	    px += pt.x;
+	    py += pt.y;
+	} else {
+	    px = (W_VIEW_WIDTH(scrPtr->rootView)-W_VIEW_WIDTH(view))/2;
+	    py = (W_VIEW_HEIGHT(scrPtr->rootView)-W_VIEW_HEIGHT(view))/2;
+	}
+	WMSetWindowInitialPosition(panel->win, px, py);
+    }
+
     WMMapWidget(panel->win);
 
-    scrPtr->modal = 1;
-    while (!panel->done || WMScreenPending(scrPtr)) {
-	XEvent event;
-	
-	WMNextEvent(scrPtr->display, &event);
-	WMHandleEvent(&event);
-    }
-    scrPtr->modal = 0;
-
+    WMRunModalLoop(scrPtr, W_VIEW(panel->win));
+    
     tmp = panel->result;
 
     WMDestroyAlertPanel(panel);
@@ -87,22 +101,21 @@ WMCreateAlertPanel(WMScreen *scrPtr, WMWindow *owner,
 		   char *alternateButton, char *otherButton)
 {
     WMAlertPanel *panel;
-    int x, dw=0, aw=0, ow=0, w;
-    
-    
+    int dw=0, aw=0, ow=0, w;
+    WMBox *hbox;
+    WMPixmap *icon;
+
+
     panel = wmalloc(sizeof(WMAlertPanel));
     memset(panel, 0, sizeof(WMAlertPanel));
 
-        
-    panel->retKey = XKeysymToKeycode(scrPtr->display, XK_Return);
-    panel->escKey = XKeysymToKeycode(scrPtr->display, XK_Escape);
-
-    if (owner)
+    if (owner) {
 	panel->win = WMCreatePanelWithStyleForWindow(owner, "alertPanel",
 						     WMTitledWindowMask);
-    else
+    } else {
 	panel->win = WMCreateWindowWithStyle(scrPtr, "alertPanel",
 					     WMTitledWindowMask);
+    }
 
     WMSetWindowInitialPosition(panel->win,
 	 (scrPtr->rootView->size.width - WMWidgetWidth(panel->win))/2,
@@ -110,14 +123,27 @@ WMCreateAlertPanel(WMScreen *scrPtr, WMWindow *owner,
 
     WMSetWindowTitle(panel->win, "");
 
-    if (scrPtr->applicationIcon) {		
-	panel->iLbl = WMCreateLabel(panel->win);
-	WMResizeWidget(panel->iLbl, scrPtr->applicationIcon->width,
-		       scrPtr->applicationIcon->height);
-	WMMoveWidget(panel->iLbl, 8 + (64 - scrPtr->applicationIcon->width)/2,
-		     (75 - scrPtr->applicationIcon->height)/2);
-	WMSetLabelImage(panel->iLbl, scrPtr->applicationIcon);
-	WMSetLabelImagePosition(panel->iLbl, WIPImageOnly);
+    panel->vbox = WMCreateBox(panel->win);
+    WMSetViewExpandsToParent(WMWidgetView(panel->vbox), 0, 0, 0, 0);
+    WMSetBoxHorizontal(panel->vbox, False);
+    WMMapWidget(panel->vbox);
+
+    hbox = WMCreateBox(panel->vbox);
+    WMSetBoxBorderWidth(hbox, 5);
+    WMSetBoxHorizontal(hbox, True);
+    WMMapWidget(hbox);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(hbox), False, True, 74, 0, 5);
+
+    panel->iLbl = WMCreateLabel(hbox);
+    WMSetLabelImagePosition(panel->iLbl, WIPImageOnly);
+    WMMapWidget(panel->iLbl);
+    WMAddBoxSubview(hbox, WMWidgetView(panel->iLbl), False, True, 64, 0, 10);
+    icon = WMCreateApplicationIconBlendedPixmap(scrPtr, (RColor*)NULL);
+    if (icon) {
+        WMSetLabelImage(panel->iLbl, icon);
+        WMReleasePixmap(icon);
+    } else {
+	WMSetLabelImage(panel->iLbl, scrPtr->applicationIconPixmap);
     }
 
     if (title) {
@@ -125,38 +151,47 @@ WMCreateAlertPanel(WMScreen *scrPtr, WMWindow *owner,
 	
 	largeFont = WMBoldSystemFontOfSize(scrPtr, 24);
 
-	panel->tLbl = WMCreateLabel(panel->win);
-	WMMoveWidget(panel->tLbl, 80, (80 - WMFontHeight(largeFont))/2);
-	WMResizeWidget(panel->tLbl, 400 - 70, WMFontHeight(largeFont)+4);
+	panel->tLbl = WMCreateLabel(hbox);
+	WMMapWidget(panel->tLbl);
+	WMAddBoxSubview(hbox, WMWidgetView(panel->tLbl), True, True, 
+			64, 0, 0);
 	WMSetLabelText(panel->tLbl, title);
 	WMSetLabelTextAlignment(panel->tLbl, WALeft);
 	WMSetLabelFont(panel->tLbl, largeFont);
-	
+
 	WMReleaseFont(largeFont);
     }
 
+    /* create divider line */
+    
+    panel->line = WMCreateFrame(panel->win);
+    WMMapWidget(panel->line);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(panel->line), False, True,
+		    2, 2, 5);
+    WMSetFrameRelief(panel->line, WRGroove);
+
 
     if (msg) {
-	panel->mLbl = WMCreateLabel(panel->win);
-	WMMoveWidget(panel->mLbl, 10, 83);
-	WMResizeWidget(panel->mLbl, 380, WMFontHeight(scrPtr->normalFont)*4);
+	panel->mLbl = WMCreateLabel(panel->vbox);
+	WMSetLabelWraps(panel->mLbl, True);
+	WMMapWidget(panel->mLbl);
+	WMAddBoxSubview(panel->vbox, WMWidgetView(panel->mLbl), True, True,
+			WMFontHeight(scrPtr->normalFont)*4, 0, 5);
 	WMSetLabelText(panel->mLbl, msg);
 	WMSetLabelTextAlignment(panel->mLbl, WACenter);
     }
     
+    hbox = WMCreateBox(panel->vbox);
+    WMSetBoxBorderWidth(hbox, 10);
+    WMSetBoxHorizontal(hbox, True);
+    WMMapWidget(hbox);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(hbox), False, True, 44, 0, 0);
     
-    /* create divider line */
-    
-    panel->line = WMCreateFrame(panel->win);
-    WMMoveWidget(panel->line, 0, 80);
-    WMResizeWidget(panel->line, 400, 2);
-    WMSetFrameRelief(panel->line, WRGroove);
-
     /* create buttons */
     if (otherButton) 
-	ow = WMWidthOfString(scrPtr->normalFont, otherButton, 
+	ow = WMWidthOfString(scrPtr->normalFont, otherButton,
 			     strlen(otherButton));
-    
+
     if (alternateButton)
 	aw = WMWidthOfString(scrPtr->normalFont, alternateButton, 
 			     strlen(alternateButton));
@@ -165,47 +200,52 @@ WMCreateAlertPanel(WMScreen *scrPtr, WMWindow *owner,
 	dw = WMWidthOfString(scrPtr->normalFont, defaultButton,
 			     strlen(defaultButton));
     
-    w = dw + (scrPtr->buttonArrow ? scrPtr->buttonArrow->width : 0);
-    if (aw > w)
-	w = aw;
-    if (ow > w)
-	w = ow;
+    dw = dw + (scrPtr->buttonArrow ? scrPtr->buttonArrow->width : 0);
+    
+    aw += 30;
+    ow += 30;
+    dw += 30;
+    
+    w = WMAX(dw, WMAX(aw, ow));
+    if ((w+10)*3 < 400) {
+	aw = w;
+	ow = w;
+	dw = w;
+    } else {
+	int t;
 
-    w += 30;
-    x = 400;
+	t = 400 - 40 - aw - ow - dw;
+	aw += t/3;
+	ow += t/3;
+	dw += t/3;
+    }
 
     if (defaultButton) {
-	x -= w + 10;
-	
-	panel->defBtn = WMCreateCommandButton(panel->win);
+	panel->defBtn = WMCreateCommandButton(hbox);
 	WMSetButtonAction(panel->defBtn, alertPanelOnClick, panel);
-	WMMoveWidget(panel->defBtn, x, 144);
-	WMResizeWidget(panel->defBtn, w, 24);
+	WMAddBoxSubviewAtEnd(hbox, WMWidgetView(panel->defBtn),
+			     False, True, dw, 0, 0);
 	WMSetButtonText(panel->defBtn, defaultButton);
 	WMSetButtonImage(panel->defBtn, scrPtr->buttonArrow);
 	WMSetButtonAltImage(panel->defBtn, scrPtr->pushedButtonArrow);
 	WMSetButtonImagePosition(panel->defBtn, WIPRight);
     }
     if (alternateButton) {
-	x -= w + 10;
-
-	panel->altBtn = WMCreateCommandButton(panel->win);
-	WMMoveWidget(panel->altBtn, x, 144);
-	WMResizeWidget(panel->altBtn, w, 24);
+	panel->altBtn = WMCreateCommandButton(hbox);
+	WMAddBoxSubviewAtEnd(hbox, WMWidgetView(panel->altBtn),
+			     False, True, aw, 0, 5);
 	WMSetButtonAction(panel->altBtn, alertPanelOnClick, panel);
 	WMSetButtonText(panel->altBtn, alternateButton);
     }
     if (otherButton) {
-	x -= w + 10;
-	
-	panel->othBtn = WMCreateCommandButton(panel->win);
+	panel->othBtn = WMCreateCommandButton(hbox);
 	WMSetButtonAction(panel->othBtn, alertPanelOnClick, panel);
-	WMMoveWidget(panel->othBtn, x, 144);
-	WMResizeWidget(panel->othBtn, w, 24);
-	WMSetButtonText(panel->othBtn, otherButton);	
+	WMAddBoxSubviewAtEnd(hbox, WMWidgetView(panel->othBtn),
+			     False, True, ow, 0, 5);
+	WMSetButtonText(panel->othBtn, otherButton);
     }
-
-    panel->done = 0;
+    
+    WMMapSubwidgets(hbox);
     
     WMCreateEventHandler(W_VIEW(panel->win), KeyPressMask,
 			 handleKeyPress, panel);
@@ -225,7 +265,7 @@ inputBoxOnClick(WMWidget *self, void *clientData)
 {
     WMInputPanel *panel = clientData;
 
-    panel->done = 1;
+    WMBreakModalLoop(WMWidgetScreen(self));
     if (self == panel->defBtn) {
 	panel->result = WAPRDefault;
     } else if (self == panel->altBtn) {
@@ -239,16 +279,18 @@ static void
 handleKeyPress2(XEvent *event, void *clientData)
 {
     WMInputPanel *panel = (WMInputPanel*)clientData;
-
-    if (event->xkey.keycode == panel->retKey && panel->defBtn) {
+    KeySym ksym;
+    
+    XLookupString(&event->xkey, NULL, 0, &ksym, NULL);
+    
+    if (ksym == XK_Return && panel->defBtn) {
 	WMPerformButtonClick(panel->defBtn);
-    }
-    if (event->xkey.keycode == panel->escKey) {
+    } else if (ksym == XK_Escape) {
 	if (panel->altBtn) {
             WMPerformButtonClick(panel->altBtn);
 	} else {
 	    /*           printf("got esc\n");*/
-	    panel->done = 1;
+	    WMBreakModalLoop(WMWidgetScreen(panel->win));
 	    panel->result = WAPRDefault;
 	}
     }
@@ -266,15 +308,30 @@ WMRunInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title,
     panel = WMCreateInputPanel(scrPtr, owner, title, msg, defaultText, 
 			       okButton, cancelButton);
 
-    WMMapWidget(panel->win);
 
-    while (!panel->done || WMScreenPending(scrPtr)) {
-	XEvent event;
+    {
+	int px, py;
+	WMView *view = WMWidgetView(panel->win);
 	
-	WMNextEvent(scrPtr->display, &event);
-	WMHandleEvent(&event);
+	if (owner) {
+	    WMView *oview = WMWidgetView(owner);
+	    WMPoint pt = WMGetViewScreenPosition(oview);
+
+	    px = (W_VIEW_WIDTH(oview)-W_VIEW_WIDTH(view))/2;
+	    py = (W_VIEW_HEIGHT(oview)-W_VIEW_HEIGHT(view))/2;
+	    
+	    px += pt.x;
+	    py += pt.y;
+	} else {
+	    px = (W_VIEW_WIDTH(scrPtr->rootView)-W_VIEW_WIDTH(view))/2;
+	    py = (W_VIEW_HEIGHT(scrPtr->rootView)-W_VIEW_HEIGHT(view))/2;
+	}
+	WMSetWindowInitialPosition(panel->win, px, py);
     }
 
+    WMMapWidget(panel->win);
+
+    WMRunModalLoop(scrPtr, W_VIEW(panel->win));
 
     if (panel->result == WAPRDefault)
 	tmp = WMGetTextFieldText(panel->text);
@@ -312,7 +369,7 @@ endedEditingObserver(void *observerData, WMNotification *notification)
 	if (panel->altBtn)
 	    WMPerformButtonClick(panel->altBtn);
 	else {
-	    panel->done = 1;
+	    WMBreakModalLoop(WMWidgetScreen(panel->win));
 	    panel->result = WAPRDefault;
 	}
 	break;
@@ -333,9 +390,6 @@ WMCreateInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title, char *msg,
     panel = wmalloc(sizeof(WMInputPanel));
     memset(panel, 0, sizeof(WMInputPanel));
 
-    panel->retKey = XKeysymToKeycode(scrPtr->display, XK_Return);
-    panel->escKey = XKeysymToKeycode(scrPtr->display, XK_Escape);
-
     if (owner)
 	panel->win = WMCreatePanelWithStyleForWindow(owner, "inputPanel",
 						     WMTitledWindowMask);
@@ -419,8 +473,6 @@ WMCreateInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title, char *msg,
 	WMSetButtonText(panel->altBtn, cancelButton);
     }
 
-    panel->done = 0;
-    
     WMCreateEventHandler(W_VIEW(panel->win), KeyPressMask,
 			 handleKeyPress2, panel);
 
@@ -434,3 +486,172 @@ WMCreateInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title, char *msg,
 
 
 
+
+static void
+handleKeyPress3(XEvent *event, void *clientData)
+{
+    WMGenericPanel *panel = (WMGenericPanel*)clientData;
+    KeySym ksym;
+    
+    XLookupString(&event->xkey, NULL, 0, &ksym, NULL);
+        
+    if (ksym == XK_Return && panel->defBtn) {
+	WMPerformButtonClick(panel->defBtn);
+    } else if (ksym == XK_Escape) {
+	if (panel->altBtn) {
+	    WMPerformButtonClick(panel->altBtn);
+	} else {
+	    panel->result = WAPRDefault;
+	    WMBreakModalLoop(WMWidgetScreen(panel->win));
+	}
+    }
+}
+
+
+void
+WMDestroyGenericPanel(WMGenericPanel *panel)
+{
+    WMUnmapWidget(panel->win);
+    WMDestroyWidget(panel->win);
+    wfree(panel);
+}
+
+
+WMGenericPanel* 
+WMCreateGenericPanel(WMScreen *scrPtr, WMWindow *owner, 
+		     char *title, char *defaultButton,
+		     char *alternateButton)
+{
+    WMGenericPanel *panel;
+    int dw=0, aw=0, w;
+    WMBox *hbox;
+    WMPixmap *icon;
+
+
+    panel = wmalloc(sizeof(WMGenericPanel));
+    memset(panel, 0, sizeof(WMGenericPanel));
+
+    if (owner) {
+	panel->win = WMCreatePanelWithStyleForWindow(owner, "genericPanel",
+						     WMTitledWindowMask);
+    } else {
+	panel->win = WMCreateWindowWithStyle(scrPtr, "genericPanel",
+					     WMTitledWindowMask);
+    }
+
+    WMSetWindowInitialPosition(panel->win,
+	 (scrPtr->rootView->size.width - WMWidgetWidth(panel->win))/2,
+	 (scrPtr->rootView->size.height - WMWidgetHeight(panel->win))/2);
+
+    WMSetWindowTitle(panel->win, "");
+
+    panel->vbox = WMCreateBox(panel->win);
+    WMSetViewExpandsToParent(WMWidgetView(panel->vbox), 0, 0, 0, 0);
+    WMSetBoxHorizontal(panel->vbox, False);
+    WMMapWidget(panel->vbox);
+
+    hbox = WMCreateBox(panel->vbox);
+    WMSetBoxBorderWidth(hbox, 5);
+    WMSetBoxHorizontal(hbox, True);
+    WMMapWidget(hbox);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(hbox), False, True, 74, 0, 5);
+
+    panel->iLbl = WMCreateLabel(hbox);
+    WMSetLabelImagePosition(panel->iLbl, WIPImageOnly);
+    WMMapWidget(panel->iLbl);
+    WMAddBoxSubview(hbox, WMWidgetView(panel->iLbl), False, True, 64, 0, 10);
+    icon = WMCreateApplicationIconBlendedPixmap(scrPtr, (RColor*)NULL);
+    if (icon) {
+        WMSetLabelImage(panel->iLbl, icon);
+        WMReleasePixmap(icon);
+    } else {
+        WMSetLabelImage(panel->iLbl, scrPtr->applicationIconPixmap);
+    }
+
+    if (title) {
+	WMFont *largeFont;
+	
+	largeFont = WMBoldSystemFontOfSize(scrPtr, 24);
+
+	panel->tLbl = WMCreateLabel(hbox);
+	WMMapWidget(panel->tLbl);
+	WMAddBoxSubview(hbox, WMWidgetView(panel->tLbl), True, True, 
+			64, 0, 0);
+	WMSetLabelText(panel->tLbl, title);
+	WMSetLabelTextAlignment(panel->tLbl, WALeft);
+	WMSetLabelFont(panel->tLbl, largeFont);
+
+	WMReleaseFont(largeFont);
+    }
+
+    /* create divider line */
+    
+    panel->line = WMCreateFrame(panel->vbox);
+    WMMapWidget(panel->line);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(panel->line), False, True,
+		    2, 2, 5);
+    WMSetFrameRelief(panel->line, WRGroove);
+
+    
+    panel->content = WMCreateFrame(panel->vbox);
+    WMMapWidget(panel->content);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(panel->content), True, True,
+		    50, 0, 5);
+    WMSetFrameRelief(panel->content, WRFlat);
+    
+    hbox = WMCreateBox(panel->vbox);
+    WMSetBoxBorderWidth(hbox, 10);
+    WMSetBoxHorizontal(hbox, True);
+    WMMapWidget(hbox);
+    WMAddBoxSubview(panel->vbox, WMWidgetView(hbox), False, True, 44, 0, 0);
+    
+    /* create buttons */
+    if (defaultButton) 
+	dw = WMWidthOfString(scrPtr->normalFont, defaultButton,
+			     strlen(defaultButton));
+
+    if (alternateButton)
+	aw = WMWidthOfString(scrPtr->normalFont, alternateButton, 
+			     strlen(alternateButton));
+    
+    
+    dw = dw + (scrPtr->buttonArrow ? scrPtr->buttonArrow->width : 0);
+    
+    aw += 30;
+    dw += 30;
+    
+    w = WMAX(dw, aw);
+    if ((w+10)*2 < 400) {
+	aw = w;
+	dw = w;
+    } else {
+	int t;
+
+	t = 400 - 40 - aw - dw;
+	aw += t/2;
+	dw += t/2;
+    }
+
+    if (defaultButton) {
+	panel->defBtn = WMCreateCommandButton(hbox);
+	WMSetButtonAction(panel->defBtn, alertPanelOnClick, panel);
+	WMAddBoxSubviewAtEnd(hbox, WMWidgetView(panel->defBtn),
+			     False, True, dw, 0, 0);
+	WMSetButtonText(panel->defBtn, defaultButton);
+	WMSetButtonImage(panel->defBtn, scrPtr->buttonArrow);
+	WMSetButtonAltImage(panel->defBtn, scrPtr->pushedButtonArrow);
+	WMSetButtonImagePosition(panel->defBtn, WIPRight);
+    }
+
+    WMMapSubwidgets(hbox);
+
+    WMCreateEventHandler(W_VIEW(panel->win), KeyPressMask,
+			 handleKeyPress3, panel);
+
+    WMRealizeWidget(panel->win);
+    WMMapSubwidgets(panel->win);
+
+    return panel;
+}
+
+
