@@ -65,6 +65,11 @@ typedef struct W_TextField {
 
 	unsigned int pointerGrabbed:1;
 
+	unsigned int ownsSelection:1;
+
+	unsigned int waitingSelection:1; /* requested selection, but
+					  * didnt get yet */
+
 	/**/
 	unsigned int notIllegalMovement:1;
     } flags;
@@ -116,6 +121,20 @@ struct W_ViewDelegate _TextFieldViewDelegate = {
 };
 
 
+
+static void lostHandler(WMView *view, Atom selection, void *cdata);
+
+static WMData *requestHandler(WMView *view, Atom selection, Atom target,
+			      void *cdata, Atom *type);
+
+
+static WMSelectionProcs selectionHandler = {
+    requestHandler,
+	lostHandler,
+	NULL
+};
+
+
 #define TEXT_WIDTH(tPtr, start)	(WMWidthOfString((tPtr)->font, \
 		   &((tPtr)->text[(start)]), (tPtr)->textLen - (start) + 1))
 
@@ -210,75 +229,76 @@ decrToFit(TextField *tPtr)
 #undef TEXT_WIDTH
 #undef TEXT_WIDTH2
 
-static Bool
-requestHandler(WMWidget *w, Atom selection, Atom target, Atom *type,
-	       void **value, unsigned *length, int *format) 
+
+
+static WMData*
+requestHandler(WMView *view, Atom selection, Atom target, void *cdata, 
+	       Atom *type)
 {
-    TextField *tPtr = w;
+    TextField *tPtr = view->self;
     int count;
     Display *dpy = tPtr->view->screen->display;
     Atom _TARGETS;
-    char *text;
-    text = XGetAtomName(tPtr->view->screen->display,target);
-    XFree(text);
-    text = XGetAtomName(tPtr->view->screen->display,selection);
-    XFree(text);
+    Atom TEXT = XInternAtom(dpy, "TEXT", False);
+    Atom COMPOUND_TEXT = XInternAtom(dpy, "COMPOUND_TEXT", False);
+    WMData *data;
 
-    *format = 32;
-    *length = 0;
-    *value = NULL;
     count = tPtr->selection.count < 0
         ? tPtr->selection.position + tPtr->selection.count
         : tPtr->selection.position;
 
-    if (target == XA_STRING ||
-            target == XInternAtom(dpy, "TEXT", False) ||
-            target == XInternAtom(dpy, "COMPOUND_TEXT", False)) {
-        *value = wstrdup(&(tPtr->text[count]));
-        *length = abs(tPtr->selection.count);
-        *format = 8;
+    if (target == XA_STRING || target == TEXT || target == COMPOUND_TEXT) {
+	
+	data = WMCreateDataWithBytes(&(tPtr->text[count]), 
+				      abs(tPtr->selection.count));
+	WMSetDataFormat(data, 8);
         *type = target;
-        return True;
+	
+        return data;
     }
     
     _TARGETS = XInternAtom(dpy, "TARGETS", False);
     if (target == _TARGETS) {
-        int *ptr;
+        Atom *ptr;
 
-        *length = 4;
-        ptr = *value = (char *) wmalloc(4 * sizeof(Atom));
+        ptr = wmalloc(4 * sizeof(Atom));
         ptr[0] = _TARGETS;
         ptr[1] = XA_STRING;
-        ptr[2] = XInternAtom(dpy, "TEXT", False);
-        ptr[3] = XInternAtom(dpy, "COMPOUND_TEXT", False);
+        ptr[2] = TEXT;
+        ptr[3] = COMPOUND_TEXT;
+	
+	data = WMCreateDataWithBytes(ptr, 4*4);
+	WMSetDataFormat(data, 32);
+	
         *type = target;
-        return True;
+        return data;
     }
-    /*
-    *target = XA_PRIMARY;
-    */
-    return False;
+
+    return NULL;
 
 }
 
 
 static void
-lostHandler(WMWidget *w, Atom selection)
+lostHandler(WMView *view, Atom selection, void *cdata)
 {
-    TextField *tPtr = (WMTextField*)w;
+    TextField *tPtr = (WMTextField*)view->self;
 
+    tPtr->flags.ownsSelection = 0;
     tPtr->selection.count = 0;
     paintTextField(tPtr);
 }
 
+
 static void
 _notification(void *observerData, WMNotification *notification)
 {
     WMTextField *to = (WMTextField*)observerData;
     WMTextField *tw = (WMTextField*)WMGetNotificationClientData(notification);
-    if (to != tw) lostHandler(to, 0);
+    if (to != tw) lostHandler(to->view, XA_PRIMARY, NULL);
 }
 
+
 WMTextField*
 WMCreateTextField(WMWidget *parent)
 {
@@ -327,8 +347,6 @@ WMCreateTextField(WMWidget *parent)
 			 |ButtonReleaseMask|ButtonPressMask|KeyPressMask|Button1MotionMask,
 			 handleTextFieldActionEvents, tPtr);
     
-    WMCreateSelectionHandler(tPtr, XA_PRIMARY, CurrentTime, requestHandler,
-             lostHandler, NULL);
     WMAddNotificationObserver(_notification, tPtr, "_lostOwnership", tPtr);
 
 
@@ -980,40 +998,70 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     int count, refresh = 0;
     int control_pressed = 0;
     int cancelSelection = 1;
+    Bool shifted, controled, modified;
+    Bool relay = True;
+    WMScreen *scr = tPtr->view->screen;
 
+    event->xkey.state &= ~scr->ignoredModifierMask;
+    
     /*printf("(%d,%d) -> ", tPtr->selection.position, tPtr->selection.count);*/
     if (((XKeyEvent *) event)->state & WM_EMACSKEYMASK)
 	control_pressed = 1;
+    
+    shifted = event->xkey.state & ShiftMask;
+    controled = event->xkey.state & ControlMask;
+    if ((event->xkey.state & ~(ShiftMask|ControlMask)) != 0) {
+	modified = True;
+    } else {
+	modified = False;
+    }
 
     count = XLookupString(&event->xkey, buffer, 63, &ksym, NULL);
     buffer[count] = '\0';
-
+    
     switch (ksym) {
      case XK_Tab:
 #ifdef XK_ISO_Left_Tab
      case XK_ISO_Left_Tab:
 #endif
-	if (event->xkey.state & ShiftMask) {
-	    if (tPtr->view->prevFocusChain) {
-		W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
-				     tPtr->view->prevFocusChain);
-		tPtr->flags.notIllegalMovement = 1;
-            }
-            data = (void*)WMBacktabTextMovement;
-	} else {
-	    if (tPtr->view->nextFocusChain) {
-		W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
-				     tPtr->view->nextFocusChain);
-		tPtr->flags.notIllegalMovement = 1;
-            }
-            data = (void*)WMTabTextMovement;
+	if (!controled && !modified) {
+	    if (shifted) {
+		if (tPtr->view->prevFocusChain) {
+		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
+					 tPtr->view->prevFocusChain);
+		    tPtr->flags.notIllegalMovement = 1;
+		}
+		data = (void*)WMBacktabTextMovement;
+	    } else {
+		if (tPtr->view->nextFocusChain) {
+		    W_SetFocusOfTopLevel(W_TopLevelOfView(tPtr->view),
+					 tPtr->view->nextFocusChain);
+		    tPtr->flags.notIllegalMovement = 1;
+		}
+		data = (void*)WMTabTextMovement;
+	    }
+	    textEvent = WMTextDidEndEditingNotification;
+
+	    relay = False;
+	}
+	break;
+
+     case XK_Escape:
+	if (!shifted && !controled && !modified) {
+	    data = (void*)WMEscapeTextMovement;
+	    textEvent = WMTextDidEndEditingNotification;
+	    
+	    relay = False;
 	}
-        textEvent = WMTextDidEndEditingNotification;
 	break;
 
      case XK_Return:
-        data = (void*)WMReturnTextMovement;
-        textEvent = WMTextDidEndEditingNotification;
+	if (!shifted && !controled && !modified) {
+	    data = (void*)WMReturnTextMovement;
+	    textEvent = WMTextDidEndEditingNotification;
+	    
+	    relay = False;
+	}
 	break;
 
      case WM_EMACSKEY_LEFT:
@@ -1024,26 +1072,30 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
      case XK_KP_Left:
 #endif
      case XK_Left:
-	if (tPtr->cursorPosition > 0) {
-	    paintCursor(tPtr);
-            if (event->xkey.state & ControlMask) {
-    	    	int i = tPtr->cursorPosition - 1;
-				
-		while (i > 0 && tPtr->text[i] != ' ') i--;
-		while (i > 0 && tPtr->text[i] == ' ') i--;
-
-		tPtr->cursorPosition = (i > 0) ? i + 1 : 0;
-            } else
-		tPtr->cursorPosition--;
-
-	    if (tPtr->cursorPosition < tPtr->viewPosition) {
-		tPtr->viewPosition = tPtr->cursorPosition;
-		refresh = 1;
-	    } else
+	if (!modified) {
+	    if (tPtr->cursorPosition > 0) {
 		paintCursor(tPtr);
+		if (event->xkey.state & ControlMask) {
+		    int i = tPtr->cursorPosition - 1;
+		    
+		    while (i > 0 && tPtr->text[i] != ' ') i--;
+		    while (i > 0 && tPtr->text[i] == ' ') i--;
+		    
+		    tPtr->cursorPosition = (i > 0) ? i + 1 : 0;
+		} else
+		    tPtr->cursorPosition--;
+		
+		if (tPtr->cursorPosition < tPtr->viewPosition) {
+		    tPtr->viewPosition = tPtr->cursorPosition;
+		    refresh = 1;
+		} else
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
-	if (event->xkey.state & ShiftMask)
-	    cancelSelection = 0;
 	break;
 
     case WM_EMACSKEY_RIGHT:
@@ -1054,30 +1106,34 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case XK_KP_Right:
 #endif
     case XK_Right:
-	if (tPtr->cursorPosition < tPtr->textLen) {
-	    paintCursor(tPtr);
-            if (event->xkey.state & ControlMask) {
-    	    	int i = tPtr->cursorPosition;
-				
-    	    	while (tPtr->text[i] && tPtr->text[i] != ' ') i++;
-    	    	while (tPtr->text[i] == ' ') i++;
-
-    	    	tPtr->cursorPosition = i;
-            } else {
-               tPtr->cursorPosition++;
-            }
-	    while (WMWidthOfString(tPtr->font,
-				&(tPtr->text[tPtr->viewPosition]),
-				tPtr->cursorPosition-tPtr->viewPosition)
-		   > tPtr->usableWidth) {
-		tPtr->viewPosition++;
-		refresh = 1;
-	    }
-	    if (!refresh)
+	if (!modified) {
+	    if (tPtr->cursorPosition < tPtr->textLen) {
 		paintCursor(tPtr);
+		if (event->xkey.state & ControlMask) {
+		    int i = tPtr->cursorPosition;
+		    
+		    while (tPtr->text[i] && tPtr->text[i] != ' ') i++;
+		    while (tPtr->text[i] == ' ') i++;
+		    
+		    tPtr->cursorPosition = i;
+		} else {
+		    tPtr->cursorPosition++;
+		}
+		while (WMWidthOfString(tPtr->font,
+				       &(tPtr->text[tPtr->viewPosition]),
+				       tPtr->cursorPosition-tPtr->viewPosition)
+		       > tPtr->usableWidth) {
+		    tPtr->viewPosition++;
+		    refresh = 1;
+		}
+		if (!refresh)
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
-	if (event->xkey.state & ShiftMask)
-	    cancelSelection = 0;
 	break;
 	
     case WM_EMACSKEY_HOME:
@@ -1088,17 +1144,21 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case XK_KP_Home:
 #endif
     case XK_Home:
-	if (tPtr->cursorPosition > 0) {
-	    paintCursor(tPtr);
-	    tPtr->cursorPosition = 0;
-	    if (tPtr->viewPosition > 0) {
-		tPtr->viewPosition = 0;
-		refresh = 1;
-	    } else
+	if (!modified && !controled) {
+	    if (tPtr->cursorPosition > 0) {
 		paintCursor(tPtr);
+		tPtr->cursorPosition = 0;
+		if (tPtr->viewPosition > 0) {
+		    tPtr->viewPosition = 0;
+		    refresh = 1;
+		} else
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
-	if (event->xkey.state & ShiftMask)
-	    cancelSelection = 0;
 	break;
 	
     case WM_EMACSKEY_END:
@@ -1109,22 +1169,26 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case XK_KP_End:
 #endif
     case XK_End:
-	if (tPtr->cursorPosition < tPtr->textLen) {
-	    paintCursor(tPtr);
-	    tPtr->cursorPosition = tPtr->textLen;
-	    tPtr->viewPosition = 0;
-	    while (WMWidthOfString(tPtr->font,
-				   &(tPtr->text[tPtr->viewPosition]),
-				   tPtr->textLen-tPtr->viewPosition)
-		   > tPtr->usableWidth) {
-		tPtr->viewPosition++;
-		refresh = 1;
-	    }
-	    if (!refresh)
+	if (!modified && !controled) {
+	    if (tPtr->cursorPosition < tPtr->textLen) {
 		paintCursor(tPtr);
+		tPtr->cursorPosition = tPtr->textLen;
+		tPtr->viewPosition = 0;
+		while (WMWidthOfString(tPtr->font,
+				       &(tPtr->text[tPtr->viewPosition]),
+				       tPtr->textLen-tPtr->viewPosition)
+		       > tPtr->usableWidth) {
+		    tPtr->viewPosition++;
+		    refresh = 1;
+		}
+		if (!refresh)
+		    paintCursor(tPtr);
+	    }
+	    if (event->xkey.state & ShiftMask)
+		cancelSelection = 0;
+	    
+	    relay = False;
 	}
-	if (event->xkey.state & ShiftMask)
-	    cancelSelection = 0;
 	break;
 	
      case WM_EMACSKEY_BS:
@@ -1132,17 +1196,21 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
              goto normal_key;
          }
      case XK_BackSpace:
-     	if (tPtr->selection.count) {
-	    WMDeleteTextFieldRange(tPtr, tPtr->selection);
-            data = (void*)WMDeleteTextEvent;
-            textEvent = WMTextDidChangeNotification;
-	} else if (tPtr->cursorPosition > 0) {
-            WMRange range;
-            range.position = tPtr->cursorPosition - 1;
-            range.count = 1;
-	    WMDeleteTextFieldRange(tPtr, range);
-            data = (void*)WMDeleteTextEvent;
-            textEvent = WMTextDidChangeNotification;
+	if (!modified && !shifted && !controled) {
+	    if (tPtr->selection.count) {
+		WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    } else if (tPtr->cursorPosition > 0) {
+		WMRange range;
+		range.position = tPtr->cursorPosition - 1;
+		range.count = 1;
+		WMDeleteTextFieldRange(tPtr, range);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    }
+	    
+	    relay = False;
 	}
 	break;
 	
@@ -1154,56 +1222,63 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
     case XK_KP_Delete:
 #endif
     case XK_Delete:
-     	if (tPtr->selection.count) {
-	    WMDeleteTextFieldRange(tPtr, tPtr->selection);
-            data = (void*)WMDeleteTextEvent;
-            textEvent = WMTextDidChangeNotification;
-	} else if (tPtr->cursorPosition < tPtr->textLen) {
-            WMRange range;
-            range.position = tPtr->cursorPosition;
-            range.count = 1;
-	    WMDeleteTextFieldRange(tPtr, range);
-            data = (void*)WMDeleteTextEvent;
-            textEvent = WMTextDidChangeNotification;
+	if (!modified && !controled && !shifted) {
+	    if (tPtr->selection.count) {
+		WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    } else if (tPtr->cursorPosition < tPtr->textLen) {
+		WMRange range;
+		range.position = tPtr->cursorPosition;
+		range.count = 1;
+		WMDeleteTextFieldRange(tPtr, range);
+		data = (void*)WMDeleteTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    }
+	    
+	    relay = False;
         }
 	break;
 
     normal_key:
      default:
-	if (count > 0 && isprint(buffer[0])) {
-            if (tPtr->selection.count)
-                WMDeleteTextFieldRange(tPtr, tPtr->selection);
-	    WMInsertTextFieldText(tPtr, buffer, tPtr->cursorPosition);
-            data = (void*)WMInsertTextEvent;
-            textEvent = WMTextDidChangeNotification;
-        } else {
-            /* should we rather break and goto cancel selection below? -Dan */
-            return;
+	if (!controled && !modified) {
+	    if (count > 0 && isprint(buffer[0])) {
+		if (tPtr->selection.count)
+		    WMDeleteTextFieldRange(tPtr, tPtr->selection);
+		WMInsertTextFieldText(tPtr, buffer, tPtr->cursorPosition);
+		data = (void*)WMInsertTextEvent;
+		textEvent = WMTextDidChangeNotification;
+	    
+		relay = False;
+	    }
         }
 	break;
     }
 
-    if (!cancelSelection) {
-    	if (tPtr->selection.count != tPtr->cursorPosition - tPtr->selection.position) {
-	    WMNotification *notif;
-	    
+    if (relay) {
+	WMRelayToNextResponder(W_VIEW(tPtr), event);
+	return;
+    }
+
+    /* Do not allow text selection in secure text fields */
+    if (cancelSelection || tPtr->flags.secure) {
+        lostHandler(tPtr->view, XA_PRIMARY, NULL);
+
+        if (tPtr->selection.count) {
+            tPtr->selection.count = 0;
+            refresh = 1;
+        }
+        tPtr->selection.position = tPtr->cursorPosition;
+    } else {
+        if (tPtr->selection.count != tPtr->cursorPosition - tPtr->selection.position) {
+
             tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
 
-            XSetSelectionOwner(tPtr->view->screen->display,
-                    	       XA_PRIMARY, tPtr->view->window,
-			       event->xbutton.time);
-            notif = WMCreateNotification("_lostOwnership", NULL, tPtr);
-            WMPostNotification(notif);
-            WMReleaseNotification(notif);
+            WMPostNotificationName("_lostOwnership", NULL, tPtr);
 
             refresh = 1;
         }
-    } else {
-    	if (tPtr->selection.count) {
-    	    tPtr->selection.count = 0;
-            refresh = 1;
-	}
-    	tPtr->selection.position = tPtr->cursorPosition;
     }
 
     /*printf("(%d,%d)\n", tPtr->selection.position, tPtr->selection.count);*/
@@ -1215,9 +1290,11 @@ handleTextFieldKeyPress(TextField *tPtr, XEvent *event)
             if (textEvent==WMTextDidBeginEditingNotification &&
                 tPtr->delegate->didBeginEditing)
                 (*tPtr->delegate->didBeginEditing)(tPtr->delegate, notif);
+
             else if (textEvent==WMTextDidEndEditingNotification &&
                      tPtr->delegate->didEndEditing)
                 (*tPtr->delegate->didEndEditing)(tPtr->delegate, notif);
+
             else if (textEvent==WMTextDidChangeNotification &&
                      tPtr->delegate->didChange)
                 (*tPtr->delegate->didChange)(tPtr->delegate, notif);
@@ -1264,25 +1341,57 @@ pointToCursorPosition(TextField *tPtr, int x)
 }
 
 
+
+static void 
+pasteText(WMView *view, Atom selection, Atom target, Time timestamp, 
+	  void *cdata, WMData *data)
+{
+    TextField *tPtr = (TextField*)view->self;
+    char *str;
+    
+    tPtr->flags.waitingSelection = 0;
+
+    if (data != NULL) {
+	str = (char*)WMDataBytes(data);
+
+	WMInsertTextFieldText(tPtr, str, tPtr->cursorPosition);
+	NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+	       (void*)WMInsertTextEvent);
+    } else {
+	int n;
+	
+	str = XFetchBuffer(tPtr->view->screen->display, &n, 0);
+		
+	if (str != NULL) {
+	    str[n] = 0;
+	    WMInsertTextFieldText(tPtr, str, tPtr->cursorPosition);
+	    XFree(str);
+	    NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+		   (void*)WMInsertTextEvent);
+	}
+    }
+}
+
+
 static void
 handleTextFieldActionEvents(XEvent *event, void *data)
 {
     TextField *tPtr = (TextField*)data;
     static int move = 0;
     static Time lastButtonReleasedEvent = 0;
+    Display *dpy = event->xany.display;
     
     CHECK_CLASS(data, WC_TextField);
 
     switch (event->type) {
      case KeyPress:
+	if (tPtr->flags.waitingSelection) {
+	    return;
+	}
         if (tPtr->flags.enabled && tPtr->flags.focused) {
             handleTextFieldKeyPress(tPtr, event);
-	    XGrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen),
-			 W_VIEW(tPtr)->window, False, 
-			 PointerMotionMask|ButtonPressMask|ButtonReleaseMask,
-			 GrabModeAsync, GrabModeAsync, None, 
-			 W_VIEW(tPtr)->screen->invisibleCursor,
-			 CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->invisibleCursor);
 	    tPtr->flags.pointerGrabbed = 1;
 	}
         break;
@@ -1291,7 +1400,11 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 	
 	if (tPtr->flags.pointerGrabbed) {
 	    tPtr->flags.pointerGrabbed = 0;
-	    XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	}
+	if (tPtr->flags.waitingSelection) {
+	    return;
 	}
 
         if (tPtr->flags.enabled && (event->xmotion.state & Button1Mask)) {
@@ -1309,40 +1422,41 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 		tPtr->viewPosition--;
 	    }
 
-	    /*if (!tPtr->selection.count) {
-		tPtr->selection.position = tPtr->cursorPosition;
-	    }*/
-	    
 	    tPtr->cursorPosition = 
 		pointToCursorPosition(tPtr, event->xmotion.x);
-	    
-	    tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
-	    /*printf("(%d,%d)\n", tPtr->selection.position, tPtr->selection.count);*/
-	    
-	    /*
-	     printf("notify %d %d\n",event->xmotion.x,tPtr->usableWidth);
-	     */
+
+            /* Do not allow text selection in secure textfields */
+            if (tPtr->flags.secure) {
+                tPtr->selection.position = tPtr->cursorPosition;
+            }
+
+            tPtr->selection.count = tPtr->cursorPosition - tPtr->selection.position;
+
+	    if (tPtr->selection.count != 0) {
+		if (!tPtr->flags.ownsSelection) {
+		    WMCreateSelectionHandler(tPtr->view,
+					     XA_PRIMARY,
+					     event->xbutton.time,
+					     &selectionHandler, NULL);
+		    tPtr->flags.ownsSelection = 1;
+		}
+	    }
 	    
 	    paintCursor(tPtr);
 	    paintTextField(tPtr);
 
 	}
-        if (move) {
-            XSetSelectionOwner(tPtr->view->screen->display,
-                       XA_PRIMARY, tPtr->view->window,  event->xmotion.time);
-            {
-                WMNotification *notif = WMCreateNotification("_lostOwnership",
-                        NULL,tPtr);
-                WMPostNotification(notif);
-                WMReleaseNotification(notif);
-            }
-        }
 	break;
 
      case ButtonPress:
 	if (tPtr->flags.pointerGrabbed) {
 	    tPtr->flags.pointerGrabbed = 0;
-	    XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	    break;
+	}
+	
+	if (tPtr->flags.waitingSelection) {
 	    break;
 	}
 
@@ -1357,22 +1471,13 @@ handleTextFieldActionEvents(XEvent *event, void *data)
 		tPtr->selection.position = tPtr->cursorPosition;
                 tPtr->selection.count = 0;
             }
-            if(textWidth < tPtr->usableWidth){
+            if (textWidth < tPtr->usableWidth) {
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
 				     event->xbutton.x - tPtr->usableWidth
                                      + textWidth);
-            }
-            else tPtr->cursorPosition = pointToCursorPosition(tPtr, 
+            } else tPtr->cursorPosition = pointToCursorPosition(tPtr,
                                      event->xbutton.x);
-            /*
-            tPtr->cursorPosition = pointToCursorPosition(tPtr, 
-                                     event->xbutton.x);
-                tPtr->cursorPosition += tPtr->usableWidth - textWidth;
-            }
 
-            tPtr->cursorPosition = pointToCursorPosition(tPtr, 
-                                     event->xbutton.x);
-                                     */
             paintTextField(tPtr);
             break;
         
@@ -1382,7 +1487,8 @@ handleTextFieldActionEvents(XEvent *event, void *data)
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
                                      event->xbutton.x);
                 paintTextField(tPtr);
-            } else if (tPtr->flags.focused) {
+            } else if (tPtr->flags.focused 
+		       && event->xbutton.button == Button1) {
                 tPtr->cursorPosition = pointToCursorPosition(tPtr, 
                                      event->xbutton.x);
 		tPtr->selection.position = tPtr->cursorPosition;
@@ -1391,51 +1497,73 @@ handleTextFieldActionEvents(XEvent *event, void *data)
             }
             if (event->xbutton.button == Button2 && tPtr->flags.enabled) {
                 char *text;
+		int n;
 
-                text = W_GetTextSelection(tPtr->view->screen, XA_PRIMARY);
-                
-                if (!text) {
-                    text = W_GetTextSelection(tPtr->view->screen,
-                              tPtr->view->screen->clipboardAtom);
-                }
-                if (!text) {
-		    text = W_GetTextSelection(tPtr->view->screen, 
-					      XA_CUT_BUFFER0);
-                }
-                if (text) {
-		    WMInsertTextFieldText(tPtr, text, tPtr->cursorPosition);
-		    XFree(text);
-                    NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
-                           (void*)WMInsertTextEvent);
-                }
+		if (!WMRequestSelection(tPtr->view, XA_PRIMARY, XA_STRING, 
+					event->xbutton.time,
+					pasteText, NULL)) {
+		    text = XFetchBuffer(tPtr->view->screen->display, &n, 0);
+		
+		    if (text) {
+			text[n] = 0;
+			WMInsertTextFieldText(tPtr, text, tPtr->cursorPosition);
+			XFree(text);
+			NOTIFY(tPtr, didChange, WMTextDidChangeNotification,
+			       (void*)WMInsertTextEvent);
+		    }
+		} else {
+		    tPtr->flags.waitingSelection = 1;
+		}
             }
             break;
+	 default:
+	    break;
         }
 	break;
 
-	case ButtonRelease:
-	    if (tPtr->flags.pointerGrabbed) {
-	    	tPtr->flags.pointerGrabbed = 0;
-	    	XUngrabPointer(WMScreenDisplay(W_VIEW(tPtr)->screen), CurrentTime);
-	    }
+     case ButtonRelease:
+	if (tPtr->flags.pointerGrabbed) {
+	    tPtr->flags.pointerGrabbed = 0;
+	    XDefineCursor(dpy, W_VIEW(tPtr)->window, 
+			  W_VIEW(tPtr)->screen->textCursor);
+	}
+	if (tPtr->flags.waitingSelection) {
+	    break;
+	}
+	
+	if (tPtr->selection.count != 0) {
+	    int start, count;
+	    XRotateBuffers(dpy, 1);
+	    
+	    count = abs(tPtr->selection.count);
+	    if (tPtr->selection.count < 0)
+		start = tPtr->selection.position - count;
+	    else
+		start = tPtr->selection.position;
+	    
+	    XStoreBuffer(dpy, &tPtr->text[start], count, 0);
+	}
 
-            move = 0;
+	move = 0;
 		
-	    if (event->xbutton.time - lastButtonReleasedEvent
-	    	<= WINGsConfiguration.doubleClickDelay) {
-	    	tPtr->selection.position = 0;
-	    	tPtr->selection.count = tPtr->textLen;
-	    	paintTextField(tPtr);
-            	XSetSelectionOwner(tPtr->view->screen->display,
-                       XA_PRIMARY, tPtr->view->window,  event->xbutton.time);
-            	{
-            	    WMNotification *notif = WMCreateNotification("_lostOwnership",
-                        NULL,tPtr);
-                    WMPostNotification(notif);
-                    WMReleaseNotification(notif);
-            	}
+        if (!tPtr->flags.secure &&
+            event->xbutton.time - lastButtonReleasedEvent
+            <= WINGsConfiguration.doubleClickDelay) {
+	    tPtr->selection.position = 0;
+	    tPtr->selection.count = tPtr->textLen;
+	    paintTextField(tPtr);
+	    
+	    if (!tPtr->flags.ownsSelection) {
+		WMCreateSelectionHandler(tPtr->view,
+					 XA_PRIMARY,
+					 event->xbutton.time,
+					 &selectionHandler, NULL);
+		tPtr->flags.ownsSelection = 1;
 	    }
-	    lastButtonReleasedEvent = event->xbutton.time;
+	    
+	    WMPostNotificationName("_lostOwnership", NULL, tPtr);
+	}
+	lastButtonReleasedEvent = event->xbutton.time;
 		
         break;
     }
@@ -1451,7 +1579,7 @@ destroyTextField(TextField *tPtr)
 #endif
 
     WMReleaseFont(tPtr->font);
-    WMDeleteSelectionHandler(tPtr, XA_PRIMARY);
+    WMDeleteSelectionHandler(tPtr->view, XA_PRIMARY, CurrentTime);
     WMRemoveNotificationObserver(tPtr);
 
     if (tPtr->text)
