@@ -28,7 +28,7 @@
 
 
 typedef struct _Panel {
-    WMFrame *frame;
+    WMBox *box;
 
     char *sectionName;
 
@@ -36,7 +36,7 @@ typedef struct _Panel {
 
     CallbackRec callbacks;
 
-    WMWindow *win;
+    WMWidget *parent;
 
     WMLabel *actL;
     WMList *actLs;
@@ -99,14 +99,13 @@ static char *keyOptions[] = {
 	"WindowShortcut2Key",
 	"WindowShortcut3Key",
 	"WindowShortcut4Key",
-#ifdef EXTEND_WINDOWSHORTCUT
 	"WindowShortcut5Key",
 	"WindowShortcut6Key",
 	"WindowShortcut7Key",
 	"WindowShortcut8Key",
 	"WindowShortcut9Key",
 	"WindowShortcut10Key",
-#endif
+	"ScreenSwitchKey",
 	"ClipRaiseKey",
 	"ClipLowerKey",
 #ifndef XKB_MODELOCK
@@ -123,7 +122,7 @@ static char*
 captureShortcut(Display *dpy, _Panel *panel)
 {
     XEvent ev;
-    KeySym ksym;
+    KeySym ksym, lksym, uksym;
     char buffer[64];
     char *key = NULL;
 
@@ -132,9 +131,10 @@ captureShortcut(Display *dpy, _Panel *panel)
 	WMNextEvent(dpy, &ev);
 	if (ev.type==KeyPress && ev.xkey.keycode!=0) {
 	    ksym = XKeycodeToKeysym(dpy, ev.xkey.keycode, 0);
-	    if (!IsModifierKey(ksym)) {
-		key=XKeysymToString(ksym);
-		panel->capturing = 0;
+            if (!IsModifierKey(ksym)) {
+                XConvertCase(ksym, &lksym, &uksym);
+                key=XKeysymToString(uksym);
+                panel->capturing = 0;
 		break;
 	    }
 	}
@@ -177,14 +177,14 @@ static void
 captureClick(WMWidget *w, void *data)
 {
     _Panel *panel = (_Panel*)data;
-    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->win));
+    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->parent));
     char *shortcut;
     
     if (!panel->capturing) {
 	panel->capturing = 1;
 	WMSetButtonText(w, _("Cancel"));
 	WMSetLabelText(panel->instructionsL, _("Press the desired shortcut key(s) or click Cancel to stop capturing."));
-	XGrabKeyboard(dpy, WMWidgetXID(panel->win), True, GrabModeAsync,
+	XGrabKeyboard(dpy, WMWidgetXID(panel->parent), True, GrabModeAsync,
 		      GrabModeAsync, CurrentTime);
 	shortcut = captureShortcut(dpy, panel);
 	if (shortcut) {
@@ -193,12 +193,12 @@ captureClick(WMWidget *w, void *data)
 	    WMSetTextFieldText(panel->shoT, shortcut);
 	    if (row>=0) {
 		if (panel->shortcuts[row])
-		    free(panel->shortcuts[row]);
+		    wfree(panel->shortcuts[row]);
 		panel->shortcuts[row] = shortcut;
 
 		WMRedisplayWidget(panel->actLs);
 	    } else {
-		free(shortcut);
+		wfree(shortcut);
 	    }
 	}
     }
@@ -220,7 +220,7 @@ clearShortcut(WMWidget *w, void *data)
 
     if (row>=0) {
 	if (panel->shortcuts[row])
-	    free(panel->shortcuts[row]);
+	    wfree(panel->shortcuts[row]);
 	panel->shortcuts[row]=NULL;
 	WMRedisplayWidget(panel->actLs);
     }
@@ -238,10 +238,10 @@ typedKeys(void *observerData, WMNotification *notification)
 	return;
 
     if (panel->shortcuts[row])
-	free(panel->shortcuts[row]);
+	wfree(panel->shortcuts[row]);
     panel->shortcuts[row] = WMGetTextFieldText(panel->shoT);
     if (strlen(panel->shortcuts[row])==0) {
-	free(panel->shortcuts[row]);
+	wfree(panel->shortcuts[row]);
 	panel->shortcuts[row] = NULL;
     }
     WMRedisplayWidget(panel->actLs);
@@ -287,7 +287,7 @@ showData(_Panel *panel)
 
 	str = GetStringForKey(keyOptions[i]);
 	if (panel->shortcuts[i])
-	    free(panel->shortcuts[i]);
+	    wfree(panel->shortcuts[i]);
 	if (str)
 	    panel->shortcuts[i] = trimstr(str);
 	else
@@ -296,7 +296,7 @@ showData(_Panel *panel)
 	if (panel->shortcuts[i] &&
 	    (strcasecmp(panel->shortcuts[i], "none")==0 
 	     || strlen(panel->shortcuts[i])==0)) {
-	    free(panel->shortcuts[i]);
+	    wfree(panel->shortcuts[i]);
 	    panel->shortcuts[i] = NULL;
 	}	    
     }
@@ -341,20 +341,21 @@ static void
 createPanel(Panel *p)
 {   
     _Panel *panel = (_Panel*)p;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     WMColor *color;
     WMFont *boldFont;
 
     panel->capturing = 0;
     
-    panel->frame = WMCreateFrame(panel->win);
-    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
-    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);
+
+
+    panel->box = WMCreateBox(panel->parent);
+    WMSetViewExpandsToParent(WMWidgetView(panel->box), 2, 2, 2, 2);
 
     boldFont = WMBoldSystemFontOfSize(scr, 12);
 
     /* **************** Actions **************** */
-    panel->actL = WMCreateLabel(panel->frame);
+    panel->actL = WMCreateLabel(panel->box);
     WMResizeWidget(panel->actL, 280, 20);
     WMMoveWidget(panel->actL, 20, 10);
     WMSetLabelFont(panel->actL, boldFont);
@@ -368,7 +369,7 @@ createPanel(Panel *p)
     WMSetLabelTextColor(panel->actL, color);
     WMReleaseColor(color);
     
-    panel->actLs = WMCreateList(panel->frame);
+    panel->actLs = WMCreateList(panel->box);
     WMResizeWidget(panel->actLs, 280, 190);
     WMMoveWidget(panel->actLs, 20, 32);
     WMSetListUserDrawProc(panel->actLs, paintItem);
@@ -408,14 +409,13 @@ createPanel(Panel *p)
     WMAddListItem(panel->actLs, _("Shortcut for window 2"));
     WMAddListItem(panel->actLs, _("Shortcut for window 3"));
     WMAddListItem(panel->actLs, _("Shortcut for window 4"));
-#ifdef EXTEND_WINDOWSHORTCUT
     WMAddListItem(panel->actLs, _("Shortcut for window 5"));
     WMAddListItem(panel->actLs, _("Shortcut for window 6"));
     WMAddListItem(panel->actLs, _("Shortcut for window 7"));
     WMAddListItem(panel->actLs, _("Shortcut for window 8"));
     WMAddListItem(panel->actLs, _("Shortcut for window 9"));
     WMAddListItem(panel->actLs, _("Shortcut for window 10"));
-#endif
+    WMAddListItem(panel->actLs, _("Switch to Next Screen/Monitor"));
     WMAddListItem(panel->actLs, _("Raise Clip"));
     WMAddListItem(panel->actLs, _("Lower Clip"));
     WMAddListItem(panel->actLs, _("Raise/Lower Clip"));
@@ -431,7 +431,7 @@ createPanel(Panel *p)
 
     /***************** Shortcut ****************/
 
-    panel->shoF = WMCreateFrame(panel->frame);
+    panel->shoF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->shoF, 190, 210);
     WMMoveWidget(panel->shoF, 315, 10);
     WMSetFrameTitle(panel->shoF, _("Shortcut"));
@@ -458,14 +458,15 @@ createPanel(Panel *p)
     WMResizeWidget(panel->instructionsL, 160, 55);
     WMMoveWidget(panel->instructionsL, 15, 140);
     WMSetLabelTextAlignment(panel->instructionsL, WACenter);
+    WMSetLabelWraps(panel->instructionsL, True);
     WMSetLabelText(panel->instructionsL, _("Click Capture to interactively define the shortcut key."));
 
     WMMapSubwidgets(panel->shoF);
     
     WMReleaseFont(boldFont);
     
-    WMRealizeWidget(panel->frame);
-    WMMapSubwidgets(panel->frame);
+    WMRealizeWidget(panel->box);
+    WMMapSubwidgets(panel->box);
     
     
     showData(panel);
@@ -483,13 +484,13 @@ storeData(_Panel *panel)
 	if (panel->shortcuts[i]) {
 	    str = trimstr(panel->shortcuts[i]);
 	    if (strlen(str)==0) {
-		free(str);
+		wfree(str);
 		str = NULL;
 	    }
 	}
 	if (str) {
 	    SetStringForKey(str, keyOptions[i]);
-	    free(str);
+	    wfree(str);
 	} else {
 	    SetStringForKey("None", keyOptions[i]);
 	}
@@ -499,7 +500,7 @@ storeData(_Panel *panel)
 
 
 Panel*
-InitKeyboardShortcuts(WMScreen *scr, WMWindow *win)
+InitKeyboardShortcuts(WMScreen *scr, WMWidget *parent)
 {
     _Panel *panel;
 
@@ -511,7 +512,7 @@ InitKeyboardShortcuts(WMScreen *scr, WMWindow *win)
     panel->description = _("Change the keyboard shortcuts for actions such\n"
 			   "as changing workspaces and opening menus.");
 
-    panel->win = win;
+    panel->parent = parent;
     
     panel->callbacks.createWidgets = createPanel;
     panel->callbacks.updateDomain = storeData;
