@@ -1,3 +1,4 @@
+
 /* MouseSettings.c- mouse options (some are equivalent to xset)
  * 
  *  WPrefs - Window Maker Preferences Program
@@ -37,17 +38,19 @@
 
 
 typedef struct _Panel {
-    WMFrame *frame;
+    WMBox *box;
 
     char *sectionName;
 
+    char *description;
+
     CallbackRec callbacks;
     
-    WMWindow *win;
+    WMWidget *parent;
     
     WMFrame *speedF;
     WMLabel *speedL;
-    WMButton *speedB[5];
+    WMSlider *speedS;
     WMLabel *acceL;
     WMTextField *acceT;
     WMLabel *threL;
@@ -60,24 +63,21 @@ typedef struct _Panel {
     DoubleTest *tester;
     
     WMFrame *menuF;
-    WMLabel *listL;
-    WMLabel *appL;
-    WMLabel *selL;
-    WMLabel *mblL;
-    WMLabel *mbmL;
-    WMLabel *mbrL;
-    WMButton *lmb[3];
-    WMButton *amb[3];
-    WMButton *smb[3];
+    WMLabel *button1L;
+    WMLabel *button2L;
+    WMLabel *button3L;
+    WMLabel *wheelL;
+    WMPopUpButton *button1P;
+    WMPopUpButton *button2P;
+    WMPopUpButton *button3P;
+    WMPopUpButton *wheelP;
 
-    
     WMButton *disaB;
 
     WMFrame *grabF;
     WMPopUpButton *grabP;
 
     /**/
-    WMButton *lastClickedSpeed;
     int maxThreshold;
     float acceleration;
 } _Panel;
@@ -86,31 +86,22 @@ typedef struct _Panel {
 #define ICON_FILE "mousesettings"
 
 #define SPEED_ICON_FILE "mousespeed"
-#define SPEED_IMAGE "speed%i"
-#define SPEED_IMAGE_S "speed%is"
 
 #define DELAY_ICON "timer%i"
 #define DELAY_ICON_S "timer%is"
 
-#define MOUSEB_L "minimouseleft"
-#define MOUSEB_M "minimousemiddle"
-#define MOUSEB_R "minimouseright"
 
 /* need access to the double click variables */
-#include "WINGsP.h"
+#include <WINGs/WINGsP.h>
 
 
 
-static char *modifierNames[] = {
-    "Shift",
-	"Lock",
-	"Control",
-	"Mod1",
-	"Mod2",
-	"Mod3",
-	"Mod4",
-	"Mod5"
-};
+static char *modifierNames[8]; 
+
+
+static char *buttonActions[4];
+
+static char *wheelActions[2];
 
 
 #define DELAY(i)		((i)*75+170)
@@ -132,7 +123,7 @@ setMouseAccel(WMScreen *scr, float accel, int threshold)
 
 
 static void
-speedClick(WMWidget *w, void *data)
+speedChange(WMWidget *w, void *data)
 {
     _Panel *panel = (_Panel*)data;
     int i;
@@ -149,34 +140,31 @@ speedClick(WMWidget *w, void *data)
 			    _("Error"),
 			    _("Invalid mouse acceleration value. Must be a positive real value."),
 			    _("OK"), NULL, NULL);
-	    free(tmp);
+	    wfree(tmp);
 	    return;
 	}
 	panel->acceleration = accel;
-	free(tmp);
+	wfree(tmp);
     } else {
-	for (i=0; i<5; i++) {
-	    if (panel->speedB[i]==w)
-		break;
-	}
+	i = (int)WMGetSliderValue(panel->speedS);
     
-	panel->acceleration = 0.5+(i*0.5);
+	panel->acceleration = 0.25+(i*0.25);
 
-	sprintf(buffer, "%.2f", 0.5+(i*0.5));
+	sprintf(buffer, "%.2f", 0.25+(i*0.25));
 	WMSetTextFieldText(panel->acceT, buffer);
     }
 
     tmp = WMGetTextFieldText(panel->threT);
     if (sscanf(tmp, "%i", &threshold)!=1 || threshold < 0 
 	|| threshold > panel->maxThreshold) {
-	WMRunAlertPanel(WMWidgetScreen(panel->win), GetWindow(panel), _("Error"),
+	WMRunAlertPanel(WMWidgetScreen(panel->parent), GetWindow(panel), _("Error"),
 			_("Invalid mouse acceleration threshold value. Must be the number of pixels to travel before accelerating."),
 			_("OK"), NULL, NULL);
     } else {
-	setMouseAccel(WMWidgetScreen(panel->win), panel->acceleration, 
+	setMouseAccel(WMWidgetScreen(panel->parent), panel->acceleration, 
 		      threshold);
     }
-    free(tmp);
+    wfree(tmp);
 }
 
 
@@ -185,7 +173,7 @@ returnPressed(void *observerData, WMNotification *notification)
 {
     _Panel *panel = (_Panel*)observerData;
 
-    speedClick(NULL, panel);
+    speedChange(NULL, panel);
 }
 
 
@@ -210,30 +198,37 @@ doubleClick(WMWidget *w, void *data)
 
 
 int
-getbutton(char *str)
+getButtonAction(char *str)
 {
     if (!str)
 	return -2;
 
-    if (strcasecmp(str, "left")==0)
+    if (strcasecmp(str, "None")==0)
 	return 0;
-    else if (strcasecmp(str, "middle")==0)
+    else if (strcasecmp(str, "OpenApplicationsMenu")==0)
 	return 1;
-    else if (strcasecmp(str, "right")==0)
+    else if (strcasecmp(str, "OpenWindowListMenu")==0)
 	return 2;
-    else if (strcasecmp(str, "button1")==0)
-	return 0;
-    else if (strcasecmp(str, "button2")==0)
-	return 1;
-    else if (strcasecmp(str, "button3")==0)
-	return 2;
-    else if (strcasecmp(str, "button4")==0
-	     || strcasecmp(str, "button5")==0) {
-	wwarning(_("mouse button %s not supported by WPrefs."), str);
+    else if (strcasecmp(str, "SelectWindows")==0)
+	return 3;
+    else
+        return -1;
+
+}
+
+
+getWheelAction(char *str)
+{
+    if (!str)
 	return -2;
-    } else {
+
+    if (strcasecmp(str, "None")==0)
+	return 0;
+    else if (strcasecmp(str, "SwitchWorkspaces")==0)
+	return 1;
+    else
 	return -1;
-    }
+
 }
 
 
@@ -254,51 +249,59 @@ showData(_Panel *panel)
 {
     char *str;
     int i;
-    int a=-1, b=-1, c=-1;
+    int a=-1, b=-1, c=-1, w=-1;
     float accel;
     char buffer[32];
-    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->win));
+    Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->parent));
 
-    str = GetStringForKey("SelectWindowsMouseButton");
-    if (!str)
-	str = "left";
-    i = getbutton(str);
-    if (i==-1) {
-	a = 0;
-	wwarning(_("bad value %s for option %s"),str, "SelectWindowsMouseButton");
-	WMPerformButtonClick(panel->smb[0]);
-    } else if (i>=0) {
-	a = i;
-	WMPerformButtonClick(panel->smb[i]);
-    } 
-    
-    str = GetStringForKey("WindowListMouseButton");
-    if (!str)
-	str = "middle";
-    i = getbutton(str);
-    if (i==-1) {
-	b = 0;
-	wwarning(_("bad value %s for option %s"), str, "WindowListMouseButton");
-	WMPerformButtonClick(panel->lmb[1]);
-    } else if (i>=0) {
-	b = i;
-	WMPerformButtonClick(panel->lmb[i]);
-    } 
-    
-    str = GetStringForKey("ApplicationMenuMouseButton");
-    if (!str)
-	str = "right";
-    i = getbutton(str);
-    if (i==-1) {
-	c = 0;
-	wwarning(_("bad value %s for option %s"), str, "ApplicationMenuMouseButton");
-	WMPerformButtonClick(panel->amb[2]);
-    } else if (i>=0) {
-	c = i;
-	WMPerformButtonClick(panel->amb[i]);
+    str = GetStringForKey("MouseLeftButtonAction");
+    i = getButtonAction(str);
+    if (i<0) {
+        a = 3;
+        if (i==-1) {
+            wwarning(_("bad value %s for option %s"), str, "MouseLeftButtonAction");
+        }
+    } else {
+        a = i;
     }
+    WMSetPopUpButtonSelectedItem(panel->button1P, a);
     
+    str = GetStringForKey("MouseMiddleButtonAction");
+    i = getButtonAction(str);
+    if (i<0) {
+        b = 2;
+        if (i==-1) {
+            wwarning(_("bad value %s for option %s"), str, "MouseMiddleButtonAction");
+        }
+    } else {
+        b = i;
+    }
+    WMSetPopUpButtonSelectedItem(panel->button2P, b);
     
+    str = GetStringForKey("MouseRightButtonAction");
+    i = getButtonAction(str);
+    if (i<0) {
+        c = 1;
+        if (i==-1) {
+            wwarning(_("bad value %s for option %s"), str, "MouseRightButtonAction");
+        }
+    } else {
+        c = i;
+    }
+    WMSetPopUpButtonSelectedItem(panel->button3P, c);
+
+    str = GetStringForKey("MouseWheelAction");
+    i = getWheelAction(str);
+    if (i<0) {
+        w = 0;
+        if (i==-1) {
+            wwarning(_("bad value %s for option %s"), str, "MouseWheelAction");
+        }
+    } else {
+        w = i;
+    }
+    WMSetPopUpButtonSelectedItem(panel->wheelP, w);
+
     WMSetButtonSelected(panel->disaB, GetBoolForKey("DisableWSMouseActions"));
     
     /**/
@@ -310,20 +313,12 @@ showData(_Panel *panel)
     sprintf(buffer, "%i", a);
     WMSetTextFieldText(panel->threT, buffer);
 
-    a = -1;
-    for (i=0; i<5; i++) {
-	if (0.5+(float)i*0.5 == accel)
-	    a = i;
-    }
-    if (a >= 0) {
-	WMPerformButtonClick(panel->speedB[a]);
-	panel->lastClickedSpeed = panel->speedB[a];
-    }
+    WMSetSliderValue(panel->speedS, (accel - 0.25)/0.25);
+
     panel->acceleration = accel;
     sprintf(buffer, "%.2f", accel);
     WMSetTextFieldText(panel->acceT, buffer);
 
-    speedClick(panel->lastClickedSpeed, panel);
     /**/
     b = GetIntegerForKey("DoubleClickTime");
     /* find best match */
@@ -430,21 +425,24 @@ fillModifierPopUp(WMPopUpButton *pop)
 	    ptr = strstr(tmp, "_R");
 	    if (ptr)
 		*ptr = 0;
-	    sprintf(buffer, "%s (%s)", modifierNames[j], tmp);
+            sprintf(buffer, "%s (%s)", modifierNames[j], tmp);
+            /*sprintf(buffer, "%s", tmp);*/
 	    WMAddPopUpButtonItem(pop, buffer);
 	    for (i=k+1; i<a; i++) {
+		if (array[i] == NULL)
+		    continue;
 		if (strstr(array[i], tmp)) {
-		    free(array[i]);
+		    wfree(array[i]);
 		    array[i]=NULL;
 		    break;
 		}
 	    }
-	    free(tmp);
+	    wfree(tmp);
 	}
 	
 	while (--a>0) {
 	    if (array[a])
-		free(array[a]);
+		wfree(array[a]);
 	}
     }
     
@@ -454,83 +452,28 @@ fillModifierPopUp(WMPopUpButton *pop)
 
 
 
-static void
-mouseButtonClickA(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->amb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->lmb[i]))
-	WMSetButtonSelected(panel->lmb[i], False);
-    if (WMGetButtonSelected(panel->smb[i]))
-	WMSetButtonSelected(panel->smb[i], False);
-}
-
-static void
-mouseButtonClickL(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->lmb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->smb[i]))
-	WMSetButtonSelected(panel->smb[i], False);
-    if (WMGetButtonSelected(panel->amb[i]))
-	WMSetButtonSelected(panel->amb[i], False);    
-}
-
-static void
-mouseButtonClickS(WMWidget *w, void *data)
-{
-    _Panel *panel = (_Panel*)data;
-    int i;
-    
-    for (i=0; i<3; i++) {
-	if (panel->smb[i]==w)
-	    break;
-    }
-    if (i==3)
-	return;
-    if (WMGetButtonSelected(panel->lmb[i]))
-	WMSetButtonSelected(panel->lmb[i], False);
-    if (WMGetButtonSelected(panel->amb[i]))
-	WMSetButtonSelected(panel->amb[i], False);    
-}
-
 static void
 createPanel(Panel *p)
 {
     _Panel *panel = (_Panel*)p;
-    WMScreen *scr = WMWidgetScreen(panel->win);
+    WMScreen *scr = WMWidgetScreen(panel->parent);
     WMPixmap *icon;
     char *buf1, *buf2;
     int i;
     RColor color;
     char *path;
 
-    color.red = 0xaa;
-    color.green = 0xae;
-    color.blue = 0xaa;
+    color.red = 0xae;
+    color.green = 0xaa;
+    color.blue = 0xae;
     
-    panel->frame = WMCreateFrame(panel->win);
-    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
-    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);
+    panel->box = WMCreateBox(panel->parent);
+    WMSetViewExpandsToParent(WMWidgetView(panel->box), 2, 2, 2, 2);
     
     /**************** Mouse Speed ****************/
-    panel->speedF = WMCreateFrame(panel->frame);
+    panel->speedF = WMCreateFrame(panel->box);
     WMResizeWidget(panel->speedF, 245, 100);
-    WMMoveWidget(panel->speedF, 15, 15);
+    WMMoveWidget(panel->speedF, 15, 5);
     WMSetFrameTitle(panel->speedF, _("Mouse Speed"));
     
     panel->speedL = WMCreateLabel(panel->speedF);
@@ -546,50 +489,17 @@ createPanel(Panel *p)
 	} else {
 	    wwarning(_("could not load icon %s"), path);
 	}
-	free(path);
+	wfree(path);
     }
 
-    buf1 = wmalloc(strlen(SPEED_IMAGE)+2);
-    buf2 = wmalloc(strlen(SPEED_IMAGE_S)+2);
+    panel->speedS = WMCreateSlider(panel->speedF);
+    WMResizeWidget(panel->speedS, 160, 15);
+    WMMoveWidget(panel->speedS, 70, 35);
+    WMSetSliderMinValue(panel->speedS, 0);
+    WMSetSliderMaxValue(panel->speedS, 40);
+    WMSetSliderContinuous(panel->speedS, False);
+    WMSetSliderAction(panel->speedS, speedChange, panel);
 
-    for (i = 0; i < 5; i++) {
-	panel->speedB[i] = WMCreateCustomButton(panel->speedF,
-						WBBStateChangeMask);
-	WMResizeWidget(panel->speedB[i], 26, 26);
-	WMMoveWidget(panel->speedB[i], 60+(35*i), 25);
-	WMSetButtonBordered(panel->speedB[i], False);
-	WMSetButtonImagePosition(panel->speedB[i], WIPImageOnly);
-	WMSetButtonAction(panel->speedB[i], speedClick, panel);
-	if (i > 0) {
-	    WMGroupButtons(panel->speedB[0], panel->speedB[i]);
-	}
-	sprintf(buf1, SPEED_IMAGE, i);
-	sprintf(buf2, SPEED_IMAGE_S, i);
-	path = LocateImage(buf1);
-	if (path) {
-	    icon = WMCreatePixmapFromFile(scr, path);
-	    if (icon) {
-		WMSetButtonImage(panel->speedB[i], icon);
-		WMReleasePixmap(icon);
-	    } else {
-		wwarning(_("could not load icon file %s"), path);
-	    }
-	    free(path);
-	}
-	path = LocateImage(buf2);
-	if (path) {
-	    icon = WMCreatePixmapFromFile(scr, path);
-	    if (icon) {
-		WMSetButtonAltImage(panel->speedB[i], icon);
-		WMReleasePixmap(icon);
-	    } else {
-		wwarning(_("could not load icon file %s"), path);
-	    }
-	}
-    }
-    free(buf1);
-    free(buf2);
-    
     panel->acceL = WMCreateLabel(panel->speedF);
     WMResizeWidget(panel->acceL, 70, 16);
     WMMoveWidget(panel->acceL, 10, 67);
@@ -619,9 +529,9 @@ createPanel(Panel *p)
     
     /***************** Doubleclick Delay ****************/
 
-    panel->ddelaF = WMCreateFrame(panel->frame);
-    WMResizeWidget(panel->ddelaF, 245, 95);
-    WMMoveWidget(panel->ddelaF, 15, 125);
+    panel->ddelaF = WMCreateFrame(panel->box);
+    WMResizeWidget(panel->ddelaF, 245, 105);
+    WMMoveWidget(panel->ddelaF, 15, 115);
     WMSetFrameTitle(panel->ddelaF, _("Double-Click Delay"));
     
     buf1 = wmalloc(strlen(DELAY_ICON)+2);
@@ -631,7 +541,7 @@ createPanel(Panel *p)
 	panel->ddelaB[i] = WMCreateCustomButton(panel->ddelaF, 
 						WBBStateChangeMask);
 	WMResizeWidget(panel->ddelaB[i], 25, 25);
-	WMMoveWidget(panel->ddelaB[i], 30+(40*i), 20);
+	WMMoveWidget(panel->ddelaB[i], 30+(40*i), 25);
 	WMSetButtonBordered(panel->ddelaB[i], False);
 	WMSetButtonImagePosition(panel->ddelaB[i], WIPImageOnly);
 	WMSetButtonAction(panel->ddelaB[i], doubleClick, panel);
@@ -649,7 +559,7 @@ createPanel(Panel *p)
 	    } else {
 		wwarning(_("could not load icon file %s"), path);
 	    }
-	    free(path);
+	    wfree(path);
 	}
 	path = LocateImage(buf2);
 	if (path) {
@@ -660,23 +570,23 @@ createPanel(Panel *p)
 	    } else {
 		wwarning(_("could not load icon file %s"), path);
 	    }
-	    free(path);
+	    wfree(path);
 	}
     }
-    free(buf1);
-    free(buf2);
+    wfree(buf1);
+    wfree(buf2);
 
     panel->tester = CreateDoubleTest(panel->ddelaF, _("Test"));
     WMResizeWidget(panel->tester, 84, 29);
-    WMMoveWidget(panel->tester, 35, 55);
+    WMMoveWidget(panel->tester, 35, 60);
 
     panel->ddelaT = WMCreateTextField(panel->ddelaF);
     WMResizeWidget(panel->ddelaT, 40, 20);
-    WMMoveWidget(panel->ddelaT, 140, 60);
+    WMMoveWidget(panel->ddelaT, 140, 65);
 
     panel->ddelaL = WMCreateLabel(panel->ddelaF);
     WMResizeWidget(panel->ddelaL, 40, 16);
-    WMMoveWidget(panel->ddelaL, 185, 65);
+    WMMoveWidget(panel->ddelaL, 185, 70);
     {
 	WMFont *font;
 	WMColor *color;
@@ -688,131 +598,94 @@ createPanel(Panel *p)
 	WMReleaseFont(font);
 	WMReleaseColor(color);
     }
-    WMSetLabelText(panel->ddelaL, "msec");
+    WMSetLabelText(panel->ddelaL, _("msec"));
 
     WMMapSubwidgets(panel->ddelaF);
-    
+
     /* ************** Workspace Action Buttons **************** */
-    panel->menuF = WMCreateFrame(panel->frame);
-    WMResizeWidget(panel->menuF, 240, 145);
-    WMMoveWidget(panel->menuF, 270, 15);
+    panel->menuF = WMCreateFrame(panel->box);
+    WMResizeWidget(panel->menuF, 240, 160);
+    WMMoveWidget(panel->menuF, 270, 5);
     WMSetFrameTitle(panel->menuF, _("Workspace Mouse Actions"));
 
     panel->disaB = WMCreateSwitchButton(panel->menuF);
     WMResizeWidget(panel->disaB, 205, 18);
-    WMMoveWidget(panel->disaB, 10, 20);
+    WMMoveWidget(panel->disaB, 10, 18);
     WMSetButtonText(panel->disaB, _("Disable mouse actions"));
 
-    panel->mblL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mblL, 16, 22);
-    WMMoveWidget(panel->mblL, 135, 40);
-    WMSetLabelImagePosition(panel->mblL, WIPImageOnly);
-    path = LocateImage(MOUSEB_L);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mblL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
-    }
-    panel->mbmL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mbmL, 16, 22);
-    WMMoveWidget(panel->mbmL, 170, 40);
-    WMSetLabelImagePosition(panel->mbmL, WIPImageOnly);
-    path = LocateImage(MOUSEB_M);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mbmL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
+    panel->button1L = WMCreateLabel(panel->menuF);
+    WMResizeWidget(panel->button1L, 87, 20);
+    WMMoveWidget(panel->button1L, 5, 45);
+    WMSetLabelTextAlignment(panel->button1L, WARight);
+    WMSetLabelText(panel->button1L, _("Left Button"));
+
+    panel->button1P = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->button1P, 135, 20);
+    WMMoveWidget(panel->button1P, 95, 45);
+    
+    panel->button2L = WMCreateLabel(panel->menuF);
+    WMResizeWidget(panel->button2L, 87, 20);
+    WMMoveWidget(panel->button2L, 5, 73);
+    WMSetLabelTextAlignment(panel->button2L, WARight);
+    WMSetLabelText(panel->button2L, _("Middle Button"));
+    
+    panel->button2P = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->button2P, 135, 20);
+    WMMoveWidget(panel->button2P, 95, 73);
+
+    panel->button3L = WMCreateLabel(panel->menuF);
+    WMResizeWidget(panel->button3L, 87, 20);
+    WMMoveWidget(panel->button3L, 5, 101);
+    WMSetLabelTextAlignment(panel->button3L, WARight);
+    WMSetLabelText(panel->button3L, _("Right Button"));
+
+    panel->button3P = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->button3P, 135, 20);
+    WMMoveWidget(panel->button3P, 95, 101);
+
+    panel->wheelL = WMCreateLabel(panel->menuF);
+    WMResizeWidget(panel->wheelL, 87, 20);
+    WMMoveWidget(panel->wheelL, 5, 129);
+    WMSetLabelTextAlignment(panel->wheelL, WARight);
+    WMSetLabelText(panel->wheelL, _("Mouse Wheel"));
+
+    panel->wheelP = WMCreatePopUpButton(panel->menuF);
+    WMResizeWidget(panel->wheelP, 135, 20);
+    WMMoveWidget(panel->wheelP, 95, 129);
+
+    for (i = 0; i < sizeof(buttonActions)/sizeof(char*); i++) {
+        WMAddPopUpButtonItem(panel->button1P, buttonActions[i]);
+        WMAddPopUpButtonItem(panel->button2P, buttonActions[i]);
+        WMAddPopUpButtonItem(panel->button3P, buttonActions[i]);
     }
 
-    panel->mbrL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->mbrL, 16, 22);
-    WMMoveWidget(panel->mbrL, 205, 40);
-    WMSetLabelImagePosition(panel->mbrL, WIPImageOnly);
-    path = LocateImage(MOUSEB_R);
-    if (path) {
-	icon = WMCreatePixmapFromFile(scr, path);
-	if (icon) {
-	    WMSetLabelImage(panel->mbrL, icon);
-	    WMReleasePixmap(icon);
-	} else {
-	    wwarning(_("could not load icon file %s"), path);
-	}
-	free(path);
+    for (i = 0; i < sizeof(wheelActions)/sizeof(char*); i++) {
+        WMAddPopUpButtonItem(panel->wheelP, wheelActions[i]);
     }
 
-    panel->appL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->appL, 125, 16);
-    WMMoveWidget(panel->appL, 5, 65);
-    WMSetLabelTextAlignment(panel->appL, WARight);
-    WMSetLabelText(panel->appL, _("Applications menu"));
-    
-    panel->listL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->listL, 125, 16);
-    WMMoveWidget(panel->listL, 5, 90);
-    WMSetLabelTextAlignment(panel->listL, WARight);
-    WMSetLabelText(panel->listL, _("Window list menu"));
-    
-    panel->selL = WMCreateLabel(panel->menuF);
-    WMResizeWidget(panel->selL, 125, 16);
-    WMMoveWidget(panel->selL, 5, 115);
-    WMSetLabelTextAlignment(panel->selL, WARight);
-    WMSetLabelText(panel->selL, _("Select windows"));
-
-    
-    for (i=0; i<3; i++) {
-	panel->amb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->amb[i], 24, 24);
-	WMMoveWidget(panel->amb[i], 135+35*i, 65);
-	WMSetButtonText(panel->amb[i], NULL);
-	WMSetButtonAction(panel->amb[i], mouseButtonClickA, panel);
-
-	panel->lmb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->lmb[i], 24, 24);
-	WMMoveWidget(panel->lmb[i], 135+35*i, 90);
-	WMSetButtonText(panel->lmb[i], NULL);	
-	WMSetButtonAction(panel->lmb[i], mouseButtonClickL, panel);
-
-	panel->smb[i] = WMCreateRadioButton(panel->menuF);
-	WMResizeWidget(panel->smb[i], 24, 24);
-	WMMoveWidget(panel->smb[i], 135+35*i, 115);
-	WMSetButtonText(panel->smb[i], NULL);	
-	WMSetButtonAction(panel->smb[i], mouseButtonClickS, panel);
-
-	if (i>0) {
-	    WMGroupButtons(panel->lmb[0], panel->lmb[i]);
-	    WMGroupButtons(panel->amb[0], panel->amb[i]);
-	    WMGroupButtons(panel->smb[0], panel->smb[i]);	    
-	}
-    }
-    
     WMMapSubwidgets(panel->menuF);
-    
+
     /* ************** Grab Modifier **************** */
-    panel->grabF = WMCreateFrame(panel->frame);
-    WMResizeWidget(panel->grabF, 240, 55);
-    WMMoveWidget(panel->grabF, 270, 165);
+    panel->grabF = WMCreateFrame(panel->box);
+    WMResizeWidget(panel->grabF, 240, 50);
+    WMMoveWidget(panel->grabF, 270, 170);
     WMSetFrameTitle(panel->grabF, _("Mouse Grab Modifier"));
-    
+
+    WMSetBalloonTextForView(_("Keyboard modifier to use for actions that\n"
+			      "involve dragging windows with the mouse,\n"
+			      "clicking inside the window."),
+			    WMWidgetView(panel->grabF));
+
     panel->grabP = WMCreatePopUpButton(panel->grabF);
     WMResizeWidget(panel->grabP, 160, 20);
-    WMMoveWidget(panel->grabP, 40, 25);
+    WMMoveWidget(panel->grabP, 40, 20);
 
     fillModifierPopUp(panel->grabP);
     
     WMMapSubwidgets(panel->grabF);
     
-    WMRealizeWidget(panel->frame);
-    WMMapSubwidgets(panel->frame);
+    WMRealizeWidget(panel->box);
+    WMMapSubwidgets(panel->box);
     
     
     showData(panel);
@@ -823,14 +696,11 @@ static void
 storeCommandInScript(char *cmd, char *line)
 {
     char *path;
-    char *p;
     FILE *f;
     char buffer[128];
 
-    p = wusergnusteppath();
-    path = wmalloc(strlen(p)+64);
-    sprintf(path, "%s/Library/WindowMaker/autostart", p);
-
+    path = wstrconcat(wusergnusteppath(), "/Library/WindowMaker/autostart");
+    
     f = fopen(path, "r");
     if (!f) {
 	f = fopen(path, "w");
@@ -847,18 +717,25 @@ storeCommandInScript(char *cmd, char *line)
 	char *tmppath;
 	FILE *fo;
 
-	tmppath = wmalloc(strlen(p)+64);
-	sprintf(tmppath, "%s/Library/WindowMaker/autostart.tmp", p);
+	tmppath = wstrconcat(wusergnusteppath(), 
+			     "/Library/WindowMaker/autostart.tmp");
 	fo = fopen(tmppath, "w");
 	if (!fo) {
-	    wsyserror(_("could not create temporary file %s"), tmppath);
+            wsyserror(_("could not create temporary file %s"), tmppath);
+            wfree(tmppath);
 	    goto end;
 	}
 
 	while (!feof(f)) {
 	    if (!fgets(buffer, 127, f)) {
 		break;
-	    }
+            }
+            if (buffer[0] == '\n') {
+                /* don't write empty lines, else the file will grow
+                 * indefinitely (one '\n' added at end of file on each save).
+                 */
+                continue;
+            }
 	    if (strncmp(buffer, cmd, len)==0) {
 		if (!ok) {
 		    fputs(line, fo);
@@ -878,14 +755,13 @@ storeCommandInScript(char *cmd, char *line)
 	if (rename(tmppath, path)!=0) {
 	    wsyserror(_("could not rename file %s to %s\n"), tmppath, path);
 	}
-	free(tmppath);
+	wfree(tmppath);
     }
     sprintf(buffer, "chmod u+x %s", path);
     system(buffer);
 
 end:
-    free(p);
-    free(path);
+    wfree(path);
     if (f)
 	fclose(f);
 }
@@ -897,13 +773,14 @@ storeData(_Panel *panel)
     char buffer[64];
     int i;
     char *tmp, *p;
-    static char *button[3] = {"left", "middle", "right"};
+    static char *button[4] = {"None", "OpenApplicationsMenu", "OpenWindowListMenu", "SelectWindows"};
+    static char *wheel[2] = {"None", "SwitchWorkspaces"};
     WMUserDefaults *udb = WMGetStandardUserDefaults();
 
     if (!WMGetUDBoolForKey(udb, "NoXSetStuff")) {
 	tmp = WMGetTextFieldText(panel->threT);
 	if (strlen(tmp)==0) {
-	    free(tmp);
+	    wfree(tmp);
 	    tmp = wstrdup("4");
 	}
 
@@ -911,7 +788,7 @@ storeData(_Panel *panel)
 		10, tmp);
 	storeCommandInScript(XSET" m", buffer);
 
-	free(tmp);
+	wfree(tmp);
     }
 
     tmp = WMGetTextFieldText(panel->ddelaT);
@@ -920,28 +797,19 @@ storeData(_Panel *panel)
 
     SetBoolForKey(WMGetButtonSelected(panel->disaB), "DisableWSMouseActions");
 
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->amb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "ApplicationMenuMouseButton");
+    i = WMGetPopUpButtonSelectedItem(panel->button1P);
+    SetStringForKey(button[i], "MouseLeftButtonAction");
+
+    i = WMGetPopUpButtonSelectedItem(panel->button2P);
+    SetStringForKey(button[i], "MouseMiddleButtonAction");
     
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->lmb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "WindowListMouseButton");
+    i = WMGetPopUpButtonSelectedItem(panel->button3P);
+    SetStringForKey(button[i], "MouseRightButtonAction");
     
-    for (i=0; i<3; i++) {
-	if (WMGetButtonSelected(panel->smb[i]))
-	    break;
-    }
-    if (i<3)
-	SetStringForKey(button[i], "SelectWindowsMouseButton");
+    i = WMGetPopUpButtonSelectedItem(panel->wheelP);
+    SetStringForKey(wheel[i], "MouseWheelAction");
     
-    tmp = WMGetPopUpButtonItem(panel->grabP, 
+    tmp = WMGetPopUpButtonItem(panel->grabP,
 			       WMGetPopUpButtonSelectedItem(panel->grabP));
     tmp = wstrdup(tmp);
     p = strchr(tmp, ' ');
@@ -949,22 +817,42 @@ storeData(_Panel *panel)
 
     SetStringForKey(tmp, "ModifierKey");
 
-    free(tmp);
+    wfree(tmp);
 }
 
 
 Panel*
-InitMouseSettings(WMScreen *scr, WMWindow *win)
+InitMouseSettings(WMScreen *scr, WMWidget *parent)
 {
     _Panel *panel;
-    
+
+    modifierNames[0] = wstrdup(_("Shift"));
+    modifierNames[1] = wstrdup(_("Lock"));
+    modifierNames[2] = wstrdup(_("Control"));
+    modifierNames[3] = wstrdup(_("Mod1"));
+    modifierNames[4] = wstrdup(_("Mod2"));
+    modifierNames[5] = wstrdup(_("Mod3"));
+    modifierNames[6] = wstrdup(_("Mod4"));
+    modifierNames[7] = wstrdup(_("Mod5"));
+
+    buttonActions[0] = wstrdup(_("None"));
+    buttonActions[1] = wstrdup(_("Applications Menu"));
+    buttonActions[2] = wstrdup(_("Window List Menu"));
+    buttonActions[3] = wstrdup(_("Select Windows"));
+
+    wheelActions[0] = wstrdup(_("None"));
+    wheelActions[1] = wstrdup(_("Switch Workspaces"));
+
     panel = wmalloc(sizeof(_Panel));
     memset(panel, 0, sizeof(_Panel));
 
     panel->sectionName = _("Mouse Preferences");
 
-    panel->win = win;
-    
+    panel->description = _("Mouse speed/acceleration, double click delay,\n"
+			   "mouse button bindings etc.");
+
+    panel->parent = parent;
+
     panel->callbacks.createWidgets = createPanel;
     panel->callbacks.updateDomain = storeData;
 
