@@ -31,14 +31,12 @@
 #include <X11/Xlib.h>
 
 
-#include <WINGs.h>
-#include <WUtil.h>
+#include <WINGs/WINGs.h>
 
 #include "WPrefs.h"
 
 #include "TexturePanel.h"
 
-
 #define MAX_SECTION_PARTS 5
 
 typedef struct _TexturePanel {
@@ -175,25 +173,25 @@ updateGradButtons(TexturePanel *panel)
 	image = RRenderMultiGradient(80, 30, colors, RHorizontalGradient);
 	pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 					  image, 128);
-	RDestroyImage(image);
+	RReleaseImage(image);
 	WMSetButtonImage(panel->dirhB, pixmap);
 	WMReleasePixmap(pixmap);
 
 	image = RRenderMultiGradient(80, 30, colors, RVerticalGradient);
 	pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 					  image, 128);
-	RDestroyImage(image);
+	RReleaseImage(image);
 	WMSetButtonImage(panel->dirvB, pixmap);
 	WMReleasePixmap(pixmap);
 
 	image = RRenderMultiGradient(80, 30, colors, RDiagonalGradient);
 	pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 					  image, 128);
-	RDestroyImage(image);
+	RReleaseImage(image);
 	WMSetButtonImage(panel->dirdB, pixmap);
 	WMReleasePixmap(pixmap);
 
-	free(colors);
+	wfree(colors);
     }
 }
 
@@ -240,14 +238,14 @@ updateTGradImage(TexturePanel *panel)
 
     RCombineImagesWithOpaqueness(image, gradient,
 				 WMGetSliderValue(panel->topaS));
-    RDestroyImage(gradient);
+    RReleaseImage(gradient);
     pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->win),
 				      image, 128);
 
     WMSetLabelImage(panel->imageL, pixmap);
     WMReleasePixmap(pixmap);
     WMResizeWidget(panel->imageL, image->width, image->height);
-    RDestroyImage(image);
+    RReleaseImage(image);
 }
 
 
@@ -273,21 +271,21 @@ updateSGradButtons(TexturePanel *panel)
     image = RRenderGradient(80, 30, &from, &to, RHorizontalGradient);
     pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 				      image, 128);
-    RDestroyImage(image);
+    RReleaseImage(image);
     WMSetButtonImage(panel->dirhB, pixmap);
     WMReleasePixmap(pixmap);
     
     image = RRenderGradient(80, 30, &from, &to, RVerticalGradient);
     pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 				      image, 128);
-    RDestroyImage(image);
+    RReleaseImage(image);
     WMSetButtonImage(panel->dirvB, pixmap);
     WMReleasePixmap(pixmap);
     
     image = RRenderGradient(80, 30, &from, &to, RDiagonalGradient);
     pixmap = WMCreatePixmapFromRImage(WMWidgetScreen(panel->gcolL),
 				      image, 128);
-    RDestroyImage(image);
+    RReleaseImage(image);
     WMSetButtonImage(panel->dirdB, pixmap);
     WMReleasePixmap(pixmap);
 }
@@ -318,7 +316,7 @@ updateSVSlider(WMSlider *sPtr, Bool saturation, WMFont *font, RHSVColor *hsv)
     }
     image = RRenderGradient(130, 16, &from, &to, RHorizontalGradient);
     pixmap = WMCreatePixmapFromRImage(scr, image, 128);
-    RDestroyImage(image);
+    RReleaseImage(image);
 
     if (hsv->value < 128 || !saturation) {
 	WMColor *col = WMWhiteColor(scr);
@@ -360,7 +358,7 @@ updateHueSlider(WMSlider *sPtr, WMFont *font, RHSVColor *hsv)
 
     image = RRenderMultiGradient(130, 16, colors, RGRD_HORIZONTAL);
     pixmap = WMCreatePixmapFromRImage(scr, image, 128);
-    RDestroyImage(image);
+    RReleaseImage(image);
 
     if (hsv->value < 128) {
 	WMColor *col = WMWhiteColor(scr);
@@ -379,7 +377,7 @@ updateHueSlider(WMSlider *sPtr, WMFont *font, RHSVColor *hsv)
     WMReleasePixmap(pixmap);
 
     for (i = 0; i <= 6; i++)
-	free(colors[i]);
+	wfree(colors[i]);
 }
 
 
@@ -444,9 +442,9 @@ sliderChangeCallback(WMWidget *w, void *data)
     image = RRenderMultiGradient(30, i*WMGetListItemHeight(panel->gcolL), 
 				 colors, RVerticalGradient);
     RConvertImage(WMScreenRContext(scr), image, &panel->gimage);
-    RDestroyImage(image);
+    RReleaseImage(image);
 
-    free(colors);
+    wfree(colors);
 
     WMRedisplayWidget(panel->gcolL);
 
@@ -554,7 +552,7 @@ gradDeleteCallback(WMWidget *w, void *data)
 	return;
 
     item = WMGetListItem(panel->gcolL, row);
-    free(item->clientData);
+    wfree(item->clientData);
 
     WMRemoveListItem(panel->gcolL, row);
 
@@ -602,7 +600,7 @@ updateImage(TexturePanel *panel, char *path)
 	if (!image) {
 	    char *message;
 	    
-	    message = wstrappend(_("Could not load the selected file: "),
+	    message = wstrconcat(_("Could not load the selected file: "),
 				 (char*)RMessageForError(RErrorCode));
 
 	    WMRunAlertPanel(scr, panel->win, _("Error"), message,
@@ -610,13 +608,15 @@ updateImage(TexturePanel *panel, char *path)
 
 	    if (!panel->image)
 		WMSetButtonEnabled(panel->okB, False);
-	    return;
+
+            wfree(message);
+            return;
 	}
 
 	WMSetButtonEnabled(panel->okB, True);
 
 	if (panel->image)
-	    RDestroyImage(panel->image);
+	    RReleaseImage(panel->image);
 	panel->image = image;
     } else {
 	image = panel->image;
@@ -654,7 +654,7 @@ browseImageCallback(WMWidget *w, void *data)
 	ipath = wstrdup(wgethomedir());
     
     if (WMRunModalFilePanelForDirectory(opanel, panel->win, ipath,
-					"Open Image", NULL)) {
+					_("Open Image"), NULL)) {
 	char *path, *fullpath;
 	char *tmp, *tmp2;
 
@@ -663,7 +663,7 @@ browseImageCallback(WMWidget *w, void *data)
 	    return;
 	fullpath = tmp;
 
-	free(ipath);
+	wfree(ipath);
 	ipath = fullpath;
 
 	path = wstrdup(fullpath);
@@ -676,20 +676,20 @@ browseImageCallback(WMWidget *w, void *data)
 
 	if (tmp) {
 	    if (strcmp(fullpath, tmp)==0) {
-		free(path);
+		wfree(path);
 		path = tmp2;
 	    }
-	    free(tmp);
+	    wfree(tmp);
 	}
 
 	if (!RGetImageFileFormat(fullpath)) {
 	    WMRunAlertPanel(scr, panel->win, _("Error"),
 			    _("The selected file does not contain a supported image."),
 			    _("OK"), NULL, NULL);
-	    free(path);
+	    wfree(path);
 	} else {
 	    updateImage(panel, fullpath);
-	    free(panel->imageFile);
+	    wfree(panel->imageFile);
 	    panel->imageFile = path;
 
 	    WMSetTextFieldText(panel->imageT, path);
@@ -891,7 +891,7 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
 	WMSetTextFieldText(panel->imageT,
 			   PLGetString(PLGetArrayElement(texture, 1)));
 	if (panel->imageFile)
-	    free(panel->imageFile);
+	    wfree(panel->imageFile);
 	panel->imageFile = wstrdup(PLGetString(PLGetArrayElement(texture, 1)));
 
 
@@ -927,7 +927,7 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
 			   PLGetString(PLGetArrayElement(texture, 1)));
 	
     if (panel->imageFile)
-        free(panel->imageFile);
+        wfree(panel->imageFile);
     if ((panel->imageFile = wfindfileinarray(panel->pathList,
                 PLGetString(PLGetArrayElement(texture, 1)))) != NULL) {
 
@@ -935,7 +935,7 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
         updateTGradImage(panel);
 
         updateSGradButtons(panel);
-    } else wwarning("could not load file '%s': %s", panel->imageFile,
+    } else wwarning(_("could not load file '%s': %s"), panel->imageFile,
 		     RMessageForError(RErrorCode));
 
     /*...............................................*/
@@ -946,7 +946,7 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
 
 	for (i = 0; i < WMGetListNumberOfRows(panel->gcolL); i++) {
 	    item = WMGetListItem(panel->gcolL, i);
-	    free(item->clientData);
+	    wfree(item->clientData);
 	}
 	WMClearList(panel->gcolL);
 
@@ -1019,7 +1019,7 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
 			   PLGetString(PLGetArrayElement(texture, 1)));
 	
 	if (panel->imageFile)
-	    free(panel->imageFile);
+	    wfree(panel->imageFile);
 	panel->imageFile = wfindfileinarray(panel->pathList,
 			    PLGetString(PLGetArrayElement(texture, 1)));
 
@@ -1054,8 +1054,8 @@ SetTexturePanelTexture(TexturePanel *panel, char *name, proplist_t texture)
 
  bad_texture:
     str = PLGetDescription(texture);
-    wwarning("error creating texture %s", str);
-    free(str);
+    wwarning(_("error creating texture %s"), str);
+    wfree(str);
 
 }
 
@@ -1087,7 +1087,7 @@ GetTexturePanelTexture(TexturePanel *panel)
 	str = WMGetColorRGBDescription(color);
 	prop = PLMakeArrayFromElements(PLMakeString("solid"),
 				       PLMakeString(str), NULL);
-	free(str);
+	wfree(str);
 
 	break;
 
@@ -1117,7 +1117,7 @@ GetTexturePanelTexture(TexturePanel *panel)
 					   PLMakeString(str), NULL);
 	    break;
 	}
-	free(str);
+	wfree(str);
 	break;
 
      case TYPE_TGRADIENT:
@@ -1148,8 +1148,8 @@ GetTexturePanelTexture(TexturePanel *panel)
 					   PLMakeString(str),
 					   PLMakeString(str2), NULL);
 	}
-	free(str);
-	free(str2);
+	wfree(str);
+	wfree(str2);
 	break;
 
 
@@ -1173,8 +1173,8 @@ GetTexturePanelTexture(TexturePanel *panel)
 					   PLMakeString(str),
 					   PLMakeString(str2), NULL);
 	}
-	free(str);
-	free(str2);
+	wfree(str);
+	wfree(str2);
 	break;
 
      case TYPE_GRADIENT:
@@ -1191,7 +1191,7 @@ GetTexturePanelTexture(TexturePanel *panel)
 	    prop = PLMakeArrayFromElements(PLMakeString("mhgradient"),
 					   PLMakeString(str), NULL);
 	}
-	free(str);
+	wfree(str);
 
 	for (i = 0; i < WMGetListNumberOfRows(panel->gcolL); i++) {
 	    RColor *rgb;
@@ -1608,7 +1608,7 @@ testOKButton(WMWidget *self, void *data)
     XCopyArea(dpy, pix, win, (WMScreenRContext(WMWidgetScreen(panel->okB)))->copy_gc, 0, 0, image->width, image->height, 
 	      0, 0);
     
-    free (test);
+    wfree (test);
     
 }
 
