@@ -2,7 +2,7 @@
  * 
  *  WPrefs - Window Maker Preferences Program
  * 
- *  Copyright (c) 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998-2000 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,40 +25,45 @@
 #include <assert.h>
 
 
-extern Panel *InitWindowHandling(WMScreen *scr, WMWindow *win);
+extern Panel *InitWindowHandling(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMouseSettings(WMScreen *scr, WMWindow *win);
+extern Panel *InitMouseSettings(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWindow *win);
+extern Panel *InitKeyboardShortcuts(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitWorkspace(WMScreen *scr, WMWindow *win);
+extern Panel *InitWorkspace(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitFocus(WMScreen *scr, WMWindow *win);
+extern Panel *InitFocus(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitText(WMScreen *scr, WMWindow *win);
+extern Panel *InitFont(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitConfigurations(WMScreen *scr, WMWindow *win);
+extern Panel *InitConfigurations(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitPaths(WMScreen *scr, WMWindow *win);
+extern Panel *InitPaths(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenu(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenu(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitExpert(WMScreen *scr, WMWindow *win);
+extern Panel *InitExpert(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitMenuPreferences(WMScreen *scr, WMWindow *win);
+extern Panel *InitMenuPreferences(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitIcons(WMScreen *scr, WMWindow *win);
+extern Panel *InitIcons(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitThemes(WMScreen *scr, WMWindow *win);
+extern Panel *InitThemes(WMScreen *scr, WMWidget *parent);
 
-extern Panel *InitAppearance(WMScreen *scr, WMWindow *win);
+extern Panel *InitAppearance(WMScreen *scr, WMWidget *parent);
 
 
 
+
+#define ICON_TITLE_FONT "-adobe-helvetica-bold-r-*-*-10-*"
+#define ICON_TITLE_VFONT "-adobe-helvetica-bold-r-*-*-10-[]-*"
+
+
 #define MAX_SECTIONS 16
 
 
@@ -75,13 +80,15 @@ typedef struct _WPrefs {
     WMButton *closeBtn;
     WMButton *undoBtn;
     WMButton *undosBtn;
-    
+
+    WMButton *balloonBtn;
+
     WMFrame *banner;
     WMLabel *nameL;
     WMLabel *versionL;
     WMLabel *creditsL;
     WMLabel *statusL;
-    
+        
     Panel *currentPanel;
 } _WPrefs;
 
@@ -106,10 +113,13 @@ static void loadConfigurations(WMScreen *scr, WMWindow *mainw);
 
 static void savePanelData(Panel *panel);
 
+static void prepareForClose();
 
 void
 quit(WMWidget *w, void *data)
 {
+    prepareForClose();
+
     exit(0);
 }
 
@@ -121,6 +131,8 @@ save(WMWidget *w, void *data)
     proplist_t p1, p2;
     proplist_t keyList;
     proplist_t key;
+    char *msg = "Reconfigure";
+    XEvent ev;
 
 
 /*    puts("gathering data");*/
@@ -154,9 +166,27 @@ save(WMWidget *w, void *data)
 /*    puts("storing data");*/
 
     PLSave(WindowMakerDB, YES);
+    
+
+    memset(&ev, 0, sizeof(XEvent));
+	    
+    ev.xclient.type = ClientMessage;
+    ev.xclient.message_type = XInternAtom(WMScreenDisplay(WMWidgetScreen(w)),
+					  "_WINDOWMAKER_COMMAND", False);
+    ev.xclient.window = DefaultRootWindow(WMScreenDisplay(WMWidgetScreen(w)));
+    ev.xclient.format = 8;
+
+    for (i = 0; i <= strlen(msg); i++) {
+	ev.xclient.data.b[i] = msg[i];
+    }
+    XSendEvent(WMScreenDisplay(WMWidgetScreen(w)), 
+	       DefaultRootWindow(WMScreenDisplay(WMWidgetScreen(w))),
+	       False, SubstructureRedirectMask, &ev);
+    XFlush(WMScreenDisplay(WMWidgetScreen(w)));
 }
 
 
+
 static void
 undo(WMWidget *w, void *data)
 {
@@ -187,6 +217,36 @@ undoAll(WMWidget *w, void *data)
 }
 
 
+
+static void
+prepareForClose()
+{
+    int i;
+
+    for (i=0; i<WPrefs.sectionCount; i++) {
+	PanelRec *rec = WMGetHangedData(WPrefs.sectionB[i]);
+
+	if (rec->callbacks.prepareForClose 
+	    && (rec->callbacks.flags & INITIALIZED_PANEL))
+	    (*rec->callbacks.prepareForClose)((Panel*)rec);
+    }
+}
+
+
+void
+toggleBalloons(WMWidget *w, void *data)
+{
+    WMUserDefaults *udb = WMGetStandardUserDefaults();
+    Bool flag;
+
+    flag = WMGetButtonSelected(WPrefs.balloonBtn);
+
+    WMSetBalloonEnabled(WMWidgetScreen(WPrefs.win), flag);
+
+    WMSetUDBoolForKey(udb, flag, "BalloonHelp");
+}
+
+
 static void
 createMainWindow(WMScreen *scr)
 {
@@ -201,7 +261,7 @@ createMainWindow(WMScreen *scr)
     WMSetWindowMaxSize(WPrefs.win, 520, 390);
     WMSetWindowMinSize(WPrefs.win, 520, 390);
     WMSetWindowMiniwindowTitle(WPrefs.win, "Preferences");
-    WMSetWindowMiniwindowImage(WPrefs.win, WMGetApplicationIconImage(scr));
+    WMSetWindowMiniwindowPixmap(WPrefs.win, WMGetApplicationIconPixmap(scr));
     
     WPrefs.scrollV = WMCreateScrollView(WPrefs.win);
     WMResizeWidget(WPrefs.scrollV, 500, 87);
@@ -240,14 +300,29 @@ createMainWindow(WMScreen *scr)
     WMMoveWidget(WPrefs.closeBtn, 425, 350);
     WMSetButtonText(WPrefs.closeBtn, _("Close"));
     WMSetButtonAction(WPrefs.closeBtn, quit, NULL);
-    
+
+
+    WPrefs.balloonBtn = WMCreateSwitchButton(WPrefs.win);
+    WMResizeWidget(WPrefs.balloonBtn, 200, 28);
+    WMMoveWidget(WPrefs.balloonBtn, 15, 350);
+    WMSetButtonText(WPrefs.balloonBtn, _("Balloon Help"));
+    WMSetButtonAction(WPrefs.balloonBtn, toggleBalloons, NULL);
+    {
+	WMUserDefaults *udb = WMGetStandardUserDefaults();
+	Bool flag = WMGetUDBoolForKey(udb, "BalloonHelp");
+
+	WMSetButtonSelected(WPrefs.balloonBtn, flag);
+	WMSetBalloonEnabled(scr, flag);
+    }
+
     /* banner */
     WPrefs.banner = WMCreateFrame(WPrefs.win);
     WMResizeWidget(WPrefs.banner, FRAME_WIDTH, FRAME_HEIGHT);
     WMMoveWidget(WPrefs.banner, FRAME_LEFT, FRAME_TOP);
     WMSetFrameRelief(WPrefs.banner, WRFlat);
 
-    font = WMCreateFont(scr, "-*-times-bold-r-*-*-24-*-*-*-*-*-*-*");
+    font = WMCreateFont(scr, "-*-times-bold-r-*-*-24-*-*-*-*-*-*-*,"
+			"-*-fixed-medium-r-normal-*-24-*");
     if (!font)
 	font = WMBoldSystemFontOfSize(scr, 24);
     WPrefs.nameL = WMCreateLabel(WPrefs.banner);
@@ -262,7 +337,8 @@ createMainWindow(WMScreen *scr)
     WMResizeWidget(WPrefs.versionL, FRAME_WIDTH-20, 20);
     WMMoveWidget(WPrefs.versionL, 10, 65);
     WMSetLabelTextAlignment(WPrefs.versionL, WACenter);
-    sprintf(buffer, _("Version %s for Window Maker %s"), WVERSION, WMVERSION);
+    sprintf(buffer, _("Version %s for Window Maker %s or newer"), WVERSION,
+								 WMVERSION);
     WMSetLabelText(WPrefs.versionL, buffer);
 
     WPrefs.statusL = WMCreateLabel(WPrefs.banner);
@@ -276,8 +352,8 @@ createMainWindow(WMScreen *scr)
     WMMoveWidget(WPrefs.creditsL, 10, FRAME_HEIGHT-60);
     WMSetLabelTextAlignment(WPrefs.creditsL, WACenter);
     WMSetLabelText(WPrefs.creditsL, _("Programming/Design: Alfredo K. Kojima\n"
-		   		"Artwork: Marco van Hylckama Vlieg\n"
-				      "More Programming: James Thompson"));
+		   		"Artwork: Marco van Hylckama Vlieg, Largo et al\n"
+				      "More Programming: James Thompson et al"));
    
     
     WMMapSubwidgets(WPrefs.win);
@@ -300,7 +376,10 @@ showPanel(Panel *panel)
 
     WMSetWindowTitle(WPrefs.win, rec->sectionName);
     
-    WMMapWidget(rec->frame);
+    if (rec->callbacks.showPanel)
+	(*rec->callbacks.showPanel)(panel);
+
+    WMMapWidget(rec->box);
 }
 
 
@@ -310,7 +389,10 @@ hidePanel(Panel *panel)
 {
     PanelRec *rec = (PanelRec*)panel;    
     
-    WMUnmapWidget(rec->frame);
+    WMUnmapWidget(rec->box);
+    
+    if (rec->callbacks.hidePanel)
+	(*rec->callbacks.hidePanel)(panel);
 }
 
 
@@ -328,17 +410,25 @@ savePanelData(Panel *panel)
 static void 
 changeSection(WMWidget *self, void *data)
 {
-    if (WPrefs.banner) {
-	WMDestroyWidget(WPrefs.banner);
-	WPrefs.banner = NULL;
+    if (WPrefs.currentPanel == data)
+	return;
+	
+    if (WPrefs.currentPanel == NULL) {
+	WMDestroyWidget(WPrefs.nameL);
+	WMDestroyWidget(WPrefs.creditsL);
+	WMDestroyWidget(WPrefs.versionL);
+	WMDestroyWidget(WPrefs.statusL);
+
+	WMSetFrameRelief(WPrefs.banner, WRGroove);
+
 /*	WMMapWidget(WPrefs.undosBtn);
 	WMMapWidget(WPrefs.undoBtn);
  */
 	WMMapWidget(WPrefs.saveBtn);
     }
-    
+
     showPanel(data);
-    
+
     if (WPrefs.currentPanel)
 	hidePanel(WPrefs.currentPanel);
     WPrefs.currentPanel = data;
@@ -346,8 +436,6 @@ changeSection(WMWidget *self, void *data)
 
 
 
-
-
 char*
 LocateImage(char *name)
 {
@@ -361,7 +449,7 @@ LocateImage(char *name)
 	sprintf(tmp, "%s.xpm", name);
 	path = WMPathForResourceOfType(tmp, "xpm");
     }
-    free(tmp);
+    wfree(tmp);
     if (!path) {
 	wwarning(_("could not locate image file %s\n"), name);
     }
@@ -370,10 +458,90 @@ LocateImage(char *name)
 }
 
 
+
+static WMPixmap*
+makeTitledIcon(WMScreen *scr, WMPixmap *icon, char *title1, char *title2)
+{
+    return WMRetainPixmap(icon);
+    
+#if 0
+    static GC gc = NULL;
+    static XFontStruct *hfont = NULL;
+    static XFontStruct *vfont = NULL;
+    WMPixmap *tmp;
+    Pixmap pix, mask;
+    Display *dpy = WMScreenDisplay(scr);
+    WMColor *black = WMBlackColor(scr);
+    GC fgc;
+    WMSize size = WMGetPixmapSize(icon);
+
+    
+    tmp = WMCreatePixmap(scr, 60, 60, WMScreenDepth(scr), True);
+
+    pix = WMGetPixmapXID(tmp);
+    mask = WMGetPixmapMaskXID(tmp);
+
+    if (gc == NULL) {
+	gc = XCreateGC(dpy, mask, 0, NULL);
+
+	hfont = XLoadQueryFont(dpy, ICON_TITLE_FONT);
+	vfont = XLoadQueryFont(dpy, ICON_TITLE_VFONT);
+    }
+    
+    if (hfont == NULL) {
+	return WMRetainPixmap(icon);
+    }
+    
+    XSetForeground(dpy, gc, 0);
+    XFillRectangle(dpy, mask, gc, 0, 0, 60, 60);
+    
+    fgc = WMColorGC(black);
+    
+    XSetForeground(dpy, gc, 1);
+    
+    XCopyArea(dpy, WMGetPixmapXID(icon), pix, fgc, 0, 0,
+	      size.width, size.height, 12, 12);
+    
+    if (WMGetPixmapMaskXID(icon) != None)
+	XCopyPlane(dpy, WMGetPixmapMaskXID(icon), mask, gc, 0, 0,
+		   size.width, size.height, 12, 12, 1);
+    else
+	XFillRectangle(dpy, mask, gc, 12, 12, 48, 48);
+
+
+    if (title1) {
+	XSetFont(dpy, fgc, vfont->fid);
+	XSetFont(dpy, gc, vfont->fid);
+
+	XDrawString(dpy, pix, fgc, 0, vfont->ascent, 
+		    title1, strlen(title1));
+	
+	XDrawString(dpy, mask, gc, 0, vfont->ascent, 
+		    title1, strlen(title1));
+    }
+
+    if (title2) {
+	XSetFont(dpy, fgc, hfont->fid);
+	XSetFont(dpy, gc, hfont->fid);
+
+	XDrawString(dpy, pix, fgc, (title1 ? 12 : 0), hfont->ascent, 
+		    title2, strlen(title2));
+	
+	XDrawString(dpy, mask, gc, (title1 ? 12 : 0), hfont->ascent, 
+		    title2, strlen(title2));
+    }
+
+    return tmp;
+#endif
+}
+
+
 void
-SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file)
+SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file,
+		    char *title1, char *title2)
 {
     WMPixmap *icon;
+    WMPixmap *icon2;
     RColor color;
     char *iconPath;
 
@@ -390,11 +558,19 @@ SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file)
     } else {
 	icon = NULL;
     }
+    
+    if (icon) {
+	icon2 = makeTitledIcon(scr, icon, title1, title2);
+	if (icon)
+	    WMReleasePixmap(icon);
+    } else {
+	icon2 = NULL;
+    }
+    
+    WMSetButtonImage(bPtr, icon2);
 
-    WMSetButtonImage(bPtr, icon);
-
-    if (icon)
-	WMReleasePixmap(icon);
+    if (icon2)
+	WMReleasePixmap(icon2);
 
     color.red = 0xff;
     color.green = 0xff;
@@ -414,7 +590,7 @@ SetButtonAlphaImage(WMScreen *scr, WMButton *bPtr, char *file)
 	WMReleasePixmap(icon);
 
     if (iconPath)
-	free(iconPath);
+	wfree(iconPath);
 }
 
 
@@ -434,8 +610,22 @@ AddSection(Panel *panel, char *iconFile)
     WMSetButtonAction(bPtr, changeSection, panel);
     WMHangData(bPtr, panel);
 
-    SetButtonAlphaImage(WMWidgetScreen(bPtr), bPtr, iconFile);
+    WMSetBalloonTextForView(((PanelRec*)panel)->description, 
+		    WMWidgetView(bPtr));
 
+    {
+	char *t1, *t2;
+	
+	t1 = wstrdup(((PanelRec*)panel)->sectionName);
+	t2 = strchr(t1, ' ');
+	if (t2) {
+	    *t2 = 0;
+	    t2++;
+	}
+	SetButtonAlphaImage(WMWidgetScreen(bPtr), bPtr, iconFile,
+			    t1, t2);
+	wfree(t1);
+    }
     WMMapWidget(bPtr);
 
     WPrefs.sectionB[WPrefs.sectionCount] = bPtr;
@@ -443,9 +633,9 @@ AddSection(Panel *panel, char *iconFile)
     if (WPrefs.sectionCount > 0) {
 	WMGroupButtons(WPrefs.sectionB[0], bPtr);
     }
-    
+
     WPrefs.sectionCount++;
-    
+
     WMResizeWidget(WPrefs.buttonF, WPrefs.sectionCount*64, 64);
 }
 
@@ -480,13 +670,13 @@ Initialize(WMScreen *scr)
 		     RMessageForError(RErrorCode));
 	} else {
 	    icon = WMCreatePixmapFromRImage(scr, tmp, 0);
-	    RDestroyImage(tmp);
+	    RReleaseImage(tmp);
 	    if (icon) {
-	        WMSetApplicationIconImage(scr, icon);
+	        WMSetApplicationIconPixmap(scr, icon);
 	        WMReleasePixmap(icon);
 	    }
 	}
-	free(path);
+	wfree(path);
     }
     
     memset(&WPrefs, 0, sizeof(_WPrefs));
@@ -501,39 +691,40 @@ Initialize(WMScreen *scr)
 
     WMSetLabelText(WPrefs.statusL, _("Initializing configuration panels..."));
 
-    InitWindowHandling(scr, WPrefs.win);
-    InitFocus(scr, WPrefs.win);
-    InitMenuPreferences(scr, WPrefs.win);
-    InitIcons(scr, WPrefs.win);
-    InitPreferences(scr, WPrefs.win);
+    InitFocus(scr, WPrefs.banner);
+    InitWindowHandling(scr, WPrefs.banner);
 
-    InitPaths(scr, WPrefs.win);    
-    InitWorkspace(scr, WPrefs.win);
-    InitConfigurations(scr, WPrefs.win);
+    InitMenuPreferences(scr, WPrefs.banner);
+    InitIcons(scr, WPrefs.banner);
+    InitPreferences(scr, WPrefs.banner);
 
-    InitMenu(scr, WPrefs.win);
+    InitPaths(scr, WPrefs.banner);    
+    InitWorkspace(scr, WPrefs.banner);
+    InitConfigurations(scr, WPrefs.banner);
+
+    InitMenu(scr, WPrefs.banner);
 
 #ifdef not_yet_fully_implemented
-    InitKeyboardSettings(scr, WPrefs.win);
+    InitKeyboardSettings(scr, WPrefs.banner);
 #endif
-    InitKeyboardShortcuts(scr, WPrefs.win);
-    InitMouseSettings(scr, WPrefs.win);
+    InitKeyboardShortcuts(scr, WPrefs.banner);
+    InitMouseSettings(scr, WPrefs.banner);
 
+    InitAppearance(scr, WPrefs.banner);
+    
+#ifdef akk
+    InitFont(scr, WPrefs.banner);
+#endif
 #ifdef not_yet_fully_implemented
-    InitAppearance(scr, WPrefs.win);
-
-    InitText(scr, WPrefs.win);
-    InitThemes(scr, WPrefs.win);
+    InitThemes(scr, WPrefs.banner);
 #endif
-    InitExpert(scr, WPrefs.win);
+    InitExpert(scr, WPrefs.banner);
 
     WMRealizeWidget(WPrefs.scrollV);
 
     WMSetLabelText(WPrefs.statusL, 
 		   _("WPrefs is free software and is distributed WITHOUT ANY\n"
-		   "WARRANTY under the terms of the GNU General Public License.\n"
-		   "The icons in this program are licensed through the\n"
-		   "OpenContent License."));
+	   "WARRANTY under the terms of the GNU General Public License."));
 }
 
 
@@ -569,9 +760,18 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 		path);
 	WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
     }
-    free(path);
+    wfree(path);
 
-    file = popen("wmaker --version", "r");
+    path = getenv("WMAKER_BIN_NAME");
+    if (!path)
+	path = "wmaker";
+    {
+	char *command;
+
+	command = wstrconcat(path, " --version");
+	file = popen(command, "r");
+	wfree(command);
+    }
     if (!file || !fgets(buffer, 1023, file)) {
 	wsyserror(_("could not extract version information from Window Maker"));
 	wfatal(_("Make sure wmaker is in your search path."));
@@ -606,9 +806,15 @@ loadConfigurations(WMScreen *scr, WMWindow *mainw)
 	WMRunAlertPanel(scr, mainw, _("Warning"), mbuf, _("OK"), NULL, NULL);
     }
 
-    file = popen("wmaker --global_defaults_path", "r");
+    {
+	char *command;
+	
+	command = wstrconcat(path, " --global_defaults_path");
+	file = popen(command, "r");
+	wfree(command);
+    }
     if (!file || !fgets(buffer, 1023, file)) {
-	wsyserror(_("could not run \"wmaker --global_defaults_path\"."));
+	wsyserror(_("could not run \"%s --global_defaults_path\"."), path);
 	exit(1);
     } else {
 	char *ptr;
