@@ -44,18 +44,12 @@
  * set by configure */
 #undef PIXMAPDIR
 
-/*
- * define REDUCE_APPICONS if you want apps with the same WM_INSTANCE &&
- * WM_CLASS to share an appicon
- */
-#undef REDUCE_APPICONS
-
 /* Internationalization (I18N) support 
  * set by configure */
 #undef I18N
 
 /* define if you want sound support */
-#undef WMSOUND
+#undef WSOUND
 
 /* define if you want the 'lite' version */
 #undef LITE
@@ -93,9 +87,6 @@
 /* define if your X server has XInternAtoms() */
 #undef HAVE_XINTERNATOMS
 
-/* define if processor is x86 */
-#undef ASM_X86
-
-/* define if processor is x86 with MMX(tm) support */
-#undef ASM_X86_MMX
+/* define if you want Hermes support */
+#undef HAVE_HERMES
 
