@@ -2,7 +2,8 @@
  * 
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998, 1999 Alfredo K. Kojima
+ *  Copyright (c) 1998       Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -45,7 +46,6 @@
 #include "dock.h"
 #include "appmenu.h"
 #include "winspector.h"
-#include "list.h"
 #include "workspace.h"
 
 #ifdef GNOME_STUFF
@@ -55,8 +55,8 @@
 # include "kwm.h"
 #endif
 
-#ifdef WMSOUND
-#include "wmsound.h"
+#ifdef WSOUND
+#include "wsound.h"
 #endif
 
 
@@ -133,11 +133,18 @@ processEvents(int event_count)
 void
 wSetFocusTo(WScreen *scr, WWindow  *wwin)
 {
+    static WScreen *old_scr=NULL;
+
+    WWindow *old_focused;
     WWindow *focused=scr->focused_window;
     int timestamp=LastTimestamp;
     WApplication *oapp=NULL, *napp=NULL;
     int wasfocused;
 
+    if (!old_scr)
+      old_scr=scr;
+    old_focused=old_scr->focused_window;
+
     LastFocusChange = timestamp;
 
 /*
@@ -148,13 +155,13 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 */
       timestamp = CurrentTime;
 
-    if (focused)
-	oapp = wApplicationOf(focused->main_window);
+    if (old_focused)
+	oapp = wApplicationOf(old_focused->main_window);
 
     if (wwin == NULL) {
 	XSetInputFocus(dpy, scr->no_focus_win, RevertToParent, timestamp);
-	if (focused) {
-	    wWindowUnfocus(focused);
+	if (old_focused) {
+	    wWindowUnfocus(old_focused);
 	}
 	if (oapp) {
 	    wAppMenuUnmap(oapp->menu);
@@ -167,7 +174,10 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
 	wKWMSendEventMessage(NULL, WKWMFocusWindow);
 #endif
 	return;
+    } else if (old_scr != scr && old_focused) {
+        wWindowUnfocus(old_focused);
     }
+
     wasfocused = wwin->flags.focused;
     napp = wApplicationOf(wwin->main_window);
 
@@ -202,7 +212,6 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     } else {
 	XSetInputFocus(dpy, scr->no_focus_win, RevertToParent, timestamp);
     }
-
     if (WFLAGP(wwin, no_focusable))
 	return;
 
@@ -231,7 +240,12 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     wWindowFocus(wwin, focused);
 
     if (napp && !wasfocused) {
-	wAppMenuMap(napp->menu, wwin);
+#ifdef USER_MENU
+	wUserMenuRefreshInstances(napp->menu, wwin);
+#endif /* USER_MENU */
+
+    if (wwin->flags.mapped)
+    	wAppMenuMap(napp->menu, wwin);
 #ifdef NEWAPPICON
 	wApplicationActivate(napp);
 #endif
@@ -241,6 +255,7 @@ wSetFocusTo(WScreen *scr, WWindow  *wwin)
     wKWMSendEventMessage(wwin, WKWMFocusWindow);
 #endif
     XFlush(dpy);
+    old_scr=scr;
 }
 
 
@@ -257,8 +272,8 @@ wShadeWindow(WWindow  *wwin)
 
     XLowerWindow(dpy, wwin->client_win);    
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_SHADE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_SHADE);
 #endif
 
 #ifdef ANIMATIONS
@@ -344,8 +359,8 @@ wUnshadeWindow(WWindow  *wwin)
     wwin->flags.mapped = 1;
     XMapWindow(dpy, wwin->client_win);
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNSHADE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNSHADE);
 #endif
 
 #ifdef ANIMATIONS
@@ -362,7 +377,7 @@ wUnshadeWindow(WWindow  *wwin)
 		   + wwin->frame->bottom_width) {
 		XResizeWindow(dpy, wwin->frame->core->window, w, h);
 		XMoveWindow(dpy, wwin->client_win, 0, y);
-		XSync(dpy, 0);
+		XFlush(dpy);
                 if (SHADE_DELAY > 0)
                     wusleep(SHADE_DELAY*2000L/3);
 		h+=s;
@@ -411,17 +426,38 @@ wMaximizeWindow(WWindow *wwin, int directions)
 {
     int new_width, new_height, new_x, new_y;
     WArea usableArea = wwin->screen_ptr->totalUsableArea;
+    WArea totalArea;
 
 
     if (WFLAGP(wwin, no_resizable))
 	return;
-
     
+    totalArea.x1 = 0;
+    totalArea.y1 = 0;
+    totalArea.x2 = wwin->screen_ptr->scr_width;
+    totalArea.y2 = wwin->screen_ptr->scr_height;
+
+#ifdef XINERAMA
+    if (wwin->screen_ptr->xine_count > 0
+	&& !(directions & MAX_IGNORE_XINERAMA)) {
+	WScreen *scr = wwin->screen_ptr;
+	WMRect rect;
+	
+	rect = wGetRectForHead(scr, wGetHeadForWindow(wwin));
+	totalArea.x1 = rect.pos.x;
+	totalArea.y1 = rect.pos.y;
+	totalArea.x2 = totalArea.x1 + rect.size.width;
+	totalArea.y2 = totalArea.y1 + rect.size.height;
+	
+	usableArea.x1 = WMAX(totalArea.x1, usableArea.x1);
+	usableArea.y1 = WMAX(totalArea.y1, usableArea.y1);
+	usableArea.x2 = WMIN(totalArea.x2, usableArea.x2);
+	usableArea.y2 = WMIN(totalArea.y2, usableArea.y2);
+    }
+#endif /* XINERAMA */
+
     if (WFLAGP(wwin, full_maximize)) {
-	usableArea.x1 = 0;
-	usableArea.y1 = 0;
-	usableArea.x2 = wwin->screen_ptr->scr_width;
-	usableArea.y2 = wwin->screen_ptr->scr_height;
+	usableArea = totalArea;
     }
 
     if (wwin->flags.shaded) {
@@ -454,14 +490,13 @@ wMaximizeWindow(WWindow *wwin, int directions)
 
 	new_height = (usableArea.y2-usableArea.y1)-FRAME_BORDER_WIDTH*2;
 	new_y = usableArea.y1;
-	if (WFLAGP(wwin, full_maximize))
+	if (WFLAGP(wwin, full_maximize)) {
 	    new_y -= wwin->frame->top_width;
-
+	    new_height += wwin->frame->bottom_width - 1;
+	}
     } else {
-
 	new_y = wwin->frame_y;
 	new_height = wwin->frame->core->height;
-
     }
 
     if (!WFLAGP(wwin, full_maximize)) {
@@ -479,8 +514,8 @@ wMaximizeWindow(WWindow *wwin, int directions)
     wKWMSendEventMessage(wwin, WKWMChangedClient);
 #endif
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_MAXIMIZE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_MAXIMIZE);
 #endif
 }
 
@@ -513,8 +548,8 @@ wUnmaximizeWindow(WWindow *wwin)
     wKWMSendEventMessage(wwin, WKWMChangedClient);
 #endif
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNMAXIMIZE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNMAXIMIZE);
 #endif
 }
 
@@ -882,6 +917,7 @@ removeIconGrabs(WIcon *icon)
 }
 #endif
 
+
 void
 wIconifyWindow(WWindow *wwin)
 {
@@ -915,11 +951,15 @@ wIconifyWindow(WWindow *wwin)
 		     ButtonMotionMask|ButtonReleaseMask, GrabModeAsync, 
 		     GrabModeAsync, None, None, CurrentTime);
     }
-    
-    if (!wwin->flags.icon_moved) {
-	PlaceIcon(wwin->screen_ptr, &wwin->icon_x, &wwin->icon_y);
+
+    if (!wPreferences.disable_miniwindows) {
+	if (!wwin->flags.icon_moved) {
+	    PlaceIcon(wwin->screen_ptr, &wwin->icon_x, &wwin->icon_y);
+	}
+	wwin->icon = wIconCreate(wwin);
+
+	wwin->icon->mapped = 1;
     }
-    wwin->icon = wIconCreate(wwin);
 
     wwin->flags.miniaturized = 1;
     wwin->flags.mapped = 0;
@@ -929,8 +969,8 @@ wIconifyWindow(WWindow *wwin)
     unmapTransientsFor(wwin);
 
     if (present) {
-#ifdef WMSOUND
-	wSoundPlay(WMSOUND_ICONIFY);
+#ifdef WSOUND
+	wSoundPlay(WSOUND_ICONIFY);
 #endif
 
 	XUngrabPointer(dpy, CurrentTime);
@@ -938,28 +978,61 @@ wIconifyWindow(WWindow *wwin)
 	/* let all Expose events arrive so that we can repaint
 	 * something before the animation starts (and the server is grabbed) */
 	XSync(dpy, 0);
-	wClientSetState(wwin, IconicState, wwin->icon->icon_win);
+
+	if (wPreferences.disable_miniwindows)
+	    wClientSetState(wwin, IconicState, None);
+	else
+	    wClientSetState(wwin, IconicState, wwin->icon->icon_win);
+
 	flushExpose();
 #ifdef ANIMATIONS
 	if (!wwin->screen_ptr->flags.startup && !wwin->flags.skip_next_animation
 	    && !wPreferences.no_animations) {
+	    int ix, iy, iw, ih;
+
+	    if (!wPreferences.disable_miniwindows) {
+		ix = wwin->icon_x;
+		iy = wwin->icon_y;
+		iw = wwin->icon->core->width;
+		ih = wwin->icon->core->height;
+	    } else {
+#ifdef KWM_HINTS
+		WArea area;
+
+		if (wKWMGetIconGeometry(wwin, &area)) {
+		    ix = area.x1;
+		    iy = area.y1;
+		    iw = area.x2 - ix;
+		    ih = area.y2 - iy;
+		} else
+#endif /* KWM_HINTS */
+		{
+		    ix = 0;
+		    iy = 0;
+		    iw = wwin->screen_ptr->scr_width;
+		    ih = wwin->screen_ptr->scr_height;
+		}
+	    }
 	    animateResize(wwin->screen_ptr, wwin->frame_x, wwin->frame_y, 
 			  wwin->frame->core->width, wwin->frame->core->height,
-			  wwin->icon_x, wwin->icon_y,
-			  wwin->icon->core->width, wwin->icon->core->height,
-			  False);
+			  ix, iy, iw, ih, False);
 	}
 #endif
     }
     
     wwin->flags.skip_next_animation = 0;
-    if (wwin->screen_ptr->current_workspace==wwin->frame->workspace ||
-	IS_OMNIPRESENT(wwin) || wPreferences.sticky_icons)
 
-        XMapWindow(dpy, wwin->icon->core->window);
-    AddToStackList(wwin->icon->core);
+    if (!wPreferences.disable_miniwindows) {
 
-    wLowerFrame(wwin->icon->core);
+	if (wwin->screen_ptr->current_workspace==wwin->frame->workspace ||
+	    IS_OMNIPRESENT(wwin) || wPreferences.sticky_icons)
+
+	    XMapWindow(dpy, wwin->icon->core->window);
+
+	AddToStackList(wwin->icon->core);
+
+	wLowerFrame(wwin->icon->core);
+    }
 
     if (present) {
 	WWindow *owner = recursiveTransientFor(wwin->screen_ptr->focused_window);
@@ -977,7 +1050,8 @@ wIconifyWindow(WWindow *wwin)
 	    tmp = wwin->prev;
 	    while (tmp) {
 		if (!WFLAGP(tmp, no_focusable)
-		    && !(tmp->flags.hidden||tmp->flags.miniaturized))
+		    && !(tmp->flags.hidden||tmp->flags.miniaturized)
+		    && (wwin->frame->workspace == tmp->frame->workspace))
 		    break;
 		tmp = tmp->prev;
 	    }
@@ -1000,8 +1074,8 @@ wIconifyWindow(WWindow *wwin)
 #endif
     }
 
-    
-    if (wwin->flags.selected)
+
+    if (wwin->flags.selected && !wPreferences.disable_miniwindows)
         wIconSelect(wwin->icon);
 
 #ifdef GNOME_STUFF
@@ -1013,10 +1087,11 @@ wIconifyWindow(WWindow *wwin)
 #endif
 
     UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_STATE);
-
 }
 
 
+
+
 void
 wDeiconifyWindow(WWindow  *wwin)
 {
@@ -1041,22 +1116,48 @@ wDeiconifyWindow(WWindow  *wwin)
     if (!wwin->flags.shaded)
 	wwin->flags.mapped = 1;
 
-    if (wwin->icon->selected)
-        wIconSelect(wwin->icon);
+    if (!wPreferences.disable_miniwindows) {
+	if (wwin->icon->selected)
+	    wIconSelect(wwin->icon);
 
-    XUnmapWindow(dpy, wwin->icon->core->window);
+	XUnmapWindow(dpy, wwin->icon->core->window);
+    }
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_DEICONIFY);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_DEICONIFY);
 #endif
 
     /* if the window is in another workspace, do it silently */
 #ifdef ANIMATIONS
-    if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations 
+    if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations
 	&& !wwin->flags.skip_next_animation) {
-	animateResize(wwin->screen_ptr, wwin->icon_x, wwin->icon_y,
-		      wwin->icon->core->width, wwin->icon->core->height,
-		      wwin->frame_x, wwin->frame_y, 
+	int ix, iy, iw, ih;
+
+	if (!wPreferences.disable_miniwindows) {
+	    ix = wwin->icon_x;
+	    iy = wwin->icon_y;
+	    iw = wwin->icon->core->width;
+	    ih = wwin->icon->core->height;
+	} else {
+#ifdef KWM_HINTS
+	    WArea area;
+
+	    if (wKWMGetIconGeometry(wwin, &area)) {
+		ix = area.x1;
+		iy = area.y1;
+		iw = area.x2 - ix;
+		ih = area.y2 - iy;
+	    } else
+#endif /* KWM_HINTS */
+	    {
+		ix = 0;
+		iy = 0;
+		iw = wwin->screen_ptr->scr_width;
+		ih = wwin->screen_ptr->scr_height;
+	    }
+	}
+	animateResize(wwin->screen_ptr, ix, iy, iw, ih,
+		      wwin->frame_x, wwin->frame_y,
 		      wwin->frame->core->width, wwin->frame->core->height,
 		      False);
     }
@@ -1072,15 +1173,17 @@ wDeiconifyWindow(WWindow  *wwin)
 	wClientSetState(wwin, NormalState, None);
     }
     mapTransientsFor(wwin);
-    RemoveFromStackList(wwin->icon->core);
-/*    removeIconGrabs(wwin->icon);*/
-    wIconDestroy(wwin->icon);
-    wwin->icon = NULL;
 
+    if (!wPreferences.disable_miniwindows) {
+	RemoveFromStackList(wwin->icon->core);
+	/*    removeIconGrabs(wwin->icon);*/
+	wIconDestroy(wwin->icon);
+	wwin->icon = NULL;
+    }
     XUngrabServer(dpy);
-    if (wPreferences.focus_mode==WKF_CLICK
-	|| wPreferences.focus_mode==WKF_SLOPPY)
-      wSetFocusTo(wwin->screen_ptr, wwin);
+
+    if (wPreferences.focus_mode==WKF_CLICK)
+	wSetFocusTo(wwin->screen_ptr, wwin);
 
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup) {
@@ -1115,9 +1218,11 @@ static void
 hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
 {
     if (wwin->flags.miniaturized) {
-	XUnmapWindow(dpy, wwin->icon->core->window);
+	if (wwin->icon) { 
+	    XUnmapWindow(dpy, wwin->icon->core->window);
+	    wwin->icon->mapped = 0;
+	}
 	wwin->flags.hidden = 1;
-	wwin->icon->mapped = 0;
 #ifdef GNOME_STUFF
 	wGNOMEUpdateClientStateHint(wwin, False);
 #endif
@@ -1128,12 +1233,7 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
     }
 
     if (wwin->flags.inspector_open) {
-        WWindow *pwin = wwin->inspector->frame;
-
-	wWindowUnmap(pwin);
-        pwin->flags.hidden = 1;
-
-        wClientSetState(pwin, IconicState, icon->icon_win);
+	wHideInspectorForWindow(wwin);
     }
 
     wwin->flags.hidden = 1;
@@ -1141,8 +1241,8 @@ hideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate)
 
     wClientSetState(wwin, IconicState, icon->icon_win);
     flushExpose();
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_HIDE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_HIDE);
 #endif
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations &&
@@ -1169,52 +1269,27 @@ wHideOtherApplications(WWindow *awin)
 {
     WWindow *wwin;
     WApplication *tapp;
-#ifdef REDUCE_APPICONS
-    char *tinstance, *tclass;
-    unsigned int brokenwin = 0, match = 0;
-#endif
 
     if (!awin)
 	return;
     wwin = awin->screen_ptr->focused_window;
 
-#ifdef REDUCE_APPICONS
-    if (awin->wm_instance == NULL || awin->wm_class == NULL)
-	brokenwin++;
-#endif
 
     while (wwin) {
 	if (wwin!=awin 
 	    && wwin->frame->workspace == awin->screen_ptr->current_workspace
             && !(wwin->flags.miniaturized||wwin->flags.hidden)
             && !wwin->flags.internal_window
-            && (!wwin->flags.inspector_open || wwin->inspector->frame!=awin)
+	    && wGetWindowOfInspectorForWindow(wwin) != awin
 	    && !WFLAGP(wwin, no_hide_others)) {
 	    
-#ifdef REDUCE_APPICONS
-	    match = 0;
-	    if (!brokenwin) {
-		if ((tinstance = wwin->wm_instance) == NULL)
-		    tinstance = "";
-		if ((tclass = wwin->wm_class) == NULL)
-		    tclass = "";
-		if ((strcmp(awin->wm_instance, tinstance) == 0) &&
-		    (strcmp(awin->wm_class, tclass) == 0) )
-			match++;
-	    }
-#endif
-
 	    if (wwin->main_window==None || WFLAGP(wwin, no_appicon)) {
 		if (!WFLAGP(wwin, no_miniaturizable)) {
 		    wwin->flags.skip_next_animation = 1;
 		    wIconifyWindow(wwin);
 		}
 	    } else if (wwin->main_window!=None
-#ifndef REDUCE_APPICONS
 		       && awin->main_window != wwin->main_window) {
-#else
-			&& (awin->main_window != wwin->main_window && !match)) {
-#endif
 		tapp = wApplicationOf(wwin->main_window);
 		if (tapp) {
 		    tapp->flags.skip_next_animation = 1;
@@ -1239,11 +1314,6 @@ wHideOtherApplications(WWindow *awin)
 void
 wHideApplication(WApplication *wapp)
 {
-#ifdef REDUCE_APPICONS
-    WApplication *tapp;
-    char *tinstance, *tclass;
-    unsigned int nowmhints = 0, matchwmhints = 0, matchworkspace = 0;
-#endif
     WScreen *scr;
     WWindow *wlist;
     int hadfocus;
@@ -1256,11 +1326,6 @@ wHideApplication(WApplication *wapp)
 	wwarning("group leader not found for window group");
 	return;
     }
-#ifdef REDUCE_APPICONS
-    if ((wapp->main_window_desc->wm_instance == NULL) ||
-        (wapp->main_window_desc->wm_class == NULL))
-        nowmhints++;
-#endif
     scr = wapp->main_window_desc->screen_ptr;
     hadfocus = 0;
     wlist = scr->focused_window;
@@ -1272,38 +1337,7 @@ wHideApplication(WApplication *wapp)
     else
 	wapp->last_focused = NULL;
     while (wlist) {
-#ifdef REDUCE_APPICONS
-	matchwmhints = matchworkspace = 0;
-	if (!nowmhints) {
-	    tapp = wApplicationOf(wlist->main_window);
-	    tinstance = tclass = NULL;
-	    if (tapp) {
-		if (tapp->main_window_desc) {
-		    tinstance = tapp->main_window_desc->wm_instance;
-		    tclass = tapp->main_window_desc->wm_class;
-		}
-	    }
-	    if (tapp == NULL || tinstance == NULL || tclass == NULL) {
-		/* Should never reach here */
-		tinstance = "";
-		tclass = "";
-	    }
-	    if ((strcmp(tinstance, wapp->main_window_desc->wm_instance) == 0) &&
-		(strcmp(tclass, wapp->main_window_desc->wm_class) == 0) )
-		matchwmhints++;
-	}
-	if (wlist->frame) {
-	    if (wlist->frame->workspace == wapp->main_window_desc->screen_ptr->current_workspace)
-		matchworkspace++;
-	}
-        if ((wlist->main_window == wapp->main_window || matchwmhints) && 
-	    matchworkspace) {
-#ifdef I_HATE_THIS
-	}
-#endif
-#else
 	if (wlist->main_window == wapp->main_window) {
-#endif
 	    if (wlist->flags.focused) {
 		hadfocus = 1;
             }
@@ -1337,6 +1371,10 @@ wHideApplication(WApplication *wapp)
     if(wPreferences.auto_arrange_icons) {
 	wArrangeIcons(scr, True);
     }
+#ifdef HIDDENDOT
+    if (wapp->app_icon)
+    	wAppIconPaint(wapp->app_icon);
+#endif
 }
 
 
@@ -1352,8 +1390,8 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
     wwin->flags.hidden=0;
     wwin->flags.mapped=1;
     
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_UNHIDE);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_UNHIDE);
 #endif
 #ifdef ANIMATIONS
     if (!wwin->screen_ptr->flags.startup && !wPreferences.no_animations 
@@ -1371,13 +1409,7 @@ unhideWindow(WIcon *icon, int icon_x, int icon_y, WWindow *wwin, int animate,
     wClientSetState(wwin, NormalState, None);
     wRaiseFrame(wwin->frame->core);
     if (wwin->flags.inspector_open) {
-        WWindow *pwin = wwin->inspector->frame;
-
-        pwin->flags.hidden = 0;
-        pwin->flags.mapped = 1;
-        XMapWindow(dpy, pwin->client_win);
-        XMapWindow(dpy, pwin->frame->core->window);
-        wClientSetState(pwin, NormalState, None);
+	wUnhideInspectorForWindow(wwin);
     }
 
 #ifdef GNOME_STUFF
@@ -1395,21 +1427,10 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
     WScreen *scr;
     WWindow *wlist, *next;
     WWindow *focused=NULL;
-#ifdef REDUCE_APPICONS
-    char *tinstance, *tclass;
-    unsigned int nowmhints = 0, matchwmhints = 0, matchworkspace = 0;
-#endif
 
     if (!wapp) {
 	return;
     }
-
-#ifdef REDUCE_APPICONS
-    if ((wapp->main_window_desc->wm_class == NULL) || 
-	(wapp->main_window_desc->wm_instance == NULL))
-	nowmhints++;
-#endif
-
     scr = wapp->main_window_desc->screen_ptr;
     wlist = scr->focused_window;
     if (!wlist) return;
@@ -1420,27 +1441,7 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
     while (wlist) {
 	next = wlist->next;
 
-#ifndef REDUCE_APPICONS	
 	if (wlist->main_window == wapp->main_window) {
-#else
-	matchwmhints = matchworkspace = 0;
-	if (!nowmhints) {
-	    if ((tinstance = wlist->wm_instance) == NULL)
-		tinstance = "";
-	    if ((tclass = wlist->wm_class) == NULL)
-		tclass = "";
-	    if ((strcmp(tinstance, wapp->main_window_desc->wm_instance) == 0) 
-		&& (strcmp(tclass, wapp->main_window_desc->wm_class) == 0) )
-		matchwmhints++;
-	}
-	if (wlist->frame) {
-	    if (wlist->frame->workspace == wapp->main_window_desc->screen_ptr->current_workspace)
-		matchworkspace++;
-	}
-
-	if ((wlist->main_window == wapp->main_window || matchwmhints) && 
-	    matchworkspace) {
-#endif
 	    if (wlist->flags.focused)
 	      focused = wlist;
 	    else if (!focused || !focused->flags.focused)
@@ -1490,6 +1491,9 @@ wUnhideApplication(WApplication *wapp, Bool miniwindows, Bool bringToCurrentWS)
     if (wPreferences.auto_arrange_icons) {
         wArrangeIcons(scr, True);
     }
+#ifdef HIDDENDOT
+    wAppIconPaint(wapp->app_icon);
+#endif 
 }
 
 
@@ -1616,8 +1620,8 @@ wArrangeIcons(WScreen *scr, Bool arrangeAll)
 
     pi = 0;
     si = 0;
-    while (aicon) {
-        if (!aicon->docked) {
+    while (aicon) {	
+        if (!aicon->docked && wAppIconIsFirstInstance(aicon)) {
             if (aicon->x_pos != X || aicon->y_pos != Y) {
 #ifdef ANIMATIONS
                 if (!wPreferences.no_animations) {
@@ -1625,7 +1629,7 @@ wArrangeIcons(WScreen *scr, Bool arrangeAll)
                                 X, Y);
                 }
 #endif /* ANIMATIONS */
-            }
+            }	    
 	    wAppIconMove(aicon, X, Y);
             pi++;
         }
@@ -1644,8 +1648,8 @@ wArrangeIcons(WScreen *scr, Bool arrangeAll)
     while (wwin && wwin->prev)
         wwin = wwin->prev;
 
-    while (wwin) { 
-        if (wwin->icon && wwin->flags.miniaturized &&/*!wwin->flags.hidden &&*/
+    while (wwin) {
+        if (wwin->icon && wwin->flags.miniaturized && !wwin->flags.hidden &&
             (wwin->frame->workspace==scr->current_workspace ||
              IS_OMNIPRESENT(wwin) || wPreferences.sticky_icons)) {
 	    
@@ -1684,15 +1688,31 @@ void
 wSelectWindow(WWindow *wwin, Bool flag)
 {
     WScreen *scr = wwin->screen_ptr;
+    
     if (flag) {
 	wwin->flags.selected = 1;
 	XSetWindowBorder(dpy, wwin->frame->core->window, scr->white_pixel);
-	scr->selected_windows = list_cons(wwin, scr->selected_windows);
+	
+	if (WFLAGP(wwin, no_border)) {	
+	    XSetWindowBorderWidth(dpy, wwin->frame->core->window, 
+				  FRAME_BORDER_WIDTH);
+	}
+
+	if (!scr->selected_windows)
+	    scr->selected_windows = WMCreateArray(4);
+	WMAddToArray(scr->selected_windows, wwin);
     } else {
 	wwin->flags.selected = 0;
-	XSetWindowBorder(dpy, wwin->frame->core->window, 
+	XSetWindowBorder(dpy, wwin->frame->core->window,
 			 scr->frame_border_pixel);
-	scr->selected_windows = list_remove_elem(scr->selected_windows, wwin);
+
+	if (WFLAGP(wwin, no_border)) {	
+	    XSetWindowBorderWidth(dpy, wwin->frame->core->window, 0);
+	}
+
+	if (scr->selected_windows) {
+	    WMRemoveFromArray(scr->selected_windows, wwin);
+	}
     }
 }
 
@@ -1700,7 +1720,8 @@ wSelectWindow(WWindow *wwin, Bool flag)
 void
 wMakeWindowVisible(WWindow *wwin)
 {
-    wWorkspaceChange(wwin->screen_ptr, wwin->frame->workspace);
+    if (wwin->frame->workspace != wwin->screen_ptr->current_workspace)
+	wWorkspaceChange(wwin->screen_ptr, wwin->frame->workspace);
 
     if (wwin->flags.shaded) {
 	wUnshadeWindow(wwin);
@@ -1714,7 +1735,8 @@ wMakeWindowVisible(WWindow *wwin)
     } else if (wwin->flags.miniaturized) {
 	wDeiconifyWindow(wwin);
     } else {
-	wSetFocusTo(wwin->screen_ptr, wwin);
+	if (!WFLAGP(wwin, no_focusable))
+	    wSetFocusTo(wwin->screen_ptr, wwin);
 	wRaiseFrame(wwin->frame->core);
     }
 }
