@@ -1,9 +1,10 @@
 /* appicon.c- icon for applications (not mini-window)
  *
  *  Window Maker window manager
- * 
+ *
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *  Copyright (c) 1998       Dan Pascu
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  *  USA.
  */
 
@@ -43,8 +44,14 @@
 #include "framewin.h"
 #include "dialog.h"
 #include "client.h"
+#ifdef XDND
+#include "xdnd.h"
+#endif
+#include "wsound.h"
+#include <WINGs/WINGsP.h>
 
-/* 
+
+/*
  * icon_file for the dock is got from the preferences file by
  * using the classname/instancename
  */
@@ -82,7 +89,7 @@ wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
     scr->app_icon_list = dicon;
 
     if (command) {
-	dicon->command = wstrdup(command);	
+	dicon->command = wstrdup(command);
     }
     if (wm_class)
       dicon->wm_class = wstrdup(wm_class);
@@ -95,15 +102,24 @@ wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
 
 	path = wDefaultGetIconFile(scr, wm_instance, wm_class, False);
     }
-    
+
     if (path)
 	path = FindImage(wPreferences.icon_path, path);
 
     dicon->icon = wIconCreateWithIconFile(scr, path, tile);
     if (path)
-	free(path);
-#ifdef REDUCE_APPICONS
-    dicon->num_apps = 0;
+	wfree(path);
+#ifdef XDND
+    wXDNDMakeAwareness(dicon->icon->core->window);
+#endif
+
+#ifdef DEMATERIALIZE_ICON
+    {
+        XSetWindowAttributes attribs;
+        attribs.save_under = True;
+        XChangeWindowAttributes(dpy, dicon->icon->core->window,
+                CWSaveUnder, &attribs);
+    }
 #endif
 
     /* will be overriden by dock */
@@ -122,30 +138,11 @@ WAppIcon*
 wAppIconCreate(WWindow *leader_win)
 {
     WAppIcon *aicon;
-#ifdef REDUCE_APPICONS
-    WAppIcon *atmp;
-    WAppIconAppList *applist;
-    char *tinstance, *tclass;
-#endif
     WScreen *scr = leader_win->screen_ptr;
 
     aicon = wmalloc(sizeof(WAppIcon));
     wretain(aicon);
     memset(aicon, 0, sizeof(WAppIcon));
-#ifdef REDUCE_APPICONS
-    applist = wmalloc(sizeof(WAppIconAppList));
-    memset(applist, 0, sizeof(WAppIconAppList));
-    applist->wapp = wApplicationOf(leader_win->main_window);
-    aicon->applist = applist;
-    if (applist->wapp == NULL) {
-	/* Something's wrong. wApplicationOf() should always return a
-	 * valid structure.  Rather than violate assumptions, bail. -cls
-	 */
-	free(applist);
-	wrelease(aicon);
-	return NULL;
-    }
-#endif
 
     aicon->yindex = -1;
     aicon->xindex = -1;
@@ -153,43 +150,7 @@ wAppIconCreate(WWindow *leader_win)
     aicon->prev = NULL;
     aicon->next = scr->app_icon_list;
     if (scr->app_icon_list) {
-#ifndef REDUCE_APPICONS
         scr->app_icon_list->prev = aicon;
-#else
-	/* If we aren't going to have a match, jump straight to new appicon */
-	if (leader_win->wm_class == NULL || leader_win->wm_class == NULL)
-	    atmp = NULL;
-	else
-	atmp = scr->app_icon_list;
-
-	while (atmp != NULL) {
-	    if ((tinstance = atmp->wm_instance) == NULL)
-		tinstance = "";
-	    if ((tclass = atmp->wm_class) == NULL)
-		tclass = "";
-	    if ((strcmp(leader_win->wm_class, tclass) == 0) &&
-		(strcmp(leader_win->wm_instance, tinstance) == 0))
-	    {
-		/* We have a winner */
-		wrelease(aicon);	
-		atmp->num_apps++;
-		applist->next = atmp->applist;
-		if (atmp->applist)
-		    atmp->applist->prev = applist;
-		atmp->applist = applist;
-
-		if (atmp->docked) {
-		    wDockSimulateLaunch(atmp->dock, atmp);
-		}
-
-		return atmp;
-	    }
-	    atmp = atmp->next;
-	}
-	if (atmp == NULL) {
-	    scr->app_icon_list->prev = aicon;
-	}
-#endif /* REDUCE_APPICONS */
     }
     scr->app_icon_list = aicon;
 
@@ -197,11 +158,19 @@ wAppIconCreate(WWindow *leader_win)
         aicon->wm_class = wstrdup(leader_win->wm_class);
     if (leader_win->wm_instance)
         aicon->wm_instance = wstrdup(leader_win->wm_instance);
-#ifdef REDUCE_APPICONS
-    aicon->num_apps = 1;
-#endif
 
     aicon->icon = wIconCreate(leader_win);
+#ifdef DEMATERIALIZE_ICON
+    {
+        XSetWindowAttributes attribs;
+        attribs.save_under = True;
+        XChangeWindowAttributes(dpy, aicon->icon->core->window,
+                CWSaveUnder, &attribs);
+    }
+#endif
+#ifdef XDND
+    wXDNDMakeAwareness(aicon->icon->core->window);
+#endif
 
     /* will be overriden if docked */
     aicon->icon->core->descriptor.handle_mousedown = appIconMouseDown;
@@ -220,33 +189,19 @@ void
 wAppIconDestroy(WAppIcon *aicon)
 {
     WScreen *scr = aicon->icon->core->screen_ptr;
-#ifdef REDUCE_APPICONS
-    WAppIconAppList *aptmp;
-#endif
 
     RemoveFromStackList(aicon->icon->core);
     wIconDestroy(aicon->icon);
     if (aicon->command)
-	free(aicon->command);
+	wfree(aicon->command);
 #ifdef OFFIX_DND
     if (aicon->dnd_command)
-	free(aicon->dnd_command);
+	wfree(aicon->dnd_command);
 #endif
     if (aicon->wm_instance)
-      free(aicon->wm_instance);
+      wfree(aicon->wm_instance);
     if (aicon->wm_class)
-      free(aicon->wm_class);
-#ifdef REDUCE_APPICONS
-    /* There should never be a list but just in case */
-    if (aicon->applist != NULL) {
-	aptmp = aicon->applist;
-    	while (aptmp->next) {
-	    aptmp = aptmp->next;
-	    free(aptmp->prev);
-	}
-	free(aptmp);
-    }
-#endif
+      wfree(aicon->wm_class);
 
     if (aicon == scr->app_icon_list) {
         if (aicon->next)
@@ -291,12 +246,61 @@ drawCorner(WIcon *icon, WWindow *wwin, int active)
 #endif /* NEWAPPICON */
 
 
-void 
+static void
+drawCorner(WIcon *icon)
+{
+    WScreen *scr = icon->core->screen_ptr;
+    XPoint points[3];
+
+    points[0].x = 1;
+    points[0].y = 1;
+    points[1].x = 12;
+    points[1].y = 1;
+    points[2].x = 1;
+    points[2].y = 12;
+    XFillPolygon(dpy, icon->core->window, scr->icon_title_texture->normal_gc,
+                 points, 3, Convex, CoordModeOrigin);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              0, 0, 0, 12);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              0, 0, 12, 0);
+    /* drawing the second line gives a weird concave look. -Dan */
+#if 0
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              1, 1, 1, 11);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              1, 1, 11, 1);
+#endif
+}
+
+
+void
 wAppIconMove(WAppIcon *aicon, int x, int y)
 {
-    XMoveWindow(dpy, aicon->icon->core->window, x, y);
-    aicon->x_pos = x;
-    aicon->y_pos = y;
+    WApplication *app;
+    WAppIcon *tmp;
+    
+    app = wApplicationOf(aicon->icon->owner->main_window);
+
+    if (!WFLAGP(app->main_window_desc, collapse_appicons)) {	
+	XMoveWindow(dpy, aicon->icon->core->window, x, y);
+	aicon->x_pos = x;
+	aicon->y_pos = y;
+    } else {
+	tmp = aicon->icon->core->screen_ptr->app_icon_list;    
+    
+	/* move all icons of the same class/name to the same pos */
+	while (tmp) {
+	    if (strcmp(tmp->wm_class, aicon->wm_class) == 0 &&
+		strcmp(tmp->wm_instance, aicon->wm_instance) == 0 &&
+		!tmp->docked) {
+		XMoveWindow(dpy, tmp->icon->core->window, x, y);
+		tmp->x_pos = x;
+		tmp->y_pos = y;
+	    }
+	    tmp = tmp->next;
+	}
+    }
 }
 
 
@@ -304,98 +308,169 @@ wAppIconMove(WAppIcon *aicon, int x, int y)
 static void
 updateDockNumbers(WScreen *scr)
 {
-   int length;
-   char *ws_numbers;
-   GC numbers_gc;
-   XGCValues my_gc_values;
-   unsigned long my_v_mask = (GCForeground);
-   WAppIcon *dicon = scr->dock->icon_array[0];
+    int length;
+    char *ws_numbers;
+    GC numbers_gc;
+    XGCValues my_gc_values;
+    unsigned long my_v_mask = (GCForeground);
+    WAppIcon *dicon = scr->dock->icon_array[0];
+    
+    my_gc_values.foreground = scr->white_pixel;
+    numbers_gc = XCreateGC(dpy, dicon->icon->core->window,
+			   my_v_mask, &my_gc_values);
 
-   my_gc_values.foreground = scr->white_pixel;
-   numbers_gc = XCreateGC(dpy, dicon->icon->core->window,
-			  my_v_mask, &my_gc_values);
-   
-   ws_numbers = malloc(20);
-   sprintf(ws_numbers, "%i [ %i ]", scr->current_workspace+1, 
-	   ((scr->current_workspace/10)+1));
-   length = strlen(ws_numbers);
+    ws_numbers = malloc(20);
+    sprintf(ws_numbers, "%i [ %i ]", scr->current_workspace+1,
+	    ((scr->current_workspace/10)+1));
+    length = strlen(ws_numbers);
+    
+    XClearArea(dpy, dicon->icon->core->window, 2, 2, 50,
+	       WMFontHeight(scr->icon_title_font)+1, False);
+    
+    XSetForeground(dpy, numbers_gc, scr->black_pixel);
+    WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
+		 scr->icon_title_font, 4, 3, ws_numbers, length);
 
-   XClearArea(dpy, dicon->icon->core->window, 2, 2, 50, 
-	      scr->icon_title_font->y+1, False);
+    XSetForeground(dpy, numbers_gc, scr->white_pixel);
+    WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
+		 scr->icon_title_font, 3, 2, ws_numbers, length);
 
-#ifndef I18N_MB
-   XSetFont(dpy, numbers_gc, scr->icon_title_font->font->fid);
-   XSetForeground(dpy, numbers_gc, scr->black_pixel);
-   XDrawString(dpy, dicon->icon->core->window,
-               numbers_gc, 4, scr->icon_title_font->y+3, ws_numbers, length);
-   XSetForeground(dpy, numbers_gc, scr->white_pixel);
-   XDrawString(dpy, dicon->icon->core->window,
-               numbers_gc, 3, scr->icon_title_font->y+2, ws_numbers, length);
-#else
-   XSetForeground(dpy, numbers_gc, scr->black_pixel);
-   XmbDrawString(dpy, dicon->icon->core->window,
-                 scr->icon_title_font->font, numbers_gc, 4,
-                 scr->icon_title_font->y+3, ws_numbers, length);
-   XSetForeground(dpy, numbers_gc, scr->white_pixel);
-   XmbDrawString(dpy, dicon->icon->core->window,
-                 scr->icon_title_font->font, numbers_gc, 3,
-                 scr->icon_title_font->y+2, ws_numbers, length);
-#endif /* I18N_MB */
-
-   XFreeGC(dpy, numbers_gc);
-   free(ws_numbers);
+    XFreeGC(dpy, numbers_gc);
+    wfree(ws_numbers);
 }
 #endif /* WS_INDICATOR */
 
 
+
+WAppIcon*
+wAppIconNextSibling(WAppIcon *icon)
+{    
+    WApplication *app;
+    WAppIcon *tmp;
+    
+    app = wApplicationOf(icon->icon->owner->main_window);
+
+    if (!WFLAGP(app->main_window_desc, collapse_appicons)) {
+	return NULL;
+    }
+    
+    tmp = icon->next;
+    while (tmp) {
+	if (strcmp(tmp->wm_class, icon->wm_class) == 0
+	    && strcmp(tmp->wm_instance, icon->wm_instance) == 0
+	    && !tmp->docked) {
+	    return tmp;
+	}
+	tmp = tmp->next;
+    }
+    
+    tmp = icon->icon->core->screen_ptr->app_icon_list;
+    while (tmp && tmp != icon) {
+	if (strcmp(tmp->wm_class, icon->wm_class) == 0
+	    && strcmp(tmp->wm_instance, icon->wm_instance) == 0
+	    && !tmp->docked) {
+	    return tmp;
+	}
+	tmp = tmp->next;
+    }
+    
+    return NULL;
+}
+
+
+Bool
+wAppIconIsFirstInstance(WAppIcon *icon)
+{
+    WAppIcon *list = icon->icon->core->screen_ptr->app_icon_list;
+    int index = 0;
+
+    if (!WFLAGP(icon->icon->owner, collapse_appicons))
+	return True;
+    
+    while (list) {
+	if (icon == list)
+	    return True;
+	
+	if (strcmp(icon->wm_instance,
+		   list->wm_instance) == 0
+	    &&
+	    strcmp(icon->wm_class,
+		   list->wm_class) == 0
+	    && !icon->docked)
+	    return False;
+
+	list = list->next;
+    }
+    
+    puts("OH SHIT!?!?!? HOW THE FUCK DID WE GET HERE!?!?!?!?!");
+    
+    return 0;
+}
+
+
+
 void
 wAppIconPaint(WAppIcon *aicon)
 {
+    WApplication *wapp;    
     WScreen *scr = aicon->icon->core->screen_ptr;
+    int index;
+
+    if (aicon->icon->owner)
+	wapp = wApplicationOf(aicon->icon->owner->main_window);
+    else
+	wapp = NULL;
 
     wIconPaint(aicon->icon);
 
 
 # ifdef WS_INDICATOR
-    if (aicon->docked && scr->dock && aicon->yindex==0)
+    if (aicon->docked && scr->dock && scr->dock==aicon->dock &&
+        aicon->yindex==0)
 	updateDockNumbers(scr);
 # endif
     if (scr->dock_dots && aicon->docked && !aicon->running
 	&& aicon->command!=NULL) {
 	XSetClipMask(dpy, scr->copy_gc, scr->dock_dots->mask);
 	XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
-	XCopyArea(dpy, scr->dock_dots->image, aicon->icon->core->window, 
+	XCopyArea(dpy, scr->dock_dots->image, aicon->icon->core->window,
 		  scr->copy_gc, 0, 0, scr->dock_dots->width,
 		  scr->dock_dots->height, 0, 0);
+    }       
+    
+#ifdef HIDDENDOT
+    if (wapp && wapp->flags.hidden) {
+	XSetClipMask(dpy, scr->copy_gc, scr->dock_dots->mask);
+	XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
+	XCopyArea(dpy, scr->dock_dots->image,
+		  aicon->icon->core->window,
+		  scr->copy_gc, 0, 0, 7,
+		  scr->dock_dots->height, 0, 0);        
     }
+#endif /* HIDDENDOT */
 
-#ifdef NEWAPPICON
-    if (!wPreferences.strict_ns && aicon->icon->owner!=NULL) {
-	int active=0;
-	if (aicon->main_window==None) {
-	    active = (aicon->icon->owner->flags.focused ? 1 : 0);
-	} else {
-	    WApplication *wapp;
-
-	    wapp = wApplicationOf(aicon->main_window);
-	    if (!wapp) {
-		active = aicon->icon->owner->flags.focused;
-		wwarning("error in wAppIconPaint(). Please report it");
-	    } else {
-		active = wapp->main_window_desc->flags.focused;
-		if (wapp->main_window_desc->flags.hidden
-		    || !wapp->main_window_desc->flags.mapped)
-		  active = -1;
-	    }
-	}
-	if (active>=0)
-	  drawCorner(aicon->icon, aicon->icon->owner, active);
+    if (wapp)
+	index = wApplicationIndexOfGroup(wapp);
+    else
+	index = 0;
+    
+    if (index > 0) {
+	char buf[16];
+	
+	sprintf(buf, "%i", index);
+	
+	WMDrawString(scr->wmscreen, aicon->icon->core->window, 
+		     scr->clip_title_gc, scr->title_font,
+		     1, 1, buf, strlen(buf));
     }
-#endif /* NEWAPPICON */
+    
+    
+    if (aicon->omnipresent)
+        drawCorner(aicon->icon);
 
     XSetClipMask(dpy, scr->copy_gc, None);
     if (aicon->launching) {
-	XFillRectangle(dpy, aicon->icon->core->window, scr->stipple_gc, 
+	XFillRectangle(dpy, aicon->icon->core->window, scr->stipple_gc,
 		       0, 0, wPreferences.icon_size, wPreferences.icon_size);
     }
 }
@@ -404,65 +479,13 @@ wAppIconPaint(WAppIcon *aicon)
 
 #define canBeDocked(wwin)  ((wwin) && ((wwin)->wm_class||(wwin)->wm_instance))
 
-#ifdef REDUCE_APPICONS
-unsigned int
-wAppIconReduceAppCount(WApplication *wapp) 
-{
-    WAppIconAppList *applist;
-
-    if (wapp == NULL)
-	return 0;
-
-    if (wapp->app_icon == NULL) 
-	return 0;
-
-    /* If given a main window, check the applist 
-     * and remove the if it exists 
-     */
-	applist = wapp->app_icon->applist;
-	while (applist != NULL) {
-	        if (applist->wapp == wapp) {
-		    /* If this app owns the appicon, change the appicon's 
-		     * owner to the next app in the list or NULL
-		     */
-	    if (wapp->app_icon->icon->owner == applist->wapp->main_window_desc)
-{
-			if (applist->next) {
-			    wapp->app_icon->icon->owner = applist->next->wapp->main_window_desc;
-			} else if (applist->prev) {
-			    wapp->app_icon->icon->owner = applist->prev->wapp->main_window_desc;
-			} else {
-			    wapp->app_icon->icon->owner = NULL;
-			}
-		    }
-		    if (applist->prev)
-			applist->prev->next = applist->next;
-
-		    if (applist->next)
-			applist->next->prev = applist->prev;
-
-	    if (applist == wapp->app_icon->applist)
-		    	wapp->app_icon->applist = applist->next;
-
-		    free(applist);
-
-		    if (wapp->app_icon->applist != NULL)
-		    	wapp->app_icon->main_window = wapp->app_icon->applist->wapp->main_window;
-
-		    return (--wapp->app_icon->num_apps);
-	    	}
-	    applist = applist->next;
-	}
-    return (--wapp->app_icon->num_apps);
-}
-#endif
 
 
 static void
 hideCallback(WMenu *menu, WMenuEntry *entry)
 {
     WApplication *wapp = (WApplication*)entry->clientdata;
-    
+
     if (wapp->flags.hidden) {
 	wWorkspaceChange(menu->menu->screen_ptr, wapp->last_workspace);
 	wUnhideApplication(wapp, False, False);
@@ -476,11 +499,21 @@ static void
 unhideHereCallback(WMenu *menu, WMenuEntry *entry)
 {
     WApplication *wapp = (WApplication*)entry->clientdata;
-    
+
     wUnhideApplication(wapp, False, True);
 }
 
 
+static void
+collapseCallback(WMenu *menu, WMenuEntry *entry)
+{
+    WApplication *wapp = (WApplication*)entry->clientdata;
+    
+    wApplicationSetCollapse(wapp, 
+			    !WFLAGP(wapp->main_window_desc, collapse_appicons));
+}
+
+
 static void
 setIconCallback(WMenu *menu, WMenuEntry *entry)
 {
@@ -495,26 +528,26 @@ setIconCallback(WMenu *menu, WMenuEntry *entry)
 	return;
     icon->editing = 1;
     scr = icon->icon->core->screen_ptr;
-    
+
     wretain(icon);
 
     result = wIconChooserDialog(scr, &file, icon->wm_instance, icon->wm_class);
-    
+
     if (result && !icon->destroyed) {
 	if (file[0]==0) {
-	    free(file);
+	    wfree(file);
 	    file = NULL;
 	}
 	if (!wIconChangeImageFile(icon->icon, file)) {
 	    wMessageDialog(scr, _("Error"),
-			   _("Could not open specified icon file"), 
+			   _("Could not open specified icon file"),
 			   _("OK"), NULL, NULL);
 	} else {
 	    wDefaultChangeIcon(scr, icon->wm_instance, icon->wm_class, file);
 	    wAppIconPaint(icon);
 	}
 	if (file)
-	    free(file);
+	    wfree(file);
     }
     icon->editing = 0;
     wrelease(icon);
@@ -531,10 +564,10 @@ killCallback(WMenu *menu, WMenuEntry *entry)
 	return;
 
     WCHANGE_STATE(WSTATE_MODAL);
-    
+
     assert(entry->clientdata!=NULL);
 
-    buffer = wstrappend(wapp->app_icon ? wapp->app_icon->wm_class : NULL,
+    buffer = wstrconcat(wapp->app_icon ? wapp->app_icon->wm_class : NULL,
 			_(" will be forcibly closed.\n"
 			  "Any unsaved changes will be lost.\n"
 			  "Please confirm."));
@@ -548,7 +581,7 @@ killCallback(WMenu *menu, WMenuEntry *entry)
     }
     wrelease(wapp->main_window_desc);
 
-    free(buffer);
+    wfree(buffer);
 
     WCHANGE_STATE(WSTATE_NORMAL);
 }
@@ -558,10 +591,11 @@ static WMenu*
 createApplicationMenu(WScreen *scr)
 {
     WMenu *menu;
-    
+
     menu = wMenuCreate(scr, NULL, False);
     wMenuAddCallback(menu, _("Unhide Here"), unhideHereCallback, NULL);
     wMenuAddCallback(menu, _("Hide"), hideCallback, NULL);
+    wMenuAddCallback(menu, _("Collapse"), collapseCallback, NULL);
     wMenuAddCallback(menu, _("Set Icon..."), setIconCallback, NULL);
     wMenuAddCallback(menu, _("Kill"), killCallback, NULL);
 
@@ -575,10 +609,10 @@ openApplicationMenu(WApplication *wapp, int x, int y)
     WMenu *menu;
     WScreen *scr = wapp->main_window_desc->screen_ptr;
     int i;
-    
+
     if (!scr->icon_menu) {
 	scr->icon_menu = createApplicationMenu(scr);
-	free(scr->icon_menu->entries[1]->text);
+	wfree(scr->icon_menu->entries[1]->text);
     }
 
     menu = scr->icon_menu;
@@ -589,6 +623,12 @@ openApplicationMenu(WApplication *wapp, int x, int y)
 	menu->entries[1]->text = _("Hide");
     }
 
+    if (WFLAGP(wapp->main_window_desc, collapse_appicons)) {
+	menu->entries[2]->text = _("Uncollapse");
+    } else {
+	menu->entries[2]->text = _("Collapse");
+    }
+    
     menu->flags.realized = 0;
     wMenuRealize(menu);
 
@@ -608,12 +648,13 @@ openApplicationMenu(WApplication *wapp, int x, int y)
 
 /******************************************************************/
 
-static void 
+static void
 iconExpose(WObjDescriptor *desc, XEvent *event)
 {
     wAppIconPaint(desc->parent);
 }
 
+
 static void
 iconDblClick(WObjDescriptor *desc, XEvent *event)
 {
@@ -622,14 +663,7 @@ iconDblClick(WObjDescriptor *desc, XEvent *event)
     WScreen *scr = aicon->icon->core->screen_ptr;
     int unhideHere;
 
-#ifndef REDUCE_APPICONS
     assert(aicon->icon->owner!=NULL);
-#else
-    if (aicon->icon->owner == NULL) {
-       fprintf(stderr, "Double-click disabled: missing main window.\n");
-       return;
-    }
-#endif 
 
     wapp = wApplicationOf(aicon->icon->owner->main_window);
 #ifdef DEBUG0
@@ -638,32 +672,23 @@ iconDblClick(WObjDescriptor *desc, XEvent *event)
 	return;
     }
 #endif
-#ifdef REDUCE_APPICONS
-    if (!wapp) {
-       fprintf(stderr, "Double-click disabled: missing wapp.\n");
-       return;
-    }
-#endif /* REDUCE_APPICONS */
-#if 0
-    /* aproveita que ta aqui pra atualizar */
-    wapp->main_window = aicon->icon->owner->main_window;
-#endif
-    
+
     unhideHere = (event->xbutton.state & ShiftMask);
-    
+
     /* go to the last workspace that the user worked on the app */
-    if (!unhideHere)
+    if (!unhideHere && wapp->last_workspace != scr->current_workspace)
 	wWorkspaceChange(scr, wapp->last_workspace);
 
     wUnhideApplication(wapp, event->xbutton.button==Button2, unhideHere);
-    
+
     if (event->xbutton.state & MOD_MASK) {
 	wHideOtherApplications(aicon->icon->owner);
     }
 }
 
 
-void 
+
+void
 appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 {
     WAppIcon *aicon = desc->parent;
@@ -680,11 +705,15 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     int ix, iy;
     int clickButton = event->xbutton.button;
     Pixmap ghost = None;
+    Window wins[2];
+    Bool movingSingle = False;
+    int oldX = x;
+    int oldY = y;
 
     if (aicon->editing || WCHECK_STATE(WSTATE_MODAL))
 	return;
-    
-    if (IsDoubleClick(scr, event)) {
+
+    if (event->xbutton.button == Button1 && IsDoubleClick(scr, event)) {
 	iconDblClick(desc, event);
 	return;
     }
@@ -696,7 +725,15 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	if (!wapp)
 	    return;
 
-	openApplicationMenu(wapp, event->xbutton.x_root, 
+	if (event->xbutton.send_event &&
+	    XGrabPointer(dpy, aicon->icon->core->window, True, ButtonMotionMask
+			 |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
+			 GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
+	    wwarning("pointer grab failed for appicon menu");
+	    return;
+	}
+
+	openApplicationMenu(wapp, event->xbutton.x_root,
 			    event->xbutton.y_root);
 
 	/* allow drag select of menu */
@@ -714,6 +751,15 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     else
 	wRaiseFrame(icon->core);
 
+
+    if (clickButton == Button2) {
+	WAppIcon *next = wAppIconNextSibling(aicon);
+	    
+	if (next) {
+	    wRaiseFrame(next->icon->core);
+	}
+	return;
+    }
     
     if (XGrabPointer(dpy, icon->core->window, True, ButtonMotionMask
 		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
@@ -726,7 +772,19 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     else
       dockable = canBeDocked(icon->owner);
 
-
+    wins[0] = icon->core->window;
+    wins[1] = scr->dock_shadow;
+    XRestackWindows(dpy, wins, 2);
+    if (superfluous) {
+        if (icon->pixmap!=None)
+            ghost = MakeGhostIcon(scr, icon->pixmap);
+        else
+            ghost = MakeGhostIcon(scr, icon->core->window);
+        XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
+                                   ghost);
+        XClearWindow(dpy, scr->dock_shadow);
+    }
+    
     while (!done) {
 	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
 		    |ButtonMotionMask|ExposureMask, &ev);
@@ -740,7 +798,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					    |ButtonReleaseMask|ButtonPressMask, 
+					    |ButtonReleaseMask|ButtonPressMask,
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -749,7 +807,12 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	    }
 	    x = ev.xmotion.x_root - dx;
 	    y = ev.xmotion.y_root - dy;
-	    XMoveWindow(dpy, icon->core->window, x, y);
+	    
+	    if (movingSingle) {
+		XMoveWindow(dpy, icon->core->window, x, y);
+	    } else {
+		wAppIconMove(aicon, x, y);
+	    }
 
 	    if (dockable) {
                 if (scr->dock && wDockSnapIcon(scr->dock, aicon, x, y,
@@ -774,20 +837,6 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
 		    XMoveWindow(dpy, scr->dock_shadow, shad_x, shad_y);
 		    if (!docking) {
-			Window wins[2];
-
-			wins[0] = icon->core->window;
-			wins[1] = scr->dock_shadow;
-			XRestackWindows(dpy, wins, 2);
-                        if (superfluous) {
-			if (icon->pixmap!=None)
-			    ghost = MakeGhostIcon(scr, icon->pixmap);
-			else
-			    ghost = MakeGhostIcon(scr, icon->core->window);
-			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
-						   ghost);
-			XClearWindow(dpy, scr->dock_shadow);
-                        }
                         XMapWindow(dpy, scr->dock_shadow);
 		    }
 		    docking = 1;
@@ -796,7 +845,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                                          &ix, &iy, False)) {
                     shad_x = workspace->clip->x_pos + ix*wPreferences.icon_size;
 		    shad_y = workspace->clip->y_pos + iy*wPreferences.icon_size;
-		    
+
                     if (scr->last_dock != workspace->clip && collapsed) {
                         scr->last_dock->collapsed = 1;
                         wDockHideIcons(scr->last_dock);
@@ -814,20 +863,6 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 
 		    XMoveWindow(dpy, scr->dock_shadow, shad_x, shad_y);
 		    if (!docking) {
-			Window wins[2];
-
-			wins[0] = icon->core->window;
-			wins[1] = scr->dock_shadow;
-			XRestackWindows(dpy, wins, 2);
-                        if (superfluous) {
-			if (icon->pixmap!=None)
-			    ghost = MakeGhostIcon(scr, icon->pixmap);
-			else
-			    ghost = MakeGhostIcon(scr, icon->core->window);
-			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
-						   ghost);
-			XClearWindow(dpy, scr->dock_shadow);
-                        }
 			XMapWindow(dpy, scr->dock_shadow);
 		    }
                     docking = 1;
@@ -865,13 +900,31 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (!docked) {
 		    /* If icon could not be docked, slide it back to the old
 		     * position */
-		    SlideWindow(icon->core->window, x, y, aicon->x_pos, 
-				aicon->y_pos);
+		    SlideWindow(icon->core->window, x, y, oldX, oldY);
+		} else {
+		    WAppIcon *nextIcon = wAppIconNextSibling(aicon);
+		    
+		    if (nextIcon) {
+			/* move the next instance back to the old position */
+			SlideWindow(nextIcon->icon->core->window, x, y,
+				    oldX, oldY);
+			wAppIconMove(nextIcon, oldX, oldY);
+		    }
 		}
+
+#ifdef WSOUND
+		wSoundPlay(WSOUND_DOCK);
+#endif
             } else {
-		XMoveWindow(dpy, icon->core->window, x, y);
-		aicon->x_pos = x;
-		aicon->y_pos = y;
+		if (movingSingle) {
+		    /* move back to its place */
+		    SlideWindow(icon->core->window, x, y, oldX, oldY);
+		    wAppIconMove(aicon, oldX, oldY);		    
+		} else {
+		    XMoveWindow(dpy, icon->core->window, x, y);
+		    aicon->x_pos = x;
+		    aicon->y_pos = y;
+		}
                 if (workspace->clip && workspace->clip->auto_raise_lower)
                     wDockLower(workspace->clip);
             }
@@ -885,6 +938,10 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		    XFreePixmap(dpy, ghost);
 		XSetWindowBackground(dpy, scr->dock_shadow, scr->white_pixel);
 	    }
+
+	    if (wPreferences.auto_arrange_icons)
+		wArrangeIcons(scr, True);
+
 	    done = 1;
 	    break;
 	}
@@ -892,5 +949,5 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 #ifdef DEBUG
     puts("End icon move");
 #endif
-    
+
 }
