@@ -25,7 +25,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "WindowMaker.h"
 #include "menu.h"
@@ -46,8 +50,8 @@
 
 #include "dock.h"
 
-#ifdef WMSOUND
-#include "wmsound.h"
+#ifdef WSOUND
+#include "wsound.h"
 #endif
 
 
@@ -163,7 +167,7 @@ saveIconNameFor(char *iconPath, char *wm_instance, char *wm_class)
     }
     
     key = PLMakeString(tmp); 
-    free(tmp);
+    wfree(tmp);
     adict = PLGetDictionaryEntry(dict, key);
     
     iconk = PLMakeString("Icon");
@@ -183,6 +187,7 @@ saveIconNameFor(char *iconPath, char *wm_instance, char *wm_class)
 	PLRelease(val);
     }
     PLRelease(key);
+    PLRelease(iconk);
 
     if (val && !wPreferences.flags.noupdates)
 	PLSave(dict, YES);
@@ -215,13 +220,13 @@ wApplicationExtractDirPackIcon(WScreen *scr, char *path,
 		iconPath = tmp;
 	}
 	if (!iconPath)
-	    free(tmp);
+	    wfree(tmp);
     }
     
     if (iconPath) {
 	saveIconNameFor(iconPath, wm_instance, wm_class);
 
-	free(iconPath);
+	wfree(iconPath);
     }
 }
 
@@ -237,7 +242,7 @@ extractClientIcon(WAppIcon *icon)
 
     saveIconNameFor(path, icon->wm_instance, icon->wm_class);
 
-    free(path);
+    wfree(path);
 
     return True;
 }
@@ -255,7 +260,7 @@ wApplicationCreate(WScreen *scr, Window main_window)
 #endif
 	return NULL;
     }
-
+    
     {
 	Window root;
 	int foo;
@@ -284,7 +289,7 @@ wApplicationCreate(WScreen *scr, Window main_window)
     wapp->main_window = main_window;
     wapp->main_window_desc = makeMainWindow(scr, main_window);
     if (!wapp->main_window_desc) {
-	free(wapp);
+	wfree(wapp);
 	return NULL;
     }
     
@@ -300,6 +305,25 @@ wApplicationCreate(WScreen *scr, Window main_window)
 #endif /* USER_MENU */
 
 
+    {
+	WApplication *tmp = scr->wapp_list;
+	
+	wapp->next = NULL;
+	wapp->prev = NULL;
+
+	/* append to app list */
+	if (!tmp) {
+	    scr->wapp_list = wapp;
+	} else {
+	    while (tmp->next) {
+		tmp = tmp->next;
+	    }
+	    tmp->next = wapp;
+	    wapp->prev = tmp;
+	}
+    }
+    
+    
     /*
      * Set application wide attributes from the leader.
      */
@@ -341,18 +365,6 @@ wApplicationCreate(WScreen *scr, Window main_window)
             wAppIconPaint(wapp->app_icon);
         } else {
 	    wapp->app_icon = wAppIconCreate(wapp->main_window_desc);
-#ifdef REDUCE_APPICONS
-	/* This is so we get the appearance of invoking the app sitting
-	 * on the dock. -cls */
-	    if (wapp->app_icon) {
-		if (wapp->app_icon->docked && wapp->app_icon->num_apps == 1) {
-		    wapp->app_icon->launching = 0;
-		    wapp->app_icon->running = 1;
-		    wapp->app_icon->icon->force_paint = 1;
-		    wAppIconPaint(wapp->app_icon);
-		}
-	    }
-#endif
 	}
     } else {
 	wapp->app_icon = NULL;
@@ -362,14 +374,7 @@ wApplicationCreate(WScreen *scr, Window main_window)
         wapp->app_icon->main_window = main_window;
     }
 
-#ifndef REDUCE_APPICONS
     if (wapp->app_icon && !wapp->app_icon->docked) {
-#else
-    if (wapp->app_icon && !wapp->app_icon->docked && wapp->app_icon->num_apps == 1) {
-#ifdef THIS_SUCKS
-    }
-#endif
-#endif
         WIcon *icon = wapp->app_icon->icon;
         WDock *clip = scr->workspaces[scr->current_workspace]->clip;
         int x=0, y=0;
@@ -398,24 +403,43 @@ wApplicationCreate(WScreen *scr, Window main_window)
     }
 
     if (wapp->app_icon) {
-	char *tmp;
+	char *tmp, *path;
+        struct stat dummy;
+        RImage *image;
 
-	/* if the displayed icon was supplied by the client, save the icon */
 	tmp = wDefaultGetIconFile(scr, wapp->app_icon->wm_instance,
 				  wapp->app_icon->wm_class, True);
-	if (!tmp)
+
+        /* If the icon was saved by us from the client supplied icon, but is
+         * missing, recreate it. */
+        if (tmp && strstr(tmp, ".AppInfo/WindowMaker")!=NULL &&
+            stat(tmp, &dummy)!=0 && errno==ENOENT) {
+            wmessage(_("recreating missing icon '%s'"), tmp);
+            path = wIconStore(wapp->app_icon->icon);
+            if (path) {
+                wfree(path);
+            }
+            image = wDefaultGetImage(scr, wapp->app_icon->wm_instance,
+                                     wapp->app_icon->wm_class);
+            if (image) {
+                wIconChangeImage(wapp->app_icon->icon, image);
+                wAppIconPaint(wapp->app_icon);
+                /* TODO:
+                 * wIconChangeImage() should be rewriten to use retain/release
+                 * The way it is now is too confusing about where the icon is
+                 * finally released.  -Dan */
+                /* --this is wrong at the moment-- RReleaseImage(image);*/
+            }
+        }
+
+	/* if the displayed icon was supplied by the client, save the icon */
+        if (!tmp)
 	    extractClientIcon(wapp->app_icon);
     }
     
-    wapp->prev = NULL; 
-    wapp->next = scr->wapp_list;
-    if (scr->wapp_list)
-	scr->wapp_list->prev = wapp;
-    scr->wapp_list = wapp;
-
     
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPSTART);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_APPSTART);
 #endif
 
 #ifdef DEBUG
@@ -431,9 +455,6 @@ wApplicationDestroy(WApplication *wapp)
     Window main_window;
     WWindow *wwin;
     WScreen *scr;
-#ifdef REDUCE_APPICONS
-    unsigned int napps;
-#endif
 
     if (!wapp)
       return;
@@ -445,9 +466,6 @@ wApplicationDestroy(WApplication *wapp)
 
     scr = wapp->main_window_desc->screen_ptr;
     main_window = wapp->main_window;
-#ifdef REDUCE_APPICONS
-    napps = wAppIconReduceAppCount(wapp);
-#endif
 
     if (wapp == scr->wapp_list) {
         if (wapp->next)
@@ -464,39 +482,21 @@ wApplicationDestroy(WApplication *wapp)
     wAppMenuDestroy(wapp->menu);
     if (wapp->app_icon) {
         if (wapp->app_icon->docked && !wapp->app_icon->attracted) {
-#ifdef REDUCE_APPICONS
-	    if (napps == 0) {
-#endif
-		wapp->app_icon->running = 0;
-		/* since we keep it, we don't care if it was attracted or not */
-		wapp->app_icon->attracted = 0;
-		wapp->app_icon->icon->shadowed = 0;
-		wapp->app_icon->main_window = None;
-		wapp->app_icon->pid = 0;
-		wapp->app_icon->icon->owner = NULL;
-		wapp->app_icon->icon->icon_win = None;
-		wapp->app_icon->icon->force_paint = 1;
-		wAppIconPaint(wapp->app_icon);
-#ifdef REDUCE_APPICONS
-	    }
-#endif
+	    wapp->app_icon->running = 0;
+	    /* since we keep it, we don't care if it was attracted or not */
+	    wapp->app_icon->attracted = 0;
+	    wapp->app_icon->icon->shadowed = 0;
+	    wapp->app_icon->main_window = None;
+	    wapp->app_icon->pid = 0;
+	    wapp->app_icon->icon->owner = NULL;
+	    wapp->app_icon->icon->icon_win = None;
+	    wapp->app_icon->icon->force_paint = 1;
+	    wAppIconPaint(wapp->app_icon);
         } else if (wapp->app_icon->docked) {
-#ifdef REDUCE_APPICONS
-	    if (napps == 0)  {
-#endif
-		wapp->app_icon->running = 0;
-		wDockDetach(wapp->app_icon->dock, wapp->app_icon);
-#ifdef REDUCE_APPICONS
-	    }
-#endif 
+	    wapp->app_icon->running = 0;
+	    wDockDetach(wapp->app_icon->dock, wapp->app_icon);
         } else {
-#ifdef REDUCE_APPICONS
-    	    if (napps == 0) {
-#endif 
-		wAppIconDestroy(wapp->app_icon);
-#ifdef REDUCE_APPICONS
-    	    }
-#endif
+	    wAppIconDestroy(wapp->app_icon);
 	}
     }
     wwin = wWindowFor(wapp->main_window_desc->client_win);
@@ -508,7 +508,7 @@ wApplicationDestroy(WApplication *wapp)
     	XSaveContext(dpy, wwin->client_win, wWinContext, 
 		     (XPointer)&wwin->client_descriptor);
     }
-    free(wapp);
+    wfree(wapp);
     
 #ifdef DEBUG
     printf("Destroyed application for %x\n", (unsigned)main_window);
@@ -517,9 +517,73 @@ wApplicationDestroy(WApplication *wapp)
         wArrangeIcons(scr, True);
     }
 
-#ifdef WMSOUND
-    wSoundPlay(WMSOUND_APPEXIT);
+#ifdef WSOUND
+    wSoundPlay(WSOUND_APPEXIT);
 #endif
 }
 
 
+
+void 
+wApplicationSetCollapse(WApplication *app, Bool flag)
+{
+    WApplication *list = app->main_window_desc->screen_ptr->wapp_list;
+    int index = 0;
+    WWindow *wwin = app->main_window_desc;
+
+    if (WFLAGP(app->main_window_desc, collapse_appicons) == flag)
+	return;
+
+
+    while (list) {
+	if (strcmp(wwin->wm_instance,
+		   list->main_window_desc->wm_instance) == 0
+	    &&
+	    strcmp(wwin->wm_class,
+		   list->main_window_desc->wm_class) == 0)
+	    WSETUFLAG(list->main_window_desc, collapse_appicons, flag);
+
+	list = list->next;
+    }
+
+    if (app->app_icon && flag)
+	wAppIconMove(app->app_icon, app->app_icon->x_pos, app->app_icon->y_pos);
+}
+
+
+
+
+/*
+ * Returns index number of the app in case there are more than
+ * one instance of the same class/name.
+ */
+int
+wApplicationIndexOfGroup(WApplication *app)
+{
+    WApplication *list = app->main_window_desc->screen_ptr->wapp_list;
+    int index = 0;
+    WWindow *wwin = app->main_window_desc;
+/*
+    if (!WFLAGP(wwin, collapse_appicons))
+	return 0;
+ */
+    
+    while (list) {
+	if (app == list)
+	    return index;
+
+	if (strcmp(wwin->wm_instance,
+		   list->main_window_desc->wm_instance) == 0
+	    &&
+	    strcmp(wwin->wm_class,
+		   list->main_window_desc->wm_class) == 0)
+	    index++;
+
+	list = list->next;
+    }
+    
+    puts("OH SHIT!?!?!? HOW THE FUCK DID WE GET HERE!?!?!?!?!");
+    
+    return 0;
+}
+
