@@ -22,7 +22,6 @@
  */
 
 #include "wconfig.h"
-#include "plugin.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -129,9 +128,6 @@ static int getString();
 static int getPathList();
 static int getEnum();
 static int getTexture();
-#ifdef DRAWSTRING_PLUGIN
-static int getTextRenderer();
-#endif
 static int getWSBackground();
 static int getWSSpecificBackground();
 static int getFont();
@@ -182,8 +178,9 @@ static int setClipTitleFont();
 static int setClipTitleColor();
 
 static int setMenuStyle();
+#if 0
 static int setMultiByte();
-
+#endif
 static int updateUsableArea();
 
 #ifdef DEFINABLE_CURSOR
@@ -265,13 +262,17 @@ static WOptionEnumeration seSpeeds[] = {
     {NULL, 0, 0}
 };
 
-static WOptionEnumeration seMouseButtons[] = {
-    {"None", -1, 0},
-    {"Left", Button1, 0}, {"Button1", Button1, 1},
-    {"Middle", Button2, 0}, {"Button2", Button2, 1},
-    {"Right", Button3, 0}, {"Button3", Button3, 1},
-    {"Button4", Button4, 0},
-    {"Button5", Button5, 0},
+static WOptionEnumeration seMouseButtonActions[] = {
+    {"None", WA_NONE, 0},
+    {"SelectWindows", WA_SELECT_WINDOWS, 0},
+    {"OpenApplicationsMenu", WA_OPEN_APPMENU, 0},
+    {"OpenWindowListMenu", WA_OPEN_WINLISTMENU, 0},
+    {NULL, 0, 0}
+};
+
+static WOptionEnumeration seMouseWheelActions[] = {
+    {"None", WA_NONE, 0},
+    {"SwitchWorkspaces", WA_SWITCH_WORKSPACES, 0},
     {NULL, 0, 0}
 };
 
@@ -319,7 +320,16 @@ static WOptionEnumeration seDisplayPositions[] = {
     {"topleft",		WD_TOPLEFT,	0},
     {"topright",	WD_TOPRIGHT, 	0},
     {"bottomleft",	WD_BOTTOMLEFT,	0},
-    {"bottomright",	WD_BOTTOMRIGHT,	0}
+    {"bottomright",	WD_BOTTOMRIGHT,	0},
+    {NULL,		0,		0}
+};
+
+static WOptionEnumeration seWorkspaceBorder[] = {
+    {"None",  		WB_NONE,	0},
+    {"LeftRight",      	WB_LEFTRIGHT,	0},
+    {"TopBottom",      	WB_TOPBOTTOM,  	0},
+    {"AllDirections",  	WB_ALLDIRS,	0},
+    {NULL,		0,		0}
 };
 
 
@@ -365,10 +375,12 @@ WDefaultEntry staticOptionList[] = {
     },
     {"DisableMiniwindows", "NO",		NULL,
 	&wPreferences.disable_miniwindows, getBool,	NULL
-    },
-    {"MultiByteText", "NO", 			NULL,
-	&wPreferences.multi_byte_text, getBool,		setMultiByte
     }
+#if 0
+    ,{"MultiByteText", "NO", 			NULL,
+	&wPreferences.multi_byte_text, getBool,	setMultiByte
+    }
+#endif
 };
 
 
@@ -381,14 +393,17 @@ WDefaultEntry optionList[] = {
     {"IconificationStyle", "Zoom",             seIconificationStyles,
          &wPreferences.iconification_style,    getEnum, NULL
     },
-    {"SelectWindowsMouseButton", "Left",	seMouseButtons,
-	  &wPreferences.select_button, 	getEnum,	NULL
+    {"MouseLeftButtonAction", "SelectWindows",	seMouseButtonActions,
+	  &wPreferences.mouse_button1, 	getEnum,	NULL
     },
-    {"WindowListMouseButton", "Middle",		seMouseButtons,
-	  &wPreferences.windowl_button, getEnum,	NULL
+    {"MouseMiddleButtonAction", "OpenWindowListMenu",	seMouseButtonActions,
+	  &wPreferences.mouse_button2, getEnum,	NULL
     },
-    {"ApplicationMenuMouseButton", "Right",	seMouseButtons,
-	  &wPreferences.menu_button,	getEnum,	NULL
+    {"MouseRightButtonAction", "OpenApplicationsMenu",	seMouseButtonActions,
+	  &wPreferences.mouse_button3,	getEnum,	NULL
+    },
+    {"MouseWheelAction", "None",	seMouseWheelActions,
+	  &wPreferences.mouse_wheel,	getEnum,	NULL
     },
     {"PixmapPath",	DEF_PIXMAP_PATHS,	NULL,
 	  &wPreferences.pixmap_path,	getPathList,	NULL
@@ -405,6 +420,9 @@ WDefaultEntry optionList[] = {
     {"RaiseDelay",	"0",			NULL,
 	  &wPreferences.raise_delay,	getInt,		NULL
     },
+    {"WindozeCycling",  "YES",			NULL,
+	  &wPreferences.windows_cycling,getBool,	NULL
+    },
     {"CirculateRaise",	"NO",			NULL,
 	  &wPreferences.circ_raise, 	getBool, 	NULL
     },
@@ -420,21 +438,30 @@ WDefaultEntry optionList[] = {
     {"WorkspaceNameDisplayPosition", "center",	seDisplayPositions,
 	  &wPreferences.workspace_name_display_position, getEnum, NULL
     },
+    {"WorkspaceBorder", "None",			seWorkspaceBorder,
+	  &wPreferences.workspace_border_position, getEnum, updateUsableArea
+    },
+    {"WorkspaceBorderSize", "0",       		NULL,
+	  &wPreferences.workspace_border_size, getInt, updateUsableArea
+    },
 #ifdef VIRTUAL_DESKTOP
     {"VirtualEdgeThickness", "1",   NULL,
         &wPreferences.vedge_thickness, getInt,        NULL
     },
+    {"VirtualEdgeExtendSpace", "0",   NULL,
+        &wPreferences.vedge_bordersize, getInt,        NULL
+    },
     {"VirtualEdgeHorizonScrollSpeed", "1",  NULL,
         &wPreferences.vedge_hscrollspeed, getInt,     NULL
     },
     {"VirtualEdgeVerticalScrollSpeed", "1", NULL,
         &wPreferences.vedge_vscrollspeed, getInt,     NULL
     },
-    {"VirtualEdgeWidth", "2000",    NULL,
-        &wPreferences.vedge_width, getInt,        NULL
+    {"VirtualEdgeMaximumWidth", "3000",    NULL,
+        &wPreferences.vedge_maxwidth, getInt,        NULL
     },
-    {"VirtualEdgeHeight", "2000",   NULL,
-        &wPreferences.vedge_height, getInt,       NULL
+    {"VirtualEdgeMaximumHeight", "3000",   NULL,
+        &wPreferences.vedge_maxheight, getInt,       NULL
     },
 #endif
     {"StickyIcons", "NO",			NULL,
@@ -594,17 +621,6 @@ WDefaultEntry optionList[] = {
     {"CClipTitleColor", "\"#454045\"",		(void*)CLIP_COLLAPSED,
 	  NULL,				getColor,	setClipTitleColor
     },
-#ifdef DRAWSTRING_PLUGIN
-    {"FTitleColor",    "white",             (void*)WS_FOCUSED,
-         NULL,          getTextRenderer,    setWTitleColor
-    },
-    {"PTitleColor",    "white",             (void*)WS_PFOCUSED,
-         NULL,          getTextRenderer,    setWTitleColor
-    },
-    {"UTitleColor",    "black",             (void*)WS_UNFOCUSED,
-         NULL,          getTextRenderer,    setWTitleColor
-    },
-#else
     {"FTitleColor",	"white",		(void*)WS_FOCUSED,
 	  NULL,				getColor,	setWTitleColor
     },
@@ -614,7 +630,6 @@ WDefaultEntry optionList[] = {
     {"UTitleColor",	"black",		(void*)WS_UNFOCUSED,
 	  NULL,				getColor,	setWTitleColor
     },
-#endif
     {"FTitleBack",	"(solid, black)",      	NULL,
 	  NULL,				getTexture,	setFTitleBack
     },
@@ -627,15 +642,9 @@ WDefaultEntry optionList[] = {
     {"ResizebarBack",	"(solid, gray)",	NULL,
 	  NULL,				getTexture,	setResizebarBack
     },
-#ifdef DRAWSTRING_PLUGIN
-    {"MenuTitleColor", "white",             NULL,
-        NULL,           getTextRenderer,    setMenuTitleColor
-    },
-#else
     {"MenuTitleColor",	"white",       		NULL,
 	  NULL,				getColor,	setMenuTitleColor
     },
-#endif
     {"MenuTextColor",	"black",       		NULL,
 	  NULL,				getColor,	setMenuTextColor
     },
@@ -768,7 +777,6 @@ WDefaultEntry optionList[] = {
     {"WindowShortcut4Key","None",		(void*)WKBD_WINDOW4,
 	    NULL, 			getKeybind,	setKeyGrab
     }
-#ifdef EXTEND_WINDOWSHORTCUT
     ,{"WindowShortcut5Key","None",		(void*)WKBD_WINDOW5,
 	    NULL, 			getKeybind,	setKeyGrab
     },
@@ -786,8 +794,10 @@ WDefaultEntry optionList[] = {
     },
     {"WindowShortcut10Key","None",		(void*)WKBD_WINDOW10,
 	    NULL, 			getKeybind,	setKeyGrab
+    },
+    {"ScreenSwitchKey",  "None",		(void*)WKBD_SWITCH_SCREEN,
+	    NULL, 			getKeybind,	setKeyGrab	
     }
-#endif /* EXTEND_WINDOWSHORTCUT */
 
 #ifdef KEEP_XKB_LOCK_STATUS
     ,{"ToggleKbdModeKey", "None",                      (void*)WKBD_TOGGLE,
@@ -800,8 +810,8 @@ WDefaultEntry optionList[] = {
 #ifdef DEFINABLE_CURSOR
     ,{"NormalCursor", "(builtin, left_ptr)",	(void*)WCUR_ROOT,
           NULL,				getCursor,	setCursor
-    }
-    ,{"MoveCursor", "(builtin, fleur)",		(void*)WCUR_MOVE,
+    },
+    {"MoveCursor", "(builtin, fleur)",		(void*)WCUR_MOVE,
           NULL,				getCursor,	setCursor
     },
 #if 0
@@ -825,14 +835,8 @@ WDefaultEntry optionList[] = {
     },
     {"WaitCursor", "(builtin, watch)",		(void*)WCUR_WAIT,
           NULL,				getCursor,	setCursor
-    }
-    ,{"ArrowCursor", "(builtin, top_left_arrow)",	(void*)WCUR_ARROW,
-          NULL,				getCursor,	setCursor
-    }
-    ,{"QuestionCursor", "(builtin, question_arrow)",	(void*)WCUR_QUESTION,
-          NULL,				getCursor,	setCursor
-    }
-    ,{"TextCursor", "(builtin, xterm)",		(void*)WCUR_TEXT,
+    },
+    {"ArrowCursor", "(builtin, top_left_arrow)",	(void*)WCUR_ARROW,
           NULL,				getCursor,	setCursor
     }
 #endif
@@ -921,8 +925,8 @@ wDefaultsDestroyDomain(WDDomain *domain)
 {
     if (domain->dictionary)
 	PLRelease(domain->dictionary);
-    free(domain->path);
-    free(domain);
+    wfree(domain->path);
+    wfree(domain);
 }
 
 
@@ -1125,8 +1129,24 @@ wDefaultsCheckDomains(void *foo)
 		WDWindowAttributes->dictionary = dict;
 		for (i=0; i<wScreenCount; i++) {
 		    scr = wScreenWithNumber(i);
-		    if (scr)
-			wDefaultUpdateIcons(scr);
+                    if (scr) {
+                        RImage *image;
+
+                        wDefaultUpdateIcons(scr);
+
+                        /* Update the panel image if changed */
+                        /* Don't worry. If the image is the same these
+                         * functions will have no performance impact. */
+                        image = wDefaultGetImage(scr, "Logo", "WMPanel");
+
+                        if (!image) {
+                            wwarning(_("could not load logo image for panels: %s"),
+                                     RMessageForError(RErrorCode));
+                        } else {
+                            WMSetApplicationIconImage(scr->wmscreen, image);
+                            RReleaseImage(image);
+                        }
+                    }
 		}
 	    }
 	} else {
@@ -1138,28 +1158,28 @@ wDefaultsCheckDomains(void *foo)
 
 #ifndef LITE
     if (stat(WDRootMenu->path, &stbuf)>=0
-	&& WDRootMenu->timestamp < stbuf.st_mtime) {
-	dict = ReadProplistFromFile(WDRootMenu->path);
+        && WDRootMenu->timestamp < stbuf.st_mtime) {
+        dict = ReadProplistFromFile(WDRootMenu->path);
 #ifdef HEARTBEAT
-	puts("Checking WMRootMenu domain");
+        puts("Checking WMRootMenu domain");
 #endif
-	if (dict) {
-	    if (!PLIsArray(dict) && !PLIsString(dict)) {
-		PLRelease(dict);
-		dict = NULL;
-		wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
-			 "WMRootMenu", WDRootMenu->path);
-	    } else {
-		if (WDRootMenu->dictionary) {
-		    PLRelease(WDRootMenu->dictionary);
-		}
-		WDRootMenu->dictionary = dict;
-	    }
-	} else {
-	    wwarning(_("could not load domain %s from user defaults database"),
-		     "WMRootMenu");
-	}
-	WDRootMenu->timestamp = stbuf.st_mtime;
+        if (dict) {
+            if (!PLIsArray(dict) && !PLIsString(dict)) {
+                PLRelease(dict);
+                dict = NULL;
+                wwarning(_("Domain %s (%s) of defaults database is corrupted!"),
+                         "WMRootMenu", WDRootMenu->path);
+            } else {
+                if (WDRootMenu->dictionary) {
+                    PLRelease(WDRootMenu->dictionary);
+                }
+                WDRootMenu->dictionary = dict;
+            }
+        } else {
+            wwarning(_("could not load domain %s from user defaults database"),
+                     "WMRootMenu");
+        }
+        WDRootMenu->timestamp = stbuf.st_mtime;
     }
 #endif /* !LITE */
 
@@ -1323,7 +1343,7 @@ wDefaultUpdateIcons(WScreen *scr)
             RImage *new_image;
 
             if (aicon->icon->file)
-                free(aicon->icon->file);
+                wfree(aicon->icon->file);
             aicon->icon->file = wstrdup(file);
 
             new_image = wDefaultGetImage(scr, aicon->wm_instance,
@@ -1348,7 +1368,7 @@ wDefaultUpdateIcons(WScreen *scr)
                 RImage *new_image;
 
                 if (wwin->icon->file)
-                    free(wwin->icon->file);
+                    wfree(wwin->icon->file);
                 wwin->icon->file = wstrdup(file);
 
                 new_image = wDefaultGetImage(scr, wwin->wm_instance,
@@ -1667,7 +1687,7 @@ again:
     ptr--; *(ptr--) = 0;
 
     if (*(char**)addr!=NULL) {
-	free(*(char**)addr);
+	wfree(*(char**)addr);
     }
     *(char**)addr = data;
 
@@ -1704,6 +1724,8 @@ getEnum(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
  * (dgradient <color> <color>)
  * (mhgradient <color> <color> ...)
  * (mvgradient <color> <color> ...)
+ * (mdgradient <color> <color> ...)
+ * (igradient <color1> <color1> <thickness1> <color2> <color2> <thickness2>)
  * (tpixmap <file> <color>)
  * (spixmap <file> <color>)
  * (cpixmap <file> <color>)
@@ -1803,7 +1825,66 @@ parse_texture(WScreen *scr, proplist_t pl)
 	color2.blue = xcolor.blue >> 8;
 
 	texture = (WTexture*)wTextureMakeGradient(scr, type, &color1, &color2);
-	
+
+    } else if (strcasecmp(val, "igradient")==0) {
+	RColor colors1[2], colors2[2];
+	int th1, th2;
+	XColor xcolor;
+        int i;
+
+	if (nelem != 7) {
+	    wwarning(_("bad number of arguments in gradient specification"));
+	    return NULL;
+	}
+
+	/* get from color */
+	for (i = 0; i < 2; i++) {
+	    elem = PLGetArrayElement(pl, 1+i);
+	    if (!elem || !PLIsString(elem))
+		return NULL;
+	    val = PLGetString(elem);
+
+	    if (!XParseColor(dpy, scr->w_colormap, val, &xcolor)) {
+		wwarning(_("\"%s\" is not a valid color name"), val);
+		return NULL;
+	    }
+	    colors1[i].alpha = 255;
+	    colors1[i].red = xcolor.red >> 8;
+	    colors1[i].green = xcolor.green >> 8;
+	    colors1[i].blue = xcolor.blue >> 8;
+	}
+	elem = PLGetArrayElement(pl, 3);
+	if (!elem || !PLIsString(elem))
+	    return NULL;
+	val = PLGetString(elem);
+	th1 = atoi(val);
+
+
+	/* get from color */
+	for (i = 0; i < 2; i++) {
+	    elem = PLGetArrayElement(pl, 4+i);
+	    if (!elem || !PLIsString(elem))
+		return NULL;
+	    val = PLGetString(elem);
+
+	    if (!XParseColor(dpy, scr->w_colormap, val, &xcolor)) {
+		wwarning(_("\"%s\" is not a valid color name"), val);
+		return NULL;
+	    }
+	    colors2[i].alpha = 255;
+	    colors2[i].red = xcolor.red >> 8;
+	    colors2[i].green = xcolor.green >> 8;
+	    colors2[i].blue = xcolor.blue >> 8;
+	}
+	elem = PLGetArrayElement(pl, 6);
+	if (!elem || !PLIsString(elem))
+	    return NULL;
+	val = PLGetString(elem);
+	th2 = atoi(val);
+
+	texture = (WTexture*)wTextureMakeIGradient(scr, th1, colors1,
+						   th2, colors2);
+
     } else if (strcasecmp(val, "mhgradient")==0
 	       || strcasecmp(val, "mvgradient")==0
 	       || strcasecmp(val, "mdgradient")==0) {
@@ -1832,9 +1913,9 @@ parse_texture(WScreen *scr, proplist_t pl)
 	    elem = PLGetArrayElement(pl, i+1);
 	    if (!elem || !PLIsString(elem)) {
 		for (--i; i>=0; --i) {
-		    free(colors[i]);
+		    wfree(colors[i]);
 		}
-		free(colors);
+		wfree(colors);
 		return NULL;
 	    }
 	    val = PLGetString(elem);
@@ -1842,9 +1923,9 @@ parse_texture(WScreen *scr, proplist_t pl)
 	    if (!XParseColor(dpy, scr->w_colormap, val, &color)) {
 		wwarning(_("\"%s\" is not a valid color name"), val);
 		for (--i; i>=0; --i) {
-		    free(colors[i]);
+		    wfree(colors[i]);
 		}
-		free(colors);
+		wfree(colors);
 		return NULL;
 	    } else {
 		colors[i] = wmalloc(sizeof(RColor));
@@ -1995,7 +2076,7 @@ parse_texture(WScreen *scr, proplist_t pl)
 	for (i = 0; i < argc - 1; i++) {
 	    elem = PLGetArrayElement(pl, 3 + i);
 	    if (!elem || !PLIsString(elem)) {
-		free(argv);
+		wfree(argv);
 
 		return NULL;
 	    }
@@ -2088,64 +2169,6 @@ again:
 }
 
 
-
-#ifdef DRAWSTRING_PLUGIN
-static int
-getTextRenderer(WScreen *scr, WDefaultEntry *entry, proplist_t value,
-        void *addr, void **ret)
-{
-    proplist_t elem;
-    char *val, *lib, *func, **argv;
-    int argc, changed;
-
-    if (strcmp(entry->key, "FTitleColor")==0) {
-        wPluginDestroyFunction(scr->drawstring_func[changed = W_STRING_FTITLE]);
-        scr->drawstring_func[W_STRING_FTITLE] = NULL;
-    } else if (strcmp(entry->key, "UTitleColor")==0) {
-        wPluginDestroyFunction(scr->drawstring_func[changed = W_STRING_UTITLE]);
-        scr->drawstring_func[W_STRING_UTITLE] = NULL;
-    } else if (strcmp(entry->key, "PTitleColor")==0) {
-        wPluginDestroyFunction(scr->drawstring_func[changed = W_STRING_PTITLE]);
-        scr->drawstring_func[W_STRING_PTITLE] = NULL;
-    } else if (strcmp(entry->key, "MenuTitleColor")==0) {
-        wPluginDestroyFunction(scr->drawstring_func[changed = W_STRING_MTITLE]);
-        scr->drawstring_func[W_STRING_MTITLE] = NULL;
-    } else if (strcmp(entry->key, "MenuPluginColor")==0) {
-        wPluginDestroyFunction(scr->drawstring_func[changed = W_STRING_MTEXT]);
-        scr->drawstring_func[W_STRING_MTEXT] = NULL;
-    }
-
-    if (PLIsArray(value)) {
-        if ((argc = PLGetNumberOfElements(value)) < 4) return False;
-        argc -= 2;
-        argv = (char **)wmalloc(argc * sizeof(char *));
-
-        elem = PLGetArrayElement(value,0);
-        if (!elem || !PLIsString(elem)) return False;
-        val = PLGetString(elem);
-        if (strcasecmp(val, "function")==0) {
-            elem = PLGetArrayElement(value, 1); /* library name */
-            if (!elem || !PLIsString(elem)) return False;
-            lib = PLGetString(elem);
-            elem = PLGetArrayElement(value, 2); /* function name */
-            if (!elem || !PLIsString(elem)) return False;
-            func = PLGetString(elem);
-            scr->drawstring_func[changed] = wPluginCreateFunction (W_FUNCTION_DRAWSTRING,
-                    lib, "initDrawString", func, NULL, value,
-                    wPluginPackInitData(3, dpy, scr->w_colormap,"-DATA-"));
-        }
-
-        return getColor(scr, entry, PLGetArrayElement(value,3), addr, ret);
-    } else if (PLIsString(value)) {
-        return getColor(scr, entry, value, addr, ret);
-    }
-    
-    return False;
-}
-#endif /* DRAWSTRING_PLUGIN */
-
-
-
 static int
 getWSBackground(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 		void *addr, void **ret)
@@ -2431,7 +2454,7 @@ getRImages(WScreen *scr, WDefaultEntry *entry, proplist_t value,
     }
 
     if (*(RImage**)addr) {
-	RDestroyImage(*(RImage**)addr);
+	RReleaseImage(*(RImage**)addr);
     }
     if (addr)
 	*(RImage**)addr = image;
@@ -2539,22 +2562,22 @@ static WCursorLookup cursor_table[] =
     { NULL,			CURSOR_ID_NONE }
 };
 
-static void check_bitmap_status(int status, char *filename, Pixmap bitmap)
+static void
+check_bitmap_status(int status, char *filename, Pixmap bitmap)
 {
-   switch(status)
-    {
+    switch(status) {
      case BitmapOpenFailed:
-       wwarning(_("failed to open bitmap file \"%s\""), filename);
-       break;
+	wwarning(_("failed to open bitmap file \"%s\""), filename);
+	break;
      case BitmapFileInvalid:
-       wwarning(_("\"%s\" is not a valid bitmap file"), filename);
-       break;
+	wwarning(_("\"%s\" is not a valid bitmap file"), filename);
+	break;
      case BitmapNoMemory:
-       wwarning(_("out of memory reading bitmap file \"%s\""), filename);
-       break;
+	wwarning(_("out of memory reading bitmap file \"%s\""), filename);
+	break;
      case BitmapSuccess:
-       XFreePixmap(dpy, bitmap);
-       break;
+	XFreePixmap(dpy, bitmap);
+	break;
     }
 }
 
@@ -2563,174 +2586,150 @@ static void check_bitmap_status(int status, char *filename, Pixmap bitmap)
  * (builtin, <cursor_name>)
  * (bitmap, <cursor_bitmap>, <cursor_mask>)
  */
-static int parse_cursor(WScreen *scr, proplist_t pl, Cursor *cursor)
+static int
+parse_cursor(WScreen *scr, proplist_t pl, Cursor *cursor)
 {
-   proplist_t elem;
-   char *val;
-   int nelem;
-   int status = 0;
-   
-   nelem = PLGetNumberOfElements(pl);
-   if (nelem < 1)
-    {
-       return(status);
+    proplist_t elem;
+    char *val;
+    int nelem;
+    int status = 0;
+    
+    nelem = PLGetNumberOfElements(pl);
+    if (nelem < 1) {
+	return(status);
     }
-   elem = PLGetArrayElement(pl, 0);
-   if (!elem || !PLIsString(elem))
-    {
-       return(status);
+    elem = PLGetArrayElement(pl, 0);
+    if (!elem || !PLIsString(elem)) {
+	return(status);
     }
-   val = PLGetString(elem);
-   
-   if (0 == strcasecmp(val, "none")) 
-    {
-       status = 1;
-       *cursor = None;
-    }
-   else if (0 == strcasecmp(val, "builtin")) 
-    {
-       int i;
-       int cursor_id = CURSOR_ID_NONE;
-       
-       if (2 != nelem)
-	{
-	   wwarning(_("bad number of arguments in cursor specification"));
-	   return(status);
+    val = PLGetString(elem);
+    
+    if (0 == strcasecmp(val, "none")) {
+	status = 1;
+	*cursor = None;
+    } else if (0 == strcasecmp(val, "builtin")) {
+	int i;
+	int cursor_id = CURSOR_ID_NONE;
+	
+	if (2 != nelem) {
+	    wwarning(_("bad number of arguments in cursor specification"));
+	    return(status);
 	}
-       elem = PLGetArrayElement(pl, 1);
-       if (!elem || !PLIsString(elem))
-	{
-	   return(status);
+	elem = PLGetArrayElement(pl, 1);
+	if (!elem || !PLIsString(elem)) {
+	    return(status);
 	}
-       val = PLGetString(elem);
-       
-       for (i = 0; NULL != cursor_table[i].name; i++)
-	{
-	   if (0 == strcasecmp(val, cursor_table[i].name))
-	    {
-	       cursor_id = cursor_table[i].id;
-	       break;
+	val = PLGetString(elem);
+
+	for (i = 0; NULL != cursor_table[i].name; i++) {
+	    if (0 == strcasecmp(val, cursor_table[i].name)) {
+		cursor_id = cursor_table[i].id;
+		break;
 	    }
 	}
-       if (CURSOR_ID_NONE == cursor_id) 
-	{
-	   wwarning(_("unknown builtin cursor name \"%s\""), val);
+	if (CURSOR_ID_NONE == cursor_id) {
+	    wwarning(_("unknown builtin cursor name \"%s\""), val);
+	} else {
+	    *cursor = XCreateFontCursor(dpy, cursor_id);
+	    status = 1;
 	}
-       else
-	{
-	   *cursor = XCreateFontCursor(dpy, cursor_id);
-	   status = 1;
+    } else if (0 == strcasecmp(val, "bitmap")) {
+	char *bitmap_name;
+	char *mask_name;
+	int bitmap_status;
+	int mask_status;
+	Pixmap bitmap;
+	Pixmap mask;
+	unsigned int w, h;
+	int x, y;
+	XColor fg, bg;
+	
+	if (3 != nelem) {
+	    wwarning(_("bad number of arguments in cursor specification"));
+	    return(status);
 	}
+	elem = PLGetArrayElement(pl, 1);
+	if (!elem || !PLIsString(elem)) {
+	    return(status);
+	}
+	val = PLGetString(elem);
+	bitmap_name = FindImage(wPreferences.pixmap_path, val);
+	if (!bitmap_name) {
+	    wwarning(_("could not find cursor bitmap file \"%s\""), val);
+	    return(status);
+	}
+	elem = PLGetArrayElement(pl, 2);
+	if (!elem || !PLIsString(elem)) {
+	    wfree(bitmap_name);
+	    return(status);
+	}
+	val = PLGetString(elem);
+	mask_name = FindImage(wPreferences.pixmap_path, val);
+	if (!mask_name) {
+	    wfree(bitmap_name);
+	    wwarning(_("could not find cursor bitmap file \"%s\""), val);
+	    return(status);
+	}
+	mask_status = XReadBitmapFile(dpy, scr->w_win, mask_name, &w, &h,
+				      &mask, &x, &y);
+	bitmap_status = XReadBitmapFile(dpy, scr->w_win, bitmap_name, &w, &h,
+					&bitmap, &x, &y);
+	if ((BitmapSuccess == bitmap_status) &&
+	    (BitmapSuccess == mask_status)) {
+	    fg.pixel = scr->black_pixel;
+	    bg.pixel = scr->white_pixel;
+	    XQueryColor(dpy, scr->w_colormap, &fg);
+	    XQueryColor(dpy, scr->w_colormap, &bg);
+	    *cursor = XCreatePixmapCursor(dpy, bitmap, mask, &fg, &bg, x, y);
+	    status = 1;
+	}
+	check_bitmap_status(bitmap_status, bitmap_name, bitmap);
+	check_bitmap_status(mask_status, mask_name, mask);
+	wfree(bitmap_name);
+	wfree(mask_name);
     }
-   else if (0 == strcasecmp(val, "bitmap"))
-    {
-       char *bitmap_name;
-       char *mask_name;
-       int bitmap_status;
-       int mask_status;
-       Pixmap bitmap;
-       Pixmap mask;
-       unsigned int w, h;
-       int x, y;
-       XColor fg, bg;
-       
-       if (3 != nelem)
-	{
-	   wwarning(_("bad number of arguments in cursor specification"));
-	   return(status);
-	}
-       elem = PLGetArrayElement(pl, 1);
-       if (!elem || !PLIsString(elem))
-	{
-	   return(status);
-	}
-       val = PLGetString(elem);
-       bitmap_name = FindImage(wPreferences.pixmap_path, val);
-       if (!bitmap_name)
-	{
-	   wwarning(_("could not find cursor bitmap file \"%s\""), val);
-	   return(status);
-	}
-       elem = PLGetArrayElement(pl, 2);
-       if (!elem || !PLIsString(elem))
-	{
-	   free(bitmap_name);
-	   return(status);
-	}
-       val = PLGetString(elem);
-       mask_name = FindImage(wPreferences.pixmap_path, val);
-       if (!mask_name)
-	{
-	   free(bitmap_name);
-	   wwarning(_("could not find cursor bitmap file \"%s\""), val);
-	   return(status);
-	}
-       mask_status = XReadBitmapFile(dpy, scr->w_win, mask_name, &w, &h,
-				     &mask, &x, &y);
-       bitmap_status = XReadBitmapFile(dpy, scr->w_win, bitmap_name, &w, &h,
-				       &bitmap, &x, &y);
-       if ((BitmapSuccess == bitmap_status) &&
-	   (BitmapSuccess == mask_status))
-	{
-	   fg.pixel = scr->black_pixel;
-	   bg.pixel = scr->white_pixel;
-	   XQueryColor(dpy, scr->w_colormap, &fg);
-	   XQueryColor(dpy, scr->w_colormap, &bg);
-	   *cursor = XCreatePixmapCursor(dpy, bitmap, mask, &fg, &bg, x, y);
-	   status = 1;
-	}
-       check_bitmap_status(bitmap_status, bitmap_name, bitmap);
-       check_bitmap_status(mask_status, mask_name, mask);
-       free(bitmap_name);
-       free(mask_name);
-    }
-   return(status);
+    return(status);
 }
 
+
 static int
 getCursor(WScreen *scr, WDefaultEntry *entry, proplist_t value, void *addr,
 	  void **ret)
 {
-   static Cursor cursor;
-   int status;
-   int changed = 0;
-   
+    static Cursor cursor;
+    int status;
+    int changed = 0;
+    
 again:
-   if (!PLIsArray(value))
-    {
-       wwarning(_("Wrong option format for key \"%s\". Should be %s."),
-		entry->key, "cursor specification");
-       if (!changed)
-	{
-	   value = entry->plvalue;
-	   changed = 1;
-	   wwarning(_("using default \"%s\" instead"), entry->default_value);
-	   goto again;
+    if (!PLIsArray(value)) {
+	wwarning(_("Wrong option format for key \"%s\". Should be %s."),
+		 entry->key, "cursor specification");
+	if (!changed) {
+	    value = entry->plvalue;
+	    changed = 1;
+	    wwarning(_("using default \"%s\" instead"), entry->default_value);
+	    goto again;
 	}
-       return(False);
+	return(False);
     }
-   status = parse_cursor(scr, value, &cursor);
-   if (!status)
-    {
-       wwarning(_("Error in cursor specification for key \"%s\""), entry->key);
-       if (!changed)
-	{
-	   value = entry->plvalue;
-	   changed = 1;
-	   wwarning(_("using default \"%s\" instead"), entry->default_value);
-	   goto again;
+    status = parse_cursor(scr, value, &cursor);
+    if (!status) {
+	wwarning(_("Error in cursor specification for key \"%s\""), entry->key);
+	if (!changed) {
+	    value = entry->plvalue;
+	    changed = 1;
+	    wwarning(_("using default \"%s\" instead"), entry->default_value);
+	    goto again;
 	}
-       return(False);
+	return(False);
     }
-   if (ret)
-    {
-       *ret = &cursor;
+    if (ret) {
+	*ret = &cursor;
     }
-   if (addr)
-    {
-       *(Cursor *)addr = cursor;
+    if (addr) {
+	*(Cursor *)addr = cursor;
     }
-   return(True);
+    return(True);
 }
 #undef CURSOR_ID_NONE
 
@@ -2739,7 +2738,7 @@ again:
 
 /* ---------------- value setting functions --------------- */
 static int
-setJustify(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
+    setJustify(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
 {
     return REFRESH_WINDOW_TITLE_COLOR;
 }
@@ -2811,7 +2810,7 @@ setIconTile(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
     
     if (scr->icon_tile) {
 	reset = 1;
-	RDestroyImage(scr->icon_tile);
+	RReleaseImage(scr->icon_tile);
 	XFreePixmap(dpy, scr->icon_tile_pixmap);
     }
 
@@ -2824,7 +2823,7 @@ setIconTile(WScreen *scr, WDefaultEntry *entry, WTexture **texture, void *foo)
 
     if (!wPreferences.flags.noclip) {
 	if (scr->clip_tile) {
-	    RDestroyImage(scr->clip_tile);
+	    RReleaseImage(scr->clip_tile);
 	}
 	scr->clip_tile = wClipMakeTile(scr, img);
     }
@@ -2993,7 +2992,7 @@ setClipTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 
 	image = RRenderGradient(as+1, as+1, &color1, &color2, RDiagonalGradient);
 	RConvertImage(scr->rcontext, image, &scr->clip_arrow_gradient);
-	RDestroyImage(image);
+	RReleaseImage(image);
     }
 #endif /* GRADIENT_CLIP_ARROW */
 
@@ -3022,11 +3021,8 @@ static int
 setMenuTitleColor(WScreen *scr, WDefaultEntry *entry, XColor *color, long index)
 {
     if (scr->menu_title_pixel[0]!=scr->white_pixel &&
-	scr->menu_title_pixel[0]!=scr->black_pixel) {
-#ifdef DRAWSTRING_PLUGIN
-        if(!scr->drawstring_func[W_STRING_MTITLE])
-#endif
-	wFreeColor(scr, scr->menu_title_pixel[0]);
+            scr->menu_title_pixel[0]!=scr->black_pixel) {
+            wFreeColor(scr, scr->menu_title_pixel[0]);
     }
     
     scr->menu_title_pixel[0] = color->pixel;
@@ -3164,18 +3160,21 @@ setWorkspaceSpecificBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 	    if (close(filedes[1]) < 0)
 		wsyserror("could not close pipe");
 
-	} else if (pid == 0) {
+        } else if (pid == 0) {
+            char *dither;
+
 	    SetupEnvironment(scr);
 
 	    if (close(0) < 0)
 		wsyserror("could not close pipe");
 	    if (dup(filedes[0]) < 0) {
 		wsyserror("dup() failed:can't set workspace specific background image");
-	    }
+            }
+            dither = wPreferences.no_dithering ? "-m" : "-d";
 	    if (wPreferences.smooth_workspace_back)
-		execlp("wmsetbg", "wmsetbg", "-helper", "-S", "-d", NULL);
+		execlp("wmsetbg", "wmsetbg", "-helper", "-S", dither, NULL);
 	    else
-		execlp("wmsetbg", "wmsetbg", "-helper", "-d", NULL);
+		execlp("wmsetbg", "wmsetbg", "-helper", dither, NULL);
 	    wsyserror("could not execute wmsetbg");
 	    exit(1);
 	} else {
@@ -3205,7 +3204,7 @@ setWorkspaceSpecificBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 
 	    SendHelperMessage(scr, 'S', i+1, str);
 
-	    free(str);
+	    wfree(str);
 	} else {
 	    SendHelperMessage(scr, 'U', i+1, NULL);
 	}
@@ -3231,26 +3230,28 @@ setWorkspaceBack(WScreen *scr, WDefaultEntry *entry, proplist_t value,
 	    str = PLGetDescription(value);
 	    if (str) {
 		SendHelperMessage(scr, 'S', 0, str);
-		free(str);
+		wfree(str);
 		SendHelperMessage(scr, 'C', scr->current_workspace+1, NULL);
 	    } else {
 		SendHelperMessage(scr, 'U', 0, NULL);
 	    }
 	}
-    } else {
+    } else if (PLGetNumberOfElements(value) > 0) {
 	char *command;
-	char *text;
+        char *text;
+        char *dither;
 
 	SetupEnvironment(scr);
 	text = PLGetDescription(value);
-	command = wmalloc(strlen(text)+40);
+        command = wmalloc(strlen(text)+40);
+        dither = wPreferences.no_dithering ? "-m" : "-d";
 	if (wPreferences.smooth_workspace_back)
-	    sprintf(command, "wmsetbg -d -S -p '%s' &", text);
+	    sprintf(command, "wmsetbg %s -S -p '%s' &", dither, text);
 	else
-	    sprintf(command, "wmsetbg -d -p '%s' &", text);
-	free(text);
+	    sprintf(command, "wmsetbg %s -p '%s' &", dither, text);
+	wfree(text);
 	system(command);
-	free(command);
+	wfree(command);
     }
     PLRelease(value);
 
@@ -3406,7 +3407,7 @@ setButtonImages(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
  * wmaker panels will have the same dbl-click values. 
  * TODO: figure a better way of dealing with it.
  */
-#include "WINGsP.h"
+#include <WINGs/WINGsP.h>
 
 static int
 setDoubleClick(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
@@ -3422,7 +3423,7 @@ setDoubleClick(WScreen *scr, WDefaultEntry *entry, int *value, void *foo)
 }
 
 
-
+#if 0
 static int
 setMultiByte(WScreen *scr, WDefaultEntry *entry, char *value, void *foo)
 {
@@ -3432,7 +3433,7 @@ setMultiByte(WScreen *scr, WDefaultEntry *entry, char *value, void *foo)
     
     return 0;
 }
-
+#endif
 
 #ifdef DEFINABLE_CURSOR
 static int
