@@ -36,6 +36,10 @@
 #include <dirent.h>
 #include <limits.h>
 
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 #include <signal.h>
 #ifdef __FreeBSD__
 #include <sys/signal.h>
@@ -75,7 +79,7 @@ wMessageDialog(WScreen *scr, char *title, char *message,
 			       defBtn, altBtn, othBtn);    
 
     parent = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 400, 180, 0, 0, 0);
-    
+
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
     wwin = wManageInternalWindow(scr, parent, None, NULL, 
@@ -84,15 +88,10 @@ wMessageDialog(WScreen *scr, char *title, char *message,
     wwin->client_leader = WMWidgetXID(panel->win);
 
     WMMapWidget(panel->win);
-    
+
     wWindowMap(wwin);
 
-    while (!panel->done) {
-	XEvent event;
-	
-	WMNextEvent(dpy, &event);
-	WMHandleEvent(&event);
-    }
+    WMRunModalLoop(WMWidgetScreen(panel->win), WMWidgetView(panel->win));
 
     result = panel->result;
 
@@ -117,7 +116,7 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
     WMInputPanel *panel;
     char *result;
 
-    
+
     panel = WMCreateInputPanel(scr->wmscreen, NULL, title, message, *text,
 			       _("OK"), _("Cancel"));
 
@@ -134,16 +133,11 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
     wwin->client_leader = WMWidgetXID(panel->win);
 
     WMMapWidget(panel->win);
-    
+
     wWindowMap(wwin);
-    
-    while (!panel->done) {
-	XEvent event;
-	
-	WMNextEvent(dpy, &event);
-	WMHandleEvent(&event);
-    }
-    
+
+    WMRunModalLoop(WMWidgetScreen(panel->win), WMWidgetView(panel->win));
+
     if (panel->result == WAPRDefault)
 	result = WMGetTextFieldText(panel->text);
     else
@@ -159,7 +153,7 @@ wInputDialog(WScreen *scr, char *title, char *message, char **text)
 	return False;
     else {
         if (*text)
-            free(*text);
+            wfree(*text);
         *text = result;
 
         return True;
@@ -228,8 +222,8 @@ listPixmaps(WScreen *scr, WMList *lPtr, char *path)
 	strcat(msg, path);
 	
 	wMessageDialog(scr, _("Error"), msg, _("OK"), NULL, NULL);
-	free(msg);
-	free(apath);
+	wfree(msg);
+	wfree(apath);
 	return;
     }
 
@@ -256,7 +250,7 @@ listPixmaps(WScreen *scr, WMList *lPtr, char *path)
     WMSortListItems(lPtr);
 
     closedir(dir);
-    free(apath);
+    wfree(apath);
     panel->preview = True;
 }
 
@@ -298,7 +292,11 @@ listCallback(void *self, void *data)
     char *path;
 
     if (lPtr==panel->dirList) {
-	path = WMGetListSelectedItem(lPtr)->text;
+	WMListItem *item = WMGetListSelectedItem(lPtr);
+	
+	if (item == NULL)
+	    return;
+	path = item->text;
 	
 	WMSetTextFieldText(panel->fileField, path);
 
@@ -310,20 +308,26 @@ listCallback(void *self, void *data)
 	listPixmaps(panel->scr, panel->iconList, path);	
     } else {
 	char *tmp, *iconFile;
+	WMListItem *item = WMGetListSelectedItem(panel->dirList);
 	
-	path = WMGetListSelectedItem(panel->dirList)->text;
+	if (item == NULL)
+	    return;
+	path = item->text;
 	tmp = wexpandpath(path);
 
-	iconFile = WMGetListSelectedItem(panel->iconList)->text;
+	item = WMGetListSelectedItem(panel->iconList);
+	if (item == NULL)
+	    return;
+	iconFile = item->text;
 
 	path = wmalloc(strlen(tmp)+strlen(iconFile)+4);
 	strcpy(path, tmp);
 	strcat(path, "/");
 	strcat(path, iconFile);
-	free(tmp);
+	wfree(tmp);
 	WMSetTextFieldText(panel->fileField, path);
 	setViewedImage(panel, path);
-	free(path);
+	wfree(path);
     }
 }
 
@@ -346,10 +350,10 @@ listIconPaths(WMList *lPtr)
 	 * directories searched */
 	if (access(tmp, X_OK)==0)
 	    WMAddListItem(lPtr, path);
-	free(tmp);
+	wfree(tmp);
     } while ((path=strtok(NULL, ":"))!=NULL);
 
-    free(paths);
+    wfree(paths);
 }
 
 
@@ -360,12 +364,13 @@ drawIconProc(WMList *lPtr, int index, Drawable d, char *text,
     IconPanel *panel = WMGetHangedData(lPtr);
     GC gc = panel->scr->draw_gc;
     GC copygc = panel->scr->copy_gc;
-    char *buffer, *dirfile;
+    char *file, *dirfile;
     WMPixmap *pixmap;
     WMColor *blackcolor;
     WMColor *whitecolor;
     WMSize size;
-    WMScreen *wmscr=WMWidgetScreen(panel->win);
+    WMScreen *wmscr = WMWidgetScreen(panel->win);
+    RColor color;
     int width;
 
     if(!panel->preview) return;
@@ -376,12 +381,20 @@ drawIconProc(WMList *lPtr, int index, Drawable d, char *text,
     whitecolor = WMWhiteColor(wmscr);
 
     dirfile = wexpandpath(WMGetListSelectedItem(panel->dirList)->text);
-    buffer = wmalloc(strlen(dirfile)+strlen(text)+4);
-    sprintf(buffer, "%s/%s", dirfile, text);
-    free(dirfile);
+    file = wmalloc(strlen(dirfile)+strlen(text)+4);
+    sprintf(file, "%s/%s", dirfile, text);
+    wfree(dirfile);
+
+    if ((state & WLDSSelected) != 0) {
+        color.red = color.green = color.blue = 0xff;
+        color.alpha = 0;
+    } else {
+        color.red = color.blue = 0xae;
+        color.green = 0xaa; color.alpha = 0;
+    }
+    pixmap = WMCreateBlendedPixmapFromFile(wmscr, file, &color);
+    wfree(file);
 
-    pixmap = WMCreatePixmapFromFile(WMWidgetScreen(panel->win), buffer);
-    free(buffer);
     if (!pixmap) {
         WMRemoveListItem(lPtr, index);
         return;
@@ -475,7 +488,7 @@ buttonCallback(void *self, void *clientData)
 	    path = WMGetFilePanelFile(op);
 	    WMSetTextFieldText(panel->fileField, path);
 	    setViewedImage(panel, path);
-	    free(path);
+	    wfree(path);
 	}
 	WMDestroyFilePanel(op);
     }
@@ -483,6 +496,83 @@ buttonCallback(void *self, void *clientData)
 }
 
 
+static void
+keyPressHandler(XEvent *event, void *data)
+{
+    IconPanel *panel = (IconPanel*)data;
+    Display *dpy = event->xany.display;
+    char buffer[32];
+    int count;
+    KeySym ksym;
+    int iidx;
+    int didx;
+    int item;
+    WMList *list = NULL;
+    
+    if (event->type == KeyRelease)
+	return;
+
+    buffer[0] = 0;
+    count = XLookupString(&event->xkey, buffer, sizeof(buffer), &ksym, NULL);
+    
+    
+    iidx = WMGetListSelectedItemRow(panel->iconList);
+    didx = WMGetListSelectedItemRow(panel->dirList);
+    
+    switch (ksym) {
+     case XK_Up:
+	if (iidx > 0)
+	    item = iidx-1;
+	else
+	    item = iidx;
+	list = panel->iconList;
+	break;
+     case XK_Down:
+	if (iidx < WMGetListNumberOfRows(panel->iconList) - 1)
+	    item = iidx+1;
+	else
+	    item = iidx;
+	list = panel->iconList;
+	break;
+     case XK_Home:
+	item = 0;
+	list = panel->iconList;
+	break;
+     case XK_End:
+	item = WMGetListNumberOfRows(panel->iconList) - 1;
+	list = panel->iconList;
+	break;
+     case XK_Next:
+	if (didx < WMGetListNumberOfRows(panel->dirList) - 1)
+	    item = didx + 1;
+	else
+	    item = didx;
+	list = panel->dirList;
+	break;
+     case XK_Prior:
+	if (didx > 0)
+	    item = didx - 1;
+	else
+	    item = 0;
+	list = panel->dirList;
+	break;
+     case XK_Return:
+	WMPerformButtonClick(panel->okButton);
+	break;
+     case XK_Escape:
+	WMPerformButtonClick(panel->cancelButton);
+	break;
+    }
+
+    if (list) {
+	WMSelectListItem(list, item);
+	WMSetListPosition(list, item - 5);
+	listCallback(list, panel);
+    }
+}
+
+
+
 Bool
 wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 {
@@ -500,6 +590,10 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
     panel->win = WMCreateWindow(scr->wmscreen, "iconChooser");
     WMResizeWidget(panel->win, 450, 280);
     
+    WMCreateEventHandler(WMWidgetView(panel->win), KeyPressMask|KeyReleaseMask,
+			 keyPressHandler, panel);
+    
+
     boldFont = WMBoldSystemFontOfSize(scr->wmscreen, 12);
     panel->normalfont = WMSystemFontOfSize(WMWidgetScreen(panel->win), 12);
     
@@ -564,6 +658,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
     WMSetLabelText(panel->fileLabel, _("File Name:"));
     
     panel->fileField = WMCreateTextField(panel->win);
+    WMSetViewNextResponder(WMWidgetView(panel->fileField), WMWidgetView(panel->win));
     WMResizeWidget(panel->fileField, 345, 20);
     WMMoveWidget(panel->fileField, 95, 210);
     WMSetTextFieldEditable(panel->fileField, False);
@@ -608,7 +703,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 	wwin = wManageInternalWindow(scr, parent, None, tmp,
 				     (scr->scr_width - 450)/2,
 				     (scr->scr_height - 280)/2, 450, 280);
-	free(tmp);
+	wfree(tmp);
     }
     
     /* put icon paths in the list */
@@ -632,7 +727,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 	 * would be loaded by default with the current search path */
 	*file = WMGetListSelectedItem(panel->iconList)->text;
 	if ((*file)[0]==0) {
-	    free(*file);
+	    wfree(*file);
 	    *file = NULL;
 	} else {
 	    defaultPath = FindImage(wPreferences.icon_path, *file);
@@ -642,9 +737,9 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 		*file = wantedPath;
 	    } else {
 		*file = wstrdup(*file);
-		free(wantedPath);
+		wfree(wantedPath);
 	    }
-	    free(defaultPath);
+	    wfree(defaultPath);
 	}
     } else {
 	*file = NULL;
@@ -658,7 +753,7 @@ wIconChooserDialog(WScreen *scr, char **file, char *instance, char *class)
 
     wUnmanageWindow(wwin, False, False);
 
-    free(panel);
+    wfree(panel);
 
     XDestroyWindow(dpy, parent);
     
@@ -696,6 +791,7 @@ typedef struct {
     RImage *icon;
     RImage *pic;
     WMPixmap *oldPix;
+    WMFont *oldFont;
     char *str;
     int x;
 #endif
@@ -704,8 +800,8 @@ typedef struct {
 
 
 #define COPYRIGHT_TEXT  \
-     "Copyright \xa9 1997~2000 Alfredo K. Kojima <kojima@windowmaker.org>\n"\
-     "Copyright \xa9 1998~2000 Dan Pascu <dan@windowmaker.org>"
+     "Copyright \xa9 1997~2001 Alfredo K. Kojima <kojima@windowmaker.org>\n"\
+     "Copyright \xa9 1998~2001 Dan Pascu <dan@windowmaker.org>"
 
  
 
@@ -721,11 +817,14 @@ destroyInfoPanel(WCoreWindow *foo, void *data, XEvent *event)
     if (thePanel->oldPix) {
 	WMReleasePixmap(thePanel->oldPix);
     }
+    if (thePanel->oldFont) {
+        WMReleaseFont(thePanel->oldFont);
+    }
     if (thePanel->icon) {
-	RDestroyImage(thePanel->icon);
+	RReleaseImage(thePanel->icon);
     }
     if (thePanel->pic) {
-	RDestroyImage(thePanel->pic);
+	RReleaseImage(thePanel->pic);
     }
 #endif /* SILLYNESS */
     WMUnmapWidget(thePanel);
@@ -734,7 +833,7 @@ destroyInfoPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     WMDestroyWidget(thePanel->win);
 
-    free(thePanel);
+    wfree(thePanel);
     
     thePanel = NULL;
 }
@@ -798,6 +897,7 @@ logoPushCallback(void *data)
     InfoPanel *panel = (InfoPanel*)data;
     char buffer[512];
     int i;
+    static int oldi = 0;
     int len;
     static int jingobeu[] = {
 	329, 150, -1, 100, 329, 150, -1, 100, 329, 300, -1, 250,
@@ -824,29 +924,41 @@ XChangeKeyboardControl(dpy,KBBellPitch|KBBellDuration|KBBellPercent,&kc);
 	}
     } else if (panel->cycle < 30) {
 	RImage *image;
-	WMPixmap *pix;
+        WMPixmap *pix;
+        RColor gray;
 
-	image = RCloneImage(panel->icon);
+        gray.red = 0xae;  gray.green = 0xaa;
+        gray.blue = 0xae; gray.alpha = 0;
+
+        image = RScaleImage(panel->icon, panel->pic->width, panel->pic->height);
 	RCombineImagesWithOpaqueness(image, panel->pic, panel->cycle*255/30);
-	pix = WMCreatePixmapFromRImage(panel->scr->wmscreen, image, 128);
-	RDestroyImage(image);
+	pix = WMCreateBlendedPixmapFromRImage(panel->scr->wmscreen, image, &gray);
+	RReleaseImage(image);
 	WMSetLabelImage(panel->logoL, pix);
 	WMReleasePixmap(pix);
     }
 
-    i = panel->cycle%200;
+    /* slow down text a little */
+    i = (int)(panel->cycle * 50.0/85.0)%200;
 
-    len = strlen(panel->str);
+    if (i != oldi) {
+        len = strlen(panel->str);
 
-    strncpy(buffer, panel->str, i<len ? i : len);
-    if (i >= len)
-	memset(&buffer[len], ' ', i-len);
+        strncpy(buffer, panel->str, i<len ? i : len);
+        if (i >= len)
+            memset(&buffer[len], ' ', i-len);
 
-    strncpy(buffer, panel->str, i<len ? i : len);
-    if (i >= len)
-	memset(&buffer[len], ' ', i-len);
-    buffer[i]=0;
-    WMSetLabelText(panel->versionL, buffer);
+        strncpy(buffer, panel->str, i<len ? i : len);
+        if (i >= len)
+            memset(&buffer[len], ' ', i-len);
+        buffer[i]=0;
+
+        WMSetLabelText(panel->versionL, buffer);
+
+        XFlush(WMScreenDisplay(WMWidgetScreen(panel->versionL)));
+
+        oldi = i;
+    }
 
     panel->timer = WMAddTimerHandler(50, logoPushCallback, panel);
     panel->cycle++;
@@ -964,67 +1076,66 @@ handleLogoPush(XEvent *event, void *data)
 "             .....XmnbN:q&Bo....             ",
 "                 ............                "};
     static char *msgs[] = {
-	    "Have a nice day!"
+	    "Have a nice day!",
+	    "Focus follow mouse users will burn in hell!!!",
+	    "Mooo Canada!!!!",
+	    "Hi! My name is bobby...",
+	    "AHH! The neurotic monkeys are after me!",
+	    "WHAT YOU SAY??",
+	    "WE GET SIGNAL",
+	    "SOMEBODY SET UP US THE BOMB",
+	    "ALL YOUR BASE ARE BELONG TO US!",
+            "Oh My God!!! Larry is back!"
     };
 
 
     clicks++;
+	       
     if (!panel->timer && !broken && clicks > 0) {
-	char *file;
-	char *path;
+        WMFont *font;
 
 	panel->x = 0;
 	clicks = 0;
 	if (!panel->icon) {
-	    file = wDefaultGetIconFile(panel->scr, "Logo", "WMPanel", False);
-	    if (!file) {
-		broken = 1;
-		return;
-	    }
-    
-	    path = FindImage(wPreferences.icon_path, file);
-	    if (!path) {
-		broken = 1;
-		return;
-	    }
-
-	    panel->icon = RLoadImage(panel->scr->rcontext, path, 0);
-	    free(path);
+            panel->icon = WMGetApplicationIconImage(panel->scr->wmscreen);
 	    if (!panel->icon) {
 		broken = 1;
 		return;
-	    }
+            } else {
+		RColor color;
+
+                color.red = 0xae;  color.green = 0xaa;
+                color.blue = 0xae; color.alpha = 0;
+
+                panel->icon = RCloneImage(panel->icon);
+		RCombineImageWithColor(panel->icon, &color);
+            }
 	}
 	if (!panel->pic) {
 	    panel->pic = RGetImageFromXPMData(panel->scr->rcontext, pic_data);
-	    if (!panel->pic || panel->icon->width!=panel->pic->width
-		|| panel->icon->height!=panel->pic->height) {
+	    if (!panel->pic) {
 		broken = 1;
-		RDestroyImage(panel->icon);
+		RReleaseImage(panel->icon);
 		panel->icon = NULL;
-		if (panel->pic) {
-		    RDestroyImage(panel->pic);
-		    panel->pic = NULL;
-		}
 		return;
 	    }
-
-	    {
-		RColor color;
-		color.red = 0xae;
-		color.green = 0xaa;
-		color.blue = 0xae;
-		color.alpha = 255;
-		RCombineImageWithColor(panel->icon, &color);
-		RCombineImageWithColor(panel->pic, &color);
-	    }
 	}
 
 	panel->str = msgs[rand()%(sizeof(msgs)/sizeof(char*))];
 
 	panel->timer = WMAddTimerHandler(50, logoPushCallback, panel);
 	panel->cycle = 0;
-	panel->oldPix = WMRetainPixmap(WMGetLabelImage(panel->logoL));
+        panel->oldPix = WMRetainPixmap(WMGetLabelImage(panel->logoL));
+        /* If we don't use a fixed font, scrolling will be jumpy */
+        /* Alternatively we can draw text in a pixmap and scroll it smoothly */
+        if ((panel->oldFont = WMGetLabelFont(panel->versionL))!=NULL)
+            WMRetainFont(panel->oldFont);
+        font = WMCreateFont(WMWidgetScreen(panel->versionL), "-*-fixed-*-*-*-*-14-*-*-*-*-*-*-*");
+        if (font) {
+            WMSetLabelFont(panel->versionL, font);
+            WMReleaseFont(font);
+        }
+        WMSetLabelText(panel->versionL, "");
     } else if (panel->timer) {
 	char version[20];
 
@@ -1037,8 +1148,14 @@ handleLogoPush(XEvent *event, void *data)
 	WMDeleteTimerHandler(panel->timer);
 	panel->timer = NULL;
 
-	sprintf(version, "Version %s", VERSION);
-	WMSetLabelText(panel->versionL, version);
+        WMSetLabelFont(panel->versionL, panel->oldFont);
+        if (panel->oldFont) {
+            WMReleaseFont(panel->oldFont);
+            panel->oldFont = NULL;
+        }
+	sprintf(version, _("Version %s"), VERSION);
+        WMSetLabelText(panel->versionL, version);
+        XFlush(WMScreenDisplay(WMWidgetScreen(panel->versionL)));
     }
 
     {
@@ -1090,7 +1207,10 @@ wShowInfoPanel(WScreen *scr)
     panel->win = WMCreateWindow(scr->wmscreen, "info");
     WMResizeWidget(panel->win, 382, 230);
     
-    logo = WMGetApplicationIconImage(scr->wmscreen);
+    logo = WMCreateApplicationIconBlendedPixmap(scr->wmscreen, (RColor*)NULL);
+    if (!logo) {
+        logo = WMRetainPixmap(WMGetApplicationIconPixmap(scr->wmscreen));
+    }
     if (logo) {
 	size = WMGetPixmapSize(logo);
 	panel->logoL = WMCreateLabel(panel->win);
@@ -1102,6 +1222,7 @@ wShowInfoPanel(WScreen *scr)
 	WMCreateEventHandler(WMWidgetView(panel->logoL), ButtonPressMask,
 			     handleLogoPush, panel);
 #endif
+        WMReleasePixmap(logo);
     }
 
     panel->name1L = WMCreateLabel(panel->win);
@@ -1139,10 +1260,10 @@ wShowInfoPanel(WScreen *scr)
 	font = NULL;
     }
     WMSetLabelTextAlignment(panel->name2L, WACenter);
-    WMSetLabelText(panel->name2L, "Window Manager for X");
+    WMSetLabelText(panel->name2L, _("Window Manager for X"));
 
     
-    sprintf(version, "Version %s", VERSION);
+    sprintf(version, _("Version %s"), VERSION);
     panel->versionL = WMCreateLabel(panel->win);
     WMResizeWidget(panel->versionL, 310, 16);
     WMMoveWidget(panel->versionL, 30, 95);
@@ -1163,32 +1284,42 @@ wShowInfoPanel(WScreen *scr)
 
     switch (scr->w_depth) {
      case 15:
-	strcpy(version, "32 thousand");
+	strcpy(version, _("32 thousand"));
 	break;
      case 16:
-	strcpy(version, "64 thousand");
+	strcpy(version, _("64 thousand"));
 	break;
      case 24:
      case 32:
-	strcpy(version, "16 million");
+	strcpy(version, _("16 million"));
 	break;
      default:
 	sprintf(version, "%d", 1<<scr->w_depth);
 	break;
     }
 
-    sprintf(buffer, "Using visual 0x%x: %s %ibpp (%s colors)\n",
+    sprintf(buffer, _("Using visual 0x%x: %s %ibpp (%s colors)\n"),
 	    (unsigned)scr->w_visual->visualid,
 	    visuals[scr->w_visual->class], scr->w_depth, version);
 
-    strcat(buffer, "Supported image formats: ");
+#if defined(HAVE_MALLOC_H) && defined(HAVE_MALLINFO)
+    {
+	struct mallinfo ma = mallinfo();
+	sprintf(buffer+strlen(buffer), 
+		_("Total allocated memory: %i kB. Total memory in use: %i kB.\n"),
+		(ma.arena+ma.hblkhd)/1024, (ma.uordblks+ma.hblkhd)/1024);
+	
+    }
+#endif
+    
+    strcat(buffer, _("Supported image formats: "));
     strl = RSupportedFileFormats();
     for (i=0; strl[i]!=NULL; i++) {
 	strcat(buffer, strl[i]);
 	strcat(buffer, " ");
     }
 
-    strcat(buffer, "\nAdditional Support For: ");
+    strcat(buffer, _("\nAdditional Support For: "));
     {
 	char *list[8];
 	char buf[80];
@@ -1206,15 +1337,15 @@ wShowInfoPanel(WScreen *scr)
 #ifdef OLWM_HINTS
 	list[j++] = "OLWM";
 #endif
-#ifdef WMSOUND
-	list[j++] = "Sound";
+#ifdef WSOUND
+	list[j++] = _("Sound");
 #endif
 
 	buf[0] = 0;
 	for (i = 0; i < j; i++) {
 	    if (i > 0) {
 		if (i == j - 1)
-		    strcat(buf, " and ");
+		    strcat(buf, _(" and "));
 		else
 		    strcat(buf, ", ");
 	    }
@@ -1243,7 +1374,7 @@ wShowInfoPanel(WScreen *scr)
 
     WMMapWidget(panel->win);
 
-    wwin = wManageInternalWindow(scr, parent, None, "Info",
+    wwin = wManageInternalWindow(scr, parent, None, _("Info"),
 				 (scr->scr_width - 382)/2,
 				 (scr->scr_height - 230)/2, 382, 230);
 
@@ -1266,7 +1397,7 @@ wShowInfoPanel(WScreen *scr)
 	panel->timer = WMAddTimerHandler(100, logoPushCallback, panel);
 	panel->cycle = 0;
 	panel->x = 1;
-	panel->str = "Merry Christmas!";
+	panel->str = _("Merry X'mas!");
 	panel->oldPix = WMRetainPixmap(WMGetLabelImage(panel->logoL));
     } 
 #endif
@@ -1290,22 +1421,6 @@ typedef struct {
 } LegalPanel;
 
 
-
-#define LICENSE_TEXT \
-	"    Window Maker is free software; you can redistribute it and/or modify "\
-	"it under the terms of the GNU General Public License as published "\
-	"by the Free Software Foundation; either version 2 of the License, "\
-	"or (at your option) any later version.\n\n\n"\
-	"    Window Maker is distributed in the hope that it will be useful, but "\
-	"WITHOUT ANY WARRANTY; without even the implied warranty of "\
-	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU "\
-	"General Public License for more details.\n\n\n"\
-	"    You should have received a copy of the GNU General Public License "\
-	"along with this program; if not, write to the Free Software "\
-	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA "\
-	"02111-1307, USA."
- 
-
 static LegalPanel *legalPanel = NULL;
 
 static void
@@ -1317,7 +1432,7 @@ destroyLegalPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     wUnmanageWindow(legalPanel->wwin, False, False);
 
-    free(legalPanel);
+    wfree(legalPanel);
     
     legalPanel = NULL;
 }
@@ -1347,10 +1462,23 @@ wShowLegalPanel(WScreen *scr)
 
     
     panel->licenseL = WMCreateLabel(panel->win);
+    WMSetLabelWraps(panel->licenseL, True);
     WMResizeWidget(panel->licenseL, 400, 230);
     WMMoveWidget(panel->licenseL, 10, 10);
     WMSetLabelTextAlignment(panel->licenseL, WALeft);
-    WMSetLabelText(panel->licenseL, LICENSE_TEXT);
+    WMSetLabelText(panel->licenseL,
+      _("    Window Maker is free software; you can redistribute it and/or\n"
+	"modify it under the terms of the GNU General Public License as\n"
+	"published by the Free Software Foundation; either version 2 of the\n"
+	"License, or (at your option) any later version.\n\n\n"
+	"    Window Maker is distributed in the hope that it will be useful,\n"
+	"but WITHOUT ANY WARRANTY; without even the implied warranty\n"
+	"of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+	"See the GNU General Public License for more details.\n\n\n"
+	"    You should have received a copy of the GNU General Public\n"
+	"License along with this program; if not, write to the Free Software\n"
+	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA\n"
+	"02111-1307, USA."));
     WMSetLabelRelief(panel->licenseL, WRGroove);
 
     WMRealizeWidget(panel->win);
@@ -1360,7 +1488,7 @@ wShowLegalPanel(WScreen *scr)
 
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-    wwin = wManageInternalWindow(scr, parent, None, "Legal",
+    wwin = wManageInternalWindow(scr, parent, None, _("Legal"),
 				 (scr->scr_width - 420)/2,
 				 (scr->scr_height - 250)/2, 420, 250);
 
@@ -1473,14 +1601,13 @@ getWindowMakerIconImage(WMScreen *scr)
         path = FindImage(wPreferences.icon_path, PLGetString(value));
 
         if (path) {
-            RImage *image;
+            RColor gray;
 
-            image = RLoadImage(WMScreenRContext(scr), path, 0);
-            if (image) {
-                pix = WMCreatePixmapFromRImage(scr, image, 0);
-                RDestroyImage(image);
-            }
-            free(path);
+            gray.red = 0xae;  gray.green = 0xaa;
+            gray.blue = 0xae; gray.alpha = 0;
+
+            pix = WMCreateBlendedPixmapFromFile(scr, path, &gray);
+            wfree(path);
         }
     }
 
@@ -1564,6 +1691,7 @@ wShowCrashingDialogPanel(int whatSig)
                    _(" This fatal error occured probably due to a bug."
                    " Please fill the included BUGFORM and "
                    "report it to bugs@windowmaker.org."));
+    WMSetLabelWraps(panel->note2L, True);
 
 
     panel->whatF = WMCreateFrame(panel->win);
@@ -1617,7 +1745,7 @@ wShowCrashingDialogPanel(int whatSig)
 
     WMUnmapWidget(panel->win);
     WMDestroyWidget(panel->win);
-    free(panel);
+    wfree(panel);
 
     return action;
 }
@@ -1670,19 +1798,6 @@ drawGNUstepLogo(Display *dpy, Drawable d, int width, int height,
 }
 
 
-
-#define GNUSTEP_TEXT \
-	"Window Maker is part of the GNUstep project.\n"\
-	"The GNUstep project aims to create a free\n"\
-	"implementation of the OpenStep(tm) specification\n"\
-	"which is a object-oriented framework for\n"\
-	"creating advanced graphical, multi-platform\n"\
-	"applications. Additionally, a development and\n"\
-	"user desktop enviroment will be created on top\n"\
-	"of the framework. For more information about\n"\
-	"GNUstep, please visit: www.gnustep.org"
-
-
 typedef struct {
     WScreen *scr;
     
@@ -1706,7 +1821,7 @@ destroyGNUstepPanel(WCoreWindow *foo, void *data, XEvent *event)
 
     wUnmanageWindow(gnustepPanel->wwin, False, False);
 
-    free(gnustepPanel);
+    wfree(gnustepPanel);
 
     gnustepPanel = NULL;
 }
@@ -1768,7 +1883,16 @@ wShowGNUstepPanel(WScreen *scr)
     WMMoveWidget(panel->textL, 30, 50);
     WMSetLabelTextAlignment(panel->textL, WARight);
     WMSetLabelImagePosition(panel->textL, WIPOverlaps);
-    WMSetLabelText(panel->textL, GNUSTEP_TEXT);
+    WMSetLabelText(panel->textL,
+	_("Window Maker is part of the GNUstep project.\n"\
+	"The GNUstep project aims to create a free\n"\
+	"implementation of the OpenStep(tm) specification\n"\
+	"which is a object-oriented framework for\n"\
+	"creating advanced graphical, multi-platform\n"\
+	"applications. Additionally, a development and\n"\
+	"user desktop enviroment will be created on top\n"\
+	"of the framework. For more information about\n"\
+	"GNUstep, please visit: www.gnustep.org"));
     WMSetLabelImage(panel->textL, pixmap);
 
     WMReleasePixmap(pixmap);
@@ -1780,7 +1904,7 @@ wShowGNUstepPanel(WScreen *scr)
 
     XReparentWindow(dpy, WMWidgetXID(panel->win), parent, 0, 0);
 
-    wwin = wManageInternalWindow(scr, parent, None, "About GNUstep",
+    wwin = wManageInternalWindow(scr, parent, None, _("About GNUstep"),
 				 (scr->scr_width - 325)/2,
 				 (scr->scr_height - 200)/2, 325, 200);
 
