@@ -29,13 +29,15 @@
 #include <unistd.h>
 #include <string.h>
 
+
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #ifdef SHAPE
-#include <X11/extensions/shape.h>
+# include <X11/extensions/shape.h>
 #endif
-#ifdef XDE_DND
-#include "xde.h"
+#ifdef XDND
+#include "xdnd.h"
 #endif
 
 #ifdef KEEP_XKB_LOCK_STATUS     
@@ -56,6 +58,7 @@
 #include "framewin.h"
 #include "properties.h"
 #include "balloon.h"
+
 #ifdef GNOME_STUFF
 # include "gnome.h"
 #endif
@@ -81,8 +84,11 @@ extern Atom _XA_WM_COLORMAP_NOTIFY;
 
 extern Atom _XA_WM_CHANGE_STATE;
 extern Atom _XA_WM_DELETE_WINDOW;
+extern Atom _XA_GNUSTEP_WM_ATTR;
 extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
+extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
+extern Atom _XA_WINDOWMAKER_COMMAND;
 
 #ifdef OFFIX_DND
 extern Atom _XA_DND_PROTOCOL;
@@ -94,6 +100,10 @@ extern Bool wShapeSupported;
 extern int wShapeEventBase;
 #endif
 
+#ifdef KEEP_XKB_LOCK_STATUS     
+extern int wXkbEventBase;
+#endif
+
 /* special flags */
 extern char WDelayedActionSet;
 
@@ -143,11 +153,10 @@ static int deadProcessPtr=0;
 typedef struct DeathHandler {
     WDeathHandler	*callback;
     pid_t 		pid;
-    struct DeathHandler *next;
     void 		*client_data;
 } DeathHandler;
 
-static DeathHandler *deathHandler=NULL;
+static WMArray *deathHandlers=NULL;
 
 
 
@@ -159,14 +168,15 @@ wAddDeathHandler(pid_t pid, WDeathHandler *callback, void *cdata)
     handler = malloc(sizeof(DeathHandler));
     if (!handler)
       return 0;
-    
+
     handler->pid = pid;
     handler->callback = callback;
     handler->client_data = cdata;
-    
-    handler->next = deathHandler;
-        
-    deathHandler = handler;
+
+    if (!deathHandlers)
+	deathHandlers = WMCreateArrayWithDestructor(8, wfree);
+
+    WMAddToArray(deathHandlers, handler);
 
     return handler;
 }
@@ -176,34 +186,22 @@ wAddDeathHandler(pid_t pid, WDeathHandler *callback, void *cdata)
 void 
 wDeleteDeathHandler(WMagicNumber id)
 {
-    DeathHandler *tmp, *handler=(DeathHandler*)id;
+    DeathHandler *handler=(DeathHandler*)id;
 
-    if (!handler || !deathHandler)
+    if (!handler || !deathHandlers)
       return;
 
-    tmp = deathHandler;
-    if (tmp==handler) {
-	deathHandler = handler->next;
-	free(handler);
-    } else {
-	while (tmp->next) {
-	    if (tmp->next==handler) {
-		tmp->next=handler->next;
-		free(handler);
-		break;
-	    }
-	    tmp = tmp->next;
-	}
-    }
+    /* array destructor will call wfree(handler) */
+    WMRemoveFromArray(deathHandlers, handler);
 }
 
 
 void
 DispatchEvent(XEvent *event)
 {
-    if (deathHandler)
+    if (deathHandlers)
 	handleDeadProcess(NULL);
-    
+
     if (WCHECK_STATE(WSTATE_NEED_EXIT)) {
 	WCHANGE_STATE(WSTATE_EXITING);
 	/* received SIGTERM */
@@ -219,15 +217,18 @@ DispatchEvent(XEvent *event)
 
 	Shutdown(WSRestartPreparationMode);
 	/* received SIGHUP */
-	Restart(NULL);
+	Restart(NULL, True);
+    } else if (WCHECK_STATE(WSTATE_NEED_REREAD)) {
+        WCHANGE_STATE(WSTATE_NORMAL);
+        wDefaultsCheckDomains("bla");
     }
 
     /* for the case that all that is wanted to be dispatched is
      * the stuff above */
     if (!event)
 	return;
-    
-    saveTimestamp(event);    
+
+    saveTimestamp(event);
     switch (event->type) {
      case MapRequest:
 	handleMapRequest(event);
@@ -298,7 +299,6 @@ DispatchEvent(XEvent *event)
      case VisibilityNotify:
 	handleVisibilityNotify(event);
 	break;
-
      default:
 	handleExtensions(event);
 	break;
@@ -374,7 +374,7 @@ handleDeadProcess(void *foo)
 	wWindowDeleteSavedStatesForPID(deadProcesses[i].pid);
     }
 
-    if (!deathHandler) {
+    if (!deathHandlers) {
 	deadProcessPtr=0;
 	return;
     }
@@ -383,11 +383,10 @@ handleDeadProcess(void *foo)
     while (deadProcessPtr>0) {
 	deadProcessPtr--;
 
-	tmp = deathHandler;
-	while (tmp) {
-	    DeathHandler *t;
-	    
-	    t = tmp->next;
+	for (i = WMGetArrayItemCount(deathHandlers)-1; i >= 0; i--) {
+	    tmp = WMGetFromArray(deathHandlers, i);
+	    if (!tmp)
+		continue;
 
 	    if (tmp->pid == deadProcesses[deadProcessPtr].pid) {
 		(*tmp->callback)(tmp->pid, 
@@ -395,7 +394,6 @@ handleDeadProcess(void *foo)
 				 tmp->client_data);
 		wDeleteDeathHandler(tmp);
 	    }
-	    tmp = t;
 	}
     }
 }
@@ -433,6 +431,9 @@ saveTimestamp(XEvent *event)
 	break;
      case SelectionNotify:
 	LastTimestamp = event->xselection.time;
+#ifdef XDND
+    wXDNDProcessSelection(event);
+#endif
 	break;
     }
 }
@@ -441,11 +442,20 @@ saveTimestamp(XEvent *event)
 static void
 handleExtensions(XEvent *event)
 {
+#ifdef KEEP_XKB_LOCK_STATUS   
+    XkbEvent *xkbevent;
+    xkbevent = (XkbEvent *)event;
+#endif /*KEEP_XKB_LOCK_STATUS*/
 #ifdef SHAPE
     if (wShapeSupported && event->type == (wShapeEventBase+ShapeNotify)) {
 	handleShapeNotify(event);
     }
 #endif	
+#ifdef KEEP_XKB_LOCK_STATUS   
+    if (wPreferences.modelock && (xkbevent->type == wXkbEventBase)){
+        handleXkbIndicatorStateNotify(event);
+    }
+#endif /*KEEP_XKB_LOCK_STATUS*/
 }
 
 
@@ -457,9 +467,8 @@ handleMapRequest(XEvent *ev)
     Window window = ev->xmaprequest.window;
 
 #ifdef DEBUG
-    printf("got map request for %x\n", (unsigned)window);
+    L("got map request for %x\n", (unsigned)window);
 #endif
-
     if ((wwin = wWindowFor(window))) {
 	if (wwin->flags.shaded) {
 	    wUnshadeWindow(wwin);
@@ -470,14 +479,9 @@ handleMapRequest(XEvent *ev)
 	} else if (wwin->flags.hidden) {
             WApplication *wapp = wApplicationOf(wwin->main_window);
             /* go to the last workspace that the user worked on the app */
-#ifndef REDUCE_APPICONS
-            /* This severely breaks REDUCE_APPICONS.  last_workspace is a neat
-             * concept but it needs to be reworked to handle REDUCE_APPICONS -cls
-             */
             if (wapp) {
                 wWorkspaceChange(wwin->screen_ptr, wapp->last_workspace);
             }
-#endif
 	    wUnhideApplication(wapp, False, False);
 	}
 	return;
@@ -536,9 +540,8 @@ handleDestroyNotify(XEvent *event)
     Window window = event->xdestroywindow.window;
 
 #ifdef DEBUG
-    puts("got destroy notify");
-#endif	    
-
+    L("got destroy notify");
+#endif
     wwin = wWindowFor(window);
     if (wwin) {
 	wUnmanageWindow(wwin, False, True);
@@ -572,10 +575,9 @@ handleExpose(XEvent *event)
     WObjDescriptor *desc;
     XEvent ev;
 
-#ifdef DEBUG    
-    puts("got expose");
+#ifdef DEBUG
+    L("got expose");
 #endif
-
     while (XCheckTypedWindowEvent(dpy, event->xexpose.window, Expose, &ev));
 
     if (XFindContext(dpy, event->xexpose.window, wWinContext, 
@@ -588,6 +590,38 @@ handleExpose(XEvent *event)
     }
 }
 
+static void
+executeButtonAction(WScreen *scr, XEvent *event, int action)
+{
+    switch(action) {
+    case WA_SELECT_WINDOWS:
+        wUnselectWindows(scr);
+        wSelectWindows(scr, event);
+        break;
+    case WA_OPEN_APPMENU:
+        OpenRootMenu(scr, event->xbutton.x_root, event->xbutton.y_root, False);
+        /* ugly hack */
+        if (scr->root_menu) {
+            if (scr->root_menu->brother->flags.mapped)
+                event->xbutton.window = scr->root_menu->brother->frame->core->window;
+            else
+                event->xbutton.window = scr->root_menu->frame->core->window;
+        }
+        break;
+    case WA_OPEN_WINLISTMENU:
+        OpenSwitchMenu(scr, event->xbutton.x_root, event->xbutton.y_root, False);
+        if (scr->switch_menu) {
+            if (scr->switch_menu->brother->flags.mapped)
+                event->xbutton.window = scr->switch_menu->brother->frame->core->window;
+            else
+                event->xbutton.window = scr->switch_menu->frame->core->window;
+        }
+        break;
+    default:
+        break;
+    }
+}
+
 
 /* bindable */
 static void
@@ -595,11 +629,10 @@ handleButtonPress(XEvent *event)
 {
     WObjDescriptor *desc;
     WScreen *scr;
-
-#ifdef DEBUG    
-    puts("got button press");
+    
+#ifdef DEBUG
+    L("got button press");
 #endif
-
     scr = wScreenForRootWindow(event->xbutton.root);
     
 #ifdef BALLOON_TEXT
@@ -608,50 +641,31 @@ handleButtonPress(XEvent *event)
 
     
 #ifndef LITE
-    if (event->xbutton.window==scr->root_win) {
-
+    if (event->xbutton.window==scr->root_win) {	
+        if (event->xbutton.button==Button1 &&
+            wPreferences.mouse_button1!=WA_NONE) {
+            executeButtonAction(scr, event, wPreferences.mouse_button1);
+        } else if (event->xbutton.button==Button2 &&
+                   wPreferences.mouse_button2!=WA_NONE) {
+            executeButtonAction(scr, event, wPreferences.mouse_button2);
+        } else if (event->xbutton.button==Button3 &&
+                   wPreferences.mouse_button3!=WA_NONE) {
+            executeButtonAction(scr, event, wPreferences.mouse_button3);
+        } else if (event->xbutton.button==Button4 &&
+                   wPreferences.mouse_wheel!=WA_NONE) {
+	    wWorkspaceRelativeChange(scr, 1);
+        } else if (event->xbutton.button==Button5 &&
+                   wPreferences.mouse_wheel!=WA_NONE) {
+            wWorkspaceRelativeChange(scr, -1);
+        }
 #ifdef GNOME_STUFF
-	if (wGNOMEProxyizeButtonEvent(scr, event))
+	else if (wGNOMEProxyizeButtonEvent(scr, event))
 	    return;
 #endif
-
-	if (event->xbutton.button==wPreferences.menu_button) {
-	    OpenRootMenu(scr, event->xbutton.x_root,
-                         event->xbutton.y_root, False);
-	    /* ugly hack */
-	    if (scr->root_menu) {
-		if (scr->root_menu->brother->flags.mapped)
-		  event->xbutton.window = scr->root_menu->brother->frame->core->window;
-		else
-		  event->xbutton.window = scr->root_menu->frame->core->window;
-	    }
-	} else if (event->xbutton.button==wPreferences.windowl_button) {
-	    OpenSwitchMenu(scr, event->xbutton.x_root,
-			   event->xbutton.y_root, False);
-	    if (scr->switch_menu) {
-		if (scr->switch_menu->brother->flags.mapped)
-		  event->xbutton.window = scr->switch_menu->brother->frame->core->window;
-		else
-		  event->xbutton.window = scr->switch_menu->frame->core->window;
-	    }
-	} else if (event->xbutton.button==wPreferences.select_button) {
-	    wUnselectWindows(scr);
-	    wSelectWindows(scr, event);
-	}
-#ifdef MOUSE_WS_SWITCH
-	else if (event->xbutton.button==Button5) {
-
-	    wWorkspaceRelativeChange(scr, -1);
-
-	} else if (event->xbutton.button==Button4) {
-
-	    wWorkspaceRelativeChange(scr, 1);
-
-	}
-#endif /* MOUSE_WS_SWITCH */
     }
 #endif /* !LITE */
 
+    desc = NULL;
     if (XFindContext(dpy, event->xbutton.subwindow, wWinContext,
 		     (XPointer *)&desc)==XCNOENT) {
 	if (XFindContext(dpy, event->xbutton.window, wWinContext,
@@ -660,10 +674,6 @@ handleButtonPress(XEvent *event)
 	}
     }
 
-    if (desc->handle_mousedown!=NULL) {
-	(*desc->handle_mousedown)(desc, event);
-    }
-
     if (desc->parent_type == WCLASS_WINDOW) {
 	XSync(dpy, 0);
     
@@ -671,12 +681,12 @@ handleButtonPress(XEvent *event)
 	    XAllowEvents(dpy, AsyncPointer, CurrentTime);
 	}
      
-	if (wPreferences.focus_mode == WKF_CLICK) {
+/*	if (wPreferences.focus_mode == WKF_CLICK) {*/
 	    if (wPreferences.ignore_focus_click) {
 		XAllowEvents(dpy, AsyncPointer, CurrentTime);
 	    }
 	    XAllowEvents(dpy, ReplayPointer, CurrentTime);
-	}
+/*	}*/
 	XSync(dpy, 0);
     } else if (desc->parent_type == WCLASS_APPICON
 	       || desc->parent_type == WCLASS_MINIWINDOW
@@ -688,6 +698,10 @@ handleButtonPress(XEvent *event)
 	}
     }
 
+    if (desc->handle_mousedown!=NULL) {
+	(*desc->handle_mousedown)(desc, event);
+    }
+
     /* save double-click information */
     if (scr->flags.next_click_is_not_double) {
 	scr->flags.next_click_is_not_double = 0;
@@ -703,11 +717,9 @@ static void
 handleMapNotify(XEvent *event)
 {
     WWindow *wwin;
-
 #ifdef DEBUG
-    puts("got map");
+    L("got map");
 #endif
-
     wwin = wWindowFor(event->xmap.event);
     if (wwin && wwin->client_win == event->xmap.event) {
 	if (wwin->flags.miniaturized) {
@@ -728,11 +740,9 @@ handleUnmapNotify(XEvent *event)
     WWindow *wwin;
     XEvent ev;
     Bool withdraw = False;
-
 #ifdef DEBUG
-    puts("got unmap");
+    L("got unmap");
 #endif
-
     /* only process windows with StructureNotify selected 
      * (ignore SubstructureNotify) */
     wwin = wWindowFor(event->xunmap.window);
@@ -782,9 +792,8 @@ static void
 handleConfigureRequest(XEvent *event)
 {
     WWindow *wwin;
-
 #ifdef DEBUG
-    puts("got configure request");
+    L("got configure request");
 #endif
     if (!(wwin=wWindowFor(event->xconfigurerequest.window))) {
 	/*
@@ -806,9 +815,8 @@ handlePropertyNotify(XEvent *event)
     int ji;
     unsigned int ju;
     WScreen *scr;
-
 #ifdef DEBUG
-    puts("got property notify");
+    L("got property notify");
 #endif
     if ((wwin=wWindowFor(event->xproperty.window))) {
 	if (!XGetGeometry(dpy, wwin->client_win, &jr, &ji, &ji,
@@ -836,9 +844,8 @@ handleClientMessage(XEvent *event)
 {
     WWindow *wwin;
     WObjDescriptor *desc;
-    
 #ifdef DEBUG
-    puts("got client message");
+    L("got client message");
 #endif
     /* handle transition from Normal to Iconic state */
     if (event->xclient.message_type == _XA_WM_CHANGE_STATE
@@ -861,6 +868,10 @@ handleClientMessage(XEvent *event)
 	} else {		       /* stopping */
 	    wColormapAllowClientInstallation(scr, False);
 	}
+    } else if (event->xclient.message_type == _XA_WINDOWMAKER_COMMAND) {
+
+	wDefaultsCheckDomains("bla");
+
     } else if (event->xclient.message_type == _XA_WINDOWMAKER_WM_FUNCTION) {
 	WApplication *wapp;
 	int done=0;
@@ -892,6 +903,34 @@ handleClientMessage(XEvent *event)
 		}
 	    }
 	}
+    } else if (event->xclient.message_type == _XA_GNUSTEP_WM_ATTR) {
+	wwin = wWindowFor(event->xclient.window);
+	if (!wwin) return;
+	switch (event->xclient.data.l[0]) {
+	 case GSWindowLevelAttr:
+	   {
+	     int level = (int)event->xclient.data.l[1];
+
+	     if (WINDOW_LEVEL(wwin) != level) {
+	       ChangeStackingLevel(wwin->frame->core, level);
+	     }
+	   }
+	   break;
+	}
+    } else if (event->xclient.message_type == _XA_GNUSTEP_TITLEBAR_STATE) {
+	wwin = wWindowFor(event->xclient.window);
+	if (!wwin) return;
+	switch (event->xclient.data.l[0]) {
+	 case WMTitleBarNormal:
+	    wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
+	    break;
+	 case WMTitleBarMain:
+	    wFrameWindowChangeState(wwin->frame, WS_PFOCUSED);
+	    break;
+	 case WMTitleBarKey:
+	    wFrameWindowChangeState(wwin->frame, WS_FOCUSED);
+	    break;
+	}
 #ifdef GNOME_STUFF
     } else if (wGNOMEProcessClientMessage(&event->xclient)) {
 	/* do nothing */
@@ -900,10 +939,10 @@ handleClientMessage(XEvent *event)
     } else if (wKWMProcessClientMessage(&event->xclient)) {
 	/* do nothing */
 #endif /* KWM_HINTS */
-#ifdef XDE_DND
-    } else if (wXDEProcessClientMessage(&event->xclient)) {
+#ifdef XDND
+    } else if (wXDNDProcessClientMessage(&event->xclient)) {
 	/* do nothing */
-#endif /* XDE_DND */
+#endif /* XDND */
 #ifdef OFFIX_DND
     } else if (event->xclient.message_type==_XA_DND_PROTOCOL) {
 	WScreen *scr = wScreenForWindow(event->xclient.window);
@@ -952,7 +991,7 @@ raiseWindow(WScreen *scr)
     if (!wwin)
 	return;
 
-    if (!wwin->flags.destroyed) {
+    if (!wwin->flags.destroyed && wwin->flags.focused) {
 	wRaiseFrame(wwin->frame->core);
 	/* this is needed or a race condition will occur */
 	XSync(dpy, False);
@@ -967,10 +1006,33 @@ handleEnterNotify(XEvent *event)
     WObjDescriptor *desc = NULL;
     XEvent ev;
     WScreen *scr = wScreenForRootWindow(event->xcrossing.root);
-
-
 #ifdef DEBUG
-    puts("got enter notify");
+    L("got enter notify");
+#endif
+
+#ifdef VIRTUAL_DESKTOP
+    /* TODO: acceleration code */
+    if (wPreferences.vedge_thickness) {
+        int x,y;
+        if (event->xcrossing.window == scr->virtual_edge_r) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, scr->scr_width - wPreferences.vedge_thickness - 1, event->xcrossing.y_root);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x + VIRTUALEDGE_SCROLL_HSTEP, y);
+        } else if (event->xcrossing.window == scr->virtual_edge_l) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, wPreferences.vedge_thickness + 1, event->xcrossing.y_root);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x - VIRTUALEDGE_SCROLL_HSTEP, y);
+        } else if (event->xcrossing.window == scr->virtual_edge_u) {
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, wPreferences.vedge_thickness + 1);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y - VIRTUALEDGE_SCROLL_VSTEP);
+        } else if (event->xcrossing.window == scr->virtual_edge_d) {
+            printf("enter bottom\n");
+            XWarpPointer(dpy, None, scr->root_win, 0,0,0,0, event->xcrossing.x_root, scr->scr_height - wPreferences.vedge_thickness - 1);
+            wWorkspaceGetViewPosition(scr, scr->current_workspace, &x, &y);
+            wWorkspaceSetViewPort(scr, scr->current_workspace, x, y + VIRTUALEDGE_SCROLL_VSTEP);
+        }
+    }
 #endif
 
     if (XCheckTypedWindowEvent(dpy, event->xcrossing.window, LeaveNotify,
@@ -1014,16 +1076,16 @@ handleEnterNotify(XEvent *event)
 	 * is for the frame window and window doesn't have focus yet */
 	if ((wPreferences.focus_mode==WKF_POINTER
 	     || wPreferences.focus_mode==WKF_SLOPPY)
-		&& wwin->frame->core->window==event->xcrossing.window
+	    && wwin->frame->core->window==event->xcrossing.window
 	    && !scr->flags.doing_alt_tab) {
-
-		if (!wwin->flags.focused)
+	    
+	    if (!wwin->flags.focused && !WFLAGP(wwin, no_focusable))
 		wSetFocusTo(scr, wwin);
-
-		if (scr->autoRaiseTimer)
-			WMDeleteTimerHandler(scr->autoRaiseTimer);
-		scr->autoRaiseTimer = NULL;
-
+	    
+	    if (scr->autoRaiseTimer)
+		WMDeleteTimerHandler(scr->autoRaiseTimer);
+	    scr->autoRaiseTimer = NULL;
+	    
 	    if (wPreferences.raise_delay && !WFLAGP(wwin, no_focusable)) {
 		scr->autoRaiseWindow = wwin->frame->core->window;
 		scr->autoRaiseTimer
@@ -1073,7 +1135,7 @@ static void
 handleLeaveNotify(XEvent *event)
 {
     WObjDescriptor *desc = NULL;
-    
+
     if (XFindContext(dpy, event->xcrossing.window, wWinContext, 
 		     (XPointer *)&desc)!=XCNOENT) {
         if(desc->handle_leavenotify)
@@ -1098,11 +1160,9 @@ handleShapeNotify(XEvent *event)
     XShapeEvent *shev = (XShapeEvent*)event;
     WWindow *wwin;
     XEvent ev;
-
 #ifdef DEBUG
-    puts("got shape notify");
+    L("got shape notify");
 #endif
-
     while (XCheckTypedWindowEvent(dpy, shev->window, event->type, &ev)) {
 	XShapeEvent *sev = (XShapeEvent*)&ev;
 
@@ -1133,6 +1193,33 @@ handleShapeNotify(XEvent *event)
 }
 #endif /* SHAPE */
 
+#ifdef KEEP_XKB_LOCK_STATUS   
+/* please help ]d if you know what to do */
+handleXkbIndicatorStateNotify(XEvent *event)
+{
+    WWindow *wwin;
+    WScreen *scr;
+    XkbStateRec staterec;
+    int i;
+
+    for (i=0; i<wScreenCount; i++) {
+        scr = wScreenWithNumber(i);
+	wwin = scr->focused_window;
+	if (wwin && wwin->flags.focused) {
+        XkbGetState(dpy,XkbUseCoreKbd,&staterec);
+        if (wwin->frame->languagemode != staterec.group) {
+            wwin->frame->last_languagemode = wwin->frame->languagemode;
+            wwin->frame->languagemode = staterec.group;
+        }
+#ifdef XKB_BUTTON_HINT
+        if (wwin->frame->titlebar) {
+            wFrameWindowPaint(wwin->frame);
+        }
+#endif
+	}
+    }
+}
+#endif /*KEEP_XKB_LOCK_STATUS*/
 
 static void 
 handleColormapNotify(XEvent *event)
@@ -1227,158 +1314,6 @@ windowUnderPointer(WScreen *scr)
     return NULL;
 }
 
-#ifdef WEENDOZE_CYCLE
-
-static WWindow*
-nextToFocusAfter(WWindow *wwin)
-{
-    WWindow *tmp = wwin->next;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    tmp = wwin;
-    /* start over from the beginning of the list */
-    while (tmp->prev)
-	tmp = tmp->prev;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->next;
-    }
-
-    return wwin;
-}
-
-
-static WWindow*
-nextToFocusBefore(WWindow *wwin)
-{
-    WWindow *tmp = wwin->prev;
-
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    /* start over from the beginning of the list */
-    tmp = wwin;
-    while (tmp->next)
-	tmp = tmp->next;
-
-    while (tmp && tmp != wwin) {
-	if (wWindowCanReceiveFocus(tmp) && !WFLAGP(tmp, skip_window_list)) {
-
-	    return tmp;
-	}
-	tmp = tmp->prev;
-    }
-
-    return wwin;
-}
-
-static void
-doWindozeCycle(WWindow *wwin, XEvent *event, Bool next)
-{
-    WScreen *scr = wScreenForRootWindow(event->xkey.root);
-    Bool done = False;
-    WWindow *newFocused;
-    WWindow *oldFocused;
-    int modifiers;
-    XModifierKeymap *keymap;
-
-    if (!wwin)
-	return;
-
-    puts("IN");
-    keymap = XGetModifierMapping(dpy);
-
-    
-    XGrabKeyboard(dpy, scr->root_win, False, GrabModeAsync, GrabModeAsync, 
-		  CurrentTime);
-
-    if (next) {
-	newFocused = nextToFocusAfter(wwin);
-    } else {
-	newFocused = nextToFocusBefore(wwin); 
-    }
-
-    scr->flags.doing_alt_tab = 1;
-
-    wWindowFocus(newFocused, scr->focused_window);
-    oldFocused = newFocused;
-
-    OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);
-
-    while (!done) {
-	XEvent ev;
-
-	WMMaskEvent(dpy,KeyPressMask|KeyReleaseMask|ExposureMask, &ev);
-/*	WMNextEvent(dpy, &ev);*/
-	if (ev.type != KeyRelease && ev.type != KeyPress) {
-	    WMHandleEvent(&ev);
-	    continue;
-	}
-puts("EV");
-	/* ignore CapsLock */
-	modifiers = ev.xkey.state & ValidModMask;
-
-	if (ev.type == KeyPress
-	    && wKeyBindings[WKBD_FOCUSNEXT].keycode == ev.xkey.keycode
-	    && wKeyBindings[WKBD_FOCUSNEXT].modifier == modifiers) {
-
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	    newFocused = nextToFocusAfter(newFocused);
-	    wWindowFocus(newFocused, oldFocused);
-	    oldFocused = newFocused;
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-
-	} else if (ev.type == KeyPress
-		   && wKeyBindings[WKBD_FOCUSPREV].keycode == ev.xkey.keycode
-		   && wKeyBindings[WKBD_FOCUSPREV].modifier == modifiers) {
-
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	    newFocused = nextToFocusBefore(newFocused);
-	    wWindowFocus(newFocused, oldFocused);
-	    oldFocused = newFocused;
-	    UpdateSwitchMenu(scr, newFocused, ACTION_CHANGE_STATE);
-	}
-	if (ev.type == KeyRelease) {
-	    int i;
-
-	    for (i = 0; i <= 8 * keymap->max_keypermod; i++) {
-		if (keymap->modifiermap[i] == ev.xkey.keycode &&
-		    wKeyBindings[WKBD_FOCUSNEXT].modifier 
-		    & 1<<(i/keymap->max_keypermod)) {
-		    done = True;
-		    break;
-		}
-	    }
-	}
-    }
-puts("OUT");
-    XFree(keymap);
-
-    XUngrabKeyboard(dpy, CurrentTime);
-    wSetFocusTo(scr, newFocused);
-    scr->flags.doing_alt_tab = 0; 
-    OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, False);   
-}
-
-
-#endif /* WEENDOZE_CYCLE */
-
 
 
 
@@ -1389,7 +1324,7 @@ handleKeyPress(XEvent *event)
     WWindow *wwin = scr->focused_window;
     int i;
     int modifiers;
-    int command=-1;
+    int command=-1, index;
 #ifdef KEEP_XKB_LOCK_STATUS   
     XkbStateRec staterec;     
 #endif /*KEEP_XKB_LOCK_STATUS*/
@@ -1408,6 +1343,7 @@ handleKeyPress(XEvent *event)
 	    break;
 	}
     }
+        
     
     if (command < 0) {
 #ifdef LITE
@@ -1444,11 +1380,13 @@ handleKeyPress(XEvent *event)
     switch (command) {
 #ifndef LITE
      case WKBD_ROOTMENU:
-	OpenRootMenu(scr, event->xkey.x_root, event->xkey.y_root, True);
-	break;
+	/*OpenRootMenu(scr, event->xkey.x_root, event->xkey.y_root, True);*/
+        OpenRootMenu(scr, scr->scr_width/2, scr->scr_height/2, True);
+        break;
      case WKBD_WINDOWLIST:
-	OpenSwitchMenu(scr, event->xkey.x_root, event->xkey.y_root, True);
-	break;
+	/*OpenSwitchMenu(scr, event->xkey.x_root, event->xkey.y_root, True);*/
+        OpenSwitchMenu(scr, scr->scr_width/2, scr->scr_height/2, True);
+        break;
 #endif /* !LITE */
      case WKBD_WINDOWMENU:
 	if (ISMAPPED(wwin) && ISFOCUSED(wwin))
@@ -1551,35 +1489,11 @@ handleKeyPress(XEvent *event)
 	}
         break;
      case WKBD_FOCUSNEXT:
-#ifdef WEENDOZE_CYCLE
-	if (wPreferences.windoze_cycling) {
-	    doWindozeCycle(wwin, event, True);
-	} else
-#endif /* WEENDOZE_CYCLE */
-	{
-	    wwin = NextFocusWindow(scr);
-	    if (wwin != NULL) {
-		wSetFocusTo(scr, wwin);
-		if (wPreferences.circ_raise)
-		    wRaiseFrame(wwin->frame->core);
-	    }
-	}
+	StartWindozeCycle(wwin, event, True);
 	break;
 
      case WKBD_FOCUSPREV:
-#ifdef WEENDOZE_CYCLE
-	if (wPreferences.windoze_cycling) {
-	    doWindozeCycle(wwin, event, False);
-	} else 
-#endif /* WEENDOZE_CYCLE */
-	{
-	    wwin = PrevFocusWindow(scr);
-	    if (wwin != NULL) {
-		wSetFocusTo(scr, wwin);
-		if (wPreferences.circ_raise)
-		    wRaiseFrame(wwin->frame->core);
-	    }
-	}
+	StartWindozeCycle(wwin, event, False);
 	break;
 
 #if (defined(__STDC__) && !defined(UNIXCPP)) || defined(ANSICPP)
@@ -1616,25 +1530,100 @@ handleKeyPress(XEvent *event)
      case WKBD_WINDOW2:
      case WKBD_WINDOW3:
      case WKBD_WINDOW4:
-#ifdef EXTEND_WINDOWSHORTCUT
      case WKBD_WINDOW5:
      case WKBD_WINDOW6:
      case WKBD_WINDOW7:
      case WKBD_WINDOW8:
      case WKBD_WINDOW9:
      case WKBD_WINDOW10:
-#endif
-        if (scr->shortcutWindow[command-WKBD_WINDOW1]) {
-            wMakeWindowVisible(scr->shortcutWindow[command-WKBD_WINDOW1]);
+
+	index = command-WKBD_WINDOW1;
+	
+        if (scr->shortcutWindows[index]) {
+            WMArray *list = scr->shortcutWindows[index];
+            int cw;
+	    int count = WMGetArrayItemCount(list);
+	    WWindow *twin;
+	    WMArrayIterator iter;
+	    WWindow *wwin;
+
+            wUnselectWindows(scr);
+            cw = scr->current_workspace;
+
+            WM_ETARETI_ARRAY(list, wwin, iter) {
+		if (count > 1)
+		    wWindowChangeWorkspace(wwin, cw);
+
+                wMakeWindowVisible(wwin);
+
+		if (count > 1)
+		    wSelectWindow(wwin, True);
+            }
+	    
+	    /* rotate the order of windows, to create a cycling effect */
+	    twin = WMGetFromArray(list, 0);
+	    WMDeleteFromArray(list, 0);
+	    WMAddToArray(list, twin);
+
         } else if (wwin && ISMAPPED(wwin) && ISFOCUSED(wwin)) {
-            scr->shortcutWindow[command-WKBD_WINDOW1] = wwin;
-	    wSelectWindow(wwin, !wwin->flags.selected);
-	    XFlush(dpy);
-	    wusleep(3000);
-	    wSelectWindow(wwin, !wwin->flags.selected);
-	    XFlush(dpy);
+            if (scr->shortcutWindows[index]) {
+                WMFreeArray(scr->shortcutWindows[index]);
+                scr->shortcutWindows[index] = NULL;
+            }
+
+            if (wwin->flags.selected && scr->selected_windows) {
+                scr->shortcutWindows[index] =
+                    WMDuplicateArray(scr->selected_windows);
+                /*WMRemoveFromArray(scr->shortcutWindows[index], wwin);
+                WMInsertInArray(scr->shortcutWindows[index], 0, wwin);*/
+            } else {
+                scr->shortcutWindows[index] = WMCreateArray(4);
+                WMAddToArray(scr->shortcutWindows[index], wwin);
+            }
+
+            wSelectWindow(wwin, !wwin->flags.selected);
+            XFlush(dpy);
+            wusleep(3000);
+            wSelectWindow(wwin, !wwin->flags.selected);
+            XFlush(dpy);
+
+        } else if (scr->selected_windows 
+		   && WMGetArrayItemCount(scr->selected_windows)) {
+
+            if (wwin->flags.selected && scr->selected_windows) {
+                if (scr->shortcutWindows[index]) {
+                    WMFreeArray(scr->shortcutWindows[index]);
+                }
+                scr->shortcutWindows[index] =
+                    WMDuplicateArray(scr->selected_windows);
+            }
         }
+
         break;
+	
+     case WKBD_SWITCH_SCREEN:
+	if (wScreenCount > 1) {
+	    WScreen *scr2;
+	    int i;
+	    
+	    /* find index of this screen */
+	    for (i = 0; i < wScreenCount; i++) {
+		if (wScreenWithNumber(i) == scr)
+		    break;
+	    }
+	    i++;
+	    if (i >= wScreenCount) {
+		i = 0;
+	    }
+	    scr2 = wScreenWithNumber(i);
+	    
+	    if (scr2) {
+		XWarpPointer(dpy, scr->root_win, scr2->root_win, 0, 0, 0, 0,
+			     scr2->scr_width/2, scr2->scr_height/2);
+	    }
+	}
+	break;
+	
      case WKBD_NEXTWSLAYER:
      case WKBD_PREVWSLAYER:
 	{
@@ -1666,11 +1655,20 @@ handleKeyPress(XEvent *event)
         break;
 #ifdef KEEP_XKB_LOCK_STATUS
      case WKBD_TOGGLE:
-        if(wPreferences.modelock){
-	    XkbGetState(dpy,XkbUseCoreKbd,&staterec);
-	    /*toggle*/
-	    XkbLockGroup(dpy,XkbUseCoreKbd,
-			 wwin->languagemode=staterec.compat_state&32?0:1);
+        if(wPreferences.modelock) {
+            /*toggle*/
+            wwin = scr->focused_window;
+
+            if (wwin && wwin->flags.mapped
+                && wwin->frame->workspace == wwin->screen_ptr->current_workspace
+                && !wwin->flags.miniaturized && !wwin->flags.hidden) {
+                XkbGetState(dpy,XkbUseCoreKbd,&staterec);
+
+                wwin->frame->languagemode = wwin->frame->last_languagemode;
+                wwin->frame->last_languagemode = staterec.group;
+                XkbLockGroup(dpy,XkbUseCoreKbd, wwin->frame->languagemode);
+                                 
+            }
         }
         break;
 #endif /* KEEP_XKB_LOCK_STATUS */
@@ -1684,18 +1682,16 @@ handleMotionNotify(XEvent *event)
 {
     WMenu *menu;
     WScreen *scr = wScreenForRootWindow(event->xmotion.root);
-    WWindow *wwin;
 
     if (wPreferences.scrollable_menus) {
-        if (event->xmotion.x_root <= 1 ||
+        if (scr->flags.jump_back_pending ||
+            event->xmotion.x_root <= 1 ||
             event->xmotion.x_root >= (scr->scr_width - 2) ||
             event->xmotion.y_root <= 1 ||
             event->xmotion.y_root >= (scr->scr_height - 2)) {
-
 #ifdef DEBUG
-            puts("pointer at screen edge");
+            L("pointer at screen edge");
 #endif
-
             menu = wMenuUnderPointer(scr);
             if (menu!=NULL)
                 wMenuScroll(menu, event);
