@@ -66,9 +66,6 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
 		   int width, int height, int *clearance, int flags,
 		   WTexture **title_texture, WTexture **resize_texture,
 		   unsigned long *color,
-#ifdef DRAWSTRING_PLUGIN
-           int function_offset,
-#endif
            GC *gc, WMFont **font)
 {
     WFrameWindow *fwin;
@@ -84,9 +81,6 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
     fwin->resizebar_texture = resize_texture;
     fwin->title_pixel = color;
     fwin->title_clearance = clearance;
-#ifdef DRAWSTRING_PLUGIN
-    fwin->drawstring_proc_offset = function_offset;
-#endif
     fwin->title_gc = gc;
     fwin->font = font;
 #ifdef KEEP_XKB_LOCK_STATUS
@@ -440,7 +434,7 @@ wFrameWindowDestroy(WFrameWindow *fwin)
     wCoreDestroy(fwin->core);
     
     if (fwin->title)
-	free(fwin->title);
+	wfree(fwin->title);
     
     for (i=0; i < (fwin->flags.single_texture ? 1 : 3); i++) {
 	FREE_PIXMAP(fwin->title_back[i]);
@@ -453,7 +447,7 @@ wFrameWindowDestroy(WFrameWindow *fwin)
         }
     }
     
-    free(fwin);
+    wfree(fwin);
 }
 
 
@@ -679,7 +673,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	    }
 	    x += limg->width;
 	    w -= limg->width;
-	    RDestroyImage(limg);
+	    RReleaseImage(limg);
 	}
 
 #ifdef XKB_BUTTON_HINT
@@ -690,7 +684,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	    }
 	    x += timg->width;
 	    w -= timg->width;
-	    RDestroyImage(timg);
+	    RReleaseImage(timg);
     }
 #endif
 	
@@ -705,7 +699,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 		wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
 	    }
 	    w -= rimg->width;
-	    RDestroyImage(rimg);
+	    RReleaseImage(rimg);
 	}
 	
 	if (w!=width) {
@@ -715,7 +709,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	    if (!RConvertImage(scr->rcontext, mimg, title)) {
 		wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
 	    }
-	    RDestroyImage(mimg);
+	    RReleaseImage(mimg);
 	} else {
 	    RBevelImage(img, RBEV_RAISED2);
 	    
@@ -731,7 +725,7 @@ renderTexture(WScreen *scr, WTexture *texture, int width, int height,
 	}
     }
 
-    RDestroyImage(img);
+    RReleaseImage(img);
 }
 
 
@@ -783,7 +777,7 @@ renderResizebarTexture(WScreen *scr, WTexture *texture, int width, int height,
 	wwarning(_("error rendering image: %s"), RMessageForError(RErrorCode));
     }
 
-    RDestroyImage(img);
+    RReleaseImage(img);
 }
 
 
@@ -977,15 +971,15 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	
 	updateTexture(fwin);
     }
-    
+
     if (fwin->titlebar && !fwin->flags.repaint_only_resizebar
 	&& fwin->title_texture[fwin->flags.state]->any.type==WTEX_SOLID) {
-	wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
-		   fwin->titlebar->height,
-		   (WTexSolid*)fwin->title_texture[fwin->flags.state], 
-		   WREL_RAISED);
+        wDrawBevel(fwin->titlebar->window, fwin->titlebar->width,
+                fwin->titlebar->height,
+                (WTexSolid*)fwin->title_texture[fwin->flags.state], 
+                WREL_RAISED);
     }
-    
+
     if (fwin->resizebar	&& !fwin->flags.repaint_only_titlebar
 	&& fwin->resizebar_texture[0]->any.type == WTEX_SOLID) {
 	Window win;
@@ -993,14 +987,14 @@ wFrameWindowPaint(WFrameWindow *fwin)
 	int cw;
 	GC light_gc, dim_gc;
 	WTexSolid *texture = (WTexSolid*)fwin->resizebar_texture[0];
-	
+
 	w = fwin->resizebar->width;
 	h = fwin->resizebar->height;
 	cw = fwin->resizebar_corner_width;
 	light_gc = texture->light_gc;
 	dim_gc = texture->dim_gc;
 	win = fwin->resizebar->window;
-	
+
 	XDrawLine(dpy, win, dim_gc, 0, 0, w, 0);
 	XDrawLine(dpy, win, light_gc, 0, 1, w, 1);
 	
@@ -1020,11 +1014,10 @@ wFrameWindowPaint(WFrameWindow *fwin)
     }
     
     
-    if (fwin->title && fwin->titlebar && !fwin->flags.repaint_only_resizebar) {
+    if (fwin->titlebar && !fwin->flags.repaint_only_resizebar) {
         int x, w;
         int lofs = 6, rofs = 6;
         int titlelen;
-        char *title;
         int allButtons = 1;
 
 
@@ -1055,52 +1048,40 @@ wFrameWindowPaint(WFrameWindow *fwin)
             scr->b_pixmaps[WBUT_XKBGROUP1 + fwin->languagemode];
 #endif
 
-	title = ShrinkString(*fwin->font, fwin->title,
-			     fwin->titlebar->width - lofs - rofs);
-	titlelen = strlen(title);
-	w = WMWidthOfString(*fwin->font, title, titlelen);
-
-	switch (fwin->flags.justification) {
-	 case WTJ_LEFT:
-	    x = lofs;
-	    break;
-
-	 case WTJ_RIGHT:
-	    x = fwin->titlebar->width - w - rofs;
-	    break;
-
-	 default:
-	    if (!allButtons)
-		x = lofs + (fwin->titlebar->width - w - lofs - rofs) / 2;
-	    else
-		x = (fwin->titlebar->width - w) / 2;
-	    break;
+	if (fwin->title) {
+	    char *title;
+	    
+	    title = ShrinkString(*fwin->font, fwin->title,
+				 fwin->titlebar->width - lofs - rofs);
+	    titlelen = strlen(title);
+	    w = WMWidthOfString(*fwin->font, title, titlelen);
+	    
+	    switch (fwin->flags.justification) {
+	     case WTJ_LEFT:
+		x = lofs;
+		break;
+		
+	     case WTJ_RIGHT:
+		x = fwin->titlebar->width - w - rofs;
+		break;
+		
+	     default:
+		if (!allButtons)
+		    x = lofs + (fwin->titlebar->width - w - lofs - rofs) / 2;
+		else
+		    x = (fwin->titlebar->width - w) / 2;
+		break;
+	    }
+	    
+	    XSetForeground(dpy, *fwin->title_gc, 
+			   fwin->title_pixel[fwin->flags.state]);
+	    
+	    WMDrawString(scr->wmscreen, fwin->titlebar->window, 
+			 *fwin->title_gc, *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE, 
+			 title, titlelen);
+	    
+	    wfree(title);
 	}
-
-
-	XSetForeground(dpy, *fwin->title_gc, 
-		       fwin->title_pixel[fwin->flags.state]);
-	
-#ifdef DRAWSTRING_PLUGIN
-    if (scr->drawstring_func[fwin->flags.state + fwin->drawstring_proc_offset]) {
-        scr->drawstring_func[fwin->flags.state + fwin->drawstring_proc_offset]->
-            proc.drawString(scr->drawstring_func[fwin->flags.state 
-                    + fwin->drawstring_proc_offset]->arg,
-                    fwin->titlebar->window, *fwin->title_gc,
-                    *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE,
-                    fwin->titlebar->width, fwin->top_width, title, titlelen);
-    } else {
-        WMDrawString(scr->wmscreen, fwin->titlebar->window, 
-                *fwin->title_gc, *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE, 
-                title, titlelen);
-    }
-#else
-    WMDrawString(scr->wmscreen, fwin->titlebar->window, 
-            *fwin->title_gc, *fwin->font, x, *fwin->title_clearance + TITLEBAR_EXTEND_SPACE, 
-            title, titlelen);
-#endif /* DRAWSTRING_PLUGIN */
-
-	free(title);
 	
 	if (fwin->left_button)
 	    handleButtonExpose(&fwin->left_button->descriptor, NULL);
@@ -1247,7 +1228,7 @@ wFrameWindowChangeTitle(WFrameWindow *fwin, char *new_title)
     }
 
     if (fwin->title)
-	free(fwin->title);
+	wfree(fwin->title);
     
     fwin->title = wstrdup(new_title);
     
