@@ -61,6 +61,7 @@
 #define WIN_HINTS_SKIP_TASKBAR    (1<<2) /*do not show on taskbar*/
 #define WIN_HINTS_GROUP_TRANSIENT (1<<3) /*Reserved - definition is unclear*/
 #define WIN_HINTS_FOCUS_ON_CLICK  (1<<4) /*app only accepts focus if clicked*/
+#define WIN_HINTS_DO_NOT_COVER    (1<<5) /* attempt to not cover this window */
 
 
 #define WIN_STATE_STICKY          (1<<0) /*everyone knows sticky*/
@@ -201,7 +202,8 @@ wGNOMEUpdateClientListHint(WScreen *scr)
     count = 0;
     wwin = scr->focused_window;
     while (wwin) {
-	if (!wwin->flags.internal_window) {
+        if (!wwin->flags.internal_window &&
+            !wwin->client_flags.skip_window_list) {
 
 	    windows[count++] = wwin->client_win;
 	}
@@ -212,7 +214,7 @@ wGNOMEUpdateClientListHint(WScreen *scr)
     XChangeProperty(dpy, scr->root_win, _XA_WIN_CLIENT_LIST, XA_CARDINAL, 32,
 		    PropModeReplace, (unsigned char *)windows, count);
 
-    free(windows);
+    wfree(windows);
     XFlush(dpy);
 }
 
@@ -288,7 +290,7 @@ getWindowLevel(int layer)
 }
 
 
-void
+Bool
 wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
 {
     Atom type_ret;
@@ -296,6 +298,7 @@ wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
     unsigned long nitems_ret;
     unsigned long bytes_after_ret;
     long flags, val, *data = 0;
+    Bool hasHints = False;
 
     /* hints */
     
@@ -308,9 +311,37 @@ wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
 	
 	XFree(data);
 	
-	if (flags & (WIN_HINTS_SKIP_FOCUS|WIN_HINTS_SKIP_WINLIST)) {
+	if (flags & (WIN_HINTS_SKIP_FOCUS|WIN_HINTS_SKIP_WINLIST
+				|WIN_HINTS_SKIP_TASKBAR)) {
 	    wwin->client_flags.skip_window_list = 1;
 	}
+
+	/* client reserved area, for the panel */
+	if (flags & (WIN_HINTS_DO_NOT_COVER)) {
+		WReservedArea *area;
+
+		area = malloc(sizeof(WReservedArea));
+		if (!area) {
+			wwarning(_("out of memory while updating GNOME hints"));
+		} else {
+			XWindowAttributes wattribs;
+
+			XGetWindowAttributes(dpy, wwin->client_win, &wattribs);
+			wClientGetNormalHints(wwin, &wattribs, False,
+					&area->area.x1, &area->area.y1,
+					&area->area.x2, &area->area.y2);
+			area->area.x2 = area->area.x2 + area->area.x1;
+			area->area.y2 = area->area.y2 + area->area.y1;
+
+			area->window = wwin->client_win;
+		}
+		area->next = wwin->screen_ptr->reservedAreas;
+		wwin->screen_ptr->reservedAreas = area;
+		
+		wScreenUpdateUsableArea(wwin->screen_ptr);
+	}
+
+	hasHints = True;
     }
 
     /* layer */
@@ -323,6 +354,7 @@ wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
 	XFree(data);
 
 	*layer = getWindowLevel(val);
+	hasHints = True;
     }
     
     /* workspace */
@@ -336,6 +368,7 @@ wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
 
 	if (val > 0)
 	    *workspace = val;
+	hasHints = True;
     }
 
     /* reserved area */
@@ -362,11 +395,14 @@ wGNOMECheckClientHints(WWindow *wwin, int *layer, int *workspace)
 	wwin->screen_ptr->reservedAreas = area;
 
         wScreenUpdateUsableArea(wwin->screen_ptr);
+	hasHints = True;
     }
+    
+    return hasHints;
 }
 
 
-void
+Bool
 wGNOMECheckInitialClientState(WWindow *wwin)
 {
     Atom type_ret;
@@ -379,7 +415,7 @@ wGNOMECheckInitialClientState(WWindow *wwin)
 			   XA_CARDINAL, &type_ret, &fmt_ret, &nitems_ret, 
 			   &bytes_after_ret,
 			   (unsigned char**)&data)!=Success || !data)
-	return;
+	return False;
 
     flags = *data;
 
@@ -399,6 +435,8 @@ wGNOMECheckInitialClientState(WWindow *wwin)
 
     if (flags & WIN_STATE_SHADED)
 	wwin->flags.shaded = 1;
+    
+    return True;
 }
 
 
@@ -491,7 +529,8 @@ wGNOMEProcessClientMessage(XClientMessageEvent *event)
 
 	if (mask & WIN_STATE_STICKY) {
 	    if ((flags & WIN_STATE_STICKY) != WFLAGP(wwin, omnipresent)) {
-		wwin->client_flags.omnipresent = 1;
+		wwin->client_flags.omnipresent = (flags & WIN_STATE_STICKY)!=0;
+		wGNOMEUpdateClientStateHint(wwin, False);
 		updateWindowList = True;
 	    }
 	}
@@ -576,7 +615,7 @@ wGNOMERemoveClient(WWindow *wwin)
     if (area) {
 	if (area->window == wwin->client_win) {
 	    wwin->screen_ptr->reservedAreas = area->next;
-	    free(area);
+	    wfree(area);
 	    flag = 1;
 	} else {
 	    while (area->next && area->next->window != wwin->client_win) 
@@ -586,7 +625,7 @@ wGNOMERemoveClient(WWindow *wwin)
 		WReservedArea *next;
 		
 		next = area->next->next;
-		free(area->next);
+		wfree(area->next);
 		area->next = next;
 		
 		flag = 1;
