@@ -24,6 +24,9 @@
  * 
  * Supported stuff:
  * ================
+ * 
+ * kfm icon selection from krootbgwm
+ * 
  * kwm.h function/method	Notes
  *----------------------------------------------------------------------------
  * setUnsavedDataHint() 	currently, only gives visual clue that 
@@ -115,6 +118,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
+
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+#include <signal.h>
 
 
 #include "WindowMaker.h"
@@ -130,6 +139,7 @@
 #include "actions.h"
 #include "workspace.h"
 #include "dialog.h"
+#include "stacking.h"
 
 #include "kwm.h"
 
@@ -178,6 +188,8 @@ static Atom _XA_KWM_WIN_ICONIFIED = 0;
 static Atom _XA_KWM_WIN_MAXIMIZED = 0;
 static Atom _XA_KWM_WIN_STICKY = 0;
 
+static Atom _XA_KWM_WIN_ICON_GEOMETRY = 0;
+
 static Atom _XA_KWM_CURRENT_DESKTOP = 0;
 static Atom _XA_KWM_NUMBER_OF_DESKTOPS = 0;
 static Atom _XA_KWM_DESKTOP_NAME_[MAX_WORKSPACES];
@@ -210,9 +222,12 @@ static KWMModuleList *KWMDockWindows = NULL;
 /* window decoration types */
 enum {
     KWMnoDecoration = 0, 
-	KWMnormalDecoration = 1, 
-	KWMtinyDecoration = 2,
-	KWMnoFocus = 256
+    KWMnormalDecoration = 1, 
+    KWMtinyDecoration = 2,
+    KWMnoFocus = 256,
+    KWMstandaloneMenuBar = 512, 
+    KWMdesktopIcon = 1024,
+    KWMstaysOnTop = 2048
 };
 
 
@@ -412,7 +427,7 @@ removeModule(WScreen *scr, Window window)
 	if (KWMModules->title)
 	    XFree(KWMModules->title);
 #endif
-	free(KWMModules);
+	wfree(KWMModules);
 	KWMModules = next;
     } else {
 	KWMModuleList *ptr;
@@ -426,7 +441,7 @@ removeModule(WScreen *scr, Window window)
 		if (ptr->next->title)
 		    XFree(ptr->next->title);
 #endif
-		free(ptr->next);
+		wfree(ptr->next);
 		ptr->next->next = next;
 		break;
 	    }
@@ -481,7 +496,7 @@ removeDockWindow(WScreen *scr, Window window)
 			  window);
 
 	next = KWMDockWindows->next;
-	free(KWMDockWindows);
+	wfree(KWMDockWindows);
 	KWMDockWindows = next;
 
     } else {
@@ -493,7 +508,7 @@ removeDockWindow(WScreen *scr, Window window)
 		sendClientMessage(scr, scr->kwm_dock,
 				  _XA_KWM_MODULE_DOCKWIN_REMOVE, window);
 		next = ptr->next->next;
-		free(ptr->next);
+		wfree(ptr->next);
 		ptr->next = next;
 		return;
 	    }
@@ -558,6 +573,9 @@ wKWMInitStuff(WScreen *scr)
 
 	_XA_KWM_WIN_MAXIMIZED = XInternAtom(dpy, "KWM_WIN_MAXIMIZED", False);
 
+	_XA_KWM_WIN_ICON_GEOMETRY = XInternAtom(dpy, "KWM_WIN_ICON_GEOMETRY",
+						False);
+
 	_XA_KWM_COMMAND = XInternAtom(dpy, "KWM_COMMAND", False);
 
 	_XA_KWM_ACTIVE_WINDOW = XInternAtom(dpy, "KWM_ACTIVE_WINDOW", False);
@@ -625,12 +643,11 @@ wKWMInitStuff(WScreen *scr)
 void
 wKWMSendStacking(WScreen *scr, Window module)
 {
-    int i;
+    WMBagIterator i;
     WCoreWindow *core;
 
-    for (i = 0; i < MAX_WINDOW_LEVELS; i++) {
-	for (core = scr->stacking_list[i]; core != NULL; 
-	     core = core->stacking->under) {
+    WM_ITERATE_BAG(scr->stacking_list, core, i) {
+	for (; core != NULL; core = core->stacking->under) {
 	    WWindow *wwin;
 
 	    wwin = wWindowFor(core->window);
@@ -711,7 +728,7 @@ wKWMShutdown(WScreen *scr, Bool closeModules)
 
 
 void
-wKWMCheckClientHints(WWindow *wwin, int *workspace)
+wKWMCheckClientHints(WWindow *wwin, int *layer, int *workspace)
 {
     long val;
 
@@ -732,6 +749,21 @@ wKWMCheckClientHints(WWindow *wwin, int *workspace)
 	 case KWMtinyDecoration:
 	    wwin->client_flags.no_resizebar = 1;
 	    break;
+	 case KWMstandaloneMenuBar:
+	    wwin->client_flags.no_titlebar = 1;
+	    wwin->client_flags.no_resizebar = 1;
+	    wwin->client_flags.no_resizable = 1;
+	    wwin->client_flags.skip_window_list = 1;
+	    wwin->client_flags.no_hide_others = 1;
+	    wwin->flags.kwm_menubar = 1;
+	    *layer = WMMainMenuLevel;
+	    break;
+	 case KWMdesktopIcon:
+	    *layer = WMDesktopLevel;
+	    break;
+	 case KWMstaysOnTop:
+	    *layer = WMFloatingLevel;
+	    break;
 	 case KWMnormalDecoration:
 	 default:
 	    break;
@@ -747,20 +779,34 @@ void
 wKWMCheckClientInitialState(WWindow *wwin)
 {
     long val;
+    WArea area;
 
     if (getSimpleHint(wwin->client_win, _XA_KWM_WIN_STICKY, &val) && val) {
 
 	wwin->client_flags.omnipresent = 1;
     }
-    if (getSimpleHint(wwin->client_win, _XA_KWM_WIN_ICONIFIED, &val) 
-	&& val) {
+    if (getSimpleHint(wwin->client_win, _XA_KWM_WIN_ICONIFIED, &val) && val) {
 
 	wwin->flags.miniaturized = 1;
     }
-    if (getSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED, &val) 
-	&& val) {
+    if (getSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED, &val)) {
+	if (val == 2)
+	    wwin->flags.maximized = MAX_VERTICAL;
+	else if (val == 1)
+	    wwin->flags.maximized = MAX_HORIZONTAL;
+	else if (val == 3)
+	    wwin->flags.maximized = MAX_VERTICAL|MAX_HORIZONTAL;
+    }
+    if (getAreaHint(wwin->client_win, _XA_KWM_WIN_GEOMETRY_RESTORE, &area)
+	&& (wwin->old_geometry.x != area.x1
+	    || wwin->old_geometry.y != area.y1
+	    || wwin->old_geometry.width != area.x2 - area.x1
+	    || wwin->old_geometry.height != area.y2 - area.y1)) {
 
-	wwin->flags.maximized = MAX_VERTICAL|MAX_HORIZONTAL;
+	wwin->old_geometry.x = area.x1;
+	wwin->old_geometry.y = area.y1;
+	wwin->old_geometry.width = area.x2 - area.x1;
+	wwin->old_geometry.height = area.y2 - area.y1;
     }
 }
 
@@ -796,7 +842,7 @@ wKWMCheckClientHintChange(WWindow *wwin, XPropertyEvent *event)
 	printf("got KDE sticky change\n");
 #endif
 	flag = !getSimpleHint(wwin->client_win, _XA_KWM_WIN_STICKY, 
-			     &value) || value;
+			      &value) || value;
 
 	if (flag != wwin->client_flags.omnipresent) {
 
@@ -806,17 +852,25 @@ wKWMCheckClientHintChange(WWindow *wwin, XPropertyEvent *event)
 
 	}
     } else if (event->atom == _XA_KWM_WIN_MAXIMIZED) {
+	int bla = 0;
 
 #ifdef DEBUG1
 	printf("got KDE maximize change\n");
 #endif
-	flag = !getSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED,
-			     &value) || value;
+	flag = getSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED, &value);
 
-	if (flag != (wwin->flags.maximized!=0)) {
+	if (flag) {
+	    if (value == 3)
+		bla = MAX_VERTICAL|MAX_HORIZONTAL;
+	    else if (value == 2)
+		bla = MAX_VERTICAL;
+	    else if (value == 1)
+		bla = MAX_HORIZONTAL;
+	}
 
-	    if (flag)
-		wMaximizeWindow(wwin, flag*(MAX_VERTICAL|MAX_HORIZONTAL));
+	if (bla != wwin->flags.maximized) {
+	    if (bla != 0)
+		wMaximizeWindow(wwin, bla);
 	    else
 		wUnmaximizeWindow(wwin);
 	} 
@@ -1024,7 +1078,7 @@ performCommand(WScreen *scr, char *command, XClientMessageEvent *event)
 	cmd = ExpandOptions(scr, _("%a(Run Command,Type the command to run:)"));
 	if (cmd) {
 	    ExecuteShellCommand(scr, cmd);
-	    free(cmd);
+	    wfree(cmd);
 	}
     } else if (strcmp(command, "logout")==0) {
 
@@ -1037,7 +1091,8 @@ performCommand(WScreen *scr, char *command, XClientMessageEvent *event)
     } else if (strncmp(command, "go:", 3)==0) {
 
 	Shutdown(WSRestartPreparationMode);
-	Restart(&command[3]);
+	Restart(&command[3], False);
+	Restart(NULL, True);
 
     } else if (strcmp(command, "desktop+1")==0) {
 
@@ -1091,7 +1146,7 @@ performCommand(WScreen *scr, char *command, XClientMessageEvent *event)
     } else if (strcmp(command, "wmaker:restart")==0) {
 
 	Shutdown(WSRestartPreparationMode);
-	Restart(NULL);
+	Restart(NULL, True);
 
     } else if (strcmp(command, "wmaker:exit")==0) {
 
@@ -1279,7 +1334,7 @@ wKWMCheckRootHintChange(WScreen *scr, XPropertyEvent *event)
 
 	processed = False;
 
-	for (i = 0; i < MAX_WORKSPACES; i++) {
+	for (i = 0; i < MAX_WORKSPACES && i < scr->workspace_count; i++) {
 	     if (event->atom == _XA_KWM_DESKTOP_NAME_[i]) {
 		 char *name;
 
@@ -1339,7 +1394,7 @@ wKWMManageableClient(WScreen *scr, Window win, char *title)
      */
     if (ptr && strncmp(ptr->title, title, strlen(ptr->title))==0) {
 	next = ptr->next;
-	free(ptr);
+	wfree(ptr);
 	KWMDoNotManageCrap = next;
 #ifdef DEBUG1
 	printf("window %s not managed per KDE request\n", title);
@@ -1356,7 +1411,7 @@ wKWMManageableClient(WScreen *scr, Window win, char *title)
 		printf("window %s not managed per KDE request\n", title);
 #endif
 		next = ptr->next->next;
-		free(ptr->next);
+		wfree(ptr->next);
 		ptr->next = next;
 
 		sendToModules(scr, _XA_KWM_MODULE_WIN_ADD, NULL, win);
@@ -1388,12 +1443,26 @@ void
 wKWMUpdateActiveWindowHint(WScreen *scr)
 {
     long val;
+    WWindow *wwin, *tmp;
 
     if (!scr->focused_window || !scr->focused_window->flags.focused)
 	val = None;
-    else
+    else {
 	val = (long)(scr->focused_window->client_win);
 
+	/* raise the menubar thing */
+	wwin = scr->focused_window;
+	tmp = wwin->prev;
+	while (tmp) {
+	    if (tmp->flags.kwm_menubar 
+		&& tmp->transient_for == wwin->client_win) {
+		wRaiseFrame(tmp->frame->core);
+		break;
+	    }
+	    tmp = tmp->prev;
+	}
+    }
+    
     XChangeProperty(dpy, scr->root_win, _XA_KWM_ACTIVE_WINDOW,
 		    _XA_KWM_ACTIVE_WINDOW, 32, PropModeReplace, 
 		    (unsigned char*)&val, 1);
@@ -1496,8 +1565,14 @@ wKWMUpdateClientStateHint(WWindow *wwin, WKWMStateFlag flags)
 		      IS_OMNIPRESENT(wwin));
     }
     if (flags & KWMMaximizedFlag) {
-	setSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED, 
-		      wwin->flags.maximized!=0);
+	int value = 0;
+
+	if (wwin->flags.maximized & MAX_VERTICAL)
+	    value |= 2;
+	if (wwin->flags.maximized & MAX_HORIZONTAL)
+	    value |= 1;
+
+	setSimpleHint(wwin->client_win, _XA_KWM_WIN_MAXIMIZED, value);
     }
 }
 
@@ -1517,6 +1592,15 @@ wKWMGetUsableArea(WScreen *scr, WArea *area)
 }
 
 
+
+Bool
+wKWMGetIconGeometry(WWindow *wwin, WArea *area)
+{
+    return getAreaHint(wwin->client_win, _XA_KWM_WIN_ICON_GEOMETRY, area);
+}
+
+
+
 #ifdef not_used
 void
 wKWMSetUsableAreaHint(WScreen *scr, int workspace)
@@ -1593,4 +1677,118 @@ wKWMSendEventMessage(WWindow *wwin, WKWMEventMessage message)
 }
 
 
+static void
+writeSocket(int sock, char *data)
+{
+    char buffer[128];
+
+    sprintf(buffer, "%i ", strlen(data));
+    write(sock, buffer, strlen(buffer));
+    write(sock, data, strlen(data));
+}
+
+
+static int
+connectKFM(WScreen *scr)
+{
+    char *path;
+    char buffer[128];
+    char *ptr;
+    FILE *f;
+    int pid;
+    int sock = 0;
+    struct sockaddr_un addr;
+
+    path = wstrconcat(wgethomedir(), "/.kde/share/apps/kfm/pid");
+    strcpy(buffer, getenv("DISPLAY"));
+
+    ptr = strchr(buffer, ':');
+    if (ptr)
+	*ptr = '_';
+
+    ptr = strrchr(buffer, '.');
+    if (ptr)
+	*ptr = 0;
+    {
+	char b[32];
+	
+	sprintf(b, ".%i", scr->screen);
+	strcat(buffer, b);
+    }
+    ptr = path;
+    path = wstrconcat(ptr, buffer);
+    wfree(ptr);
+
+    /* pid file */
+    f = fopen(path, "r");
+    wfree(path);
+    if (!f)
+	return -1;
+
+    buffer[0] = 0;
+    fgets(buffer, 123, f);
+    pid = atoi(buffer);
+    if (pid <= 0)
+	return -1;
+
+    if (kill(pid, 0) != 0)
+	return -1;
+
+    buffer[0] = 0;
+    fscanf(f, "%s", buffer);
+    fclose(f);
+
+    sock = socket(AF_UNIX, SOCK_STREAM, 0);
+    if (sock < 0)
+	return -1;
+    addr.sun_family = AF_UNIX;
+    strcpy(addr.sun_path, buffer);
+
+    if (connect(sock, (struct sockaddr*)&addr, sizeof(addr)) < 0) {
+	close(sock);
+	return -1;
+    }
+
+    path = wstrconcat(wgethomedir(), "/.kde/share/apps/kfm/magic");
+    f = fopen(path, "r");
+    if (!f) {
+	return -1;
+    }
+    ptr = fgets(buffer, 123, f);
+    fclose(f);
+    if (!ptr) {
+	return -1;
+    }
+    puts(buffer);
+
+    ptr = wstrconcat("auth", buffer);
+
+    writeSocket(sock, ptr);
+    wfree(ptr);
+
+    return sock;
+}
+
+
+void
+wKWMSelectRootRegion(WScreen *scr, int x, int y, int w, int h, Bool control)
+{
+#if 0
+    char buffer[128];
+    int sock;
+
+    puts("CONNECTING");
+    sock = connectKFM(scr);
+    if (sock < 0)
+	return;
+    puts("SENDING DATA");
+    
+    sprintf(buffer, "selectRootIcons %i %i %i %i %c", x, y, w, h, control);
+    writeSocket(sock, buffer);
+    
+    close(sock);
+#endif
+}
+
+
 #endif /* KWM_HINTS */
