@@ -31,8 +31,9 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <ctype.h>
+#if 0
 #include <nana.h>
-
+#endif
 #include "WindowMaker.h"
 #include "wcore.h"
 #include "framewin.h"
@@ -173,9 +174,6 @@ wMenuCreate(WScreen *screen, char *title, int main_menu)
 	wFrameWindowCreate(screen, tmp, 8, 2, 1, 1, &wPreferences.menu_title_clearance, flags,
 			   screen->menu_title_texture, NULL,
                            screen->menu_title_pixel,
-#ifdef DRAWSTRING_PLUGIN
-			   W_STRING_MTITLE,
-#endif
 			   &screen->menu_title_gc,
                            &screen->menu_title_font);
 			   
@@ -416,15 +414,15 @@ wMenuRemoveItem(WMenu *menu, int index)
     /* destroy unshared data */
 
     if (menu->entries[index]->text)
-      free(menu->entries[index]->text);
+      wfree(menu->entries[index]->text);
     
     if (menu->entries[index]->rtext)
-      free(menu->entries[index]->rtext);
+      wfree(menu->entries[index]->rtext);
 	
     if (menu->entries[index]->free_cdata && menu->entries[index]->clientdata)
       (*menu->entries[index]->free_cdata)(menu->entries[index]->clientdata);
 
-    free(menu->entries[index]);
+    wfree(menu->entries[index]);
 	
     for (i=index; i<menu->entry_no-1; i++) {
 	menu->entries[i+1]->order--;
@@ -486,7 +484,7 @@ renderTexture(WMenu *menu)
     if (!RConvertImage(scr->rcontext, img, &pix)) {
 	wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
     }
-    RDestroyImage(img);
+    RReleaseImage(img);
 
     return pix;
 }
@@ -627,10 +625,10 @@ wMenuDestroy(WMenu *menu, int recurse)
     if (!menu->flags.brother) {
 	for (i=0; i<menu->entry_no; i++) {
 
-	    free(menu->entries[i]->text);
+	    wfree(menu->entries[i]->text);
 
 	    if (menu->entries[i]->rtext)
-	      free(menu->entries[i]->rtext);
+	      wfree(menu->entries[i]->rtext);
 #ifdef USER_MENU
 
 	    if (menu->entries[i]->instances){
@@ -641,7 +639,7 @@ wMenuDestroy(WMenu *menu, int recurse)
 	    if (menu->entries[i]->free_cdata && menu->entries[i]->clientdata) {
 		(*menu->entries[i]->free_cdata)(menu->entries[i]->clientdata);
 	    }
-	    free(menu->entries[i]);
+	    wfree(menu->entries[i]);
 	}
 
 	if (recurse) {
@@ -656,14 +654,14 @@ wMenuDestroy(WMenu *menu, int recurse)
 	}
 	
 	if (menu->entries)
-	    free(menu->entries);
+	    wfree(menu->entries);
 	
     }
 
     FREE_PIXMAP(menu->menu_texture_data);
     
     if (menu->cascades)
-      free(menu->cascades);
+        wfree(menu->cascades);
 
     wCoreDestroy(menu->menu);
     wFrameWindowDestroy(menu->frame);
@@ -672,7 +670,7 @@ wMenuDestroy(WMenu *menu, int recurse)
     if (!menu->flags.brother && menu->brother)
 	wMenuDestroy(menu->brother, False);
     
-    free(menu);
+    wfree(menu);
 }
 
 
@@ -682,7 +680,7 @@ wMenuDestroy(WMenu *menu, int recurse)
 #define F_NONE		3
 
 static void
-drawFrame(WScreen *scr, Window win, int y, int w, int h, int type)
+drawFrame(WScreen *scr, Drawable win, int y, int w, int h, int type)
 {
     XSegment segs[2];
     int i;
@@ -837,11 +835,11 @@ paintEntry(WMenu *menu, int index, int selected)
     /* draw right text */
     
     if (entry->rtext && entry->cascade<0) {
+
 	tw = WMWidthOfString(scr->menu_entry_font, entry->rtext,
 			     strlen(entry->rtext));
-
-	WMDrawString(scr->wmscreen, win, textGC, scr->menu_entry_font, w-6-tw, 
-		     y + 3 + wPreferences.menu_text_clearance, entry->rtext, strlen(entry->rtext));
+    WMDrawString(scr->wmscreen, win, textGC, scr->menu_entry_font, w-6-tw, 
+            y + 3 + wPreferences.menu_text_clearance, entry->rtext, strlen(entry->rtext));
     }
 }
 
@@ -971,13 +969,17 @@ keyboardMenu(WMenu *menu)
 		break;
 
 	     case XK_Home:
+#ifdef XK_KP_Home
 	     case XK_KP_Home:
+#endif
 		selectEntry(menu, 0);
 		makeVisible(menu);
 		break;
 		
 	     case XK_End:
+#ifdef XK_KP_End
 	     case XK_KP_End:
+#endif
 		selectEntry(menu, menu->entry_no-1);
 		makeVisible(menu);
 		break;
@@ -986,7 +988,9 @@ keyboardMenu(WMenu *menu)
 #ifdef ARROWLESS_KBD
 	     case XK_k:
 #endif
+#ifdef XK_KP_Up
 	     case XK_KP_Up:
+#endif
 		if (menu->selected_entry <= 0)
                     selectEntry(menu, menu->entry_no-1);
 		else
@@ -998,7 +1002,9 @@ keyboardMenu(WMenu *menu)
 #ifdef ARROWLESS_KBD
 	     case XK_j:
 #endif
+#ifdef XK_KP_Down
 	     case XK_KP_Down:
+#endif
 		if (menu->selected_entry<0) 
                     selectEntry(menu, 0);
                 else if (menu->selected_entry == menu->entry_no-1)
@@ -1012,7 +1018,9 @@ keyboardMenu(WMenu *menu)
 #ifdef ARROWLESS_KBD
 	     case XK_l:
 #endif
+#ifdef XK_KP_Right
 	     case XK_KP_Right:
+#endif
 		if (menu->selected_entry>=0) {
 		    WMenuEntry *entry;
 		    entry = menu->entries[menu->selected_entry];
@@ -1037,7 +1045,9 @@ keyboardMenu(WMenu *menu)
 #ifdef ARROWLESS_KBD
 	     case XK_h:
 #endif
+#ifdef XK_KP_Left
 	     case XK_KP_Left:
+#endif
 		if (menu->parent!=NULL && menu->parent->selected_entry>=0) {
 		    selectEntry(menu, -1);
 		    move_menus(menu, old_pos_x, old_pos_y);
@@ -1242,7 +1252,7 @@ editEntry(WMenu *menu, WMenuEntry *entry)
     
     if (XGrabKeyboard(dpy, text->core->window, True, GrabModeAsync,
 		      GrabModeAsync, CurrentTime)!=GrabSuccess) {
-	wwarning("could not grab keyboard");
+	wwarning(_("could not grab keyboard"));
 	wTextDestroy(text);
 	
 	wSetFocusTo(menu->frame->screen_ptr, 
@@ -1288,7 +1298,7 @@ editEntry(WMenu *menu, WMenuEntry *entry)
     /* if !t, the user has canceled editing */
     if (t) {
 	if (entry->text)
-	    free(entry->text);
+	    wfree(entry->text);
 	entry->text = wstrdup(t);
 	
 	menu->flags.realized = 0;
@@ -1581,8 +1591,6 @@ wMenuUnderPointer(WScreen *screen)
 
 
 
-#define MIN(a,b)  (((a) > (b)) ? (b) : (a))
-
 
 static void
 getPointerPosition(WScreen *scr, int *x, int *y)
@@ -1612,31 +1620,14 @@ getScrollAmount(WMenu *menu, int *hamount, int *vamount)
     
     getPointerPosition(scr, &xroot, &yroot);
 
-#ifdef VIRTUAL_DESKTOP
-    if (wPreferences.vedge_thickness) {
-        if (xroot <= wPreferences.vedge_thickness + 1 && menuX1 < wPreferences.vedge_thickness) {
-            /* scroll to the right */
-            *hamount = MIN(MENU_SCROLL_STEP, abs(menuX1));
-
-        } else if (xroot >= screenW-2-wPreferences.vedge_thickness && menuX2 > screenW-1-wPreferences.vedge_thickness) {
-            /* scroll to the left */
-            *hamount = MIN(MENU_SCROLL_STEP, abs(menuX2-screenW-1));
-
-            if (*hamount==0)
-                *hamount = 1;
-
-            *hamount = -*hamount;
-        }
-    } else
-#endif
 
     if (xroot <= 1 && menuX1 < 0) {
 	/* scroll to the right */
-	*hamount = MIN(MENU_SCROLL_STEP, abs(menuX1));
+	*hamount = WMIN(MENU_SCROLL_STEP, abs(menuX1));
 
     } else if (xroot >= screenW-2 && menuX2 > screenW-1) {
 	/* scroll to the left */
-	*hamount = MIN(MENU_SCROLL_STEP, abs(menuX2-screenW-1));
+	*hamount = WMIN(MENU_SCROLL_STEP, abs(menuX2-screenW-1));
 	
 	if (*hamount==0)
 	    *hamount = 1;
@@ -1646,11 +1637,11 @@ getScrollAmount(WMenu *menu, int *hamount, int *vamount)
     
     if (yroot <= 1 && menuY1 < 0) {
 	/* scroll down */
-	*vamount = MIN(MENU_SCROLL_STEP, abs(menuY1));
+	*vamount = WMIN(MENU_SCROLL_STEP, abs(menuY1));
         
     } else if (yroot >= screenH-2 && menuY2 > screenH-1) {
 	/* scroll up */
-	*vamount = MIN(MENU_SCROLL_STEP, abs(menuY2-screenH-2));
+	*vamount = WMIN(MENU_SCROLL_STEP, abs(menuY2-screenH-2));
         
 	*vamount = -*vamount;
     }
@@ -1712,6 +1703,10 @@ scrollMenuCallback(void *data)
     int hamount = 0;		       /* amount to scroll */
     int vamount = 0;
 
+#ifdef VIRTUAL_DESKTOP
+    /* don't scroll if it is in vdesk mode */
+    if (!wPreferences.vedge_thickness)
+#endif
     getScrollAmount(menu, &hamount, &vamount);
     
     if (hamount != 0 || vamount != 0) {
@@ -1766,7 +1761,7 @@ _leaving(_delay *dl)
     wMenuMove(dl->menu, dl->ox, dl->oy, True);
     dl->menu->jump_back=NULL;
     dl->menu->menu->screen_ptr->flags.jump_back_pending = 0;
-    free(dl);
+    wfree(dl);
 }
 
 
@@ -1804,69 +1799,69 @@ wMenuScroll(WMenu *menu, XEvent *event)
 
         WMNextEvent(dpy, &ev);
         switch (ev.type) {
-	 case EnterNotify:
-            WMHandleEvent(&ev);
-	 case MotionNotify:
-            x = (ev.type==MotionNotify) ? ev.xmotion.x_root : ev.xcrossing.x_root;
-            y = (ev.type==MotionNotify) ? ev.xmotion.y_root : ev.xcrossing.y_root;
+            case EnterNotify:
+                WMHandleEvent(&ev);
+            case MotionNotify:
+                x = (ev.type==MotionNotify) ? ev.xmotion.x_root : ev.xcrossing.x_root;
+                y = (ev.type==MotionNotify) ? ev.xmotion.y_root : ev.xcrossing.y_root;
 
-	    /* on_border is != 0 if the pointer is between the menu
-	     * and the screen border and is close enough to the border */
-	    on_border = isPointNearBoder(menu, x, y);
+                /* on_border is != 0 if the pointer is between the menu
+                 * and the screen border and is close enough to the border */
+                on_border = isPointNearBoder(menu, x, y);
 
-            smenu = wMenuUnderPointer(scr);
-	    	    
-            if ((smenu==NULL && !on_border) || (smenu && parentMenu(smenu)!=omenu)) {
-                done = 1;
+                smenu = wMenuUnderPointer(scr);
+
+                if ((smenu==NULL && !on_border) || (smenu && parentMenu(smenu)!=omenu)) {
+                    done = 1;
+                    break;
+                }
+
+                on_x_edge = x <= 1 || x >= scr->scr_width - 2;
+                on_y_edge = y <= 1 || y >= scr->scr_height - 2;
+                on_border = on_x_edge || on_y_edge;
+
+                if (!on_border && !jump_back) {
+                    done = 1;
+                    break;
+                }
+
+                if (menu->timer && (smenu!=menu || (!on_y_edge && !on_x_edge))) {
+                    WMDeleteTimerHandler(menu->timer);
+                    menu->timer = NULL;
+                }
+
+                if (smenu != NULL)
+                    menu = smenu;
+
+                if (!menu->timer)
+                    scrollMenuCallback(menu);
                 break;
-            }
-
-            on_x_edge = x <= 1 || x >= scr->scr_width - 2;
-            on_y_edge = y <= 1 || y >= scr->scr_height - 2;
-            on_border = on_x_edge || on_y_edge;
-
-            if (!on_border && !jump_back) {
-                done = 1;
+            case ButtonPress:
+                /* True if we push on title, or drag the omenu to other position */
+                on_title = ev.xbutton.x_root >= omenu->frame_x &&
+                    ev.xbutton.x_root <= omenu->frame_x + MENUW(omenu) &&
+                    ev.xbutton.y_root >= omenu->frame_y &&
+                    ev.xbutton.y_root <= omenu->frame_y + omenu->frame->top_width;
+                WMHandleEvent(&ev);
+                smenu = wMenuUnderPointer(scr);
+                if (smenu == NULL || (smenu && smenu->flags.buttoned && smenu != omenu))
+                    done = 1;
+                else if (smenu==omenu && on_title) {
+                    jump_back = 0;
+                    done = 1;
+                }
                 break;
-            }
-
-            if (menu->timer && (smenu!=menu || (!on_y_edge && !on_x_edge))) {
-                WMDeleteTimerHandler(menu->timer);
-                menu->timer = NULL;
-            }
-
-            if (smenu != NULL)
-                menu = smenu;
-
-	    if (!menu->timer)
-		scrollMenuCallback(menu);
-            break;
-        case ButtonPress:
-            /* True if we push on title, or drag the omenu to other position */
-            on_title = ev.xbutton.x_root >= omenu->frame_x &&
-                ev.xbutton.x_root <= omenu->frame_x + MENUW(omenu) &&
-                ev.xbutton.y_root >= omenu->frame_y &&
-                ev.xbutton.y_root <= omenu->frame_y + omenu->frame->top_width;
-            WMHandleEvent(&ev);
-            smenu = wMenuUnderPointer(scr);
-            if (smenu == NULL || (smenu && smenu->flags.buttoned && smenu != omenu))
+            case KeyPress:
                 done = 1;
-            else if (smenu==omenu && on_title) {
-                jump_back = 0;
-                done = 1;
-            }
-            break;
-        case KeyPress:
-            done = 1;
-        default:
-            WMHandleEvent(&ev);
-            break;
+            default:
+                WMHandleEvent(&ev);
+                break;
         }
     }
 
     if (menu->timer) {
         WMDeleteTimerHandler(menu->timer);
-	menu->timer = NULL;
+        menu->timer = NULL;
     }
 
     if (jump_back) {
@@ -2004,6 +1999,12 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
 		dragScrollMenuCallback(menu);
         }
     }
+
+#ifdef VIRTUAL_DESKTOP
+    if (wPreferences.vedge_thickness) {
+        wWorkspaceLowerEdge(scr);
+    }
+#endif
     
     prevx = bev->x_root;
     prevy = bev->y_root;
@@ -2014,142 +2015,148 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
 
 	WMMaskEvent(dpy, ExposureMask|ButtonMotionMask|ButtonReleaseMask
 		    |ButtonPressMask, &ev);
-	switch (ev.type) {
-	 case MotionNotify:
+    switch (ev.type) {
+        case MotionNotify:
             smenu = findMenu(scr, &x, &y);
 
-	    if (smenu == NULL) {
-		/* moved mouse out of menu */
-		
-		if (!delayed_select && d_data.magic) {
-		    WMDeleteTimerHandler(d_data.magic);
-		    d_data.magic = NULL;
-		}
-		if (menu==NULL
-		    || (menu->selected_entry>=0
-			&& menu->entries[menu->selected_entry]->cascade>=0)) {
-		    prevx = ev.xmotion.x_root;
-		    prevy = ev.xmotion.y_root;
-		    
-		    break;
-		}
-		selectEntry(menu, -1);
-		menu = smenu;
-		prevx = ev.xmotion.x_root;
-		prevy = ev.xmotion.y_root;
-		break;
-	    } else if (menu && menu!=smenu
-		       && (menu->selected_entry<0
-			   || menu->entries[menu->selected_entry]->cascade<0)) {
-		selectEntry(menu, -1);
+            if (smenu == NULL) {
+                /* moved mouse out of menu */
 
-		if (!delayed_select && d_data.magic) {
-		    WMDeleteTimerHandler(d_data.magic);
-		    d_data.magic = NULL;
-		}
-	    } else {
-		
-		/* hysteresis for item selection */
+                if (!delayed_select && d_data.magic) {
+                    WMDeleteTimerHandler(d_data.magic);
+                    d_data.magic = NULL;
+                }
+                if (menu==NULL
+                        || (menu->selected_entry>=0
+                            && menu->entries[menu->selected_entry]->cascade>=0)) {
+                    prevx = ev.xmotion.x_root;
+                    prevy = ev.xmotion.y_root;
 
-		/* check if the motion was to the side, indicating that 
-		 * the user may want to cross to a submenu */
-		if (!delayed_select && menu) {
-		    int dx;
-		    Bool moved_to_submenu;/* moved to direction of submenu */
+                    break;
+                }
+                selectEntry(menu, -1);
+                menu = smenu;
+                prevx = ev.xmotion.x_root;
+                prevy = ev.xmotion.y_root;
+                break;
+            } else if (menu && menu!=smenu
+                    && (menu->selected_entry<0
+                        || menu->entries[menu->selected_entry]->cascade<0)) {
+                selectEntry(menu, -1);
 
-		    dx = abs(prevx - ev.xmotion.x_root);
+                if (!delayed_select && d_data.magic) {
+                    WMDeleteTimerHandler(d_data.magic);
+                    d_data.magic = NULL;
+                }
+            } else {
 
-		    moved_to_submenu = False;
-		    if (dx > 0 /* if moved enough to the side */
-			/* maybe a open submenu */
-			&& menu->selected_entry>=0 
-			/* moving to the right direction */
-			&& (wPreferences.align_menus
-			    || ev.xmotion.y_root >= prevy)) {
-			int index;
-			
-			index = menu->entries[menu->selected_entry]->cascade;
-			if (index>=0) {
-			    if (menu->cascades[index]->frame_x>menu->frame_x) {
-				if (prevx < ev.xmotion.x_root)
-				    moved_to_submenu = True;
-			    } else {
-				if (prevx > ev.xmotion.x_root)
-				    moved_to_submenu = True;
-			    }
-			}
-		    }
-		    
+                /* hysteresis for item selection */
+
+                /* check if the motion was to the side, indicating that 
+                 * the user may want to cross to a submenu */
+                if (!delayed_select && menu) {
+                    int dx;
+                    Bool moved_to_submenu;/* moved to direction of submenu */
+
+                    dx = abs(prevx - ev.xmotion.x_root);
+
+                    moved_to_submenu = False;
+                    if (dx > 0 /* if moved enough to the side */
+                            /* maybe a open submenu */
+                            && menu->selected_entry>=0 
+                            /* moving to the right direction */
+                            && (wPreferences.align_menus
+                                || ev.xmotion.y_root >= prevy)) {
+                        int index;
+
+                        index = menu->entries[menu->selected_entry]->cascade;
+                        if (index>=0) {
+                            if (menu->cascades[index]->frame_x>menu->frame_x) {
+                                if (prevx < ev.xmotion.x_root)
+                                    moved_to_submenu = True;
+                            } else {
+                                if (prevx > ev.xmotion.x_root)
+                                    moved_to_submenu = True;
+                            }
+                        }
+                    }
+
+
+                    if (menu != smenu) {
+                        if (d_data.magic) {
+                            WMDeleteTimerHandler(d_data.magic);
+                        }
+                        d_data.magic = NULL;
+                    } else if (moved_to_submenu) {
+                        /* while we are moving, postpone the selection */
+                        if (d_data.magic) {
+                            WMDeleteTimerHandler(d_data.magic);
+                        }
+                        d_data.delayed_select = NULL;
+                        d_data.menu = menu;
+                        d_data.magic = WMAddTimerHandler(MENU_SELECT_DELAY, 
+                                delaySelection,
+                                &d_data);
+                        prevx = ev.xmotion.x_root;
+                        prevy = ev.xmotion.y_root;
+                        break;
+                    } else {
+                        if (d_data.magic)
+                            WMDeleteTimerHandler(d_data.magic);
+                        d_data.magic = NULL;
+                    }
+                }
+            }
+            prevx = ev.xmotion.x_root;
+            prevy = ev.xmotion.y_root;
+            if (menu!=smenu) {
+                /* pointer crossed menus */
+                if (menu && menu->timer) {
+                    WMDeleteTimerHandler(menu->timer);
+                    menu->timer = NULL;
+                }
+                if (smenu)
+                    dragScrollMenuCallback(smenu);
+            }
+            menu = smenu;
+            if (!menu->timer)
+                dragScrollMenuCallback(menu);
 
-		    if (menu != smenu) {
-			if (d_data.magic) {
-			    WMDeleteTimerHandler(d_data.magic);
-			}
-			d_data.magic = NULL;
-		    } else if (moved_to_submenu) {
-			/* while we are moving, postpone the selection */
-			if (d_data.magic) {
-			    WMDeleteTimerHandler(d_data.magic);
-			}
-			d_data.delayed_select = NULL;
-			d_data.menu = menu;
-			d_data.magic = WMAddTimerHandler(MENU_SELECT_DELAY, 
-							 delaySelection,
-							 &d_data);
-			prevx = ev.xmotion.x_root;
-			prevy = ev.xmotion.y_root;
-			break;
-		    } else {
-			if (d_data.magic)
-			    WMDeleteTimerHandler(d_data.magic);
-			d_data.magic = NULL;
-		    }
-		}
-	    }
-	    prevx = ev.xmotion.x_root;
-	    prevy = ev.xmotion.y_root;
-	    if (menu!=smenu) {
-		/* pointer crossed menus */
-		if (menu && menu->timer) {
-		    WMDeleteTimerHandler(menu->timer);
-		    menu->timer = NULL;
-		}
-		if (smenu)
-		    dragScrollMenuCallback(smenu);
-	    }
-	    menu = smenu;
-	    if (!menu->timer)
-		dragScrollMenuCallback(menu);
-	    
             if (!delayed_select) {
                 entry_no = getEntryAt(menu, x, y);
                 if (entry_no>=0) {
                     entry = menu->entries[entry_no];
                     if (entry->flags.enabled && entry->cascade>=0 &&
-                        menu->cascades) {
+                            menu->cascades) {
                         WMenu *submenu = menu->cascades[entry->cascade];
                         if (submenu->flags.mapped && !submenu->flags.buttoned
-                            && menu->selected_entry!=entry_no) {
+                                && menu->selected_entry!=entry_no) {
                             wMenuUnmap(submenu);
                         }
                     }
                 }
                 selectEntry(menu, entry_no);
             }
-	    break;
-
-	 case ButtonPress:
-	    break;
-
-	 case ButtonRelease:
-	    if (ev.xbutton.button == event->xbutton.button)
-		done=1;
             break;
 
-	 case Expose: 
+        case ButtonPress:
+            break;
+
+        case ButtonRelease:
+            if (ev.xbutton.button == event->xbutton.button)
+                done=1;
+            break;
+
+        case Expose: 
             WMHandleEvent(&ev);
+#ifdef VIRTUAL_DESKTOP
+            /* since expose will raise edge up.. I need another ugly hack here */
+            if (wPreferences.vedge_thickness) {
+                wWorkspaceLowerEdge(scr);
+            }
+#endif
             break;
- 	}
+    }
     }
 
     if (menu && menu->timer) {
@@ -2213,6 +2220,11 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
 
 byebye:    
     ((WMenu*)desc->parent)->flags.inside_handler = 0;
+#ifdef VIRTUAL_DESKTOP
+    if (wPreferences.vedge_thickness) {
+        wWorkspaceRaiseEdge(scr);
+    }
+#endif
 }
 
 
@@ -2716,8 +2728,6 @@ wMenuRestoreState(WScreen *scr)
     }
     restoreMenuRecurs(scr, menus, scr->root_menu, "");
 #endif /* !LITE */
-
-    PLRelease(skey);
 }
 
 
