@@ -28,12 +28,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdarg.h>
 #include <pwd.h>
 #include <math.h>
 #include <time.h>
 
+#include <WINGs/WUtil.h>
 #include <wraster.h>
 
+
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
@@ -46,7 +49,6 @@
 #include "xutil.h"
 #include "xmodifier.h"
 
-#include "list.h"
 
 /**** global variables *****/
 
@@ -114,6 +116,7 @@ MakeCPPArgs(char *path)
     int i;
     char buffer[MAXLINE], *buf, *line;
     Visual *visual;
+    char *tmp;
     
     line = wmalloc(MAXLINE);
     *line = 0;
@@ -148,13 +151,21 @@ MakeCPPArgs(char *path)
     putidef(line, " -DSCR_HEIGHT=", 
 	    HeightOfScreen(DefaultScreenOfDisplay(dpy)));
 
-    #if 0
-    strcpy(buffer, path);    
-    buf = strrchr(buffer, '/');
-    if (buf) *buf = 0; /* trunc filename */
-    putdef(line, " -I", buffer);
-    #endif
-
+    /* put the dir where the menu is being read from to the
+     * search path */
+    if (path) {
+	tmp = wstrdup(path);
+	buf = strchr(tmp+1, ' ');
+	if (buf) {
+	    *buf = 0;
+	}
+	buf = strrchr(tmp, '/');
+	if (buf) {
+	    *buf = 0; /* trunc filename */
+	    putdef(line, " -I", tmp);
+	}
+	wfree(tmp);
+    }
 
 
     /* this should be done just once, but it works this way */
@@ -189,69 +200,6 @@ MakeCPPArgs(char *path)
 #endif /* USECPP */
 
 
-WWindow*
-NextFocusWindow(WScreen *scr)
-{
-    WWindow *tmp, *wwin, *closest, *min;
-    Window d;
-
-    if (!(wwin = scr->focused_window))
-        return NULL;
-    tmp = wwin->prev;
-    closest = NULL;
-    min = wwin;
-    d = 0xffffffff;
-    while (tmp) {
-        if (wWindowCanReceiveFocus(tmp) 
-	    && (!WFLAGP(tmp, skip_window_list)|| tmp->flags.internal_window)) {
-	    if (min->client_win > tmp->client_win)
-	      min = tmp;
-	    if (tmp->client_win > wwin->client_win
-		&& (!closest
-		    || (tmp->client_win - wwin->client_win) < d)) {
-		closest = tmp;
-		d = tmp->client_win - wwin->client_win;
-	    }
-	}
-	tmp = tmp->prev;
-    }
-    if (!closest||closest==wwin)
-      return min;
-    return closest;
-}
-
-
-WWindow*
-PrevFocusWindow(WScreen *scr)
-{
-    WWindow *tmp, *wwin, *closest, *max;
-    Window d;
-    
-    if (!(wwin = scr->focused_window))
-        return NULL;
-    tmp = wwin->prev;
-    closest = NULL;
-    max = wwin;
-    d = 0xffffffff;
-    while (tmp) {
-	if (wWindowCanReceiveFocus(tmp) &&
-            (!WFLAGP(tmp, skip_window_list) || tmp->flags.internal_window)) {
-	    if (max->client_win < tmp->client_win)
-	      max = tmp;
-	    if (tmp->client_win < wwin->client_win
-		&& (!closest
-		    || (wwin->client_win - tmp->client_win) < d)) {
-		closest = tmp;
-		d = wwin->client_win - tmp->client_win;
-	    }
-	}
-	tmp = tmp->prev;
-    }
-    if (!closest||closest==wwin)
-      return max;
-    return closest;
-}
-
 
 
 #if 0
@@ -289,7 +237,8 @@ isBelow(WWindow *win1, WWindow *win2)
  *  XFetchName Wrapper
  *
  */
-Bool wFetchName(dpy, win, winname)
+Bool
+wFetchName(dpy, win, winname)
 Display *dpy;
 Window win;
 char **winname;
@@ -333,7 +282,8 @@ char **winname;
  *
  */
 
-Bool wGetIconName(dpy, win, iconname)
+Bool
+wGetIconName(dpy, win, iconname)
 Display *dpy;
 Window win;
 char **iconname;
@@ -363,27 +313,6 @@ char **iconname;
 }
 
 
-#if 0
-#ifdef I18N_MB
-void
-wTextWidth(XFontSet font, char *text, int length) 
-{
-    XRectangle rect;
-    XRectangle AIXsucks;
-    
-    XmbTextExtents(font, text, length, &AIXsucks, &rec);
-
-    return rect.width;
-}
-#else
-void
-wTextWidth(XFontStruct *font, char *text, int length) 
-{
-    return XTextWidth(font, text, length);
-}
-#endif
-#endif
-
 static void 
 eatExpose()
 {
@@ -491,21 +420,19 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 
 
 char*
-ShrinkString(WFont *font, char *string, int width)
+ShrinkString(WMFont *font, char *string, int width)
 {
-#ifndef I18N_MB
     int w, w1=0;
     int p;
     char *pos;
     char *text;
     int p1, p2, t;
-#endif
 
-#ifdef I18N_MB
-    return wstrdup(string);
-#else
+    if (wPreferences.multi_byte_text)
+	return wstrdup(string);
+
     p = strlen(string);
-    w = wTextWidth(font->font, string, p);
+    w = WMWidthOfString(font, string, p);
     text = wmalloc(strlen(string)+8);
     strcpy(text, string);
     if (w<=width)
@@ -518,8 +445,8 @@ ShrinkString(WFont *font, char *string, int width)
     if (pos) {
 	*pos = 0;
 	p = strlen(text);
-	w1=wTextWidth(font->font, text, p);
-	if (w1>width) {
+    w1 = WMWidthOfString(font, text, p);
+	if (w1 > width) {
 	    w1 = 0;
 	    p = 0;
 	    *pos = ' ';
@@ -535,14 +462,13 @@ ShrinkString(WFont *font, char *string, int width)
 	*text=0;
     }
     strcat(text, "...");
-    width -= wTextWidth(font->font, "...", 3);
-    
+    width -= WMWidthOfString(font, "...", 3);
     pos = string;
     p1=0;
     p2=p;
     t = (p2-p1)/2;
     while (p2>p1 && p1!=t) {
-	w = wTextWidth(font->font, &string[p-t], t);
+	w = WMWidthOfString(font, &string[p-t], t);
 	if (w>width) {
 	    p2 = t;
 	    t = p1+(p2-p1)/2;
@@ -553,8 +479,8 @@ ShrinkString(WFont *font, char *string, int width)
 	  p2=p1=t;
     }
     strcat(text, &string[p-p1]);
+
     return text;
-#endif /* I18N_MB */
 }
 
 
@@ -577,199 +503,113 @@ FindImage(char *paths, char *file)
 }
 
 
-char*
-FlattenStringList(char **list, int count)
-{
-    int i, j;
-    char *flat_string, *wspace;
-
-    j = 0;
-    for (i=0; i<count; i++) {
-        if (list[i]!=NULL && list[i][0]!=0) {
-            j += strlen(list[i]);
-            if (strpbrk(list[i], " \t"))
-                j += 2;
-        }
-    }
-    
-    flat_string = malloc(j+count+1);
-    if (!flat_string) {
-	return NULL;
-    }
-
-    *flat_string = 0;
-    for (i=0; i<count; i++) {
-	if (list[i]!=NULL && list[i][0]!=0) {
-	    if (i>0)
-		strcat(flat_string, " ");
-            wspace = strpbrk(list[i], " \t");
-            if (wspace)
-                strcat(flat_string, "\"");
-	    strcat(flat_string, list[i]);
-            if (wspace)
-                strcat(flat_string, "\"");
-	}
-    }
-
-    return flat_string;
-}
-
-
-
-/*
- *----------------------------------------------------------------------
- * ParseCommand --
- * 	Divides a command line into a argv/argc pair.
- *---------------------------------------------------------------------- 
- */
-#define PRC_ALPHA	0
-#define PRC_BLANK	1
-#define PRC_ESCAPE	2
-#define PRC_DQUOTE	3
-#define PRC_EOS		4
-#define PRC_SQUOTE	5
-
-typedef struct {
-    short nstate;
-    short output;
-} DFA;
-
-
-static DFA mtable[9][6] = {
-    {{3,1},{0,0},{4,0},{1,0},{8,0},{6,0}},
-    {{1,1},{1,1},{2,0},{3,0},{5,0},{1,1}},
-    {{1,1},{1,1},{1,1},{1,1},{5,0},{1,1}},
-    {{3,1},{5,0},{4,0},{1,0},{5,0},{6,0}},
-    {{3,1},{3,1},{3,1},{3,1},{5,0},{3,1}},
-    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
-    {{6,1},{6,1},{7,0},{6,1},{5,0},{3,0}},
-    {{6,1},{6,1},{6,1},{6,1},{5,0},{6,1}},
-    {{-1,-1},{0,0},{0,0},{0,0},{0,0},{0,0}}, /* final state */
-};
-
-char*
-next_token(char *word, char **next)
-{
-    char *ptr;
-    char *ret, *t;
-    int state, ctype;
-
-    t = ret = wmalloc(strlen(word)+1);
-    ptr = word;
-    
-    state = 0;
-    *t = 0;
-    while (1) {
-	if (*ptr==0) 
-	    ctype = PRC_EOS;
-	else if (*ptr=='\\')
-	    ctype = PRC_ESCAPE;
-	else if (*ptr=='"')
-	    ctype = PRC_DQUOTE;
-	else if (*ptr=='\'')
-	    ctype = PRC_SQUOTE;
-	else if (*ptr==' ' || *ptr=='\t')
-	    ctype = PRC_BLANK;
-	else
-	    ctype = PRC_ALPHA;
-
-	if (mtable[state][ctype].output) {
-	    *t = *ptr; t++;
-	    *t = 0;
-	}
-	state = mtable[state][ctype].nstate;
-	ptr++;
-	if (mtable[state][0].output<0) {
-	    break;
-	}
-    }
-
-    if (*ret==0)
-	t = NULL;
-    else
-	t = wstrdup(ret);
-
-    free(ret);
-    
-    if (ctype==PRC_EOS)
-	*next = NULL;
-    else
-	*next = ptr;
-    
-    return t;
-}
-
-
-void
-ParseCommand(char *command, char ***argv, int *argc)
-{
-    LinkedList *list = NULL;
-    char *token, *line;
-    int count, i;
-
-    line = command;
-    do {
-	token = next_token(line, &line);
-	if (token) {	    
-	    list = list_cons(token, list);
-	}
-    } while (token!=NULL && line!=NULL);
-
-    count = list_length(list);
-    *argv = wmalloc(sizeof(char*)*count);
-    i = count;
-    while (list!=NULL) {
-	(*argv)[--i] = list->head;
-	list_remove_head(&list);
-    }
-    *argc = count;
-}
-
-#if 0
 static void
-timeup(void *foo)
+timeoutHandler(void *data)
 {
-    *(int*)foo=1;
+    *(int*)data = 1;
 }
-#endif
+
+
+static char*
+getTextSelection(WScreen *screen, Atom selection)
+{
+    int buffer = -1;
+    
+    switch (selection) {
+     case XA_CUT_BUFFER0:
+	buffer = 0;
+	break;
+     case XA_CUT_BUFFER1:
+	buffer = 1;
+	break;
+     case XA_CUT_BUFFER2:
+	buffer = 2;
+	break;
+     case XA_CUT_BUFFER3:
+	buffer = 3;
+	break;
+     case XA_CUT_BUFFER4:
+	buffer = 4;
+	break;
+     case XA_CUT_BUFFER5:
+	buffer = 5;
+	break;
+     case XA_CUT_BUFFER6:
+	buffer = 6;
+	break;
+     case XA_CUT_BUFFER7:
+	buffer = 7;
+	break;
+    }
+    if (buffer >= 0) {
+	char *data;
+	int size;
+
+	data = XFetchBuffer(dpy, &size, buffer);
+
+	return data;
+    } else {
+	char *data;
+	int bits;
+	Atom rtype;
+	unsigned long len, bytes;
+	WMHandlerID timer;
+	int timeout = 0;
+	XEvent ev;
+	static Atom clipboard = 0;
+	
+	if (!clipboard) 
+	    clipboard = XInternAtom(dpy, "CLIPBOARD", False);
+	
+	XDeleteProperty(dpy, screen->info_window, clipboard);
+	
+	XConvertSelection(dpy, selection, XA_STRING,
+			  clipboard, screen->info_window,
+			  CurrentTime);
+	
+	timer = WMAddTimerHandler(1000, timeoutHandler, &timeout);
+	
+	while (!XCheckTypedWindowEvent(dpy, screen->info_window,
+				       SelectionNotify, &ev) && !timeout);
+	
+	if (!timeout) {
+	    WMDeleteTimerHandler(timer);
+	} else {
+	    wwarning("selection retrieval timed out");
+	    return NULL;
+	}
+
+	/* nobody owns the selection or the current owner has
+	 * nothing to do with what we need */
+	if (ev.xselection.property == None) {
+	    return NULL;
+	}
+
+	if (XGetWindowProperty(dpy, screen->info_window,
+			       clipboard, 0, 1024,
+			       False, XA_STRING, &rtype, &bits, &len,
+			       &bytes, (unsigned char**)&data)!=Success) {
+	    return NULL;
+	}
+	if (rtype!=XA_STRING || bits!=8) {
+	    wwarning("invalid data in text selection");
+	    if (data)
+		XFree(data);
+	    return NULL;
+	}
+	return data;
+    }
+}
+
 static char*
 getselection(WScreen *scr)
 {
     char *tmp;
-    extern char *W_GetTextSelection(); /* in WINGs */
     
-    tmp = W_GetTextSelection(scr->wmscreen, XA_PRIMARY);
+    tmp = getTextSelection(scr, XA_PRIMARY);
     if (!tmp)
-	tmp = W_GetTextSelection(scr->wmscreen, XA_CUT_BUFFER0);
+	tmp = getTextSelection(scr, XA_CUT_BUFFER0);
     return tmp;
-
-#if 0
-    XEvent event;
-    int timeover=0;
-    WMHandlerID *id;
-    
-#ifdef DEBUG
-    puts("getting selection");
-#endif
-    RequestSelection(dpy, scr->no_focus_win, LastTimestamp);
-    /* timeout on 1 sec. */
-    id = WMAddTimerHandler(1000, timeup, &timeover);
-    while (!timeover) {
-	WMNextEvent(dpy, &event);
-	if (event.type == SelectionNotify 
-	    && event.xany.window==scr->no_focus_win) {
-	    WMDeleteTimerHandler(id);
-#ifdef DEBUG
-	    puts("selection ok");
-#endif
-	    return GetSelection(dpy, scr->no_focus_win);
-	} else {
-	    WMHandleEvent(&event);
-	}
-    }
-    wwarning(_("selection timed-out"));
-    return NULL;
-#endif
 }
 
 
@@ -780,7 +620,7 @@ getuserinput(WScreen *scr, char *line, int *ptr)
     char *title;
     char *prompt;
     int j, state;
-    int begin;
+    int begin = 0;
     char tbuffer[256], pbuffer[256];
 
     title = _("Program Arguments");
@@ -873,11 +713,6 @@ get_dnd_selection(WScreen *scr)
     char *flat_string;
     int count;
     
-#ifdef XDE_DND
-    if(scr->xdestring) {
-	return (wstrdup(scr->xdestring));
-    }
-#endif
     result=XGetTextProperty(dpy, scr->root_win, &text_ret, _XA_DND_SELECTION);
     
     if (result==0 || text_ret.value==NULL || text_ret.encoding==None
@@ -894,7 +729,7 @@ get_dnd_selection(WScreen *scr)
 	return NULL;
     }
 
-    flat_string = FlattenStringList(list, count);
+    flat_string = wtokenjoin(list, count);
     if (!flat_string) {
 	wwarning(_("out of memory while getting data from DND drop"));
     }
@@ -931,7 +766,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
     char *out, *nout;
     char *selection=NULL;
     char *user_input=NULL;
-#ifdef OFFIX_DND
+#if defined(OFFIX_DND) || defined(XDND)
     char *dropped_thing=NULL;
 #endif
     char tmpbuf[TMPBUFSIZE];
@@ -1042,8 +877,13 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		}
 		break;
 
-#ifdef OFFIX_DND
+#if defined(OFFIX_DND) || defined(XDND)
 	     case 'd':
+#ifdef XDND
+		if(scr->xdestring) {
+		    dropped_thing = wstrdup(scr->xdestring);
+		}
+#endif
 		if (!dropped_thing) {
 		    dropped_thing = get_dnd_selection(scr);
 		}
@@ -1083,6 +923,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		strcat(out,selection);
 		optr+=slen;
 		break;
+
 	     default:
 		out[optr++]='%';
 		out[optr++]=cmdline[ptr];
@@ -1097,7 +938,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
     return out;
     
     error:
-    free(out);
+    wfree(out);
     if (selection)
       XFree(selection);
     return NULL;
@@ -1181,6 +1022,7 @@ keysymToString(KeySym keysym, unsigned int state)
 }
 #endif
 
+
 char*
 GetShortcutString(char *text)
 {
@@ -1240,7 +1082,7 @@ GetShortcutString(char *text)
     puts(tmp);
     buffer = wstrappend(buffer, tmp);
 */
-    free(tmp);
+    wfree(tmp);
 
     return buffer;
 }
@@ -1258,7 +1100,7 @@ EscapeWM_CLASS(char *name, char *class)
 
     if (name) {
 	l = strlen(name);
-	ename = wmalloc(l*2);
+	ename = wmalloc(l*2+1);
 	j = 0;
 	for (i=0; i<l; i++) {
 	    if (name[i]=='\\') {
@@ -1272,7 +1114,7 @@ EscapeWM_CLASS(char *name, char *class)
     }
     if (class) {
 	l = strlen(class);
-	eclass = wmalloc(l*2);
+	eclass = wmalloc(l*2+1);
 	j = 0;
 	for (i=0; i<l; i++) {
 	    if (class[i]=='\\') {
@@ -1288,14 +1130,14 @@ EscapeWM_CLASS(char *name, char *class)
     if (ename && eclass) {
 	ret = wmalloc(strlen(ename)+strlen(eclass)+4);
 	sprintf(ret, "%s.%s", ename, eclass);
-	free(ename);
-	free(eclass);
+	wfree(ename);
+	wfree(eclass);
     } else if (ename) {
 	ret = wstrdup(ename);
-	free(ename);
+	wfree(ename);
     } else {
 	ret = wstrdup(eclass);
-	free(eclass);
+	wfree(eclass);
     }
 
     return ret;
@@ -1363,11 +1205,11 @@ UnescapeWM_CLASS(char *str, char **name, char **class)
     (*class)[k] = 0;
     
     if (!*name) {
-	free(*name);
+	wfree(*name);
 	*name = NULL;
     }
     if (!*class) {
-	free(*class);
+	wfree(*class);
 	*class = NULL;
     }
 }
@@ -1404,40 +1246,5 @@ SendHelperMessage(WScreen *scr, char type, int workspace, char *msg)
     if (write(scr->helper_fd, buffer, len+4) < 0) {
 	wsyserror(_("could not send message to background image helper"));
     }
-    free(buffer);
-}
-
-
-void
-ExecuteShellCommand(WScreen *scr, char *command)
-{
-    static char *shell = NULL;
-    pid_t pid;
-
-    /*
-     * This have a problem: if the shell is tcsh (not sure about others)
-     * and ~/.tcshrc have /bin/stty erase ^H somewhere on it, the shell
-     * will block and the command will not be executed.
-    if (!shell) {
-	shell = getenv("SHELL");
-	if (!shell)
-	    shell = "/bin/sh";
-    }
-     */
-    shell = "/bin/sh";
-
-    pid = fork();
-    if (pid==0) {
-
-	SetupEnvironment(scr);
-
-#ifdef HAVE_SETPGID
-	setpgid(0, 0);
-#endif
-	execl(shell, shell, "-c", command, NULL);
-	wsyserror("could not execute %s -c %s", shell, command);
-	Exit(-1);
-    } else if (pid < 0) {
-	wsyserror("cannot fork a new process");
-    }
+    wfree(buffer);
 }
