@@ -148,7 +148,13 @@ showPosition(WWindow *wwin, int x, int y)
 		  scr->scr_height);
 #endif
     } else {
+#ifdef VIRTUAL_DESKTOP
+	WSetGeometryViewShownPosition(scr->gview,
+            x + scr->workspaces[scr->current_workspace]->view_x,
+            y + scr->workspaces[scr->current_workspace]->view_y);
+#else
 	WSetGeometryViewShownPosition(scr->gview, x, y);
+#endif
     }
 }
 
@@ -384,19 +390,19 @@ mapGeometryDisplay(WWindow *wwin, int x, int y, int w, int h)
 
 
 static void
-doWindowMove(WWindow *wwin, WMBag *bag, int dx, int dy)
+doWindowMove(WWindow *wwin, WMArray *array, int dx, int dy)
 {
     WWindow *tmpw;
     int x, y;
     int scr_width = wwin->screen_ptr->scr_width;
     int scr_height = wwin->screen_ptr->scr_height;
 
-    if (!bag || !WMGetBagItemCount(bag)) {
+    if (!array || !WMGetArrayItemCount(array)) {
         wWindowMove(wwin, wwin->frame_x + dx, wwin->frame_y + dy);
     } else {
-	int i;
-	for (i = 0; i < WMGetBagItemCount(bag); i++) {
-	    tmpw = WMGetFromBag(bag, i);
+        WMArrayIterator iter;
+
+        WM_ITERATE_ARRAY(array, tmpw, iter) {
 	    x = tmpw->frame_x + dx;
 	    y = tmpw->frame_y + dy;
 	    
@@ -449,14 +455,14 @@ drawTransparentFrame(WWindow *wwin, int x, int y, int width, int height)
 
 
 static void
-drawFrames(WWindow *wwin, WMBag *bag, int dx, int dy)
+drawFrames(WWindow *wwin, WMArray *array, int dx, int dy)
 {
     WWindow *tmpw;
     int scr_width = wwin->screen_ptr->scr_width;
     int scr_height = wwin->screen_ptr->scr_height;
     int x, y;
     
-    if (!bag) {
+    if (!array) {
 
         x = wwin->frame_x + dx;
         y = wwin->frame_y + dy;
@@ -466,9 +472,9 @@ drawFrames(WWindow *wwin, WMBag *bag, int dx, int dy)
 			     wwin->frame->core->height);
 	
     } else {
-	int i;
-	for (i = 0; i < WMGetBagItemCount(bag); i++) {
-	    tmpw = WMGetFromBag(bag, i);
+        WMArrayIterator iter;
+
+        WM_ITERATE_ARRAY(array, tmpw, iter) {
 	    x = tmpw->frame_x + dx;
 	    y = tmpw->frame_y + dy;
 	    
@@ -706,13 +712,13 @@ static void
 freeMoveData(MoveData *data)
 {
     if (data->topList)
-	free(data->topList);
+	wfree(data->topList);
     if (data->leftList)
-	free(data->leftList);
+	wfree(data->leftList);
     if (data->rightList)
-	free(data->rightList);
+	wfree(data->rightList);
     if (data->bottomList)
-	free(data->bottomList);
+	wfree(data->bottomList);
 }
 
 
@@ -1252,241 +1258,265 @@ wKeyboardMoveResizeWindow(WWindow *wwin)
     ww=w;
     wh=h;
     while(1) {
-	/*
-	 looper.ox=off_x;
-	 looper.oy=off_y;
-	 */
-	WMMaskEvent(dpy, KeyPressMask | ButtonReleaseMask 
-		    | ButtonPressMask | ExposureMask, &event);
-    	if (wwin->flags.shaded || scr->selected_windows) {
-	    if(scr->selected_windows)
-	        drawFrames(wwin,scr->selected_windows,off_x,off_y);
-	    else drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
-	    /*** I HATE EDGE RESISTANCE - ]d ***/
-	}
-	else {
-	    drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, ww, wh);
-	}
-
-	if(ctrlmode)
-	    showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
-
-	XUngrabServer(dpy);
-	XSync(dpy, False);
-
-	switch (event.type) {
-     case KeyPress:
-        /* accelerate */
-        if (event.xkey.time - lastTime > 50) {
-            kspeed/=(1 + (event.xkey.time - lastTime)/100);
-        } else {
-            if (kspeed < 20) {
-                kspeed++;
+        /*
+           looper.ox=off_x;
+           looper.oy=off_y;
+         */
+        do {
+            WMMaskEvent(dpy, KeyPressMask | ButtonReleaseMask 
+                    | ButtonPressMask | ExposureMask, &event);
+            if (event.type == Expose) {
+                WMHandleEvent(&event);
             }
-        }
-        if (kspeed < _KS) kspeed = _KS;
-        lastTime = event.xkey.time;
+        } while (event.type == Expose);
 
-        if (event.xkey.state & ControlMask && !wwin->flags.shaded) {
-            ctrlmode=1;
-            wUnselectWindows(scr);
+
+        if (wwin->flags.shaded || scr->selected_windows) {
+            if(scr->selected_windows)
+                drawFrames(wwin,scr->selected_windows,off_x,off_y);
+            else drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
+            /*** I HATE EDGE RESISTANCE - ]d ***/
         }
         else {
-            ctrlmode=0;
+            drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, ww, wh);
         }
-        if (event.xkey.keycode == shiftl || event.xkey.keycode == shiftr) {
-            if (ctrlmode)
-                cycleGeometryDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh, 0);
-            else
-                cyclePositionDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
-        }
-        else {
 
-            keysym = XLookupKeysym(&event.xkey, 0);
-            switch (keysym) {
-             case XK_Return:
-                done=2;
+        if(ctrlmode)
+            showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
+
+        XUngrabServer(dpy);
+        XSync(dpy, False);
+
+        switch (event.type) {
+            case KeyPress:
+                /* accelerate */
+                if (event.xkey.time - lastTime > 50) {
+                    kspeed/=(1 + (event.xkey.time - lastTime)/100);
+                } else {
+                    if (kspeed < 20) {
+                        kspeed++;
+                    }
+                }
+                if (kspeed < _KS) kspeed = _KS;
+                lastTime = event.xkey.time;
+
+                if (event.xkey.state & ControlMask && !wwin->flags.shaded) {
+                    ctrlmode=1;
+                    wUnselectWindows(scr);
+                }
+                else {
+                    ctrlmode=0;
+                }
+                if (event.xkey.keycode == shiftl || event.xkey.keycode == shiftr) {
+                    if (ctrlmode)
+                        cycleGeometryDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh, 0);
+                    else
+                        cyclePositionDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
+                }
+                else {
+
+                    keysym = XLookupKeysym(&event.xkey, 0);
+                    switch (keysym) {
+                        case XK_Return:
+                            done=2;
+                            break;
+                        case XK_Escape:
+                            done=1;
+                            break;
+                        case XK_Up:
+#ifdef XK_KP_Up
+                        case XK_KP_Up:
+#endif
+                        case XK_k:
+                            if (ctrlmode){
+                                if (moment != UP)
+                                    h = wh;
+                                h-=kspeed;
+                                moment = UP;
+                                if (h < 1) h = 1;
+                            }
+                            else off_y-=kspeed;
+                            break;
+                        case XK_Down:
+#ifdef XK_KP_Down
+                        case XK_KP_Down:
+#endif
+                        case XK_j:
+                            if (ctrlmode){
+                                if (moment != DOWN)
+                                    h = wh;
+                                h+=kspeed;
+                                moment = DOWN;
+                            }
+                            else off_y+=kspeed;
+                            break;
+                        case XK_Left:
+#ifdef XK_KP_Left
+                        case XK_KP_Left:
+#endif
+                        case XK_h:
+                            if (ctrlmode) {
+                                if (moment != LEFT)
+                                    w = ww;
+                                w-=kspeed;
+                                if (w < 1) w = 1;
+                                moment = LEFT;
+                            }
+                            else off_x-=kspeed;
+                            break;
+                        case XK_Right:
+#ifdef XK_KP_Right
+                        case XK_KP_Right:
+#endif
+                        case XK_l:
+                            if (ctrlmode) {
+                                if (moment != RIGHT)
+                                    w = ww;
+                                w+=kspeed;
+                                moment = RIGHT;
+                            }
+                            else off_x+=kspeed;
+                            break;
+                    }
+
+                    ww=w;wh=h;
+                    wh-=vert_border;
+                    wWindowConstrainSize(wwin, &ww, &wh);
+                    wh+=vert_border;
+
+                    if (wPreferences.ws_cycle){
+                        if (src_x + off_x + ww < 20){
+                            if(!scr->current_workspace) {
+                                wWorkspaceChange(scr, scr->workspace_count-1);
+                            }
+                            else wWorkspaceChange(scr, scr->current_workspace-1);
+                            off_x += scr_width;
+                        }
+                        else if (src_x + off_x + 20 > scr_width){
+                            if(scr->current_workspace == scr->workspace_count-1) {
+                                wWorkspaceChange(scr, 0);
+                            }
+                            else wWorkspaceChange(scr, scr->current_workspace+1);
+                            off_x -= scr_width;
+                        }
+                    }
+                    else {
+                        if (src_x + off_x + ww < 20)
+                            off_x = 20 - ww - src_x;
+                        else if (src_x + off_x + 20 > scr_width)
+                            off_x = scr_width - 20 - src_x;
+                    }
+
+                    if (src_y + off_y + wh < 20) {
+                        off_y = 20 - wh - src_y;
+                    }
+                    else if (src_y + off_y + 20 > scr_height) {
+                        off_y = scr_height - 20 - src_y;
+                    }
+                }
                 break;
-             case XK_Escape:
+            case ButtonPress:
+            case ButtonRelease:
                 done=1;
                 break;
-             case XK_Up:
-             case XK_KP_Up:
-             case XK_k:
-                if (ctrlmode){
-                if (moment != UP)
-                    h = wh;
-                h-=kspeed;
-                moment = UP;
-                if (h < 1) h = 1;
+            case Expose:
+                WMHandleEvent(&event);
+                while (XCheckTypedEvent(dpy, Expose, &event)) {
+                    WMHandleEvent(&event);
                 }
-                else off_y-=kspeed;
                 break;
-             case XK_Down:
-             case XK_KP_Down:
-             case XK_j:
-                if (ctrlmode){
-                if (moment != DOWN)
-                    h = wh;
-                h+=kspeed;
-                moment = DOWN;
-                }
-                else off_y+=kspeed;
-                break;
-             case XK_Left:
-             case XK_KP_Left:
-             case XK_h:
-                if (ctrlmode) {
-                if (moment != LEFT)
-                    w = ww;
-                w-=kspeed;
-                if (w < 1) w = 1;
-                moment = LEFT;
-                }
-                else off_x-=kspeed;
-                break;
-             case XK_Right:
-             case XK_KP_Right:
-             case XK_l:
-                if (ctrlmode) {
-                if (moment != RIGHT)
-                    w = ww;
-                w+=kspeed;
-                moment = RIGHT;
-                }
-                else off_x+=kspeed;
-                break;
-            }
 
-            ww=w;wh=h;
-            wh-=vert_border;
-            wWindowConstrainSize(wwin, &ww, &wh);
-            wh+=vert_border;
-        
-            if (wPreferences.ws_cycle){
-                if (src_x + off_x + ww < 20){
-                    if(!scr->current_workspace) {
-                        wWorkspaceChange(scr, scr->workspace_count-1);
-                    }
-                    else wWorkspaceChange(scr, scr->current_workspace-1);
-                    off_x += scr_width;
-                }
-                else if (src_x + off_x + 20 > scr_width){
-                    if(scr->current_workspace == scr->workspace_count-1) {
-                        wWorkspaceChange(scr, 0);
-                    }
-                    else wWorkspaceChange(scr, scr->current_workspace+1);
-                    off_x -= scr_width;
-                }
-            }
-            else {
-                if (src_x + off_x + ww < 20)
-                    off_x = 20 - ww - src_x;
-                else if (src_x + off_x + 20 > scr_width)
-                    off_x = scr_width - 20 - src_x;
-            }
-        
-            if (src_y + off_y + wh < 20) {
-                off_y = 20 - wh - src_y;
-            }
-            else if (src_y + off_y + 20 > scr_height) {
-                off_y = scr_height - 20 - src_y;
+            default:
+                WMHandleEvent(&event);
+                break;
+        }
+
+        XGrabServer(dpy);
+        /*xxx*/
+
+        if (wwin->flags.shaded && !scr->selected_windows){
+            moveGeometryDisplayCentered(scr, src_x+off_x + w/2, src_y+off_y + h/2);
+        } else {
+            if (ctrlmode) {
+                WMUnmapWidget(scr->gview);
+                mapGeometryDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
+            } else if(!scr->selected_windows) {
+                WMUnmapWidget(scr->gview);
+                mapPositionDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
             }
         }
-        break;
-	 case ButtonPress:
-	 case ButtonRelease:
-	    done=1;
-	    break;
-	 default:
-	    WMHandleEvent(&event);
-	    break;
-	}
 
-	XGrabServer(dpy);
-	/*xxx*/
+        if (wwin->flags.shaded || scr->selected_windows) {
+            if (scr->selected_windows)
+                drawFrames(wwin,scr->selected_windows,off_x,off_y);
+            else 
+                drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
+        } else {
+            drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, ww, wh);
+        }
 
-	if (wwin->flags.shaded && !scr->selected_windows){
-	    moveGeometryDisplayCentered(scr, src_x+off_x + w/2, src_y+off_y + h/2);
-	} else {
-	    if (ctrlmode) {
-		WMUnmapWidget(scr->gview);
-		mapGeometryDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
-	    } else if(!scr->selected_windows) {
-		WMUnmapWidget(scr->gview);
-		mapPositionDisplay(wwin, src_x+off_x, src_y+off_y, ww, wh);
-	    }
-	}
-	
-	if (wwin->flags.shaded || scr->selected_windows) {
-	    if (scr->selected_windows)
-		drawFrames(wwin,scr->selected_windows,off_x,off_y);
-	    else 
-		drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
-	} else {
-	    drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, ww, wh);
-	}
-	
-	
-	if (ctrlmode) {
-	    showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
-	} else if(!scr->selected_windows)
-	    showPosition(wwin, src_x+off_x, src_y+off_y);
-	/**/
-	
-	if(done){
-	    scr->keymove_tick=0;
-	    /*
-	     WMDeleteTimerWithClientData(&looper);
-	     */
+
+        if (ctrlmode) {
+            showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
+        } else if(!scr->selected_windows)
+            showPosition(wwin, src_x+off_x, src_y+off_y);
+        /**/
+
+        if (done) {
+            scr->keymove_tick=0;
+            /*
+               WMDeleteTimerWithClientData(&looper);
+             */
             if (wwin->flags.shaded || scr->selected_windows) {
-	        if(scr->selected_windows)
-	            drawFrames(wwin,scr->selected_windows,off_x,off_y);
-	        else drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
+                if(scr->selected_windows)
+                    drawFrames(wwin,scr->selected_windows,off_x,off_y);
+                else drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, w, h);
             }
             else {
                 drawTransparentFrame(wwin, src_x+off_x, src_y+off_y, ww, wh);
             }
 
-	    if (ctrlmode) {
-		showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
-		WMUnmapWidget(scr->gview);
-	    } else
-		WMUnmapWidget(scr->gview);
-	    
-	    XUngrabKeyboard(dpy, CurrentTime);
-	    XUngrabPointer(dpy, CurrentTime);
-	    XUngrabServer(dpy);
+            if (ctrlmode) {
+                showGeometry(wwin, src_x+off_x, src_y+off_y, src_x+off_x+ww, src_y+off_y+wh,0);
+                WMUnmapWidget(scr->gview);
+            } else
+                WMUnmapWidget(scr->gview);
 
-	    if(done==2) {
-		if (wwin->flags.shaded || scr->selected_windows) {
-		    if (!scr->selected_windows) {
-			wWindowMove(wwin, src_x+off_x, src_y+off_y);
-		        wWindowSynthConfigureNotify(wwin);
-		    } else {
-			int i;
-			WMBag *bag = scr->selected_windows;
-			doWindowMove(wwin,scr->selected_windows,off_x,off_y);
-			for (i = 0; i < WMGetBagItemCount(bag); i++) {
-			    wWindowSynthConfigureNotify(WMGetFromBag(bag, i));
-			}
-		    }
-		} else {
-		    if (wwin->client.width != ww)
-			wwin->flags.user_changed_width = 1;
-		    
-		    if (wwin->client.height != wh - vert_border)
-			wwin->flags.user_changed_height = 1;
-		    
-		    wWindowConfigure(wwin, src_x+off_x, src_y+off_y, 
-				     ww, wh - vert_border);
-		    wWindowSynthConfigureNotify(wwin);
-		}
-		wWindowChangeWorkspace(wwin, scr->current_workspace);
-		wSetFocusTo(scr, wwin);
-	    }
-	    return 1;
-	}
+            XUngrabKeyboard(dpy, CurrentTime);
+            XUngrabPointer(dpy, CurrentTime);
+            XUngrabServer(dpy);
+
+            if(done==2) {
+                if (wwin->flags.shaded || scr->selected_windows) {
+                    if (!scr->selected_windows) {
+                        wWindowMove(wwin, src_x+off_x, src_y+off_y);
+                        wWindowSynthConfigureNotify(wwin);
+                    } else {
+                        WMArrayIterator iter;
+                        WWindow *foo;
+
+                        doWindowMove(wwin, scr->selected_windows, off_x, off_y);
+
+                        WM_ITERATE_ARRAY(scr->selected_windows, foo, iter) {
+                            wWindowSynthConfigureNotify(foo);
+                        }
+                    }
+                } else {
+                    if (wwin->client.width != ww)
+                        wwin->flags.user_changed_width = 1;
+
+                    if (wwin->client.height != wh - vert_border)
+                        wwin->flags.user_changed_height = 1;
+
+                    wWindowConfigure(wwin, src_x+off_x, src_y+off_y, 
+                            ww, wh - vert_border);
+                    wWindowSynthConfigureNotify(wwin);
+                }
+                wWindowChangeWorkspace(wwin, scr->current_workspace);
+                wSetFocusTo(scr, wwin);
+            }
+            return 1;
+        }
     }
 }
 
@@ -2068,16 +2098,14 @@ wUnselectWindows(WScreen *scr)
     if (!scr->selected_windows)
 	return;
     
-    while (WMGetBagItemCount(scr->selected_windows)) {
-	WMBagIterator dummy;
-	
-	wwin = WMBagFirst(scr->selected_windows, &dummy);
+    while (WMGetArrayItemCount(scr->selected_windows)) {
+	wwin = WMGetFromArray(scr->selected_windows, 0);
         if (wwin->flags.miniaturized && wwin->icon && wwin->icon->selected)
             wIconSelect(wwin->icon);
 
 	wSelectWindow(wwin, False);
     }
-    WMFreeBag(scr->selected_windows);
+    WMFreeArray(scr->selected_windows);
     scr->selected_windows = NULL;
 }
 
