@@ -144,9 +144,6 @@ static Shortcut *shortcutList = NULL;
  *							     
  */
 
-#define MAX(a,b)	((a)>(b) ? (a) : (b))
-
-
 #define M_QUICK		1
 
 /* menu commands */
@@ -165,7 +162,7 @@ execCommand(WMenu *menu, WMenuEntry *entry)
 
     if (cmdline) {
 	ExecuteShellCommand(menu->frame->screen_ptr, cmdline);
-	free(cmdline);
+	wfree(cmdline);
     }
     XUngrabPointer(dpy, CurrentTime);
     XSync(dpy, 0);
@@ -455,7 +452,7 @@ removeShortcutsForMenu(WMenu *menu)
     while (ptr!=NULL) {
 	tmp = ptr->next;
 	if (ptr->menu == menu) {
-	    free(ptr);
+	    wfree(ptr);
 	} else {
 	    ptr->next = newList;
 	    newList = ptr;
@@ -490,7 +487,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
 	mod = wXModifierFromKey(b);
 	if (mod<0) {
 	    wwarning(_("%s:invalid key modifier \"%s\""), file, b);
-	    free(ptr);
+	    wfree(ptr);
 	    return False;
 	}
 	ptr->modifier |= mod;
@@ -504,7 +501,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     if (ksym==NoSymbol) {
 	wwarning(_("%s:invalid kbd shortcut specification \"%s\" for entry %s"),
 		 file, shortcutDefinition, entry->text);
-	free(ptr);
+	wfree(ptr);
 	return False;
     }
     
@@ -512,7 +509,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
     if (ptr->keycode==0) {
 	wwarning(_("%s:invalid key in shortcut \"%s\" for entry %s"), file,
 		 shortcutDefinition, entry->text);
-	free(ptr);
+	wfree(ptr);
 	return False;
     }
 
@@ -601,7 +598,7 @@ static void
 separateCommand(char *line, char ***file, char **command)
 {
     char *token, *tmp = line;
-    WMBag *bag = WMCreateBag(4);
+    WMArray *array = WMCreateArray(4);
     int count, i;
 
     *file = NULL;
@@ -616,19 +613,19 @@ separateCommand(char *line, char ***file, char **command)
                     wwarning(_("%s: missing command"), line);
                 break;
             }
-	    WMPutInBag(bag, token);
+	    WMAddToArray(array, token);
         }
     } while (token!=NULL && tmp!=NULL);
 
-    count = WMGetBagItemCount(bag);
+    count = WMGetArrayItemCount(array);
     if (count>0) {
         *file = wmalloc(sizeof(char*)*(count+1));
         (*file)[count] = NULL;
         for (i = 0; i < count; i++) {
-            (*file)[i] = WMGetFromBag(bag, i);
+            (*file)[i] = WMGetFromArray(array, i);
         }
     }
-    WMFreeBag(bag);
+    WMFreeArray(array);
 }
 
 
@@ -677,10 +674,10 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 	    }
 	    
 	    tmp = wexpandpath(path[i]);
-	    free(path[i]);
+	    wfree(path[i]);
 	    lpath = getLocalizedMenuFile(tmp);
 	    if (lpath) {
-		free(tmp);
+		wfree(tmp);
 		path[i] = lpath;
 		lpath = NULL;
 	    } else {
@@ -741,10 +738,10 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 finish:
     i = 0;
     while (path[i]!=NULL)
-        free(path[i++]);
-    free(path);
+        wfree(path[i++]);
+    wfree(path);
     if (cmd)
-	free(cmd);
+	wfree(cmd);
 }
 
 
@@ -770,6 +767,32 @@ addWorkspaceMenu(WScreen *scr, WMenu *menu, char *title)
     return entry;
 }
 
+static WMenuEntry*
+addWindowsMenu(WScreen *scr, WMenu *menu, char *title)
+{
+    WMenu *wwmenu;
+    WWindow *wwin;
+    WMenuEntry *entry;
+
+    if (scr->flags.added_windows_menu) {
+	wwarning(_("There are more than one WINDOWS_MENU commands in the applications menu. Only one is allowed."));
+	return NULL;
+    } else {
+	scr->flags.added_windows_menu = 1;
+
+	wwmenu = wMenuCreate(scr, _("Window List"), False);
+	scr->switch_menu = wwmenu;
+	wwin = scr->focused_window;
+	while (wwin) {
+	    UpdateSwitchMenu(scr, wwin, ACTION_ADD);
+
+	    wwin = wwin->prev;
+	}
+	entry = wMenuAddCallback(menu, title, NULL, NULL);
+	wMenuEntrySetCascade(menu, entry, wwmenu);
+    }
+    return entry;
+}
 
 static WMenuEntry*
 addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
@@ -806,7 +829,7 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
 		     file_name, command);
 	else {
 	    entry = wMenuAddCallback(menu, title, execCommand, 
-				     wstrappend("exec ", params));
+				     wstrconcat("exec ", params));
 	    entry->free_cdata = free;
 	    shortcutOk = True;
 	}
@@ -844,6 +867,10 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
     } else if (strcmp(command, "WORKSPACE_MENU")==0) {
 	entry = addWorkspaceMenu(scr, menu, title);
 	
+	shortcutOk = True;
+    } else if (strcmp(command, "WINDOWS_MENU")==0) {
+	entry = addWindowsMenu(scr, menu, title);
+
 	shortcutOk = True;
     } else if (strcmp(command, "ARRANGE_ICONS")==0) {
 	entry = wMenuAddCallback(menu, title, arrangeIconsCommand, NULL);
@@ -1105,7 +1132,7 @@ readMenuFile(WScreen *scr, char *file_name)
 	    wwarning(_("could not make arguments for menu file preprocessor"));
 	} else {
 	    sprintf(command, "%s %s %s", CPP_PATH, args, file_name);
-	    free(args);
+	    wfree(args);
 	    file = popen(command, "r");
 	    if (!file) {
 		wsyserror(_("%s:could not open/preprocess menu file"), 
@@ -1205,7 +1232,7 @@ readMenuPipe(WScreen *scr, char **file_name)
        } else {
            sprintf(command, "%s | %s %s", filename, CPP_PATH, args);
 
-           free(args);
+           wfree(args);
            file = popen(command, "r");
            if (!file) {
                wsyserror(_("%s:could not open/preprocess menu file"), filename);
@@ -1304,14 +1331,15 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
     struct stat stat_buf;
     WMenu *menu=NULL;
     char *buffer;
-    WMBag *dirs = NULL, *files = NULL;
+    WMArray *dirs = NULL, *files = NULL;
+    WMArrayIterator iter;
     int length, i, have_space=0;
     dir_data *data;
     int stripExtension = 0;
 
     
-    dirs = WMCreateBag(16);
-    files = WMCreateBag(16);
+    dirs = WMCreateArray(16);
+    files = WMCreateArray(16);
     
     i=0;
     while (path[i]!=NULL) {
@@ -1365,7 +1393,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 			data->name = wstrdup(dentry->d_name);
 			data->index = i;
 
-			WMPutInBag(dirs, data);
+			WMAddToArray(dirs, data);
                     }
                 } else if (S_ISREG(stat_buf.st_mode) || isFilePack) {
                     /* Hack because access always returns X_OK success for user root */
@@ -1378,35 +1406,33 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 			data->name = wstrdup(dentry->d_name);
 			data->index = i;
 
-			WMPutInBag(files, data);
+			WMAddToArray(files, data);
                     }
                 }
             }
-            free(buffer);
+            wfree(buffer);
         }
 
         closedir(dir);
         i++;
     }
     
-    if (!WMGetBagItemCount(dirs) && !WMGetBagItemCount(files)) {
-	WMFreeBag(dirs);
-	WMFreeBag(files);
+    if (!WMGetArrayItemCount(dirs) && !WMGetArrayItemCount(files)) {
+	WMFreeArray(dirs);
+	WMFreeArray(files);
         return NULL;
     }
 
-    WMSortBag(dirs, myCompare);
-    WMSortBag(files, myCompare);
+    WMSortArray(dirs, myCompare);
+    WMSortArray(files, myCompare);
 
     menu = wMenuCreate(scr, title, False);
     menu->on_destroy = removeShortcutsForMenu;
 
-    for (i = 0; i < WMGetBagItemCount(dirs); i++) {
+    WM_ITERATE_ARRAY(dirs, data, iter) {
         /* New directory. Use same OPEN_MENU command that was used
          * for the current directory. */
-        dir_data *d = (dir_data*)WMGetFromBag(dirs, i);
-
-        length = strlen(path[d->index])+strlen(d->name)+6;
+        length = strlen(path[data->index])+strlen(data->name)+6;
 	if (stripExtension)
 	    length += 7;
         if (command)
@@ -1414,7 +1440,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
         buffer = malloc(length);
         if (!buffer) {
             wsyserror(_("out of memory while constructing directory menu %s"),
-                     path[d->index]);
+                     path[data->index]);
             break;
         }
 	
@@ -1422,15 +1448,15 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
         if (stripExtension)
             strcat(buffer, "-noext ");
 
-        have_space = strchr(path[d->index], ' ')!=NULL ||
-                     strchr(d->name, ' ')!=NULL;
+        have_space = strchr(path[data->index], ' ')!=NULL ||
+                     strchr(data->name, ' ')!=NULL;
 
 	if (have_space)
             strcat(buffer, "\"");
-	strcat(buffer, path[d->index]);
+	strcat(buffer, path[data->index]);
 
         strcat(buffer, "/");
-        strcat(buffer, d->name);
+        strcat(buffer, data->name);
         if (have_space)
             strcat(buffer, "\"");
         if (command) {
@@ -1438,66 +1464,64 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
             strcat(buffer, command);
         }
 
-        addMenuEntry(menu, d->name, NULL, "OPEN_MENU", buffer, path[d->index]);
+        addMenuEntry(menu, data->name, NULL, "OPEN_MENU", buffer, path[data->index]);
 
-        free(buffer);
-	if (d->name)
-	    free(d->name);
-	free(d);
+        wfree(buffer);
+	if (data->name)
+	    wfree(data->name);
+	wfree(data);
     }
 
-    for (i = 0; i < WMGetBagItemCount(files); i++) {
+    WM_ITERATE_ARRAY(files, data, iter) {
         /* executable: add as entry */
-        dir_data *f = (dir_data*)WMGetFromBag(files, i);
-
-        length = strlen(path[f->index])+strlen(f->name)+6;
+        length = strlen(path[data->index])+strlen(data->name)+6;
         if (command)
             length += strlen(command);
 
         buffer = malloc(length);
         if (!buffer) {
             wsyserror(_("out of memory while constructing directory menu %s"),
-                     path[f->index]);
+                     path[data->index]);
             break;
         }
 
-        have_space = strchr(path[f->index], ' ')!=NULL ||
-                     strchr(f->name, ' ')!=NULL;
+        have_space = strchr(path[data->index], ' ')!=NULL ||
+                     strchr(data->name, ' ')!=NULL;
         if (command!=NULL) {
             strcpy(buffer, command);
             strcat(buffer, " ");
             if (have_space)
                 strcat(buffer, "\"");
-            strcat(buffer, path[f->index]);
+            strcat(buffer, path[data->index]);
         } else {
             if (have_space) {
                 buffer[0] = '"';
                 buffer[1] = 0;
-                strcat(buffer, path[f->index]);
+                strcat(buffer, path[data->index]);
             } else {
-                strcpy(buffer, path[f->index]);
+                strcpy(buffer, path[data->index]);
             }
         }
         strcat(buffer, "/");
-        strcat(buffer, f->name);
+        strcat(buffer, data->name);
         if (have_space)
             strcat(buffer, "\"");
 
 	if (stripExtension) {
-	    char *ptr = strrchr(f->name, '.');
-	    if (ptr && ptr!=f->name)
+	    char *ptr = strrchr(data->name, '.');
+	    if (ptr && ptr!=data->name)
 		*ptr = 0;
 	}
-        addMenuEntry(menu, f->name, NULL, "SHEXEC", buffer, path[f->index]);
+        addMenuEntry(menu, data->name, NULL, "SHEXEC", buffer, path[data->index]);
 
-        free(buffer);
-	if (f->name)
-	    free(f->name);
-	free(f);
+        wfree(buffer);
+	if (data->name)
+	    wfree(data->name);
+	wfree(data);
     }
     
-    WMFreeBag(files);
-    WMFreeBag(dirs);
+    WMFreeArray(files);
+    WMFreeArray(dirs);
 
     return menu;
 }
@@ -1561,14 +1585,14 @@ configureMenu(WScreen *scr, proplist_t definition)
 	if (!path) {
 	    wsyserror(_("could not find menu file \"%s\" referenced in WMRootMenu"),
 		     tmp);
-	    free(tmp);
+	    wfree(tmp);
 	    return NULL;
 	}
 
 	if (stat(path, &stat_buf)<0) {
 	    wsyserror(_("could not access menu \"%s\" referenced in WMRootMenu"), path);
-	    free(path);
-	    free(tmp);
+	    wfree(path);
+	    wfree(tmp);
 	    return NULL;
 	}
 
@@ -1583,12 +1607,12 @@ configureMenu(WScreen *scr, proplist_t definition)
 
 	    menu = readMenuFile(scr, path);
 	    if (menu)
-		menu->timestamp = MAX(stat_buf.st_mtime, WDRootMenu->timestamp);
+		menu->timestamp = WMAX(stat_buf.st_mtime, WDRootMenu->timestamp);
 	} else {
 	    menu = NULL;
 	}
-	free(path);
-	free(tmp);
+	wfree(path);
+	wfree(tmp);
 
 	return menu;
     }
@@ -1602,7 +1626,7 @@ configureMenu(WScreen *scr, proplist_t definition)
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
 		 "WMRootMenu", tmp);
-	free(tmp);
+	wfree(tmp);
 	return NULL;
     }
     mtitle = PLGetString(elem);
@@ -1678,7 +1702,7 @@ configureMenu(WScreen *scr, proplist_t definition)
 	tmp = PLGetDescription(elem);
 	wwarning(_("%s:format error in root menu configuration \"%s\""), 
 		 "WMRootMenu", tmp);
-	free(tmp);
+	wfree(tmp);
     }
     
     return menu;
@@ -1779,7 +1803,18 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
 	scr->root_menu = menu;
     }
     if (menu) {
-	wMenuMapAt(menu, x-menu->frame->core->width/2, y, keyboard);
+        int newx, newy;
+
+        if (keyboard && x==0 && y==0) {
+            newx = newy = 0;
+        } else if (keyboard && x==scr->scr_width/2 && y==scr->scr_height/2) {
+            newx = x - menu->frame->core->width/2;
+            newy = y - menu->frame->core->height/2;
+        } else {
+            newx = x - menu->frame->core->width/2;
+            newy = y;
+        }
+        wMenuMapAt(menu, newx, newy, keyboard);
     }
     
     if (scr->flags.root_menu_changed_shortcuts)
