@@ -402,9 +402,7 @@ static void
 createPixmaps(WScreen *scr)
 {
     WPixmap *pix;
-    WMPixmap *wmpix;
     RImage *image;
-    Pixmap p, m;
 
     /* load pixmaps */
     pix = wPixmapCreateFromXBMData(scr, (char*)MENU_RADIO_INDICATOR_XBM_DATA,
@@ -413,7 +411,7 @@ createPixmaps(WScreen *scr)
 				   MENU_RADIO_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
-      pix->shared = 1;
+        pix->shared = 1;
     scr->menu_radio_indicator = pix;
 
 
@@ -423,7 +421,7 @@ createPixmaps(WScreen *scr)
 				   MENU_CHECK_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
-      pix->shared = 1;
+        pix->shared = 1;
     scr->menu_check_indicator = pix;
 
     pix = wPixmapCreateFromXBMData(scr, (char*)MENU_MINI_INDICATOR_XBM_DATA,
@@ -432,25 +430,25 @@ createPixmaps(WScreen *scr)
 				   MENU_MINI_INDICATOR_XBM_SIZE,
 				   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
-      pix->shared = 1;
+        pix->shared = 1;
     scr->menu_mini_indicator = pix;
 
     pix = wPixmapCreateFromXBMData(scr, (char*)MENU_HIDE_INDICATOR_XBM_DATA,
-				   (char*)MENU_HIDE_INDICATOR_XBM_DATA,
-				   MENU_HIDE_INDICATOR_XBM_SIZE,
-				   MENU_HIDE_INDICATOR_XBM_SIZE,
-				   scr->black_pixel, scr->white_pixel);
+                                   (char*)MENU_HIDE_INDICATOR_XBM_DATA,
+                                   MENU_HIDE_INDICATOR_XBM_SIZE,
+                                   MENU_HIDE_INDICATOR_XBM_SIZE,
+                                   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
-      pix->shared = 1;
+        pix->shared = 1;
     scr->menu_hide_indicator = pix;
 
     pix = wPixmapCreateFromXBMData(scr, (char*)MENU_SHADE_INDICATOR_XBM_DATA,
-                                  (char*)MENU_SHADE_INDICATOR_XBM_DATA,
-                                  MENU_SHADE_INDICATOR_XBM_SIZE,
-                                  MENU_SHADE_INDICATOR_XBM_SIZE,
-                                  scr->black_pixel, scr->white_pixel);
+                                   (char*)MENU_SHADE_INDICATOR_XBM_DATA,
+                                   MENU_SHADE_INDICATOR_XBM_SIZE,
+                                   MENU_SHADE_INDICATOR_XBM_SIZE,
+                                   scr->black_pixel, scr->white_pixel);
     if (pix!=NULL)
-      pix->shared = 1;
+        pix->shared = 1;
     scr->menu_shade_indicator = pix;
 
 
@@ -460,16 +458,8 @@ createPixmaps(WScreen *scr)
 	wwarning(_("could not load logo image for panels: %s"),
 		 RMessageForError(RErrorCode));
     } else {
-	if (!RConvertImageMask(scr->rcontext, image, &p, &m, 128)) {
-	    wwarning(_("error making logo image for panel:%s"), RMessageForError(RErrorCode));
-	} else {
-	    wmpix = WMCreatePixmapFromXPixmaps(scr->wmscreen, p, m,
-					       image->width, image->height, 
-					       scr->depth);
-	    WMSetApplicationIconImage(scr->wmscreen, wmpix); 
-	    WMReleasePixmap(wmpix);
-	}
-	RDestroyImage(image);
+        WMSetApplicationIconImage(scr->wmscreen, image);
+	RReleaseImage(image);
     }
 
     scr->dock_dots = make3Dots(scr);
@@ -672,7 +662,7 @@ wScreenInit(int screen_number)
 
 #if 0
     if (!aquireManagerSelection(scr)) {
-	free(scr);
+	wfree(scr);
 
 	return NULL;
     }
@@ -703,7 +693,7 @@ wScreenInit(int screen_number)
     XSetErrorHandler(oldHandler);
     
     if (CantManageScreen) {
-	free(scr);
+	wfree(scr);
 	return NULL;
     }
 
@@ -722,8 +712,13 @@ wScreenInit(int screen_number)
     if (rattr.colors_per_channel<2)
 	rattr.colors_per_channel = 2;
 
+    
     /* will only be accounted for in PseudoColor */
-    rattr.standard_colormap_mode = RCreateStdColormap;
+    if (wPreferences.flags.create_stdcmap) {
+	rattr.standard_colormap_mode = RCreateStdColormap;
+    } else {
+	rattr.standard_colormap_mode = RUseStdColormap;
+    }
 
     if (wVisualID>=0) {
 	rattr.flags |= RC_VisualID;
@@ -934,7 +929,7 @@ wScreenUpdateUsableArea(WScreen *scr)
 	lw = area->area.x1;
 	rw = scr->scr_width - area->area.x2;
 
-	if (WMIN(th, bh) < WMIN(lw, rw)) {
+	if (WMIN(th, bh) <= WMIN(lw, rw)) {
 	    /* horizontal */
 	    if (th < bh) {
 		/* on top */
@@ -982,6 +977,23 @@ wScreenUpdateUsableArea(WScreen *scr)
     }
 #endif
 #endif
+
+    {
+        unsigned size = wPreferences.workspace_border_size;
+        unsigned position = wPreferences.workspace_border_position;
+
+        if (size>0 && position!=WB_NONE) {
+            if (position & WB_LEFTRIGHT) {
+                scr->totalUsableArea.x1 += size;
+                scr->totalUsableArea.x2 -= size;
+            }
+            if (position & WB_TOPBOTTOM) {
+                scr->totalUsableArea.y1 += size;
+                scr->totalUsableArea.y2 -= size;
+            }
+        }
+    }
+
 }
 
 
@@ -1007,13 +1019,13 @@ wScreenRestoreState(WScreen *scr)
 	path = wdefaultspathfordomain(buf);
     }
     scr->session_state = PLGetProplistWithPath(path);
-    free(path);
+    wfree(path);
     if (!scr->session_state && wScreenCount>1) {
 	char buf[16];
 	sprintf(buf, "WMState.%i", scr->screen);
 	path = wdefaultspathfordomain(buf);
 	scr->session_state = PLGetProplistWithPath(path);
-	free(path);
+	wfree(path);
     }
 
     if (!wPreferences.flags.noclip) {
@@ -1042,7 +1054,6 @@ wScreenRestoreState(WScreen *scr)
 void
 wScreenSaveState(WScreen *scr)
 {
-    WWorkspaceState wstate;
     WWindow *wwin;
     char *str;
     proplist_t path, old_state, foo;
@@ -1051,13 +1062,9 @@ wScreenSaveState(WScreen *scr)
 
     make_keys();
 
-/*
- * Save current workspace, so can go back to it upon restart.
- */
-    wstate.workspace = scr->current_workspace;
-
-    data[0] = wstate.flags;
-    data[1] = wstate.workspace;
+    /* Save current workspace, so can go back to it upon restart. */
+    data[0] = scr->current_workspace;
+    data[1] = WFLAGS_NONE;
 
     XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_STATE,
                     _XA_WINDOWMAKER_STATE, 32, PropModeReplace,
@@ -1122,7 +1129,7 @@ wScreenSaveState(WScreen *scr)
 	str = wdefaultspathfordomain(buf);
     }
     path = PLMakeString(str);
-    free(str);
+    wfree(str);
     PLSetFilename(scr->session_state, path);
     if (!PLSave(scr->session_state, YES)) {
 	wsyserror(_("could not save session state in %s"), PLGetString(path));
