@@ -1,7 +1,7 @@
 /*
  *  Window Maker window manager
  * 
- *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997-2001 Alfredo K. Kojima
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,11 +24,14 @@
 
 #include "wconfig.h"
 #include "WindowMaker.h"
-#include "plugin.h"
 #include <sys/types.h>
 
 #include <proplist.h>
 
+#ifdef XINERAMA
+#include <X11/Xinerama.h>
+#endif
+
 
 #define WTB_LEFT	0
 #define WTB_RIGHT	1
@@ -90,6 +93,12 @@ typedef struct _WScreen {
     Visual *w_visual;
     int  w_depth;
     Colormap w_colormap;	       /* our colormap */
+    
+#ifdef XINERAMA
+    XineramaScreenInfo *xine_screens;
+    int xine_count;		       /* 0 means not active */
+    int xine_primary_screen;	       /* main working screen */
+#endif
 
     Window no_focus_win;	       /* window to get focus when nobody
 					* else can do it */
@@ -98,8 +107,8 @@ typedef struct _WScreen {
 					* Use this list if you want to
 					* traverse the entire window list
 					*/
-    
-    WMBag *selected_windows;
+
+    WMArray *selected_windows;
 
     struct WAppIcon *app_icon_list;    /* list of all app-icons on screen */
 
@@ -161,10 +170,6 @@ typedef struct _WScreen {
     WMPixel dtext_pixel;	        /* disabled menu item text */
     WMPixel line_pixel;
     WMPixel frame_border_pixel;	       /* frame border */
-#ifdef DRAWSTRING_PLUGIN
-    WFunction *drawstring_func[W_STRING_MEMBERS];
-                                    /* ftitle, utitle, ptitle, mtitle, mtext */
-#endif
 
 
     union WTexture *menu_title_texture[3];/* menu titlebar texture (tex, -, -) */
@@ -286,7 +291,7 @@ typedef struct _WScreen {
 					scrolled down for titlebar access */
 
     /* for window shortcuts */
-    WMBag *shortcutWindows[MAX_WINDOW_SHORTCUTS];
+    WMArray *shortcutWindows[MAX_WINDOW_SHORTCUTS];
 
 #ifdef XDND
     char *xdestring;
@@ -305,6 +310,7 @@ typedef struct _WScreen {
 	unsigned int dnd_data_convertion_status:1;
 	unsigned int root_menu_changed_shortcuts:1;
 	unsigned int added_workspace_menu:1;
+	unsigned int added_windows_menu:1;
 	unsigned int startup2:1;       /* startup phase 2 */
 	unsigned int supports_tiff:1;
 	unsigned int clip_balloon_mapped:1;
