@@ -226,7 +226,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
         win = wwin->client_win;
 
     if (XGetCommand(dpy, win, &argv, &argc) && argc>0) {
-        command = FlattenStringList(argv, argc);
+        command = wtokenjoin(argv, argc);
         XFreeStringList(argv);
     }
     if (!command)
@@ -256,7 +256,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
 	
 	for (mask = 0, i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
 	    if (scr->shortcutWindows[i] != NULL &&
-		WMGetFirstInBag(scr->shortcutWindows[i], wwin) != WBNotFound) {
+		WMGetFirstInArray(scr->shortcutWindows[i], wwin) != WANotFound) {
 		mask |= 1<<i;
 	    }
 	}
@@ -302,7 +302,7 @@ makeWindowState(WWindow *wwin, WApplication *wapp)
 
     if (instance) XFree(instance);
     if (class) XFree(class);
-    if (command) free(command);
+    if (command) wfree(command);
 
     return win_state;
 }
@@ -314,7 +314,7 @@ wSessionSaveState(WScreen *scr)
     WWindow *wwin = scr->focused_window;
     proplist_t win_info, wks;
     proplist_t list=NULL;
-    WMBag *wapp_list=NULL;
+    WMArray *wapp_list=NULL;
 
 
     make_keys();
@@ -327,13 +327,13 @@ wSessionSaveState(WScreen *scr)
 
     list = PLMakeArrayFromElements(NULL);
 
-    wapp_list = WMCreateBag(16);
+    wapp_list = WMCreateArray(16);
 
     while (wwin) {
         WApplication *wapp=wApplicationOf(wwin->main_window);
 
         if (wwin->transient_for==None 
-	    && WMGetFirstInBag(wapp_list, wapp)==WBNotFound
+	    && WMGetFirstInArray(wapp_list, wapp)==WANotFound
 	    && !WFLAGP(wwin, dont_save_session)) {
             /* A entry for this application was not yet saved. Save one. */
             if ((win_info = makeWindowState(wwin, wapp))!=NULL) {
@@ -344,7 +344,7 @@ wSessionSaveState(WScreen *scr)
                  * application list, so no multiple entries for the same
                  * application are saved.
                  */
-		WMPutInBag(wapp_list, wapp);
+		WMAddToArray(wapp_list, wapp);
             }
         }
         wwin = wwin->prev;
@@ -357,7 +357,7 @@ wSessionSaveState(WScreen *scr)
     PLInsertDictionaryEntry(scr->session_state, sWorkspace, wks);
     PLRelease(wks);
 
-    WMFreeBag(wapp_list);
+    WMFreeArray(wapp_list);
 }
 
 
@@ -381,7 +381,7 @@ execCommand(WScreen *scr, char *command, char *host)
     char **argv;
     int argc;
 
-    ParseCommand(command, &argv, &argc);
+    wtokensplit(command, &argv, &argc);
 
     if (argv==NULL) {
         return 0;
@@ -404,8 +404,8 @@ execCommand(WScreen *scr, char *command, char *host)
         exit(111);
     }
     while (argc > 0)
-        free(argv[--argc]);
-    free(argv);
+        wfree(argv[--argc]);
+    wfree(argv);
     return pid;
 }
 
@@ -442,24 +442,19 @@ getWindowState(WScreen *scr, proplist_t win_state)
         state->miniaturized = getBool(value);
     if ((value = PLGetDictionaryEntry(win_state, sHidden))!=NULL)
         state->hidden = getBool(value);
-    if ((value = PLGetDictionaryEntry(win_state, sHidden))!=NULL) {
+    if ((value = PLGetDictionaryEntry(win_state, sShortcutMask))!=NULL) {
         mask = getInt(value);
 	state->window_shortcuts = mask;
     }
-
+    
     value = PLGetDictionaryEntry(win_state, sGeometry);
     if (value && PLIsString(value)) {
-        if (sscanf(PLGetString(value), "%ix%i+%i+%i",
+        if (!(sscanf(PLGetString(value), "%ix%i+%i+%i",
                    &state->w, &state->h, &state->x, &state->y)==4 &&
-            (state->w>0 && state->h>0)) {
-            state->use_geometry = 1;
-        } else if (sscanf(PLGetString(value), "%i,%i,%i,%i",
-                   &state->x, &state->y, &state->w, &state->h)==4 &&
-            (state->w>0 && state->h>0)) { 
-	    /* TODO: remove redundant sscanf() in version 0.20.x */
-            state->use_geometry = 1;
-        }
-
+	      (state->w>0 && state->h>0))) {
+	    state->w = 0;
+	    state->h = 0;
+	}
     }
 
     return state;
@@ -562,11 +557,11 @@ wSessionRestoreState(WScreen *scr)
         } else if ((pid = execCommand(scr, command, host)) > 0) {
             wWindowAddSavedState(instance, class, command, pid, state);
         } else {
-            free(state);
+            wfree(state);
         }
 
-        if (instance) free(instance);
-        if (class) free(class);
+        if (instance) wfree(instance);
+        if (class) wfree(class);
     }
     /* clean up */
     PLSetStringCmpHook(StringCompareHook);
@@ -1155,17 +1150,17 @@ fail:
     SmcSaveYourselfDone(smc_conn, ok);
     
     if (prop[0].vals)
-	free(prop[0].vals);
+	wfree(prop[0].vals);
     if (prop[1].vals)
-	free(prop[1].vals);
+	wfree(prop[1].vals);
     if (discardCmd)
-	free(discardCmd);
+	wfree(discardCmd);
 
     if (!ok) {
 	remove(statefile);
     }
     if (statefile)
-	free(statefile);
+	wfree(statefile);
 }
 
 
