@@ -2,6 +2,7 @@
  *  Window Maker window manager
  *
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1999       Dan Pascu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <errno.h>
 #include <signal.h>
 #include <sys/wait.h>
 #ifdef __FreeBSD__
@@ -55,13 +57,13 @@
 #include "session.h"
 #include "defaults.h"
 #include "properties.h"
+#include "dialog.h"
+#ifdef XDND
+#include "xdnd.h"
+#endif
 
 #include "xutil.h"
 
-#ifdef WMSOUND
-#include "wmsound.h"
-#endif
-
 #ifdef KWM_HINTS
 #include "kwm.h"
 #endif
@@ -77,6 +79,10 @@ extern const char * const sys_siglist[];
 # define SA_RESTART 0
 #endif
 
+/* Just in case, for weirdo systems */
+#ifndef SA_NODEFER
+# define SA_NODEFER 0
+#endif
 
 /****** Global Variables ******/
 
@@ -96,6 +102,11 @@ extern Bool wShapeSupported;
 extern int wShapeEventBase;
 #endif
 
+#ifdef KEEP_XKB_LOCK_STATUS
+extern Bool wXkbSupported;
+extern int wXkbEventBase;
+#endif
+
 /* contexts */
 extern XContext wWinContext;
 extern XContext wAppWinContext;
@@ -119,21 +130,17 @@ extern Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
 extern Atom _XA_WINDOWMAKER_STATE;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
 extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
+extern Atom _XA_WINDOWMAKER_COMMAND;
+extern Atom _XA_WINDOWMAKER_ICON_SIZE;
+extern Atom _XA_WINDOWMAKER_ICON_TILE;
 
 extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
+extern Atom _XA_GNUSTEP_TITLEBAR_STATE;
 
 #ifdef OFFIX_DND
 extern Atom _XA_DND_PROTOCOL;
 extern Atom _XA_DND_SELECTION;
 #endif
-#ifdef XDE_DND
-extern Atom _XA_XDE_REQUEST;
-extern Atom _XA_XDE_ENTER;
-extern Atom _XA_XDE_LEAVE;
-extern Atom _XA_XDE_DATA_AVAILABLE;
-extern Atom _XDE_FILETYPE;
-extern Atom _XDE_URLTYPE;
-#endif
 
 
 /* cursors */
@@ -152,7 +159,7 @@ static unsigned int _ScrollLockMask = 0;
 
 
 
-static void manageAllWindows();
+static void manageAllWindows(WScreen *scr, int crashed);
 
 extern void NotifyDeadProcess(pid_t pid, unsigned char status);
 
@@ -204,7 +211,7 @@ catchXError(Display *dpy, XErrorEvent *error)
  *---------------------------------------------------------------------- 
  */
 static int
-handleXIO(Display *dpy)
+handleXIO(Display *xio_dpy)
 {
     dpy = NULL;
     Exit(0);
@@ -221,7 +228,10 @@ handleXIO(Display *dpy)
 static void
 delayedAction(void *cdata)
 {
-    WDelayedActionSet = 0;
+    if (WDelayedActionSet == 0) {
+	return;
+    }
+    WDelayedActionSet--;
     /* 
      * Make the event dispatcher do whatever it needs to do,
      * including handling zombie processes, restart and exit 
@@ -231,6 +241,58 @@ delayedAction(void *cdata)
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ * handleExitSig--
+ * 	User generated exit signal handler.
+ *---------------------------------------------------------------------- 
+ */
+static RETSIGTYPE
+handleExitSig(int sig)
+{
+    sigset_t sigs;
+
+    sigfillset(&sigs);
+    sigprocmask(SIG_BLOCK, &sigs, NULL);
+    
+    if (sig == SIGUSR1) {
+#ifdef SYS_SIGLIST_DECLARED
+        wwarning(_("got signal %i (%s) - restarting\n"), sig, sys_siglist[sig]);
+#else
+        wwarning(_("got signal %i - restarting\n"), sig);
+#endif
+
+	SIG_WCHANGE_STATE(WSTATE_NEED_RESTART);
+	/* setup idle handler, so that this will be handled when
+	 * the select() is returned becaused of the signal, even if
+	 * there are no X events in the queue */
+	WDelayedActionSet++;
+    } else if (sig == SIGUSR2) {
+#ifdef SYS_SIGLIST_DECLARED
+        wwarning(_("got signal %i (%s) - rereading defaults\n"), sig, sys_siglist[sig]);
+#else
+        wwarning(_("got signal %i - rereading defaults\n"), sig);
+#endif
+
+        SIG_WCHANGE_STATE(WSTATE_NEED_REREAD);
+        /* setup idle handler, so that this will be handled when
+         * the select() is returned becaused of the signal, even if
+         * there are no X events in the queue */
+        WDelayedActionSet++;
+    } else if (sig == SIGINT || sig == SIGHUP) {
+#ifdef SYS_SIGLIST_DECLARED
+        wwarning(_("got signal %i (%s) - exiting...\n"), sig, sys_siglist[sig]);
+#else
+        wwarning(_("got signal %i - exiting...\n"), sig);
+#endif
+
+	SIG_WCHANGE_STATE(WSTATE_NEED_EXIT);
+
+	WDelayedActionSet++;
+    }
+    
+    sigprocmask(SIG_UNBLOCK, &sigs, NULL);
+}
 
 /*
  *----------------------------------------------------------------------
@@ -244,6 +306,7 @@ handleSig(int sig)
     static int already_crashed = 0;
     int dumpcore = 0;
 #ifndef NO_EMERGENCY_AUTORESTART
+    int crashAction;
     char *argv[2];
     
     argv[1] = NULL;
@@ -254,38 +317,6 @@ handleSig(int sig)
      * here. Xlib calls are not reentrant so the integrity of Xlib is
      * not guaranteed if a Xlib call is made from a signal handler.
      */
-    if (sig == SIGUSR1) {
-#ifdef SYS_SIGLIST_DECLARED
-        wwarning(_("got signal %i (%s) - restarting\n"), sig, sys_siglist[sig]);
-#else
-        wwarning(_("got signal %i - restarting\n"), sig);
-#endif
-
-	WCHANGE_STATE(WSTATE_NEED_RESTART);
-
-	/* setup idle handler, so that this will be handled when
-	 * the select() is returned becaused of the signal, even if
-	 * there are no X events in the queue */
-	if (!WDelayedActionSet) {
-	    WDelayedActionSet = 1;
-	    WMAddIdleHandler(delayedAction, NULL);
-	}
-        return;
-    } else if (sig == SIGTERM || sig == SIGHUP) {
-#ifdef SYS_SIGLIST_DECLARED
-        wwarning(_("got signal %i (%s) - exiting...\n"), sig, sys_siglist[sig]);
-#else
-        wwarning(_("got signal %i - exiting...\n"), sig);
-#endif
-
-	WCHANGE_STATE(WSTATE_NEED_EXIT);
-
-	if (!WDelayedActionSet) {
-	    WDelayedActionSet = 1;
-	    WMAddIdleHandler(delayedAction, NULL);
-	}
-	return;
-    }
 
 #ifdef SYS_SIGLIST_DECLARED
     wfatal(_("got signal %i (%s)\n"), sig, sys_siglist[sig]);
@@ -293,78 +324,138 @@ handleSig(int sig)
     wfatal(_("got signal %i\n"), sig);
 #endif
 
+    /* Setting the signal behaviour back to default and then reraising the
+     * signal is a cleaner way to make program exit and core dump than calling
+     * abort(), since it correctly returns from the signal handler and sets
+     * the flags accordingly. -Dan
+     */
     if (sig==SIGSEGV || sig==SIGFPE || sig==SIGBUS || sig==SIGILL
 	|| sig==SIGABRT) {
 	if (already_crashed) {
 	    wfatal(_("crashed while trying to do some post-crash cleanup. Aborting immediatelly."));
-#ifndef NO_EMERGENCY_AUTORESTART
-	    exit(1);
-#else
-	    abort();
-#endif
+            signal(sig, SIG_DFL);
+            kill(getpid(), sig);
+            return;
 	}
 	already_crashed = 1;
 
 	dumpcore = 1;
 
-	wfatal(_("a fatal error has occured, probably due to a bug. "
-		 "Please fill the included BUGFORM and report it."));
-
+	/*
+	 * Yeah, we shouldn't do this, but it's already crashed anyway :P
+	 */
+	
 #ifndef NO_EMERGENCY_AUTORESTART
-    	/* restart another window manager so that the X session doesn't
-	 * go to space */
+        /* Close the X connection and open a new one. This is to avoid messing
+         * Xlib because we call to Xlib functions in a signal handler.
+         */
+        if (dpy)
+            XCloseDisplay(dpy);
+        dpy = XOpenDisplay("");
+        if (dpy) {
+            CARD32 data[2];
+            WWindow *wwin;
+            int i;
 
-    	wwarning(_("trying to start alternative window manager..."));
-    	if (dpy)
-	    XCloseDisplay(dpy);
-    	dpy = NULL;
+	    XGrabServer(dpy);
+            crashAction = wShowCrashingDialogPanel(sig);
 
-    	argv[0] = FALLBACK_WINDOWMANAGER;
-    	execvp(FALLBACK_WINDOWMANAGER, argv);
+            /*
+             * Save current workspace, so can go back to it if restarting.
+             * Also add hint that we crashed, so that windows will be placed
+             * correctly upon restart.
+             */
+            for (i=0; i<wScreenCount; i++) {
+                data[0] = wScreen[i]->current_workspace; /* workspace number */
+                data[1] = WFLAGS_CRASHED; /* signal that we crashed */
 
-    	argv[0] = "fvwm";
-    	execvp("fvwm", argv);
+                XChangeProperty(dpy, wScreen[i]->root_win,
+                                _XA_WINDOWMAKER_STATE, _XA_WINDOWMAKER_STATE,
+                                32, PropModeReplace, (unsigned char*) data, 2);
 
-    	argv[0] = "twm";
-    	execvp("twm", argv);
+                /* save state of windows */
+                wwin = wScreen[i]->focused_window;
+                while (wwin) {
+                    wWindowSaveState(wwin);
+                    wwin = wwin->prev;
+                }
+
+            }
+    
+            XCloseDisplay(dpy);
+            dpy = NULL;
+        } else {
+            wsyserror(_("cannot open connection for crashing dialog panel. Aborting."));
+            crashAction = WMAbort;
+        }
+
+        if (crashAction == WMAbort) {
+            signal(sig, SIG_DFL);
+            kill(getpid(), sig);
+            return;
+        }
+
+        if (crashAction == WMRestart) {
+            /* we try to restart Window Maker */
+            wmessage(_("trying to restart Window Maker..."));
+            Restart(NULL, False);
+            /* fallback to alternate window manager then */
+        }
+
+    	wmessage(_("trying to start alternate window manager..."));
+
+        Restart(FALLBACK_WINDOWMANAGER, False);
+        Restart("fvwm", False);
+        Restart("twm", False);
+        wfatal(_("failed to start alternate window manager. Aborting."));
+#else
+        wfatal(_("a fatal error has occured, probably due to a bug. "
+                 "Please fill the included BUGFORM and report it."));
 #endif /* !NO_EMERGENCY_AUTORESTART */
+
+        signal(sig, SIG_DFL);
+        kill(getpid(), sig);
+        return;
+
     }
 
     wAbort(dumpcore);
 }
 
 
-static RETSIGTYPE
-ignoreSig(int signal)
-{
-    return;
-}
-
-
 static RETSIGTYPE
 buryChild(int foo)
 {
     pid_t pid;
     int status;
+    int save_errno = errno;
+    sigset_t sigs;
+
+    sigfillset(&sigs);
+    /* Block signals so that NotifyDeadProcess() doesn't get fux0red */
+    sigprocmask(SIG_BLOCK, &sigs, NULL);
     
     /* R.I.P. */
-    pid = waitpid(-1, &status, WNOHANG);
-    if (pid>0) {
+    /* If 2 or more kids exit in a small time window, before this handler gets
+     * the chance to get invoked, the SIGCHLD signals will be merged and only
+     * one SIGCHLD signal will be sent to us. We use a while loop to get all
+     * exited child status because we can't count on the number of SIGCHLD
+     * signals to know exactly how many kids have exited. -Dan
+     */
+    while ((pid=waitpid(-1, &status, WNOHANG))>0 || (pid<0 && errno==EINTR)) {
 	NotifyDeadProcess(pid, WEXITSTATUS(status));
-	/* 
-	 * Make sure that the kid will be buried even if there are
-	 * no events in the X event queue
-	 */
-	if (!WDelayedActionSet) {
-	    WDelayedActionSet = 1;
-	    WMAddIdleHandler(delayedAction, NULL);
-	}
     }
+
+    WDelayedActionSet++;
+    
+    sigprocmask(SIG_UNBLOCK, &sigs, NULL);
+
+    errno = save_errno;    
 }
 
 
 static int
-getWorkspaceState(Window root, WWorkspaceState **state)            
+getWorkspaceState(Window root, WWorkspaceState **state)
 {
     Atom type_ret;
     int fmt_ret;    
@@ -376,19 +467,18 @@ getWorkspaceState(Window root, WWorkspaceState **state)
                            True, _XA_WINDOWMAKER_STATE,
                            &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
                            (unsigned char **)&data)!=Success || !data)
-      return 0;
+        return 0;
+
+    *state = wmalloc(sizeof(WWorkspaceState));
+    (*state)->workspace = data[0];
+    (*state)->flags = data[1];
 
-    *state = malloc(sizeof(WWorkspaceState));
-    if (*state) {
-	(*state)->flags = data[0];
-	(*state)->workspace = data[1];
-    }
     XFree(data);
 
     if (*state && type_ret==_XA_WINDOWMAKER_STATE)
-      return 1;
+        return 1;
     else
-      return 0;
+        return 0;
 }
 
 
@@ -600,6 +690,31 @@ wScreenForWindow(Window window)
 }
 
 
+static char *atomNames[] = {
+    "WM_STATE", 
+    "WM_CHANGE_STATE", 
+    "WM_PROTOCOLS", 
+    "WM_TAKE_FOCUS", 
+    "WM_DELETE_WINDOW", 
+    "WM_SAVE_YOURSELF", 
+    "WM_CLIENT_LEADER", 
+    "WM_COLORMAP_WINDOWS", 
+    "WM_COLORMAP_NOTIFY", 
+
+    "_WINDOWMAKER_MENU",
+    "_WINDOWMAKER_STATE", 
+    "_WINDOWMAKER_WM_PROTOCOLS", 
+    "_WINDOWMAKER_WM_FUNCTION",
+    "_WINDOWMAKER_NOTICEBOARD",
+    "_WINDOWMAKER_COMMAND",
+    "_WINDOWMAKER_ICON_SIZE",
+    "_WINDOWMAKER_ICON_TILE",
+
+    GNUSTEP_WM_ATTR_NAME,
+    GNUSTEP_WM_MINIATURIZE_WINDOW,
+    GNUSTEP_TITLEBAR_STATE
+};
+
 
 /*
  *----------------------------------------------------------
@@ -617,6 +732,7 @@ StartUp(Bool defaultScreenOnly)
     WWorkspaceState *ws_state;
     struct sigaction sig_action;
     int j, max;
+    Atom atom[sizeof(atomNames)/sizeof(char*)];
 
     /*
      * Ignore CapsLock in modifiers
@@ -638,83 +754,100 @@ StartUp(Bool defaultScreenOnly)
 
 /*    _XA_VERSION = XInternAtom(dpy, "VERSION", False);*/
 
-    _XA_WM_STATE = XInternAtom(dpy, "WM_STATE", False);
-    _XA_WM_CHANGE_STATE = XInternAtom(dpy, "WM_CHANGE_STATE", False);
-    _XA_WM_PROTOCOLS = XInternAtom(dpy, "WM_PROTOCOLS", False);
-    _XA_WM_TAKE_FOCUS = XInternAtom(dpy, "WM_TAKE_FOCUS", False);
-    _XA_WM_DELETE_WINDOW = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
-    _XA_WM_SAVE_YOURSELF = XInternAtom(dpy, "WM_SAVE_YOURSELF", False);    
-    _XA_WM_CLIENT_LEADER = XInternAtom(dpy, "WM_CLIENT_LEADER", False);
-    _XA_WM_COLORMAP_WINDOWS = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
-    _XA_WM_COLORMAP_NOTIFY = XInternAtom(dpy, "WM_COLORMAP_NOTIFY", False);
+#ifdef HAVE_XINTERNATOMS
+    XInternAtoms(dpy, atomNames, sizeof(atomNames)/sizeof(char*),
+		 False, atom);
+#else
 
-    _XA_GNUSTEP_WM_ATTR = XInternAtom(dpy, GNUSTEP_WM_ATTR_NAME, False);
-
-    _XA_WINDOWMAKER_MENU = XInternAtom(dpy, "_WINDOWMAKER_MENU", False);
-    _XA_WINDOWMAKER_STATE = XInternAtom(dpy, "_WINDOWMAKER_STATE", False);
-
-    _XA_WINDOWMAKER_WM_PROTOCOLS =
-	XInternAtom(dpy, "_WINDOWMAKER_WM_PROTOCOLS", False);
-
-    _XA_GNUSTEP_WM_MINIATURIZE_WINDOW = 
-	XInternAtom(dpy, GNUSTEP_WM_MINIATURIZE_WINDOW, False);
+    {
+	int i;
+	for (i = 0; i < sizeof(atomNames)/sizeof(char*); i++) {
+	    atom[i] = XInternAtom(dpy, atomNames[i], False);
+	}
+    }
+#endif
     
-    _XA_WINDOWMAKER_WM_FUNCTION = XInternAtom(dpy, "_WINDOWMAKER_WM_FUNCTION",
-					      False);
+    _XA_WM_STATE = atom[0];
+    _XA_WM_CHANGE_STATE = atom[1];
+    _XA_WM_PROTOCOLS = atom[2];
+    _XA_WM_TAKE_FOCUS = atom[3];
+    _XA_WM_DELETE_WINDOW = atom[4];
+    _XA_WM_SAVE_YOURSELF = atom[5];
+    _XA_WM_CLIENT_LEADER = atom[6];
+    _XA_WM_COLORMAP_WINDOWS = atom[7];
+    _XA_WM_COLORMAP_NOTIFY = atom[8];
 
-    _XA_WINDOWMAKER_NOTICEBOARD = XInternAtom(dpy, "_WINDOWMAKER_NOTICEBOARD",
-					      False);
+    _XA_WINDOWMAKER_MENU = atom[9];
+    _XA_WINDOWMAKER_STATE = atom[10];
+    _XA_WINDOWMAKER_WM_PROTOCOLS = atom[11];
+    _XA_WINDOWMAKER_WM_FUNCTION = atom[12];
+    _XA_WINDOWMAKER_NOTICEBOARD = atom[13];
+    _XA_WINDOWMAKER_COMMAND = atom[14];
+    _XA_WINDOWMAKER_ICON_SIZE = atom[15];
+    _XA_WINDOWMAKER_ICON_TILE = atom[16];
+
+    _XA_GNUSTEP_WM_ATTR = atom[17];
+    _XA_GNUSTEP_WM_MINIATURIZE_WINDOW = atom[18];
+    _XA_GNUSTEP_TITLEBAR_STATE = atom[19];
 
 #ifdef OFFIX_DND
     _XA_DND_SELECTION = XInternAtom(dpy, "DndSelection", False);
     _XA_DND_PROTOCOL = XInternAtom(dpy, "DndProtocol", False);
 #endif
-#ifdef XDE_DND
-    _XA_XDE_ENTER = XInternAtom(dpy, "_XDE_ENTER", False);
-    _XA_XDE_REQUEST = XInternAtom(dpy, "_XDE_REQUEST", False);
-    _XA_XDE_LEAVE = XInternAtom(dpy, "_XDE_LEAVE", False);
-    _XA_XDE_DATA_AVAILABLE = XInternAtom(dpy, "_XDE_DATA_AVAILABLE", False);
-    _XDE_FILETYPE = XInternAtom(dpy, "file:ALL", False);
-    _XDE_URLTYPE = XInternAtom(dpy, "url:ALL", False);
+#ifdef XDND
+    wXDNDInitializeAtoms();
 #endif
 
 
     /* cursors */
+#ifdef DEFINABLE_CURSOR
+    wCursor[WCUR_NORMAL] = None;
+#else
     wCursor[WCUR_NORMAL] = XCreateFontCursor(dpy, XC_left_ptr);    
+#endif
+    wCursor[WCUR_ROOT] = XCreateFontCursor(dpy, XC_left_ptr);
     wCursor[WCUR_ARROW] = XCreateFontCursor(dpy, XC_top_left_arrow);
     wCursor[WCUR_MOVE] = XCreateFontCursor(dpy, XC_fleur);
     wCursor[WCUR_RESIZE] = XCreateFontCursor(dpy, XC_sizing);
+    wCursor[WCUR_TOPLEFTRESIZE] = XCreateFontCursor(dpy, XC_top_left_corner);
+    wCursor[WCUR_TOPRIGHTRESIZE] = XCreateFontCursor(dpy, XC_top_right_corner);
+    wCursor[WCUR_BOTTOMLEFTRESIZE] = XCreateFontCursor(dpy, XC_bottom_left_corner);
+    wCursor[WCUR_BOTTOMRIGHTRESIZE] = XCreateFontCursor(dpy, XC_bottom_right_corner);
+    wCursor[WCUR_VERTICALRESIZE] = XCreateFontCursor(dpy, XC_sb_v_double_arrow);
+    wCursor[WCUR_HORIZONRESIZE] = XCreateFontCursor(dpy, XC_sb_h_double_arrow);
     wCursor[WCUR_WAIT] = XCreateFontCursor(dpy, XC_watch);
     wCursor[WCUR_QUESTION] = XCreateFontCursor(dpy, XC_question_arrow);
     wCursor[WCUR_TEXT]     = XCreateFontCursor(dpy, XC_xterm); /* odd name???*/
+    wCursor[WCUR_SELECT] = XCreateFontCursor(dpy, XC_cross);
     
+    /* signal handler stuff that gets called when a signal is caught */
+    WMAddPersistentTimerHandler(500, delayedAction, NULL);
+
     /* emergency exit... */
     sig_action.sa_handler = handleSig;
     sigemptyset(&sig_action.sa_mask);
 
-    /* Here we don't care about SA_RESTART since these signals will close
-     * wmaker anyway.
-     * -Dan */
-    sig_action.sa_flags = 0;
-    sigaction(SIGINT, &sig_action, NULL);
-    sigaction(SIGTERM, &sig_action, NULL);
-    sigaction(SIGHUP, &sig_action, NULL);
+    sig_action.sa_flags = SA_RESTART;
     sigaction(SIGQUIT, &sig_action, NULL);
     sigaction(SIGSEGV, &sig_action, NULL);
     sigaction(SIGBUS, &sig_action, NULL);
     sigaction(SIGFPE, &sig_action, NULL);
-#ifndef NO_EMERGENCY_AUTORESTART
     sigaction(SIGABRT, &sig_action, NULL);
-#endif
 
+    sig_action.sa_handler = handleExitSig;
+    
     /* Here we set SA_RESTART for safety, because SIGUSR1 may not be handled
      * immediately.
      * -Dan */
     sig_action.sa_flags = SA_RESTART;
+/*    sigaction(SIGTERM, &sig_action, NULL);*/
+    sigaction(SIGINT, &sig_action, NULL);    
+    sigaction(SIGHUP, &sig_action, NULL);    
     sigaction(SIGUSR1, &sig_action, NULL);
+    sigaction(SIGUSR2, &sig_action, NULL);
 
     /* ignore dead pipe */
-    sig_action.sa_handler = ignoreSig;
+    sig_action.sa_handler = SIG_IGN;
     sig_action.sa_flags = SA_RESTART;
     sigaction(SIGPIPE, &sig_action, NULL);
 
@@ -723,6 +856,17 @@ StartUp(Bool defaultScreenOnly)
     sig_action.sa_flags = SA_NOCLDSTOP|SA_RESTART;
     sigaction(SIGCHLD, &sig_action, NULL);
 
+    /* Now we unblock all signals, that may have been blocked by the parent
+     * who exec()-ed us. This can happen for example if Window Maker crashes
+     * and restarts itself or another window manager from the signal handler.
+     * In this case, the new proccess inherits the blocked signal mask and
+     * will no longer react to that signal, until unblocked.
+     * This is because the signal handler of the proccess who crashed (parent)
+     * didn't return, and the signal remained blocked. -Dan
+     */
+    sigfillset(&sig_action.sa_mask);
+    sigprocmask(SIG_UNBLOCK, &sig_action.sa_mask, NULL);
+
     /* handle X shutdowns a such */
     XSetIOErrorHandler(handleXIO);
 
@@ -762,15 +906,18 @@ StartUp(Bool defaultScreenOnly)
 
     XSetErrorHandler((XErrorHandler)catchXError);
 
-    /* Sound init */
-#ifdef WMSOUND
-    wSoundInit(dpy);
-#endif
-
 #ifdef SHAPE
     /* ignore j */
     wShapeSupported = XShapeQueryExtension(dpy, &wShapeEventBase, &j);
 #endif
+    
+#ifdef KEEP_XKB_LOCK_STATUS
+    wXkbSupported = XkbQueryExtension(dpy, NULL, &wXkbEventBase, NULL, NULL, NULL);
+    if(wPreferences.modelock && !wXkbSupported) {
+        wwarning(_("XKB is not supported. KbdModeLock is automatically disabled."));
+        wPreferences.modelock = 0;
+    }
+#endif
 
     if (defaultScreenOnly) {
 	max = 1;
@@ -786,7 +933,7 @@ StartUp(Bool defaultScreenOnly)
 	if (defaultScreenOnly || max==1) {
 	    wScreen[wScreenCount] = wScreenInit(DefaultScreen(dpy));
 	    if (!wScreen[wScreenCount]) {
-		wfatal(_("it seems that there already is a window manager running"));
+		wfatal(_("it seems that there is already a window manager running"));
 		Exit(1);
 	    }
 	} else {
@@ -801,7 +948,9 @@ StartUp(Bool defaultScreenOnly)
 
     /* initialize/restore state for the screens */
     for (j = 0; j < wScreenCount; j++) {
-	/* restore workspace state */
+        int crashed;
+
+        /* restore workspace state */
 	if (!getWorkspaceState(wScreen[j]->root_win, &ws_state)) {
 	    ws_state = NULL;
 	}
@@ -811,29 +960,35 @@ StartUp(Bool defaultScreenOnly)
 	/* manage all windows that were already here before us */
 	if (!wPreferences.flags.nodock && wScreen[j]->dock)
 	    wScreen[j]->last_dock = wScreen[j]->dock;
-	
-	manageAllWindows(wScreen[j]);
-	
+
+        if (ws_state && (ws_state->flags & WFLAGS_CRASHED)!=0)
+            crashed = 1;
+        else
+            crashed = 0;
+
+        manageAllWindows(wScreen[j], crashed);
+
 	/* restore saved menus */
 	wMenuRestoreState(wScreen[j]);
 	
 	/* If we're not restarting restore session */
-	if (ws_state == NULL)
+	if (ws_state == NULL && !wPreferences.flags.norestore)
 	    wSessionRestoreState(wScreen[j]);
 	
-	
-	/* auto-launch apps */
-	if (!wPreferences.flags.nodock && wScreen[j]->dock) {
-	    wScreen[j]->last_dock = wScreen[j]->dock;
-	    wDockDoAutoLaunch(wScreen[j]->dock, 0);
-	}
-	/* auto-launch apps in clip */
-	if (!wPreferences.flags.noclip) {
-	    int i;
-	    for(i=0; i<wScreen[j]->workspace_count; i++) {
-		if (wScreen[j]->workspaces[i]->clip) {
-		    wScreen[j]->last_dock = wScreen[j]->workspaces[i]->clip;
-		    wDockDoAutoLaunch(wScreen[j]->workspaces[i]->clip, i);
+	if (!wPreferences.flags.noautolaunch) {
+	    /* auto-launch apps */
+	    if (!wPreferences.flags.nodock && wScreen[j]->dock) {
+		wScreen[j]->last_dock = wScreen[j]->dock;
+		wDockDoAutoLaunch(wScreen[j]->dock, 0);
+	    }
+	    /* auto-launch apps in clip */
+	    if (!wPreferences.flags.noclip) {
+		int i;
+		for(i=0; i<wScreen[j]->workspace_count; i++) {
+		    if (wScreen[j]->workspaces[i]->clip) {
+			wScreen[j]->last_dock = wScreen[j]->workspaces[i]->clip;
+			wDockDoAutoLaunch(wScreen[j]->workspaces[i]->clip, i);
+		    }
 		}
 	    }
 	}
@@ -841,7 +996,7 @@ StartUp(Bool defaultScreenOnly)
 	/* go to workspace where we were before restart */
 	if (ws_state) { 
 	    wWorkspaceForceChange(wScreen[j], ws_state->workspace);
-	    free(ws_state);
+	    wfree(ws_state);
 	} else {
 	    wSessionRestoreLastWorkspace(wScreen[j]);
 	}
@@ -856,7 +1011,7 @@ StartUp(Bool defaultScreenOnly)
 	Exit(1);
     }
 
-    if (!wPreferences.flags.noupdates) {
+    if (!wPreferences.flags.nopolling && !wPreferences.flags.noupdates) {
 	/* setup defaults file polling */
 	WMAddTimerHandler(3000, wDefaultsCheckDomains, NULL);
     }
@@ -887,7 +1042,7 @@ windowInList(Window window, Window *list, int count)
  *----------------------------------------------------------------------- 
  */
 static void
-manageAllWindows(WScreen *scr)
+manageAllWindows(WScreen *scr, int crashRecovery)
 {
     Window root, parent;
     Window *children;
@@ -950,7 +1105,16 @@ manageAllWindows(WScreen *scr)
 		wIconifyWindow(wwin);
 	    } else {
 		wClientSetState(wwin, NormalState, None);
-	    }
+            }
+            if (crashRecovery) {
+                int border;
+
+                border = (WFLAGP(wwin, no_border) ? 0 : FRAME_BORDER_WIDTH);
+
+                wWindowMove(wwin, wwin->frame_x - border,
+                            wwin->frame_y - border -
+                            wwin->frame->titlebar->height);
+            }
 	}
     }
     XUngrabServer(dpy);
