@@ -25,7 +25,7 @@
 
 #include "config.h"
 
-/*** Change it (wconfig.h) *after* you ran configure ***/
+/*** Change this file (wconfig.h) *after* you ran configure ***/
 
 /*
  *--------------------------------------------------------------------
@@ -45,7 +45,7 @@
 #define TEXTURE_PLUGIN
 
 #ifdef TEXTURE_PLUGIN
-#define DRAWSTRING_PLUGIN
+#undef DRAWSTRING_PLUGIN
 #endif
 
 
@@ -106,13 +106,6 @@
  */
 #define FALLBACK_WINDOWMANAGER "blackbox"
 
-
-/* Define if you have a 5 button mouse and want to use button 4
- * (in the root window) for switching to the previous workspace
- * and 5 for the next. This will not work if you enable
- * DisableWSMouseActions or compile wmaker with --enable-lite */
-#undef MOUSE_WS_SWITCH
- 
 /*
  * Turn on a hack to make mouse and keyboard actions work even if
  * the NumLock or ScrollLock modifiers are turned on. They might
@@ -125,13 +118,6 @@
 
 
 
-/*
- * define EXTEND_WINDOWSHORTCUT if you want 10 window shortcuts instead of 4. 
- */
-#define EXTEND_WINDOWSHORTCUT
-
-
-
 /*
  * define OPTIMIZE_SHAPE if you want the shape setting code to be optimized
  * for applications that change their shape frequently (like xdaliclock
@@ -431,6 +417,12 @@
 /* workspace name on switch display */
 #define WORKSPACE_NAME_FADE_DELAY 30
 
+#ifdef VIRTUAL_DESKTOP 
+/* workspace virtual edge speed */
+#define VIRTUALEDGE_SCROLL_VSTEP 30
+#define VIRTUALEDGE_SCROLL_HSTEP 30
+#endif
+
 #define WORKSPACE_NAME_DELAY	400
 
 /* window birth animation steps (DO NOT MAKE IT RUN-TIME) */
@@ -526,11 +518,7 @@
 
 
 /* number of window shortcuts */
-#ifdef EXTEND_WINDOWSHORTCUT
-# define MAX_WINDOW_SHORTCUTS 10
-#else
-# define MAX_WINDOW_SHORTCUTS 4
-#endif
+#define MAX_WINDOW_SHORTCUTS 10
 
 
 #define WM_PI 3.14159265358979323846
@@ -604,7 +592,7 @@
 # define NORMAL_ICON_KABOOM
 #endif
 
-#if HAVE_LIBINTL_H && I18N
+#if defined(HAVE_LIBINTL_H) && defined(I18N)
 # include <libintl.h>
 # define _(text) gettext(text)
 #else
