@@ -84,6 +84,7 @@ static proplist_t AStartHidden;	       /* app */
 static proplist_t ADontSaveSession;    /* app */
 static proplist_t AEmulateAppIcon;
 static proplist_t AFullMaximize;
+static proplist_t ACollapseAppIcons;   /* app */
 #ifdef XKB_BUTTON_HINT
 static proplist_t ANoLanguageButton;
 #endif
@@ -124,6 +125,7 @@ init_wdefaults(WScreen *scr)
     ADontSaveSession = PLMakeString("DontSaveSession");
     AEmulateAppIcon = PLMakeString("EmulateAppIcon");
     AFullMaximize = PLMakeString("FullMaximize");
+    ACollapseAppIcons = PLMakeString("CollapseAppIcons");
 #ifdef XKB_BUTTON_HINT
     ANoLanguageButton = PLMakeString("NoLanguageButton");
 #endif
@@ -205,7 +207,7 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
       char *buffer = NULL;
 	  buffer = wmalloc(strlen(class)+strlen(instance)+4);
       key1 = PLMakeString(strcat(strcat(strcpy(buffer,instance),"."),class));
-	  free(buffer);
+	  wfree(buffer);
     } else
       key1 = NULL;
 
@@ -278,6 +280,9 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
     value = get_value(dw, dc, dn, da, ANoAppIcon, No, useGlobalDefault);
     APPLY_VAL(value, no_appicon, ANoAppIcon);
 
+    value = get_value(dw, dc, dn, da, ACollapseAppIcons, No, useGlobalDefault);
+    APPLY_VAL(value, collapse_appicons, ACollapseAppIcons);
+    
     value = get_value(dw, dc, dn, da, AKeepOnTop, No, useGlobalDefault);
     APPLY_VAL(value, floating, AKeepOnTop);
 
@@ -345,7 +350,7 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option,
 
 	dict = PLGetDictionaryEntry(WDWindowAttributes->dictionary, key);
 	PLRelease(key);
-	free(buffer);
+	wfree(buffer);
 
 	if (dict) {
 	    value = PLGetDictionaryEntry(dict, option);
@@ -434,7 +439,7 @@ wDefaultGetImage(WScreen *scr, char *winstance, char *wclass)
     if (!image) {
 	wwarning(_("error loading image file \"%s\""), path, RMessageForError(RErrorCode));
     }
-    free(path);
+    wfree(path);
 
     image = wIconValidateIconSize(scr, image);
 
@@ -510,7 +515,7 @@ wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
         buffer = wmalloc(strlen(instance) + strlen(class) + 2);
         strcat(strcat(strcpy(buffer, instance), "."), class);
         key = PLMakeString(buffer);
-        free(buffer);
+        wfree(buffer);
     } else if (instance) {
         key = PLMakeString(instance);
     } else if (class) {
