@@ -227,10 +227,10 @@ wWindowDestroy(WWindow *wwin)
 	if (!wwin->screen_ptr->shortcutWindows[i])
 	    continue;
 
-	WMRemoveFromBag(wwin->screen_ptr->shortcutWindows[i], wwin);
+	WMRemoveFromArray(wwin->screen_ptr->shortcutWindows[i], wwin);
 
-	if (!WMGetBagItemCount(wwin->screen_ptr->shortcutWindows[i])) {
-	    WMFreeBag(wwin->screen_ptr->shortcutWindows[i]);
+	if (!WMGetArrayItemCount(wwin->screen_ptr->shortcutWindows[i])) {
+	    WMFreeArray(wwin->screen_ptr->shortcutWindows[i]);
 	    wwin->screen_ptr->shortcutWindows[i] = NULL;
 	}
     }
@@ -248,7 +248,7 @@ wWindowDestroy(WWindow *wwin)
       XFree(wwin->wm_class);
 
     if (wwin->wm_gnustep_attr)
-      free(wwin->wm_gnustep_attr);
+      wfree(wwin->wm_gnustep_attr);
 
     if (wwin->cmap_windows)
       XFree(wwin->cmap_windows);
@@ -396,51 +396,44 @@ wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace)
 
 	if (wwin->wm_gnustep_attr->flags & GSWindowLevelAttr) {
 
-	    switch (wwin->wm_gnustep_attr->window_level) {
-	     case WMNormalWindowLevel:
-		*level = WMNormalLevel;
-		break;
-	     case WMFloatingWindowLevel:
-		*level = WMFloatingLevel;
-		break;
-	     case WMDockWindowLevel:
-		*level = WMDockLevel;
-		break;
-	     case WMSubmenuWindowLevel:
-		*level = WMSubmenuLevel;
-		break;
-	     case WMMainMenuWindowLevel:
-		*level = WMMainMenuLevel;
-		break;
-	     default:
-		*level = WMNormalLevel;
-		break;
-	    }
+	    *level = wwin->wm_gnustep_attr->window_level;
+	    /*
+	     * INT_MIN is the only illegal window level.
+	     */
+	    if (*level == INT_MIN)
+		*level = INT_MIN + 1;
 	} else {
 	    /* setup defaults */
 	    *level = WMNormalLevel;
 	}
     } else {
 	int tmp_workspace = -1;
-	int tmp_level = -1;
+	int tmp_level = INT_MIN; /* INT_MIN is never used by the window levels */
+	Bool check;
+	
+	check = False;
 	
 #ifdef MWM_HINTS
 	wMWMCheckClientHints(wwin);
 #endif /* MWM_HINTS */
 
-#ifdef KWM_HINTS
-	wKWMCheckClientHints(wwin, &tmp_level, &tmp_workspace);
-#endif /* KWM_HINTS */
-	
 #ifdef GNOME_STUFF
-	wGNOMECheckClientHints(wwin, &tmp_level, &tmp_workspace);
+	check = wGNOMECheckClientHints(wwin, &tmp_level, &tmp_workspace);
 #endif /* GNOME_STUFF */
+	
+#ifdef KWM_HINTS
+	if (!check)
+	    wKWMCheckClientHints(wwin, &tmp_level, &tmp_workspace);
+#endif /* KWM_HINTS */
 
 #ifdef OLWM_HINTS
 	wOLWMCheckClientHints(wwin);
 #endif /* OLWM_HINTS */
 
-	if (tmp_level < 0) {
+        /* window levels are between INT_MIN+1 and INT_MAX, so if we still
+         * have INT_MIN that means that no window level was requested. -Dan
+         */
+	if (tmp_level == INT_MIN) {
 	    if (WFLAGP(wwin, floating))
 		*level = WMFloatingLevel;
 	    else if (WFLAGP(wwin, sunken))
@@ -528,6 +521,7 @@ wWindowObscuresWindow(WWindow *wwin, WWindow *obscured)
 }
 
 
+
 /*
  *----------------------------------------------------------------
  * wManageWindow--
@@ -769,12 +763,18 @@ wManageWindow(WScreen *scr, Window window)
 	wwin->flags.maximized = MAX_VERTICAL|MAX_HORIZONTAL;
     }
 
+    {
+	Bool bla;
+	
+	bla = False;
 #ifdef GNOME_STUFF
-    wGNOMECheckInitialClientState(wwin);
+	bla = wGNOMECheckInitialClientState(wwin);
 #endif
 #ifdef KWM_HINTS
-    wKWMCheckClientInitialState(wwin);
+	if (!bla)
+	    wKWMCheckClientInitialState(wwin);
 #endif
+    }
 
     /* apply previous state if it exists and we're in startup */
     if (scr->flags.startup && wm_state >= 0) {
@@ -808,6 +808,7 @@ wManageWindow(WScreen *scr, Window window)
 	    !WFLAGP(wwin, no_miniaturizable)) {
             wwin->flags.miniaturized = win_state->state->miniaturized;
         }
+
         if (!IS_OMNIPRESENT(wwin)) {
             int w = wDefaultGetStartWorkspace(scr, wwin->wm_instance,
                                               wwin->wm_class);
@@ -832,21 +833,44 @@ wManageWindow(WScreen *scr, Window window)
 	    wwin->flags.shaded = wstate->shaded;
 	    wwin->flags.hidden = wstate->hidden;
 	    wwin->flags.miniaturized = wstate->miniaturized;
-	    workspace = wstate->workspace;
+	    wwin->flags.maximized = wstate->maximized;
+	    if (wwin->flags.maximized) {
+		wwin->old_geometry.x = wstate->x;
+		wwin->old_geometry.y = wstate->y;
+		wwin->old_geometry.width = wstate->w;
+		wwin->old_geometry.height = wstate->h;
+	    }
 
-	    if (scr->flags.startup && wstate->window_shortcuts > 0) {
+	    workspace = wstate->workspace;
+	} else {
+	    wstate = NULL;
+	}
+
+	/* restore window shortcut */
+	if (wstate != NULL || win_state != NULL) {
+	    unsigned mask = 0;
+
+	    if (win_state != NULL)
+		mask = win_state->state->window_shortcuts;
+	    
+	    if (wstate != NULL && mask == 0)
+		mask = wstate->window_shortcuts;
+
+	    if (mask > 0) {
 		int i;
 
 		for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
-		    if (wstate->window_shortcuts & (1<<i)) {
+		    if (mask & (1<<i)) {
 			if (!scr->shortcutWindows[i])
-			    scr->shortcutWindows[i] = WMCreateBag(4);
+			    scr->shortcutWindows[i] = WMCreateArray(4);
 
-			WMPutInBag(scr->shortcutWindows[i], wwin);
+			WMAddToArray(scr->shortcutWindows[i], wwin);
 		    }
 		}
 	    }
-	    free(wstate);
+	}
+	if (wstate != NULL) {
+	    wfree(wstate);
 	}
     }
 
@@ -885,7 +909,7 @@ wManageWindow(WScreen *scr, Window window)
     }
 
     /* setup window geometry */
-    if (win_state && win_state->state->use_geometry) {
+    if (win_state && win_state->state->w > 0) {
         width = win_state->state->w;
         height = win_state->state->h;
     }
@@ -898,7 +922,7 @@ wManageWindow(WScreen *scr, Window window)
     {
 	Bool dontBring = False;
 
-	if (win_state && win_state->state->use_geometry) {
+	if (win_state && win_state->state->w > 0) {
 	    x = win_state->state->x;
 	    y = win_state->state->y;
 	} else if ((wwin->transient_for==None
@@ -963,13 +987,10 @@ wManageWindow(WScreen *scr, Window window)
 
     wwin->frame = wFrameWindowCreate(scr, window_level, 
 				     x, y, width, height,
-                     &wPreferences.window_title_clearance, foo,
+				     &wPreferences.window_title_clearance, foo,
 				     scr->window_title_texture,
 				     scr->resizebar_texture,
 				     scr->window_title_pixel, 
-#ifdef DRAWSTRING_PLUGIN
-                     W_STRING_FTITLE,
-#endif
 				     &scr->window_title_gc, 
 				     &scr->title_font);
 
@@ -1003,9 +1024,6 @@ wManageWindow(WScreen *scr, Window window)
 	wFrameWindowUpdatePushButton(wwin->frame, True);
 #endif /* OLWM_HINTS */
 
-    wFrameWindowChangeTitle(wwin->frame, title ? title : DEF_WINDOW_TITLE);
-    if (title)
-	XFree(title);
 
     wwin->frame->workspace = workspace;
 
@@ -1194,6 +1212,12 @@ wManageWindow(WScreen *scr, Window window)
 	wwin->prev = NULL;
     }
 
+    /* Update name must come after WApplication stuff is done */
+    wWindowUpdateName(wwin, title);
+    if (title)
+	XFree(title);
+    
+
 #ifdef GNOME_STUFF
     wGNOMEUpdateClientStateHint(wwin, True);
 #endif
@@ -1333,9 +1357,6 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
 				     scr->window_title_texture,
 				     scr->resizebar_texture,
 				     scr->window_title_pixel, 
-#ifdef DRAWSTRING_PLUGIN
-                     W_STRING_FTITLE,
-#endif
 				     &scr->window_title_gc, 
 				     &scr->title_font);
     
@@ -1418,7 +1439,7 @@ wManageInternalWindow(WScreen *scr, Window window, Window owner,
     }
 
     if (wwin->flags.is_gnustep == 0)
-      wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
+	wFrameWindowChangeState(wwin->frame, WS_UNFOCUSED);
 
 /*    if (wPreferences.auto_focus)*/
 	wSetFocusTo(scr, wwin);
@@ -1741,6 +1762,53 @@ wWindowUnfocus(WWindow *wwin)
 }
 
 
+void
+wWindowUpdateName(WWindow *wwin, char *newTitle)
+{
+    WApplication *app = wApplicationOf(wwin->main_window);
+    int instIndex = 0;
+    Bool res;
+    char prefix[32] = "";
+    char *tmp, *title;
+    
+    if (!wwin->frame)
+	return;
+    
+    if (app) 
+	instIndex = wApplicationIndexOfGroup(app);
+    
+    
+    wwin->flags.wm_name_changed = 1;
+    
+    if (!newTitle) {
+	/* the hint was removed */
+	title = DEF_WINDOW_TITLE;
+	
+#ifdef KWM_HINTS
+	wKWMSendEventMessage(wwin, WKWMChangedClient);
+#endif	
+    } else {
+	title = newTitle;
+    }
+    
+    if (instIndex > 0) {
+	sprintf(prefix, "  [%i]", instIndex);
+	
+	title = wstrconcat(title, prefix);
+    }
+    
+    if (wFrameWindowChangeTitle(wwin->frame, title)) {
+	/* only update the menu if the title has actually changed */
+	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+#ifdef KWM_HINTS
+	wKWMSendEventMessage(wwin, WKWMChangedClient);
+#endif
+    }
+    if (instIndex > 0)
+	wfree(title);
+}
+
+
 
 /*
  *----------------------------------------------------------------------
@@ -2289,10 +2357,22 @@ wWindowSaveState(WWindow *wwin)
     data[1] = wwin->flags.miniaturized;
     data[2] = wwin->flags.shaded;
     data[3] = wwin->flags.hidden;
+    data[4] = wwin->flags.maximized;
+    if (wwin->flags.maximized == 0) {
+	data[5] = wwin->frame_x;
+	data[6] = wwin->frame_y;
+	data[7] = wwin->frame->core->width;
+	data[8] = wwin->frame->core->height;
+    } else {
+	data[5] = wwin->old_geometry.x;
+	data[6] = wwin->old_geometry.y;
+	data[7] = wwin->old_geometry.width;
+	data[8] = wwin->old_geometry.height;
+    }
 
     for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
 	if (wwin->screen_ptr->shortcutWindows[i] &&
-	    WMCountInBag(wwin->screen_ptr->shortcutWindows[i], wwin))
+	    WMCountInArray(wwin->screen_ptr->shortcutWindows[i], wwin))
 	    data[9] |= 1<<i;
     }
     XChangeProperty(dpy, wwin->client_win, _XA_WINDOWMAKER_STATE,
@@ -2314,28 +2394,27 @@ getSavedState(Window window, WSavedState **state)
 			   True, _XA_WINDOWMAKER_STATE,
 			   &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
 			   (unsigned char **)&data)!=Success || !data)
-      return 0;
-    
-    *state = malloc(sizeof(WSavedState));
+        return 0;
+
+    *state = wmalloc(sizeof(WSavedState));
+
+    (*state)->workspace = data[0];
+    (*state)->miniaturized = data[1];
+    (*state)->shaded = data[2];
+    (*state)->hidden = data[3];
+    (*state)->maximized = data[4];
+    (*state)->x = data[5];
+    (*state)->y = data[6];
+    (*state)->w = data[7];
+    (*state)->h = data[8];
+    (*state)->window_shortcuts = data[9];
 
-    if (*state) {
-	(*state)->workspace = data[0];
-	(*state)->miniaturized = data[1];
-	(*state)->shaded = data[2];
-	(*state)->hidden = data[3];
-	(*state)->use_geometry = data[4];
-	(*state)->x = data[5];
-	(*state)->y = data[6];
-	(*state)->w = data[7];
-	(*state)->h = data[8];
-	(*state)->window_shortcuts = data[9];
-    }
     XFree(data);
-    
+
     if (*state && type_ret==_XA_WINDOWMAKER_STATE)
-      return 1;
+        return 1;
     else 
-      return 0;
+        return 0;
 }
 
 
@@ -2401,7 +2480,7 @@ wWindowSetShape(WWindow *wwin)
 			    0, wwin->frame->top_width, urec, count,
 			    ShapeSet, Unsorted);
     XFlush(dpy);
-    free(urec);
+    wfree(urec);
     return;
 
 alt_code:
@@ -2516,7 +2595,7 @@ wWindowResetMouseGrabs(WWindow *wwin)
     if (!wwin->flags.focused && !WFLAGP(wwin, no_focusable)
 	&& !wwin->flags.is_gnustep) {
 	/* the passive grabs to focus the window */
-	if (wPreferences.focus_mode == WKF_CLICK)
+/*	if (wPreferences.focus_mode == WKF_CLICK) */
 	    XGrabButton(dpy, AnyButton, AnyModifier, wwin->client_win,
 			True, ButtonPressMask, GrabModeSync, GrabModeAsync,
 			None, None);
@@ -2586,7 +2665,7 @@ wWindowGetSavedState(Window win)
 
     if (XGetCommand(dpy, win, &argv, &argc)) {
 	if (argc > 0)
-	    command = FlattenStringList(argv, argc);
+	    command = wtokenjoin(argv, argc);
         XFreeStringList(argv);
     }
     if (!command)
@@ -2610,7 +2689,7 @@ wWindowGetSavedState(Window win)
            class, command);
 #endif
 
-    if (command) free(command);
+    if (command) wfree(command);
     if (instance) XFree(instance);
     if (class) XFree(class);
 
@@ -2633,11 +2712,11 @@ wWindowDeleteSavedState(WMagicNumber id)
         printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-        if (wstate->instance) free(wstate->instance);
-        if (wstate->class)    free(wstate->class);
-        if (wstate->command)  free(wstate->command);
-        free(wstate->state);
-        free(wstate);
+        if (wstate->instance) wfree(wstate->instance);
+        if (wstate->class)    wfree(wstate->class);
+        if (wstate->command)  wfree(wstate->command);
+        wfree(wstate->state);
+        wfree(wstate);
     } else {
 	while (tmp->next) {
 	    if (tmp->next==wstate) {
@@ -2646,11 +2725,11 @@ wWindowDeleteSavedState(WMagicNumber id)
                 printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                        wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-                if (wstate->instance) free(wstate->instance);
-                if (wstate->class)    free(wstate->class);
-                if (wstate->command)  free(wstate->command);
-                free(wstate->state);
-                free(wstate);
+                if (wstate->instance) wfree(wstate->instance);
+                if (wstate->class)    wfree(wstate->class);
+                if (wstate->command)  wfree(wstate->command);
+                wfree(wstate->state);
+                wfree(wstate);
 		break;
 	    }
 	    tmp = tmp->next;
@@ -2675,11 +2754,11 @@ wWindowDeleteSavedStatesForPID(pid_t pid)
         printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-        if (wstate->instance) free(wstate->instance);
-        if (wstate->class)    free(wstate->class);
-        if (wstate->command)  free(wstate->command);
-        free(wstate->state);
-        free(wstate);
+        if (wstate->instance) wfree(wstate->instance);
+        if (wstate->class)    wfree(wstate->class);
+        if (wstate->command)  wfree(wstate->command);
+        wfree(wstate->state);
+        wfree(wstate);
     } else {
 	while (tmp->next) {
 	    if (tmp->next->pid==pid) {
@@ -2689,11 +2768,11 @@ wWindowDeleteSavedStatesForPID(pid_t pid)
                 printf("Deleted WindowState with ID %p, for %s.%s : \"%s\"\n",
                        wstate, wstate->instance, wstate->class, wstate->command);
 #endif
-                if (wstate->instance) free(wstate->instance);
-                if (wstate->class)    free(wstate->class);
-                if (wstate->command)  free(wstate->command);
-                free(wstate->state);
-                free(wstate);
+                if (wstate->instance) wfree(wstate->instance);
+                if (wstate->class)    wfree(wstate->class);
+                if (wstate->command)  wfree(wstate->command);
+                wfree(wstate->state);
+                wfree(wstate);
 		break;
 	    }
 	    tmp = tmp->next;
@@ -2784,10 +2863,11 @@ titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
 
 	    /* maximize window */
 	    if (dir !=0 && !WFLAGP(wwin, no_resizable)) {
-		if (wwin->flags.maximized) 
+		int ndir = dir ^ wwin->flags.maximized;
+		if (wwin->flags.maximized != 0)
 		    wUnmaximizeWindow(wwin);
-		 else
-		    wMaximizeWindow(wwin, dir);
+		if (ndir != 0)
+		    wMaximizeWindow(wwin, ndir);
 	    }
 	}
     } else if (event->xbutton.button==Button3) {
@@ -2809,8 +2889,8 @@ frameMouseDown(WObjDescriptor *desc, XEvent *event)
     
     CloseWindowMenu(wwin->screen_ptr);
 
-    if (wPreferences.focus_mode==WKF_CLICK
-	&& !(event->xbutton.state&ControlMask)
+    if (/*wPreferences.focus_mode==WKF_CLICK
+	&&*/ !(event->xbutton.state&ControlMask)
 	&& !WFLAGP(wwin, no_focusable)) {
 	wSetFocusTo(wwin->screen_ptr, wwin);
     }
