@@ -91,6 +91,10 @@ typedef struct {
     unsigned int no_resizable:1;
     unsigned int no_closable:1;
     unsigned int no_miniaturizable:1;
+    unsigned int no_border:1;	       /* 1 pixel border around window */
+#ifdef XKB_BUTTON_HINT
+    unsigned int no_language_button:1;
+#endif
 
     /* decorations */
     unsigned int no_resizebar:1;       /* draw the bottom handle? */
@@ -114,7 +118,9 @@ typedef struct {
 					* is focused */
     unsigned int no_hide_others:1;     /* hide window when doing hideothers */
     unsigned int no_appicon:1;	       /* make app icon */
-    
+
+    unsigned int collapse_appicons:1;  /* collapse icons of the same name */
+
     unsigned int dont_move_off:1;
 
     unsigned int no_focusable:1;
@@ -141,7 +147,6 @@ typedef struct {
     unsigned int olwm_transient:1;
     unsigned int olwm_warp_to_pin:1;
 #endif
-
 } WWindowAttributes;
 
 
@@ -218,10 +223,6 @@ typedef struct WWindow {
     
     FocusMode focus_mode;	       /* type of keyboard input focus */
 
-#ifdef KEEP_XKB_LOCK_STATUS
-    int languagemode;
-#endif /* KEEP_XKB_LOCK_STATUS */
-
 #ifdef OLWM_HINTS_unfinished
     struct WOLWindowState ol_window_state;
 #endif
@@ -249,6 +250,9 @@ typedef struct WWindow {
 #endif
 
 	/* info flags */
+	unsigned int is_gnustep:1;     /* 1 if the window belongs to a GNUstep
+					app */
+	
 	unsigned int buttons_dont_fit:1;
 	unsigned int rebuild_texture:1;/* the window was resized and 
 					* gradients should be re-rendered */
@@ -272,9 +276,15 @@ typedef struct WWindow {
 	unsigned int obscured:1;       /* window is obscured */
 
 	unsigned int dragged_while_fmaximized;
+
+	unsigned int user_changed_width:1;
+	unsigned int user_changed_height:1;
+	unsigned int wm_name_changed:1;
+
 #ifdef KWM_HINTS
 	unsigned int kwm_hidden_for_modules:1;
 	unsigned int kwm_managed:1;
+	unsigned int kwm_menubar:1;
 #endif
 #ifdef OLWM_HINTS
 	unsigned int olwm_push_pin_out:1;/* emulate pushpin behaviour */
@@ -302,11 +312,12 @@ typedef struct WSavedState {
     int miniaturized;
     int shaded;
     int hidden;
-    int use_geometry;
-    int x;
-    int y;
+    int maximized;
+    int x;			       /* original geometry of the */
+    int y;			       /* window if it's maximized */
     unsigned int w;
     unsigned int h;
+    unsigned window_shortcuts; /* mask like 1<<shortcut_number */
 } WSavedState;
 
 
@@ -338,6 +349,8 @@ void wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed);
 
 void wWindowFocus(WWindow *wwin, WWindow *owin);
 void wWindowUnfocus(WWindow *wwin);
+
+void wWindowUpdateName(WWindow *wwin, char *newTitle);
 void wWindowConstrainSize(WWindow *wwin, int *nwidth, int *nheight);
 void wWindowConfigure(WWindow *wwin, int req_x, int req_y, 
 		      int req_width, int req_height);
