@@ -45,6 +45,7 @@
 #include "stacking.h"
 #include "icon.h"
 
+
 #define MC_MAXIMIZE	0
 #define MC_MINIATURIZE	1
 #define MC_SHADE	2
@@ -99,6 +100,9 @@ execWindowOptionCommand(WMenu *menu, WMenuEntry *entry)
      case WO_OMNIPRESENT:
 	wwin->flags.omnipresent^=1;
 	UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE_WORKSPACE);
+#ifdef GNOME_STUFF
+	wGNOMEUpdateClientStateHint(wwin, False);
+#endif
 	break;
     }
 }
@@ -169,8 +173,7 @@ execMenuCommand(WMenu *menu, WMenuEntry *entry)
 	break;
 
      case MC_PROPERTIES:
-	if (wwin->wm_class || wwin->wm_instance)
-	    wShowInspectorForWindow(wwin);
+	wShowInspectorForWindow(wwin);
 	break;
 
      case MC_HIDE:
@@ -196,9 +199,23 @@ static void
 makeShortcutCommand(WMenu *menu, WMenuEntry *entry)
 {
     WWindow *wwin = (WWindow*)entry->clientdata;
-    
-    wwin->screen_ptr->shortcutWindow[entry->order-WO_ENTRIES] = wwin;
-    
+    WScreen *scr = wwin->screen_ptr;
+    int index = entry->order-WO_ENTRIES;
+
+    if (scr->shortcutWindows[index]) {
+	WMFreeArray(scr->shortcutWindows[index]);
+	scr->shortcutWindows[index] = NULL;
+    }
+
+    if (wwin->flags.selected && scr->selected_windows) {
+        scr->shortcutWindows[index] = WMDuplicateArray(scr->selected_windows);
+        /*WMRemoveFromArray(scr->shortcutWindows[index], wwin);
+        WMInsertInArray(scr->shortcutWindows[index], 0, wwin);*/
+    } else {
+	scr->shortcutWindows[index] = WMCreateArray(4);
+	WMAddToArray(scr->shortcutWindows[index], wwin);
+    }
+
     wSelectWindow(wwin, !wwin->flags.selected);
     XFlush(dpy);
     wusleep(3000);
@@ -220,7 +237,7 @@ updateWorkspaceMenu(WMenu *menu)
     for (i=0; i<scr->workspace_count; i++) {
 	if (i < menu->entry_no) {
 	    if (strcmp(menu->entries[i]->text,scr->workspaces[i]->name)!=0) {
-		free(menu->entries[i]->text);
+		wfree(menu->entries[i]->text);
 		strcpy(title, scr->workspaces[i]->name);
 		menu->entries[i]->text = wstrdup(title);
 		menu->flags.realized = 0;
@@ -255,23 +272,23 @@ updateMakeShortcutMenu(WMenu *menu, WWindow *wwin)
     for (i=WO_ENTRIES; i<smenu->entry_no; i++) {
 	char *tmp;
 	int shortcutNo = i-WO_ENTRIES;
-	WWindow *twin = wwin->screen_ptr->shortcutWindow[shortcutNo];
 	WMenuEntry *entry = smenu->entries[i];
+	WMArray *shortSelWindows = wwin->screen_ptr->shortcutWindows[shortcutNo];
 
 	sprintf(buffer, "%s %i", _("Set Shortcut"), shortcutNo+1);
 	
-	if (!twin) {
+	if (!shortSelWindows) {
 	    entry->flags.indicator_on = 0;
 	} else {
 	    entry->flags.indicator_on = 1;
-	    if (twin != wwin)
-		entry->flags.indicator_type = MI_CHECK;
-	    else
+	    if (WMCountInArray(shortSelWindows, wwin))
 		entry->flags.indicator_type = MI_DIAMOND;
+	    else
+		entry->flags.indicator_type = MI_CHECK;
 	}
 
 	if (strcmp(buffer, entry->text)!=0) {
-	    free(entry->text);
+	    wfree(entry->text);
 	    entry->text = wstrdup(buffer);
 	    smenu->flags.realized = 0;
 	}
@@ -282,7 +299,7 @@ updateMakeShortcutMenu(WMenu *menu, WWindow *wwin)
 	    if ((tmp = XKeysymToString(XKeycodeToKeysym(dpy, kcode, 0)))
 		&& (!entry->rtext || strcmp(tmp, entry->rtext)!=0)) {
 		if (entry->rtext)
-		    free(entry->rtext);
+		    wfree(entry->rtext);
 		entry->rtext = wstrdup(tmp);
 		smenu->flags.realized = 0;
 	    }
@@ -290,14 +307,14 @@ updateMakeShortcutMenu(WMenu *menu, WWindow *wwin)
 	} else {
 	    wMenuSetEnabled(smenu, i, False);
 	    if (entry->rtext) {
-		free(entry->rtext);
+		wfree(entry->rtext);
 		entry->rtext = NULL;
 		smenu->flags.realized = 0;
 	    }
 	}
 	entry->clientdata = wwin;
     }
-    free(buffer);
+    wfree(buffer);
     if (!smenu->flags.realized)
 	wMenuRealize(smenu);
 }
@@ -363,7 +380,6 @@ makeMakeShortcutMenu(WScreen *scr, WMenu *menu)
 
     for (i=0; i<MAX_WINDOW_SHORTCUTS; i++) {
 	WMenuEntry *entry;
-
 	entry = wMenuAddCallback(menu, "", makeShortcutCommand, NULL);
 
 	entry->flags.indicator = 1;
@@ -581,7 +597,7 @@ updateMenuForWindow(WMenu *menu, WWindow *wwin)
 
     wMenuSetEnabled(menu, MC_DUMMY_MOVETO, !IS_OMNIPRESENT(wwin));
     
-    if ((wwin->wm_class || wwin->wm_instance) && !wwin->flags.inspector_open) {
+    if (!wwin->flags.inspector_open) {
 	wMenuSetEnabled(menu, MC_PROPERTIES, True);
     } else {
 	wMenuSetEnabled(menu, MC_PROPERTIES, False);
@@ -618,9 +634,9 @@ OpenWindowMenu(WWindow *wwin, int x, int y, int keyboard)
 	scr->window_menu = createWindowMenu(scr);
 	
 	/* hack to save some memory allocation/deallocation */
-	free(scr->window_menu->entries[MC_MINIATURIZE]->text);
-	free(scr->window_menu->entries[MC_MAXIMIZE]->text);
-	free(scr->window_menu->entries[MC_SHADE]->text);
+	wfree(scr->window_menu->entries[MC_MINIATURIZE]->text);
+	wfree(scr->window_menu->entries[MC_MAXIMIZE]->text);
+	wfree(scr->window_menu->entries[MC_SHADE]->text);
     } else {
 	updateWorkspaceMenu(scr->workspace_submenu);
     }
@@ -658,9 +674,9 @@ OpenMiniwindowMenu(WWindow *wwin, int x, int y)
 	scr->window_menu = createWindowMenu(scr);
 	
 	/* hack to save some memory allocation/deallocation */
-	free(scr->window_menu->entries[MC_MINIATURIZE]->text);
-	free(scr->window_menu->entries[MC_MAXIMIZE]->text);
-	free(scr->window_menu->entries[MC_SHADE]->text);	
+	wfree(scr->window_menu->entries[MC_MINIATURIZE]->text);
+	wfree(scr->window_menu->entries[MC_MAXIMIZE]->text);
+	wfree(scr->window_menu->entries[MC_SHADE]->text);
     } else {
 	updateWorkspaceMenu(scr->workspace_submenu);
     }
