@@ -114,7 +114,7 @@ typedef struct InspectorPanel {
     
     /* 5th page. application wide attributes */
     WMFrame *appFrm;
-    WMButton *appChk[2];
+    WMButton *appChk[3];
 
     unsigned int done:1;
     unsigned int destroyed:1;
@@ -153,6 +153,7 @@ static proplist_t AStartMaximized;
 static proplist_t ADontSaveSession;
 static proplist_t AEmulateAppIcon;
 static proplist_t AFullMaximize;
+static proplist_t ACollapseAppIcons;
 #ifdef XKB_BUTTON_HINT
 static proplist_t ANoLanguageButton;
 #endif
@@ -175,6 +176,8 @@ static proplist_t Yes, No;
 
 
 
+static char *spec_text;
+
 
 static void applySettings(WMButton *button, InspectorPanel *panel);
 
@@ -218,6 +221,7 @@ make_keys()
     ADontSaveSession = PLMakeString("DontSaveSession");
     AEmulateAppIcon = PLMakeString("EmulateAppIcon");
     AFullMaximize = PLMakeString("FullMaximize");
+    ACollapseAppIcons = PLMakeString("CollapseAppIcons");    
 #ifdef XKB_BUTTON_HINT
     ANoLanguageButton = PLMakeString("NoLanguageButton");
 #endif
@@ -243,7 +247,7 @@ freeInspector(InspectorPanel *panel)
 
     XDestroyWindow(dpy, panel->parent);
 
-    free(panel);
+    wfree(panel);
 }
 
 
@@ -293,7 +297,7 @@ wDestroyInspectorPanels()
         panel->inspected->flags.inspector_open = 0;
         panel->inspected->inspector = NULL;
 
-        free(panel);
+        wfree(panel);
     }
 }
 
@@ -352,7 +356,7 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
     if ((flags & USE_TEXT_FIELD) != 0) {
         file = WMGetTextFieldText(panel->fileText);
         if (file && file[0] == 0) {
-            free(file);
+            wfree(file);
             file = NULL;
         }
     } else {
@@ -382,13 +386,13 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
 		    file);
             wMessageDialog(panel->frame->screen_ptr, _("Error"), buf, 
 			   _("OK"), NULL, NULL);
-	    free(buf);
-	    free(file);
+	    wfree(buf);
+	    wfree(file);
             return -1;
         }
 
         pixmap = WMCreatePixmapFromFile(scrPtr, path);
-        free(path);
+        wfree(path);
 
         if (!pixmap) {
 	    char *buf;
@@ -398,11 +402,11 @@ showIconFor(WMScreen *scrPtr, InspectorPanel *panel,
 		    file, RMessageForError(RErrorCode));
             wMessageDialog(panel->frame->screen_ptr, _("Error"), buf,
 			   _("OK"), NULL, NULL);
-	    free(buf);
-	    free(file);
+	    wfree(buf);
+	    wfree(file);
             return -1;
         }
-        free(file);
+        wfree(file);
     }
 
     WMSetLabelImage(panel->iconLbl, pixmap);
@@ -523,7 +527,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 		buffer = wmalloc(strlen(wwin->wm_instance)+strlen(wwin->wm_class)+4);
         strcat(strcat(strcpy(buffer, wwin->wm_instance), "."), wwin->wm_class);
         key = PLMakeString(buffer);
-		free(buffer);
+        wfree(buffer);
     } else if (WMGetButtonSelected(panel->defaultRb) != 0) {
         key = PLRetain(AnyWindow);
         flags = UPDATE_DEFAULTS;
@@ -563,7 +567,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
             different |= insertAttribute(dict, winDic, AIcon, value, flags);
             PLRelease(value);
         }
-        free(icon_file);
+        wfree(icon_file);
     }
 
     {
@@ -615,32 +619,32 @@ saveSettings(WMButton *button, InspectorPanel *panel)
     different |= insertAttribute(dict, winDic, AStartMaximized,  value, flags);
 
     value = (WMGetButtonSelected(panel->attrChk[10])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, ASkipWindowList,  value, flags);
+    different |= insertAttribute(dict, winDic, AFullMaximize,  value, flags);
 
 
     value = (WMGetButtonSelected(panel->moreChk[0])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, ANoHideOthers,    value, flags);
-
-    value = (WMGetButtonSelected(panel->moreChk[1])!=0) ? Yes : No;
     different |= insertAttribute(dict, winDic, ANoKeyBindings,   value, flags);
 
-    value = (WMGetButtonSelected(panel->moreChk[2])!=0) ? Yes : No;
+    value = (WMGetButtonSelected(panel->moreChk[1])!=0) ? Yes : No;
     different |= insertAttribute(dict, winDic, ANoMouseBindings, value, flags);
 
-    value = (WMGetButtonSelected(panel->moreChk[3])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, AKeepInsideScreen,value, flags);
+    value = (WMGetButtonSelected(panel->moreChk[2])!=0) ? Yes : No;
+    different |= insertAttribute(dict, winDic, ASkipWindowList, value, flags);
 
-    value = (WMGetButtonSelected(panel->moreChk[4])!=0) ? Yes : No;
+    value = (WMGetButtonSelected(panel->moreChk[3])!=0) ? Yes : No;
     different |= insertAttribute(dict, winDic, AUnfocusable, value, flags);
 
+    value = (WMGetButtonSelected(panel->moreChk[4])!=0) ? Yes : No;
+    different |= insertAttribute(dict, winDic, AKeepInsideScreen,value, flags);
+
     value = (WMGetButtonSelected(panel->moreChk[5])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, ADontSaveSession, value, flags);
+    different |= insertAttribute(dict, winDic, ANoHideOthers,    value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[6])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, AEmulateAppIcon,  value, flags);
+    different |= insertAttribute(dict, winDic, ADontSaveSession, value, flags);
 
     value = (WMGetButtonSelected(panel->moreChk[7])!=0) ? Yes : No;
-    different |= insertAttribute(dict, winDic, AFullMaximize, value, flags);
+    different |= insertAttribute(dict, winDic, AEmulateAppIcon,  value, flags);
 
 #ifdef XKB_BUTTON_HINT
     value = (WMGetButtonSelected(panel->moreChk[8])!=0) ? Yes : No;
@@ -656,6 +660,9 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 
 	value = (WMGetButtonSelected(panel->appChk[1])!=0) ? Yes : No;
 	different |= insertAttribute(dict, winDic, ANoAppIcon, value, flags);
+	
+	value = (WMGetButtonSelected(panel->appChk[2])!=0) ? Yes : No;
+	different |= insertAttribute(dict, winDic, ACollapseAppIcons, value, flags);
     } 
 
     PLRemoveDictionaryEntry(dict, key);
@@ -689,7 +696,7 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 	    strcat(strcpy(buffer, wapp->main_window_desc->wm_instance), ".");
 	    strcat(buffer, wapp->main_window_desc->wm_class);
 	    key = PLMakeString(buffer);
-		free(buffer);
+            wfree(buffer);
 
 	    iconFile = wDefaultGetIconFile(wwin->screen_ptr, 
 					   wapp->main_window_desc->wm_instance,
@@ -711,6 +718,10 @@ saveSettings(WMButton *button, InspectorPanel *panel)
 	    different |= insertAttribute(dict, appDic, ANoAppIcon,  value, 
 					 flags);
 
+	    value = (WMGetButtonSelected(panel->appChk[2])!=0) ? Yes : No;
+	    different |= insertAttribute(dict, appDic, ACollapseAppIcons,  value, 
+					 flags);
+	    
 	    PLRemoveDictionaryEntry(dict, key);
 	    if (different) {
 		PLInsertDictionaryEntry(dict, key, appDic);
@@ -822,16 +833,16 @@ applySettings(WMButton *button, InspectorPanel *panel)
     WSETUFLAG(wwin, omnipresent, WMGetButtonSelected(panel->attrChk[7]));
     WSETUFLAG(wwin, start_miniaturized, WMGetButtonSelected(panel->attrChk[8]));
     WSETUFLAG(wwin, start_maximized, WMGetButtonSelected(panel->attrChk[9]));
-    skip_window_list           =  WMGetButtonSelected(panel->attrChk[10]);
+    WSETUFLAG(wwin, full_maximize, WMGetButtonSelected(panel->attrChk[10]));
 
-    WSETUFLAG(wwin, no_hide_others, WMGetButtonSelected(panel->moreChk[0]));
-    WSETUFLAG(wwin, no_bind_keys, WMGetButtonSelected(panel->moreChk[1]));
-    WSETUFLAG(wwin, no_bind_mouse, WMGetButtonSelected(panel->moreChk[2]));
-    WSETUFLAG(wwin, dont_move_off, WMGetButtonSelected(panel->moreChk[3]));
-    WSETUFLAG(wwin, no_focusable, WMGetButtonSelected(panel->moreChk[4]));
-    WSETUFLAG(wwin, dont_save_session, WMGetButtonSelected(panel->moreChk[5]));
-    WSETUFLAG(wwin, emulate_appicon, WMGetButtonSelected(panel->moreChk[6]));
-    WSETUFLAG(wwin, full_maximize, WMGetButtonSelected(panel->moreChk[7]));
+    WSETUFLAG(wwin, no_bind_keys, WMGetButtonSelected(panel->moreChk[0]));
+    WSETUFLAG(wwin, no_bind_mouse, WMGetButtonSelected(panel->moreChk[1]));
+    skip_window_list           =  WMGetButtonSelected(panel->moreChk[2]);
+    WSETUFLAG(wwin, no_focusable, WMGetButtonSelected(panel->moreChk[3]));
+    WSETUFLAG(wwin, dont_move_off, WMGetButtonSelected(panel->moreChk[4]));
+    WSETUFLAG(wwin, no_hide_others, WMGetButtonSelected(panel->moreChk[5]));
+    WSETUFLAG(wwin, dont_save_session, WMGetButtonSelected(panel->moreChk[6]));
+    WSETUFLAG(wwin, emulate_appicon, WMGetButtonSelected(panel->moreChk[7]));
 #ifdef XKB_BUTTON_HINT
     WSETUFLAG(wwin, no_language_button, WMGetButtonSelected(panel->moreChk[8]));
 #endif
@@ -894,6 +905,9 @@ applySettings(WMButton *button, InspectorPanel *panel)
 
 	WSETUFLAG(wapp->main_window_desc, no_appicon,
 		  WMGetButtonSelected(panel->appChk[1]));
+
+	WSETUFLAG(wapp->main_window_desc, collapse_appicons,
+		  WMGetButtonSelected(panel->appChk[2]));
 	
         if (WFLAGP(wapp->main_window_desc, no_appicon))
             removeAppIconFor(wapp);
@@ -904,12 +918,12 @@ applySettings(WMButton *button, InspectorPanel *panel)
             char *file = WMGetTextFieldText(panel->fileText);
 
 	    if (file[0] == 0) {
-		free(file);
+		wfree(file);
 		file = NULL;
 	    }
 	    wIconChangeImageFile(wapp->app_icon->icon, file);
 	    if (file)
-		free(file);
+		wfree(file);
 	    wAppIconPaint(wapp->app_icon);
         }
     }
@@ -977,7 +991,7 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	    flag = WFLAGP(wwin, start_maximized!=0);
 	    break;
 	 case 10:
-	    flag = WFLAGP(wwin, skip_window_list);
+	    flag = WFLAGP(wwin, full_maximize);
 	    break;
 	}
         WMSetButtonSelected(panel->attrChk[i], flag);
@@ -987,28 +1001,28 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	
 	switch (i) {
 	 case 0:
-	    flag = WFLAGP(wwin, no_hide_others);
-	    break;
-	 case 1:
 	    flag = WFLAGP(wwin, no_bind_keys);
 	    break;
-	 case 2:
+	 case 1:
 	    flag = WFLAGP(wwin, no_bind_mouse);
 	    break;
-	 case 3:
-	    flag = WFLAGP(wwin, dont_move_off);
+	 case 2:
+	    flag = WFLAGP(wwin, skip_window_list);
 	    break;
-	 case 4:
+	 case 3:
 	    flag = WFLAGP(wwin, no_focusable);
 	    break;
+	 case 4:
+	    flag = WFLAGP(wwin, dont_move_off);
+	    break;
 	 case 5:
-	    flag = WFLAGP(wwin, dont_save_session);
+	    flag = WFLAGP(wwin, no_hide_others);
 	    break;
 	 case 6:
-	    flag = WFLAGP(wwin, emulate_appicon);
+	    flag = WFLAGP(wwin, dont_save_session);
 	    break;
 	 case 7:
-	    flag = WFLAGP(wwin, full_maximize);
+	    flag = WFLAGP(wwin, emulate_appicon);
 	    break;
 #ifdef XKB_BUTTON_HINT
      case 8:
@@ -1029,6 +1043,9 @@ revertSettings(WMButton *button, InspectorPanel *panel)
 	     case 1:
 		flag = WFLAGP(wapp->main_window_desc, no_appicon);
 		break;
+	     case 2:
+		flag = WFLAGP(wapp->main_window_desc, collapse_appicons);
+		break;
 	    }
 	    WMSetButtonSelected(panel->appChk[i], flag);
 	}
@@ -1075,7 +1092,7 @@ chooseIconCallback(WMWidget *self, void *clientData)
 	    WMSetTextFieldText(panel->fileText, file);
 	    showIconFor(WMWidgetScreen(self), panel, NULL, NULL,
 			USE_TEXT_FIELD);
-	    free(file);
+	    wfree(file);
 	}
 	WMSetButtonEnabled(panel->browseIconBtn, True);
     } else {
@@ -1117,27 +1134,16 @@ selectSpecification(WMWidget *bPtr, void *data)
     str = wmalloc(16 + strlen(wwin->wm_instance ? wwin->wm_instance : "?")
 		  + strlen(wwin->wm_class ? wwin->wm_class : "?"));
 
-    sprintf(str, "Inspecting  %s.%s",
+    sprintf(str, _("Inspecting  %s.%s"),
 	    wwin->wm_instance ? wwin->wm_instance : "?",
 	    wwin->wm_class ? wwin->wm_class : "?");
 
     wFrameWindowChangeTitle(panel->frame->frame, str);
 
-    free(str);
+    wfree(str);
 }
 
 
-
-
-#define SPEC_TEXT "The configuration will apply to all\n"\
-		     "windows that have their WM_CLASS property"\
-		     " set to the above selected\nname, when saved."
-
-
-#define SELEC_TEXT "Click in the window you wish to inspect."
-
-
-
 static void
 selectWindow(WMWidget *bPtr, void *data)
 {
@@ -1154,7 +1160,7 @@ selectWindow(WMWidget *bPtr, void *data)
 	return;
     }
     
-    WMSetLabelText(panel->specLbl, _(SELEC_TEXT));
+    WMSetLabelText(panel->specLbl, _("Click in the window you wish to inspect."));
 
     WMMaskEvent(dpy, ButtonPressMask, &event);
 
@@ -1172,7 +1178,7 @@ selectWindow(WMWidget *bPtr, void *data)
 						   True);
 	wCloseInspectorForWindow(wwin);
     } else {
-	WMSetLabelText(panel->specLbl, _(SPEC_TEXT));
+	WMSetLabelText(panel->specLbl, spec_text);
     }
 }
 
@@ -1193,6 +1199,10 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
     WMPixmap *pixmap;
 #endif
 
+    spec_text = _("The configuration will apply to all\n"
+			"windows that have their WM_CLASS\n"
+			"property set to the above selected\n"
+			"name, when saved.");
 
     panel = wmalloc(sizeof(InspectorPanel));
     memset(panel, 0, sizeof(InspectorPanel));
@@ -1266,15 +1276,15 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
     if (wwin->wm_class && wwin->wm_instance) {
 	char *str, *tmp;
 
-        tmp = wstrappend(wwin->wm_instance, ".");
-        str = wstrappend(tmp, wwin->wm_class);
+        tmp = wstrconcat(wwin->wm_instance, ".");
+        str = wstrconcat(tmp, wwin->wm_class);
 
 	panel->bothRb = WMCreateRadioButton(panel->specFrm);
 	WMMoveWidget(panel->bothRb, 10, 18);
 	WMResizeWidget(panel->bothRb, frame_width - (2 * 10), 20);
 	WMSetButtonText(panel->bothRb, str);
-        free(tmp);
-        free(str);
+        wfree(tmp);
+        wfree(str);
 	WMGroupButtons(panel->defaultRb, panel->bothRb);
 
 	if (!selectedBtn)
@@ -1313,16 +1323,17 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
     panel->selWinB = WMCreateCommandButton(panel->specFrm);
     WMMoveWidget(panel->selWinB, 20, 145-24 - 10);
     WMResizeWidget(panel->selWinB, frame_width - 2*10 - 20, 24);
-    WMSetButtonText(panel->selWinB, _("Select Window"));
+    WMSetButtonText(panel->selWinB, _("Select window"));
     WMSetButtonAction(panel->selWinB, selectWindow, panel);
     
     
     panel->specLbl = WMCreateLabel(panel->win);
     WMMoveWidget(panel->specLbl, 15, 210);
     WMResizeWidget(panel->specLbl, frame_width, 100);
-    WMSetLabelText(panel->specLbl, _(SPEC_TEXT));
+    WMSetLabelText(panel->specLbl, spec_text);
+    WMSetLabelWraps(panel->specLbl, True);
     
-    WMSetLabelTextAlignment(panel->specLbl, WACenter);
+    WMSetLabelTextAlignment(panel->specLbl, WALeft);
     
     /**** attributes ****/
     panel->attrFrm = WMCreateFrame(panel->win);
@@ -1337,7 +1348,7 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 
 	switch (i) {
 	 case 0:
-	    caption = _("Disable Titlebar");
+	    caption = _("Disable titlebar");
             flag = WFLAGP(wwin, no_titlebar);
 	    descr = _("Remove the titlebar of this window.\n"
 		      "To access the window commands menu of a window\n"
@@ -1346,57 +1357,59 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 		      "settings).");
 	    break;
 	 case 1:
-	    caption = _("Disable Resizebar");
+	    caption = _("Disable resizebar");
 	    flag = WFLAGP(wwin, no_resizebar);
 	    descr = _("Remove the resizebar of this window."); 
 	    break;
 	 case 2:
-	    caption = _("Disable Close Button");
+	    caption = _("Disable close button");
 	    flag = WFLAGP(wwin, no_close_button);
 	    descr = _("Remove the `close window' button of this window.");
 	    break;
 	 case 3:
-	    caption = _("Disable Miniaturize Button");
+	    caption = _("Disable miniaturize button");
 	    flag = WFLAGP(wwin, no_miniaturize_button);
 	    descr = _("Remove the `miniaturize window' button of the window.");
 	    break;
 	 case 4:
-	    caption = _("Disable Border");
+	    caption = _("Disable border");
 	    flag = WFLAGP(wwin, no_border);
 	    descr = _("Remove the 1 pixel black border around the window.");
 	    break;
 	 case 5:
-	    caption = _("Keep on Top / Floating");
+	    caption = _("Keep on top (floating)");
 	    flag = WFLAGP(wwin, floating);
 	    descr = _("Keep the window over other windows, not allowing\n"
-		      "them to covert it.");
+		      "them to cover it.");
 	    break;
 	 case 6:
-	    caption = _("Keep at Bottom / Sunken");
+	    caption = _("Keep at bottom (sunken)");
 	    flag = WFLAGP(wwin, sunken);
 	    descr = _("Keep the window under all other windows.");
 	    break;
 	 case 7:
 	    caption = _("Omnipresent");
 	    flag = WFLAGP(wwin, omnipresent);
-	    descr = _("Make window occupy all workspaces.");
+	    descr = _("Make window present in all workspaces.");
 	    break;
 	 case 8:
-	    caption = _("Start Miniaturized");
+	    caption = _("Start miniaturized");
 	    flag = WFLAGP(wwin, start_miniaturized);
 	    descr = _("Make the window be automatically miniaturized when it's\n"
 		    "first shown.");
 	    break;
 	 case 9:
-	    caption = _("Start Maximized");
+	    caption = _("Start maximized");
 	    flag = WFLAGP(wwin, start_maximized!=0);
 	    descr = _("Make the window be automatically maximized when it's\n"
 		    "first shown.");
 	    break;
 	 case 10:
-	    caption = _("Skip Window List");
-	    flag = WFLAGP(wwin, skip_window_list);
-	    descr = _("Do not list the window in the window list menu.");
+	    caption = _("Full screen maximization");
+	    flag = WFLAGP(wwin, full_maximize);
+	    descr = _("Make the window use the whole screen space when it's\n"
+		      "maximized. The titlebar and resizebar will be moved\n"
+		      "to outside the screen.");
 	    break;
 	}
 	panel->attrChk[i] = WMCreateSwitchButton(panel->attrFrm);
@@ -1428,63 +1441,61 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 	
 	switch (i) {
 	 case 0:
-	    caption = _("Ignore HideOthers");
-	    flag = WFLAGP(wwin, no_hide_others);
-	    descr = _("Do not hide the window when issuing the\n"
-		      "`HideOthers' command.");
-	    break;
-	 case 1:
-	    caption = _("Don't Bind Keyboard Shortcuts");
+	    caption = _("Do not bind keyboard shortcuts");
 	    flag = WFLAGP(wwin, no_bind_keys);
 	    descr = _("Do not bind keyboard shortcuts from Window Maker\n"
 		      "when this window is focused. This will allow the\n"
 		      "window to receive all key combinations regardless\n"
 		      "of your shortcut configuration.");
 	    break;
-	 case 2:
-	    caption = _("Don't Bind Mouse Clicks");
+	 case 1:
+	    caption = _("Do not bind mouse clicks");
 	    flag = WFLAGP(wwin, no_bind_mouse);
 	    descr = _("Do not bind mouse actions, such as `Alt'+drag\n"
 		      "in the window (when alt is the modifier you have"
 		      "configured).");
 	    break;
-	 case 3:
-	    caption = _("Keep Inside Screen");
-	    flag = WFLAGP(wwin, dont_move_off);
-	    descr = _("Do not allow the window to move itself completely\n"
-		      "outside the screen. For bug compatibility.\n");
+	 case 2:
+	    caption = _("Do not show in the window list");
+	    flag = WFLAGP(wwin, skip_window_list);
+	    descr = _("Do not list the window in the window list menu.");
 	    break;
-	 case 4:
-	    caption = _("Don't Let It Take Focus");
+	 case 3:
+	    caption = _("Do not let it take focus");
 	    flag = WFLAGP(wwin, no_focusable);
 	    descr = _("Do not let the window take keyboard focus when you\n"
 		      "click on it.");
 	    break;
+	 case 4:
+	    caption = _("Keep inside screen");
+	    flag = WFLAGP(wwin, dont_move_off);
+	    descr = _("Do not allow the window to move itself completely\n"
+		      "outside the screen. For bug compatibility.\n");
+	    break;
 	 case 5:
-	    caption = _("Don't Save Session");
+	    caption = _("Ignore 'Hide Others'");
+	    flag = WFLAGP(wwin, no_hide_others);
+	    descr = _("Do not hide the window when issuing the\n"
+		      "`HideOthers' command.");
+	    break;
+	 case 6:
+	    caption = _("Ignore 'Save Session'");
 	    flag = WFLAGP(wwin, dont_save_session);
 	    descr = _("Do not save the associated application in the\n"
 		      "session's state, so that it won't be restarted\n"
 		      "together with other applications when Window Maker\n"
 		      "starts.");
 	    break;
-	 case 6:
-	    caption = _("Emulate Application Icon");
+	 case 7:
+	    caption = _("Emulate application icon");
 	    flag = WFLAGP(wwin, emulate_appicon);
 	    descr = _("Make this window act as an application that provides\n"
 		      "enough information to Window Maker for a dockable\n"
 		      "application icon to be created.");
 	    break;
-	 case 7:
-	    caption = _("Full Screen Maximization");
-	    flag = WFLAGP(wwin, full_maximize);
-	    descr = _("Make the window use the whole screen space when it's\n"
-		      "maximized. The titlebar and resizebar will be moved\n"
-		      "to outside the screen.");
-	    break;
 #ifdef XKB_BUTTON_HINT
      case 8:
-        caption = _("Disable Language Button");
+        caption = _("Disable language button");
         flag = WFLAGP(wwin, no_language_button);
         descr = _("Remove the `toggle language' button of the window.");
         break;
@@ -1506,24 +1517,17 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
     WMSetFrameTitle(panel->iconFrm, _("Miniwindow Image"));
 
     panel->iconLbl = WMCreateLabel(panel->iconFrm);
-    WMMoveWidget(panel->iconLbl, PWIDTH - (2 * 15) - 22 - 64, 30);
+    WMMoveWidget(panel->iconLbl, PWIDTH - (2 * 15) - 22 - 64, 20);
     WMResizeWidget(panel->iconLbl, 64, 64);
     WMSetLabelRelief(panel->iconLbl, WRGroove);
     WMSetLabelImagePosition(panel->iconLbl, WIPImageOnly);
     
     panel->browseIconBtn = WMCreateCommandButton(panel->iconFrm);
     WMSetButtonAction(panel->browseIconBtn, chooseIconCallback, panel);
-    WMMoveWidget(panel->browseIconBtn, 22, 30);
-    WMResizeWidget(panel->browseIconBtn, 100, 26);
+    WMMoveWidget(panel->browseIconBtn, 22, 32);
+    WMResizeWidget(panel->browseIconBtn, 120, 26);
     WMSetButtonText(panel->browseIconBtn, _("Browse..."));
 
-#if 0
-    panel->updateIconBtn = WMCreateCommandButton(panel->iconFrm);
-    WMSetButtonAction(panel->updateIconBtn, (WMAction*)updateIcon, panel);
-    WMMoveWidget(panel->updateIconBtn, 22, 65);
-    WMResizeWidget(panel->updateIconBtn, 100, 26);
-    WMSetButtonText(panel->updateIconBtn, _("Update"));
-#endif
 #ifdef wrong_behaviour
     WMSetButtonImagePosition(panel->updateIconBtn, WIPRight);
     pixmap = WMGetSystemPixmap(scr->wmscreen, WSIReturnArrow);
@@ -1535,20 +1539,21 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 #endif
 
     panel->fileLbl = WMCreateLabel(panel->iconFrm);
-    WMMoveWidget(panel->fileLbl, 20, 95);
+    WMMoveWidget(panel->fileLbl, 20, 85);
     WMResizeWidget(panel->fileLbl, PWIDTH - (2 * 15) - (2 * 20), 14);
-    WMSetLabelText(panel->fileLbl, _("Icon File Name:"));
+    WMSetLabelText(panel->fileLbl, _("Icon filename:"));
 
     panel->fileText = WMCreateTextField(panel->iconFrm);
-    WMMoveWidget(panel->fileText, 20, 115);
-    WMResizeWidget(panel->fileText, PWIDTH - (2 * 15) - (2 * 15), 20);
+    WMMoveWidget(panel->fileText, 20, 105);
+    WMResizeWidget(panel->fileText, PWIDTH - (2 * 20) - (2 * 15), 20);
     WMSetTextFieldText(panel->fileText, NULL);
     WMAddNotificationObserver(textEditedObserver, panel, 
 			      WMTextDidEndEditingNotification, 
 			      panel->fileText);
+
     panel->alwChk = WMCreateSwitchButton(panel->iconFrm);
-    WMMoveWidget(panel->alwChk, 20, 140);
-    WMResizeWidget(panel->alwChk, PWIDTH - (2 * 15) - (2 * 15), 20);
+    WMMoveWidget(panel->alwChk, 20, 130);
+    WMResizeWidget(panel->alwChk, PWIDTH - (2 * 15) - (2 * 15), 30);
     WMSetButtonText(panel->alwChk, _("Ignore client supplied icon"));
     WMSetButtonSelected(panel->alwChk, WFLAGP(wwin, always_user_icon));
 
@@ -1582,29 +1587,35 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 	WApplication *wapp = wApplicationOf(wwin->main_window);
 	
 	panel->appFrm = WMCreateFrame(panel->win);
-	WMSetFrameTitle(panel->appFrm, _("Application Wide"));
+	WMSetFrameTitle(panel->appFrm, _("Application Attributes"));
 	WMMoveWidget(panel->appFrm, 15, 50);
 	WMResizeWidget(panel->appFrm, frame_width, 240);
 	
-	for (i=0; i < 2; i++) {
+	for (i=0; i < 3; i++) {
 	    char *caption = NULL;
 	    int flag = 0;
 	    char *descr = NULL;
 	    
 	    switch (i) {
 	     case 0:
-		caption = _("Start Hidden");
+		caption = _("Start hidden");
 		flag = WFLAGP(wapp->main_window_desc, start_hidden);
 		descr = _("Automatically hide application when it's started.");
 		break;
 	     case 1:
-		caption = _("No Application Icon");
+		caption = _("No application icon");
 		flag = WFLAGP(wapp->main_window_desc, no_appicon);
 		descr = _("Disable the application icon for the application.\n"
 			  "Note that you won't be able to dock it anymore,\n"
 			  "and any icons that are already docked will stop\n"
 			  "working correctly.");
 		break;
+	     case 2:
+		caption = _("Collapse application icons");
+		flag = WFLAGP(wapp->main_window_desc, collapse_appicons);
+		descr = _("Collapse application icons from other instances\n"
+			  "of this application into one.\n");
+		break;
 	    }
 	    panel->appChk[i] = WMCreateSwitchButton(panel->appFrm);
 	    WMMoveWidget(panel->appChk[i], 10, 20*(i+1));
@@ -1617,10 +1628,10 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 	
 	if (WFLAGP(wwin, emulate_appicon)) {
 	    WMSetButtonEnabled(panel->appChk[1], False);
-	    WMSetButtonEnabled(panel->moreChk[6], True);
+	    WMSetButtonEnabled(panel->moreChk[7], True);
 	} else {
 	    WMSetButtonEnabled(panel->appChk[1], True);
-	    WMSetButtonEnabled(panel->moreChk[6], False);
+	    WMSetButtonEnabled(panel->moreChk[7], False);
 	}
     } else {
 	int tmp;
@@ -1630,7 +1641,7 @@ createInspectorForWindow(WWindow *wwin, int xpos, int ypos,
 	    tmp = False;
 	else
 	    tmp = True;
-	WMSetButtonEnabled(panel->moreChk[6], tmp);
+	WMSetButtonEnabled(panel->moreChk[7], tmp);
 
 	WMSetPopUpButtonItemEnabled(panel->pagePopUp, 4, False);
 	panel->appFrm = NULL;
