@@ -1,9 +1,9 @@
 /* workspace.c- Workspace management
- * 
+ *
  *  Window Maker window manager
- * 
+ *
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  *  USA.
  */
 #include "wconfig.h"
@@ -64,7 +64,7 @@ static proplist_t dWorkspaces=NULL;
 static proplist_t dClip, dName;
 
 #ifdef VIRTUAL_DESKTOP
-static BOOL initVDesk=False;
+static BOOL initVDesk = False;
 #endif
 
 static void
@@ -72,7 +72,7 @@ make_keys()
 {
     if (dWorkspaces!=NULL)
 	return;
-    
+
     dWorkspaces = PLMakeString("Workspaces");
     dName = PLMakeString("Name");
     dClip = PLMakeString("Clip");
@@ -94,7 +94,7 @@ wWorkspaceNew(WScreen *scr)
 {
     WWorkspace *wspace, **list;
     int i;
-    
+
     if (scr->workspace_count < MAX_WORKSPACES) {
 	scr->workspace_count++;
 
@@ -112,10 +112,10 @@ wWorkspaceNew(WScreen *scr)
 	}
 
 
-        if (!wPreferences.flags.noclip) {
-            wspace->clip = wDockCreate(scr, WM_CLIP);
-	} else
-            wspace->clip = NULL;
+    if (!wPreferences.flags.noclip) {
+        wspace->clip = wDockCreate(scr, WM_CLIP);
+    } else
+        wspace->clip = NULL;
 
 	list = wmalloc(sizeof(WWorkspace*)*scr->workspace_count);
 
@@ -124,11 +124,16 @@ wWorkspaceNew(WScreen *scr)
 	}
 	list[i] = wspace;
 	if (scr->workspaces)
-	    free(scr->workspaces);
+	    wfree(scr->workspaces);
 	scr->workspaces = list;
 
 	wWorkspaceMenuUpdate(scr, scr->workspace_menu);
 	wWorkspaceMenuUpdate(scr, scr->clip_ws_menu);
+#ifdef VIRTUAL_DESKTOP
+    wspace->view_x = wspace->view_y = 0;
+    wspace->height = scr->scr_height;
+    wspace->width = scr->scr_width;
+#endif
 #ifdef GNOME_STUFF
 	wGNOMEUpdateWorkspaceHints(scr);
 #endif
@@ -173,7 +178,7 @@ wWorkspaceDelete(WScreen *scr, int workspace)
         wDockDestroy(scr->workspaces[workspace]->clip);
         scr->workspaces[workspace]->clip = NULL;
     }
-    
+
     list = wmalloc(sizeof(WWorkspace*)*(scr->workspace_count-1));
     j = 0;
     for (i=0; i<scr->workspace_count; i++) {
@@ -181,11 +186,11 @@ wWorkspaceDelete(WScreen *scr, int workspace)
 	    list[j++] = scr->workspaces[i];
 	else {
 	    if (scr->workspaces[i]->name)
-		free(scr->workspaces[i]->name);
-	    free(scr->workspaces[i]);
+		wfree(scr->workspaces[i]->name);
+	    wfree(scr->workspaces[i]);
 	}
     }
-    free(scr->workspaces);
+    wfree(scr->workspaces);
     scr->workspaces = list;
 
     scr->workspace_count--;
@@ -248,9 +253,9 @@ hideWorkpaceName(void *data)
 	XUnmapWindow(dpy, scr->workspace_name);
 
 	if (scr->workspace_name_data) {
-	    RDestroyImage(scr->workspace_name_data->back);
-	    RDestroyImage(scr->workspace_name_data->text);
-	    free(scr->workspace_name_data);
+	    RReleaseImage(scr->workspace_name_data->back);
+	    RReleaseImage(scr->workspace_name_data->text);
+	    wfree(scr->workspace_name_data);
 
 	    scr->workspace_name_data = NULL;
 	}
@@ -259,7 +264,7 @@ hideWorkpaceName(void *data)
 	RImage *img = RCloneImage(scr->workspace_name_data->back);
 	Pixmap pix;
 
-	scr->workspace_name_timer = 
+	scr->workspace_name_timer =
 	    WMAddTimerHandler(WORKSPACE_NAME_FADE_DELAY, hideWorkpaceName,
 			      scr);
 
@@ -268,7 +273,7 @@ hideWorkpaceName(void *data)
 
 	RConvertImage(scr->rcontext, img, &pix);
 
-	RDestroyImage(img);
+	RReleaseImage(img);
 
 	XSetWindowBackgroundPixmap(dpy, scr->workspace_name, pix);
 	XClearWindow(dpy, scr->workspace_name);
@@ -306,9 +311,9 @@ showWorkspaceName(WScreen *scr, int workspace)
 						  hideWorkpaceName, scr);
 
     if (scr->workspace_name_data) {
-	RDestroyImage(scr->workspace_name_data->back);
-	RDestroyImage(scr->workspace_name_data->text);
-	free(scr->workspace_name_data);
+	RReleaseImage(scr->workspace_name_data->back);
+	RReleaseImage(scr->workspace_name_data->text);
+	wfree(scr->workspace_name_data);
     }
 
     data = wmalloc(sizeof(WorkspaceNameData));
@@ -368,7 +373,7 @@ showWorkspaceName(WScreen *scr, int workspace)
     }
 
     XSetForeground(dpy, scr->draw_gc, scr->white_pixel);
-    WMDrawString(scr->wmscreen, text, scr->draw_gc, scr->workspace_name_font, 
+    WMDrawString(scr->wmscreen, text, scr->draw_gc, scr->workspace_name_font,
 		 2, 2, scr->workspaces[workspace]->name,
 		 strlen(scr->workspaces[workspace]->name));
 #ifdef SHAPE
@@ -423,10 +428,10 @@ erro:
 	WMDeleteTimerHandler(scr->workspace_name_timer);
 
     if (data->text)
-	RDestroyImage(data->text);
+	RReleaseImage(data->text);
     if (data->back)
-	RDestroyImage(data->back);
-    free(data);
+	RReleaseImage(data->back);
+    wfree(data);
 
     scr->workspace_name_data = NULL;
 
@@ -482,7 +487,7 @@ void
 wWorkspaceForceChange(WScreen *scr, int workspace)
 {
     WWindow *tmp, *foc=NULL, *foc2=NULL;
-    
+
     if (workspace >= MAX_WORKSPACES || workspace < 0)
 	return;
 
@@ -505,9 +510,9 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 	    || tmp->flags.changing_workspace) {
 	    foc = tmp;
 	}
- 
+
 	/* foc2 = tmp; will fix annoyance with gnome panel
-	 * but will create annoyance for every other application 
+	 * but will create annoyance for every other application
 	 */
 
 	while (tmp) {
@@ -516,23 +521,15 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 		if ((tmp->flags.mapped||tmp->flags.shaded)
 		    && !IS_OMNIPRESENT(tmp)
 		    && !tmp->flags.changing_workspace) {
-		    
+
 		    wWindowUnmap(tmp);
 		}
                 /* also unmap miniwindows not on this workspace */
-                if (tmp->flags.miniaturized && !IS_OMNIPRESENT(tmp) 
-		    && tmp->icon) {
-                    if (!wPreferences.sticky_icons) {
-                        XUnmapWindow(dpy, tmp->icon->core->window);
-			tmp->icon->mapped = 0;
-		    }
-#if 0
-		    else {
-			tmp->icon->mapped = 1;
-			/* Why is this here? -Alfredo */
-                        XMapWindow(dpy, tmp->icon->core->window);
-		    }
-#endif
+                if (!wPreferences.sticky_icons && tmp->flags.miniaturized &&
+                    tmp->icon && !IS_OMNIPRESENT(tmp)) {
+
+                    XUnmapWindow(dpy, tmp->icon->core->window);
+                    tmp->icon->mapped = 0;
                 }
 		/* update current workspace of omnipresent windows */
 		if (IS_OMNIPRESENT(tmp)) {
@@ -635,19 +632,24 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 }
 
 #ifdef VIRTUAL_DESKTOP
+/* TODO:
+* 1) Allow border around each window so the scrolling
+* won't just stop at the border.
+* 2) Make pager.
+*/
 
-void wWorkspaceManageEdge(WScreen *scr) {
-    int i;
+void wWorkspaceManageEdge(WScreen *scr)
+{
+    int w;
     int vmask;
     XSetWindowAttributes attribs;
 
-    puts("ok");
+    /* puts("wWorkspaceManageEdge()"); */
     if (wPreferences.vedge_thickness) {
-        initVDesk=True;
-        for (i = 0;i < scr->workspace_count; i++) {
-            puts("reset workspace");
-            scr->workspaces[i]->x = scr->workspaces[i]->y = 0;
-            wWorkspaceResizeViewPort(scr, i, wPreferences.vedge_width, wPreferences.vedge_height);
+        initVDesk = True;
+        for (w = 0; w < scr->workspace_count; w++) {
+            /* puts("reset workspace"); */
+            wWorkspaceSetViewPort(scr, w, 0, 0);
         }
 
         vmask = CWEventMask|CWOverrideRedirect;
@@ -677,7 +679,9 @@ void wWorkspaceManageEdge(WScreen *scr) {
     }
 }
 
-void wWorkspaceRaiseEdge(WScreen *scr) {
+void wWorkspaceRaiseEdge(WScreen *scr)
+{
+    puts("raise edge");
     if (wPreferences.vedge_thickness && initVDesk) {
         XRaiseWindow(dpy, scr->virtual_edge_u);
         XRaiseWindow(dpy, scr->virtual_edge_d);
@@ -686,47 +690,134 @@ void wWorkspaceRaiseEdge(WScreen *scr) {
     }
 }
 
-void wWorkspaceResizeViewPort(WScreen *scr, int workspace, int width, int height)
+void wWorkspaceLowerEdge(WScreen *scr)
 {
-    if (width < scr->scr_width) return;
-    if (height < scr->scr_height) return;
-
-    scr->workspaces[workspace]->width = WMAX(width,scr->scr_width);
-    scr->workspaces[workspace]->height = WMAX(height,scr->scr_height);
-
-}
-
-void wWorkspaceSetViewPort(WScreen *scr, int workspace, int x, int y)
-{
-    int diff_x, diff_y;
-    WWindow *wwin;
-
-    if (x < 0) return;
-    if (y < 0) return;
-    if (x + scr->scr_width > scr->workspaces[workspace]->width) return;
-    if (y + scr->scr_height > scr->workspaces[workspace]->height) return;
-
-    diff_x = scr->workspaces[workspace]->x - x;
-    diff_y = scr->workspaces[workspace]->y - y;
-
-    scr->workspaces[workspace]->x = WMIN(x, scr->workspaces[workspace]->width - scr->scr_width);
-    scr->workspaces[workspace]->y = WMIN(y, scr->workspaces[workspace]->height - scr->scr_height);
-
-    printf("set view %d %d, %d\n",workspace, scr->workspaces[workspace]->x, scr->workspaces[workspace]->y);
-
-    wwin = scr->focused_window;
-    while (wwin) {
-        if (wwin->frame->workspace == workspace) {
-            wWindowMove(wwin, wwin->frame_x + diff_x, wwin->frame_y + diff_y);
-            wWindowSynthConfigureNotify(wwin);
-        }
-        wwin = wwin->prev;
+    puts("lower edge");
+    if (wPreferences.vedge_thickness && initVDesk) {
+        XLowerWindow(dpy, scr->virtual_edge_u);
+        XLowerWindow(dpy, scr->virtual_edge_d);
+        XLowerWindow(dpy, scr->virtual_edge_l);
+        XLowerWindow(dpy, scr->virtual_edge_r);
     }
 }
 
-void wWorkspaceGetViewPosition(WScreen *scr, int workspace, int *x, int *y) {
-    *x = scr->workspaces[workspace]->x;
-    *y = scr->workspaces[workspace]->y;
+void wWorkspaceResizeViewPort(WScreen *scr, int workspace, int width, int height)
+{
+    scr->workspaces[workspace]->width = WMAX(width,scr->scr_width);
+    scr->workspaces[workspace]->height = WMAX(height,scr->scr_height);
+}
+
+void updateWorkspaceGeometry(WScreen *scr, int workspace, int *view_x, int *view_y) {
+    int most_left, most_right, most_top, most_bottom;
+    WWindow *wwin;
+
+    /* adjust workspace layout */
+    wwin = scr->focused_window;
+    most_right = scr->scr_width;
+    most_bottom = scr->scr_height;
+    most_left = 0;
+    most_top = 0;
+    for(;wwin; wwin = wwin->prev) {
+        if (wwin->frame->workspace == workspace) {
+            if (!wwin->flags.miniaturized
+                    && !wwin->flags.hidden) {
+                if (wwin->frame_x < most_left) { /* record positions, should this be cached? */
+                    most_left = wwin->frame_x;
+                }
+                if ((int)wwin->frame_x + (int)wwin->frame->core->width > most_right) {
+                    most_right = wwin->frame_x + wwin->frame->core->width;
+                }
+                if (wwin->frame_y < most_top) {
+                    most_top = wwin->frame_y;
+                }
+                if (wwin->frame_y + wwin->frame->core->height > most_bottom) {
+                    most_bottom = wwin->frame_y + wwin->frame->core->height;
+                }
+            }
+        }
+    }
+
+    scr->workspaces[workspace]->width = WMAX(most_right, scr->scr_width) - WMIN(most_left, 0);
+    scr->workspaces[workspace]->height = WMAX(most_bottom, scr->scr_height) - WMIN(most_top, 0);
+
+    *view_x += -most_left - scr->workspaces[workspace]->view_x;
+    scr->workspaces[workspace]->view_x = -most_left;
+
+    *view_y += -most_top - scr->workspaces[workspace]->view_y;
+    scr->workspaces[workspace]->view_y = -most_top;
+
+}
+
+typedef struct _delay_configure {
+    WWindow *wwin;
+    int delay_count;
+} _delay_configure;
+
+void _sendConfigureNotify (_delay_configure *delay) {
+    WWindow *wwin;
+
+    delay->delay_count--;
+    if (!delay->delay_count) {
+        for (wwin = delay->wwin; wwin; wwin = wwin->prev) {
+            wWindowSynthConfigureNotify(wwin);
+        }
+    }
+}
+
+Bool wWorkspaceSetViewPort(WScreen *scr, int workspace, int view_x, int view_y)
+{
+    Bool adjust_flag = False;
+    int diff_x, diff_y;
+    static _delay_configure delay_configure = {NULL, 0};
+    WWindow *wwin;
+
+    /*printf("wWorkspaceSetViewPort %d %d\n", view_x, view_y);*/
+
+    updateWorkspaceGeometry(scr, workspace, &view_x, &view_y);
+
+    if (view_x + scr->scr_width > scr->workspaces[workspace]->width) {
+        /* puts("right edge of vdesk"); */
+        view_x = scr->workspaces[workspace]->width - scr->scr_width;
+    }
+    if (view_x < 0) {
+        /* puts("left edge of vdesk"); */
+        view_x = 0;
+    }
+    if (view_y + scr->scr_height > scr->workspaces[workspace]->height) {
+        /* puts("right edge of vdesk"); */
+        view_y = scr->workspaces[workspace]->height - scr->scr_height;
+    }
+    if (view_y < 0) {
+        /* puts("left edge of vdesk"); */
+        view_y = 0;
+    }
+
+    diff_x = scr->workspaces[workspace]->view_x - view_x;
+    diff_y = scr->workspaces[workspace]->view_y - view_y;
+    if (!diff_x && !diff_y)
+        return False;
+
+    scr->workspaces[workspace]->view_x = view_x;
+    scr->workspaces[workspace]->view_y = view_y;
+
+
+    for (wwin = scr->focused_window; wwin; wwin = wwin->prev) {
+        if (wwin->frame->workspace == workspace) {
+            wWindowMove(wwin, wwin->frame_x + diff_x, wwin->frame_y + diff_y);
+        }
+    }
+    if (1) { /* if delay*/
+        delay_configure.delay_count++;
+        delay_configure.wwin = scr->focused_window;
+        WMAddTimerHandler(200, (WMCallback *)_sendConfigureNotify, &delay_configure);
+    }
+
+    return adjust_flag;
+}
+
+void wWorkspaceGetViewPosition(WScreen *scr, int workspace, int *view_x, int *view_y) {
+    if (view_x) *view_x = scr->workspaces[workspace]->view_x;
+    if (view_y) *view_y = scr->workspaces[workspace]->view_y;
 }
 
 #endif
@@ -742,7 +833,7 @@ switchWSCommand(WMenu *menu, WMenuEntry *entry)
 static void
 deleteWSCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wWorkspaceDelete(menu->frame->screen_ptr, 
+    wWorkspaceDelete(menu->frame->screen_ptr,
 		     menu->frame->screen_ptr->workspace_count-1);
 }
 
@@ -757,14 +848,14 @@ newWSCommand(WMenu *menu, WMenuEntry *foo)
     /* autochange workspace*/
     if (ws>=0)
 	wWorkspaceChange(menu->frame->screen_ptr, ws);
-    
-    
+
+
     /*
     if (ws<9) {
 	int kcode;
 	if (wKeyBindings[WKBD_WORKSPACE1+ws]) {
 	    kcode = wKeyBindings[WKBD_WORKSPACE1+ws]->keycode;
-	    entry->rtext = 
+	    entry->rtext =
 	      wstrdup(XKeysymToString(XKeycodeToKeysym(dpy, kcode, 0)));
 	}
     }*/
@@ -775,10 +866,10 @@ static char*
 cropline(char *line)
 {
     char *start, *end;
-    
+
     if (strlen(line)==0)
 	return line;
-    
+
     start = line;
     end = &(line[strlen(line)])-1;
     while (isspace(*line) && *line!=0) line++;
@@ -810,19 +901,19 @@ wWorkspaceRename(WScreen *scr, int workspace, char *name)
     buf[MAX_WORKSPACENAME_WIDTH] = 0;
 
     /* update workspace */
-    free(scr->workspaces[workspace]->name);
+    wfree(scr->workspaces[workspace]->name);
     scr->workspaces[workspace]->name = wstrdup(buf);
 
     if (scr->clip_ws_menu) {
 	if (strcmp(scr->clip_ws_menu->entries[workspace+2]->text, buf)!=0) {
-	    free(scr->clip_ws_menu->entries[workspace+2]->text);
+	    wfree(scr->clip_ws_menu->entries[workspace+2]->text);
 	    scr->clip_ws_menu->entries[workspace+2]->text = wstrdup(buf);
 	    wMenuRealize(scr->clip_ws_menu);
 	}
     }
     if (scr->workspace_menu) {
 	if (strcmp(scr->workspace_menu->entries[workspace+2]->text, buf)!=0) {
-	    free(scr->workspace_menu->entries[workspace+2]->text);
+	    wfree(scr->workspace_menu->entries[workspace+2]->text);
 	    scr->workspace_menu->entries[workspace+2]->text = wstrdup(buf);
 	    wMenuRealize(scr->workspace_menu);
 	}
@@ -832,7 +923,7 @@ wWorkspaceRename(WScreen *scr, int workspace, char *name)
 
     if (scr->clip_icon)
         wClipIconPaint(scr->clip_icon);
-    
+
 #ifdef GNOME_STUFF
     wGNOMEUpdateWorkspaceNamesHint(scr);
 #endif
@@ -865,10 +956,10 @@ wWorkspaceMenuMake(WScreen *scr, Bool titled)
 	wwarning(_("could not create Workspace menu"));
 	return NULL;
     }
-    
+
     /* callback to be called when an entry is edited */
     wsmenu->on_edit = onMenuEntryEdited;
-    
+
     wMenuAddCallback(wsmenu, _("New"), newWSCommand, NULL);
     wMenuAddCallback(wsmenu, _("Destroy Last"), deleteWSCommand, NULL);
 
@@ -910,8 +1001,8 @@ wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu)
 	}
     }
     wMenuRealize(menu);
-    
-    for (i=0; i<scr->workspace_count; i++) {	
+
+    for (i=0; i<scr->workspace_count; i++) {
 	menu->entries[i+2]->flags.indicator_on = 0;
     }
     menu->entries[scr->current_workspace+2]->flags.indicator_on = 1;
@@ -927,7 +1018,7 @@ wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu)
     /* if menu got unreachable, bring it to a visible place */
     if (menu->frame_x < tmp - (int)menu->frame->core->width)
 	wMenuMove(menu, tmp - (int)menu->frame->core->width, menu->frame_y, False);
-    
+
     wMenuPaint(menu);
 }
 
@@ -980,7 +1071,7 @@ wWorkspaceRestoreState(WScreen *scr)
 
     if (!parr)
 	return;
-    
+
     wscount = scr->workspace_count;
     for (i=0; i < WMIN(PLGetNumberOfElements(parr), MAX_WORKSPACES); i++) {
         wks_state = PLGetArrayElement(parr, i);
@@ -991,11 +1082,11 @@ wWorkspaceRestoreState(WScreen *scr)
 	if (i >= scr->workspace_count)
 	    wWorkspaceNew(scr);
 	if (scr->workspace_menu) {
-	    free(scr->workspace_menu->entries[i+2]->text);
+	    wfree(scr->workspace_menu->entries[i+2]->text);
 	    scr->workspace_menu->entries[i+2]->text = wstrdup(PLGetString(pstr));
 	    scr->workspace_menu->flags.realized = 0;
 	}
-	free(scr->workspaces[i]->name);
+	wfree(scr->workspaces[i]->name);
         scr->workspaces[i]->name = wstrdup(PLGetString(pstr));
         if (!wPreferences.flags.noclip) {
             clip_state = PLGetDictionaryEntry(wks_state, dClip);
