@@ -46,12 +46,12 @@
 
 #include "../src/wconfig.h"
 
-#include <WINGs.h>
+#include <WINGs/WINGs.h>
 #include <wraster.h>
 
 #include <proplist.h>
 
-#define PROG_VERSION	"wmsetbg (Window Maker) 2.6"
+#define PROG_VERSION	"wmsetbg (Window Maker) 2.7"
 
 
 #define WORKSPACE_COUNT (MAX_WORKSPACES+1)
@@ -111,7 +111,7 @@ loadImage(RContext *rc, char *file)
 	wwarning("%s:could not load image file used in texture:%s", path,
 		 RMessageForError(RErrorCode));
     }
-    free(path);
+    wfree(path);
 
     return image;
 }
@@ -209,12 +209,12 @@ parseTexture(RContext *rc, char *text)
 	 case 'H':
 	    gtype = RHorizontalGradient;
 	    iwidth = scrWidth;
-	    iheight = 8;
+	    iheight = 32;
 	    break;
 	 case 'V':
 	 case 'v':
 	    gtype = RVerticalGradient;
-	    iwidth = 8;
+	    iwidth = 32;
 	    iheight = scrHeight;
 	    break;
 	 default:
@@ -235,13 +235,13 @@ parseTexture(RContext *rc, char *text)
 	if (!RConvertImage(rc, image, &pixmap)) {
 	    wwarning("could not convert texture:%s", 
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(image);
+	    RReleaseImage(image);
 	    goto error;
 	}
 
 	texture->width = image->width;
 	texture->height = image->height;
-	RDestroyImage(image);
+	RReleaseImage(image);
 
 	texture->pixmap = pixmap;
     } else if (strcasecmp(type, "mvgradient")==0
@@ -268,8 +268,8 @@ parseTexture(RContext *rc, char *text)
 		wwarning("could not parse texture %s", text);
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 	    tmp = PLGetString(val);
@@ -279,16 +279,16 @@ parseTexture(RContext *rc, char *text)
 			 tmp, text);
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 	    if (!(colors[i-2] = malloc(sizeof(RColor)))) {
 		wwarning("out of memory while parsing texture");
 
 		for (j = 0; colors[j]!=NULL; j++)
-		    free(colors[j]);
-		free(colors);
+		    wfree(colors[j]);
+		wfree(colors);
 		goto error;
 	    }
 
@@ -302,12 +302,12 @@ parseTexture(RContext *rc, char *text)
 	 case 'H':
 	    gtype = RHorizontalGradient;
 	    iwidth = scrWidth;
-	    iheight = 8;
+	    iheight = 32;
 	    break;
 	 case 'V':
 	 case 'v':
 	    gtype = RVerticalGradient;
-	    iwidth = 8;
+	    iwidth = 32;
 	    iheight = scrHeight;
 	    break;
 	 default:
@@ -320,8 +320,8 @@ parseTexture(RContext *rc, char *text)
 	image = RRenderMultiGradient(iwidth, iheight, colors, gtype);
 	
 	for (j = 0; colors[j]!=NULL; j++)
-	    free(colors[j]);
-	free(colors);
+	    wfree(colors[j]);
+	wfree(colors);
 
 	if (!image) {
 	    wwarning("could not render gradient texture:%s",
@@ -332,13 +332,13 @@ parseTexture(RContext *rc, char *text)
 	if (!RConvertImage(rc, image, &pixmap)) {
 	    wwarning("could not convert texture:%s", 
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(image);
+	    RReleaseImage(image);
 	    goto error;
 	}
 
 	texture->width = image->width;
 	texture->height = image->height;
-	RDestroyImage(image);
+	RReleaseImage(image);
 
 	texture->pixmap = pixmap;
     } else if (strcasecmp(type, "cpixmap")==0
@@ -372,7 +372,7 @@ parseTexture(RContext *rc, char *text)
 
 	if (!XParseColor(dpy, DefaultColormap(dpy, scr), tmp, &color)) {
 	    wwarning("could not parse color %s in texture %s\n", tmp, text);
-	    RDestroyImage(image);
+	    RReleaseImage(image);
 	    goto error;
 	}
 	if (!XAllocColor(dpy, DefaultColormap(dpy, scr), &color)) {
@@ -397,11 +397,11 @@ parseTexture(RContext *rc, char *text)
 	    if (!pixmap && !RConvertImage(rc, image, &pixmap)) {
 		wwarning("could not convert texture:%s", 
 			 RMessageForError(RErrorCode));
-		RDestroyImage(image);
+		RReleaseImage(image);
 		goto error;
 	    }
 	    if (image)
-		RDestroyImage(image);
+		RReleaseImage(image);
 	    break;
 	 case 'S':
 	 case 'M':
@@ -427,10 +427,10 @@ parseTexture(RContext *rc, char *text)
 		if (!simage) {
 		    wwarning("could not scale image:%s", 
 			     RMessageForError(RErrorCode));
-		    RDestroyImage(image);
+		    RReleaseImage(image);
 		    goto error;
 		}
-		RDestroyImage(image);
+		RReleaseImage(image);
 		image = simage;
 	    }
 	    iwidth = image->width;
@@ -444,7 +444,7 @@ parseTexture(RContext *rc, char *text)
 		if (!pixmap && !RConvertImage(rc, image, &pixmap)) {
 		    wwarning("could not convert texture:%s", 
 			     RMessageForError(RErrorCode));
-		    RDestroyImage(image);
+		    RReleaseImage(image);
 		    goto error;
 		}
 
@@ -483,7 +483,7 @@ parseTexture(RContext *rc, char *text)
 		    pixmap = cpixmap;
 		}
 		if (image)
-		    RDestroyImage(image);
+		    RReleaseImage(image);
 
 		texture->width = scrWidth;
 		texture->height = scrHeight;
@@ -564,8 +564,8 @@ parseTexture(RContext *rc, char *text)
 	if (!gradient) {
 	    wwarning("could not render texture:%s",
 		     RMessageForError(RErrorCode));
-            RDestroyImage(gradient);
-            RDestroyImage(image);
+            RReleaseImage(gradient);
+            RReleaseImage(image);
 	    goto error;
 	}
 
@@ -573,25 +573,25 @@ parseTexture(RContext *rc, char *text)
 	if (!tiled) {
 	    wwarning("could not render texture:%s",
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(gradient);
-	    RDestroyImage(image);
+	    RReleaseImage(gradient);
+	    RReleaseImage(image);
 	    goto error;
 	}
-	RDestroyImage(image);
+	RReleaseImage(image);
 
 	RCombineImagesWithOpaqueness(tiled, gradient, opaq);
-	RDestroyImage(gradient);
+	RReleaseImage(gradient);
 
 	if (!RConvertImage(rc, tiled, &pixmap)) {
 	    wwarning("could not convert texture:%s", 
 		     RMessageForError(RErrorCode));
-	    RDestroyImage(tiled);
+	    RReleaseImage(tiled);
 	    goto error;
 	}
 	texture->width = tiled->width;
 	texture->height = tiled->height;
 
-	RDestroyImage(tiled);
+	RReleaseImage(tiled);
 
 	texture->pixmap = pixmap;	
     } else if (strcasecmp(type, "function")==0) {
@@ -658,14 +658,14 @@ function_cleanup:
         if (argv) {
 	    int i;
 	    for (i=0; i<argc; i++) {
-	        free(argv[i]);
+	        wfree(argv[i]);
 	    }
 	}
 	if (handle) {
 	    dlclose(handle);
 	}
 	if (image) {
-            RDestroyImage(image);
+            RReleaseImage(image);
 	}
 	if (!success) {
 	    goto error;
@@ -685,7 +685,7 @@ function_cleanup:
 
 error:
     if (texture)
-	free(texture);
+	wfree(texture);
     if (texarray)
 	PLRelease(texarray);
 
@@ -708,8 +708,8 @@ freeTexture(BackgroundTexture *texture)
     if (texture->pixmap) {
 	XFreePixmap(dpy, texture->pixmap);
     }
-    free(texture->spec);
-    free(texture);
+    wfree(texture->spec);
+    wfree(texture);
 }
 
 
@@ -980,7 +980,7 @@ helperLoop(RContext *rc)
 	    printf("change pixmappath %s\n", &buffer[1]);
 #endif
 	    if (PixmapPath)
-		free(PixmapPath);
+		wfree(PixmapPath);
 	    PixmapPath = wstrdup(&buffer[1]);
 	    break;
 
@@ -1010,8 +1010,9 @@ updateDomain(char *domain, char *key, char *texture)
 {
     char *program = "wdwrite";
 
-    system(wstrappend("wdwrite ", 
-		      wstrappend(domain, smooth ? " SmoothWorkspaceBack YES"
+    /* here is a mem leak */
+    system(wstrconcat("wdwrite ", 
+		      wstrconcat(domain, smooth ? " SmoothWorkspaceBack YES"
 				 : " SmoothWorkspaceBack NO")));
 
     execlp(program, program, domain, key, texture, NULL);
@@ -1047,7 +1048,7 @@ getValueForKey(char *domain, char *keyName)
     if (!d) {
 	wwarning("could not open domain file %s", path);
     }
-    free(path);
+    wfree(path);
 
     if (d && !PLIsDictionary(d)) {
 	PLRelease(d);
@@ -1066,7 +1067,7 @@ getValueForKey(char *domain, char *keyName)
 	    d = NULL;
 	} else {
 	    d = PLGetProplistWithPath(path);
-	    free(path);
+	    wfree(path);
 	}
 
 	if (d && !PLIsDictionary(d)) {
@@ -1155,21 +1156,21 @@ getFullPixmapPath(char *file)
 	char *path = wmalloc(bsize);
 	
 	while (!getcwd(path, bsize)) {
-	    free(path);
+	    wfree(path);
 	    bsize += bsize/2;
 	    path = malloc(bsize);
 	}
 
-	tmp = wstrappend(path, "/");
-	free(path);
-	path = wstrappend(tmp, file);
-	free(tmp);
+	tmp = wstrconcat(path, "/");
+	wfree(path);
+	path = wstrconcat(tmp, file);
+	wfree(tmp);
 
 	return path;
     }
     
     /* the file is in the PixmapPath */
-    free(tmp);
+    wfree(tmp);
 
     return wstrdup(file);
 }
@@ -1429,15 +1430,21 @@ main(int argc, char **argv)
     rattr.standard_colormap_mode = RCreateStdColormap;
 
     rc = RCreateContext(dpy, scr, &rattr);
+
     if (!rc) {
-	wfatal("could not initialize wrlib:",
+        rattr.standard_colormap_mode = RIgnoreStdColormap;
+        rc = RCreateContext(dpy, scr, &rattr);
+    }
+    
+    if (!rc) {
+	wfatal("could not initialize wrlib: %s",
 	       RMessageForError(RErrorCode));
 	exit(1);
     }
 
     if (helperMode) {
 	/* lower priority, so that it wont use all the CPU */
-	nice(1000);
+	nice(15);
 
 	helperLoop(rc);
     } else {
@@ -1448,7 +1455,7 @@ main(int argc, char **argv)
 	    char *image_path = getFullPixmapPath(image_name);
 	    
 	    sprintf(buffer, "(%s, \"%s\", %s)", style, image_path, back_color);
-	    free(image_path);
+	    wfree(image_path);
 	    texture = (char*)buffer;
 	}
 
