@@ -49,7 +49,7 @@ RGetPixel(RImage *image, int x, int y, RColor *color)
 
     if (image->format == RRGBAFormat) {
 	ofs = (y*image->width + x) * 4;
-	color->red = image->data[ofs];
+	color->red = image->data[ofs++];
 	color->green = image->data[ofs++];
 	color->blue = image->data[ofs++];
 	color->alpha = image->data[ofs];
@@ -57,7 +57,7 @@ RGetPixel(RImage *image, int x, int y, RColor *color)
 	ofs = (y*image->width + x) * 3;
 	color->red = image->data[ofs++];
 	color->green = image->data[ofs++];
-	color->blue = image->data[ofs++];
+	color->blue = image->data[ofs];
 	/* If the image does not have alpha channel, we consider alpha 255 */
 	color->alpha = 255;
     }
@@ -98,9 +98,9 @@ RPutPixel(RImage *image, int x, int y, RColor *color)
         alpha = color->alpha;
         nalpha = 255 - alpha;
 
-        *ptr++ = (((int)*ptr * nalpha) + (r * alpha))/256;
-        *ptr++ = (((int)*ptr * nalpha) + (g * alpha))/256;
-        *ptr++ = (((int)*ptr * nalpha) + (b * alpha))/256;
+        *ptr = (((int)*ptr * nalpha) + (r * alpha))/256; ptr++;
+        *ptr = (((int)*ptr * nalpha) + (g * alpha))/256; ptr++;
+        *ptr = (((int)*ptr * nalpha) + (b * alpha))/256; ptr++;
         if (image->format == RRGBAFormat) {
             *ptr = alpha + ((int)*ptr * nalpha)/256;
 	}
